//---------------------------------------------------------------------------------
// TabSet.js ( author : Laurent Gurin )
// TabSet object : 
//  - a TabSet is composed of 1 to N tabs
//  - each tab is associated with a TabPannel
//---------------------------------------------------------------------------------
function TabSet(sClassNormal, sClassSelected )
{
	//==============================================================================
	// INSTANCE ATTRIBUTES
	//==============================================================================
	var _currentTab      = null ; // Current selected tab object ( TD tag )
	var _currentTabIndex = -1 ;   // Current selected tab index ( 0 to N )
	var _currentPanel    = null ; // Current visible panel ( DIV tag )
		
	var _tabClassNormal   = sClassNormal ;   // CSS class for a normal tab (not selected)
	var _tabClassSelected = sClassSelected ; // CSS class for a selected tab

	//==============================================================================
	// PUBLIC / PRIVILEGED  METHODS 
	//==============================================================================		
	this.getCurrentTabIndex = function()
	{
		return _currentTabIndex ;
	}

	this.showPanel = function  ( oPanel )
	{
		//--- Cacher la page en cours 
		if ( _currentPanel != null )
		{
			_currentPanel.style.visibility = 'hidden' ;
		}
		//--- Changer de page en cours 
		_currentPanel = oPanel ;
		//--- Afficher la page en cours 
		if ( _currentPanel != null )
		{
			_currentPanel.style.visibility = 'visible' ;
		}
	}

	this.showTab = function  ( oTab )
	{
		if ( oTab != null )
		{
			//--- Neutralise l'onglet en cours 
			if ( _currentTab != null )
			{
				_currentTab.className = _tabClassNormal ;
			}
			//--- Active l'onglet en cours 
			oTab.className = _tabClassSelected ;
			//--- Set the new current tab  
			_currentTab = oTab ;
			_currentTabIndex = oTab.tabIndex ;
		}			
	}
}

function fwkInitTab ( sTabId, iTabIndex, sPanelId, oTabSet )
{
	var oTab = document.getElementById(sTabId);
	if ( oTab == null ) 
	{
		alert ( "fwkLinkTabToPanel : tab '" + sTabId + "' not found ! " );
		return ;
	}
	//--- Set the current tabset object 
	oTab.tabSet = oTabSet ;
	//--- Set the index
	oTab.tabIndex = iTabIndex ;
	//--- Link Tab object with the given panel 
	oTab.linkedPanelId = sPanelId ;
	//alert ( sTabId + " linked to " + sPanelId ) ; 
}
	
function fwkSelectTab( tab )
{
	if ( tab == null ) 
	{
		alert( "fwkSelectTab() : tab is null !" );
		return false ;
	}
	if ( typeof tab == "string" )
	{
		return fwkSelectTabById( tab );
	}
	else
	{
		return fwkSelectTabByRef( tab );
	}
}

function fwkSelectTabById( sTabId )
{
	if ( sTabId == null ) 
	{
		alert( "fwkSelectTabById() : tab id is null !" );
		return false ;
	}
	else
	{
		var oTab = document.getElementById(sTabId);
		if ( oTab == null ) 
		{
			alert ( "ERROR : Tab  "+sTabId+" doesn't exist ( check the ID ) ! " );
			return false ;
		}
		else
		{
			return fwkSelectTabByRef(oTab);
		}	
	}
}
	
function fwkSelectTabByRef( oTab )
{
	if ( oTab == null ) 
	{
		alert ( "ERROR : Tab argument is null ! " );
		return false ;
	}
	else
	{
		if ( oTab.tabSet )
		{
			//--- Check this tab is not already selected
			if ( "tabIndex" in oTab )
			{
				if ( oTab.tabIndex == oTab.tabSet.getCurrentTabIndex() )
				{
					//--- Already selected
					return false ;
				}
			}
			else
			{
				alert ( "ERROR : Tab is not initialized (no tabIndex attribute) ! " );
				return false;
			}			
			//--- Select the new tab 
			oTab.tabSet.showTab ( oTab ) ;
			//--- If the tab is linked with a panel
			if ( oTab.linkedPanelId )
			{
				//--- Get the panel object
				if ( ! oTab.linkedPanel )
				{
					//--- Not yet initialized => search by ID
					oTab.linkedPanel = document.getElementById(oTab.linkedPanelId);
				}					
				if ( oTab.linkedPanel )
				{
					//--- Show the linked panel 
					oTab.tabSet.showPanel ( oTab.linkedPanel ) ;
				}
				else
				{
					//--- Still null => invalid ID
					alert ( "fwkSelectTabByRef : pannel '" + oTab.linkedPanelId + "' not found ! " );
					return false;							
				}	
			}			
		}
		else
		{
			alert ( "ERROR : Tab is not initialized (no tabset attribute) ! " );
			return false;
		}			
	}
	return true ;
}
