//------------------------------------------------------------------------------
// fwkScreenResponse.js
//------------------------------------------------------------------------------
// Version : 1.0  ( 12/12/2005 )
// Author  : Laurent Gurin 
// Changes : none (creation)
//------------------------------------------------------------------------------

//------------------------------------------------------------------------------
// CLASS DEFINITION
//------------------------------------------------------------------------------
function ScreenResponse( oXmlHttpRequest )
{
	//==============================================================================
	// STANDARD ATTRIBUTES & METHODS
	//==============================================================================
	var _sClassName = "ScreenResponse" ;
	var _bTrace = false ;
	//------------------------------------------------------------------------------
	function _trace ( sMsg ) 
	{
		if ( _bTrace ) alert ( "TRACE : [" + _sClassName + "] \n\n" + sMsg ) ;
	}
	//------------------------------------------------------------------------------
	function _error ( sMsg ) 
	{
		alert ( "ERROR : [" + _sClassName + "] \n\n" + sMsg ) ;
	}
	//------------------------------------------------------------------------------
	
	//==============================================================================
	// PRIVATE ATTRIBUTES
	//==============================================================================
	var _iReadyState     = oXmlHttpRequest.readyState ; // 1, .. , 4 (complete)
	var _iHttpStatusCode = oXmlHttpRequest.status ; // HTTP Status Code ( ie : "200" = "OK" )
	var _sHttpStatusText = oXmlHttpRequest.statusText ; // HTTP Status Text ( ie : "OK" )
	var _sResponseText   = oXmlHttpRequest.responseText ; // Response Body as text 
	var _oResponseXML    = oXmlHttpRequest.responseXML  ; // Response Body as XML Document (DOM tree)
	var _sReturnCode     = "" ; // Telosys Return Code ( ie : "0" for "OK" )
	var _sReturnMessage  = "" ; // Telosys Return Message 
	var _htValues        = null ; // Values hashtable : init by getValues()
	
	if ( _iReadyState != 4) // 4 = complete
	{
		//--- Error
		_error("Request state is NOT COMPLETE ( readyState = " + _iReadyState + " ) ");
	}
	else
	{
		if ( _iHttpStatusCode != 200 ) // 200 = OK
		{
			//--- Error
			_error("HTTP ERROR ( Status = " + _iHttpStatusCode + " - " + _sHttpStatusText + " ) ");
		}
		else
		{
			_initReturn() ;
		}
	}
	
	//==============================================================================
	// PRIVATE METHODS 
	//==============================================================================
	function _initReturn() 
	{
		var tagReturn = _getXmlSingleTag("return");
		if ( tagReturn == null )
		{
			_error("No <return> tag in the response !");
		}
		else
		{
			//--- Get the "code" attribute
			try {
				var sRetCode = tagReturn.getAttribute("code") ;
				if ( sRetCode == null )
				{
					_sReturnCode = "" ;
					_error("No 'code' attribute in the <return> tag !");
				}
				else
				{
					_sReturnCode = sRetCode ;
				}
			} catch (e) {
				_error("Exception on getAttribute('code') in <return> tag !\nException : " + e.message );
			}
			
			//--- Get the "message" attribute
			try {
				var sRetMessage = tagReturn.getAttribute("message") ;
				if ( sRetMessage == null )
				{
					_sReturnMessage = "" ;
					_error("No 'message' attribute in the <return> tag !");
				}
				else
				{
					_sReturnMessage = sRetMessage ;
				}
			} catch (e) {
				_error("Exception on getAttribute('message') in <return> tag !\nException : " + e.message );
			}		
		}
	}
	//------------------------------------------------------------------------------
	function _getXmlSingleTag ( sTagName ) 
	{
		if ( _oResponseXML == null )
		{
			_error("_getXmlSingleTag(" + sTagName + ") : Xml response is null !");
			return null ;
		}
		var tag = null ;
		try {
			var allTags = _oResponseXML.getElementsByTagName( sTagName ) ;
			if ( allTags != null )
			{
				_trace ( "_getXmlSingleTag(" + sTagName + ") : " + allTags.length + " element(s)" );			
				//--- There is a single tag for this name so we take the first one
				tag = allTags.item(0) ;
			}
			else
			{
				_error("_getXmlSingleTag() : No tag <" + sTagName + "> in the XML response ! \n getElementsByTagName(..) return null !");
			}
		} catch (e) {
			_error("_getXmlSingleTag(" + sTagName + ") \nException : " + e.message );
		}
		if ( tag == null )
		{
			_error("No tag <" + sTagName + "> in the XML response !");
		}
		return tag ;
	}
	
	//==============================================================================
	// PUBLIC / PRIVILEGED  METHODS 
	//==============================================================================
	this.isOK = function () 
	{
		if ( ( _iReadyState == 4 ) && ( _iHttpStatusCode == 200 ) && ( _sReturnCode == "0" ) ) 
		{
			return true ;
		}
		else
		{
			return false ;
		}
	}
	//------------------------------------------------------------------------------
	// Return the response as TEXT flow
	//------------------------------------------------------------------------------
	this.getResponseText = function () 
	{
		return _sResponseText ;
	}
	//------------------------------------------------------------------------------
	this.getResponseXML = function () 
	{
		return _oResponseXML ;
	}
	//------------------------------------------------------------------------------
	//------------------------------------------------------------------------------
	this.getReadyState = function () 
	{
		return _iReadyState ;
	}
	//------------------------------------------------------------------------------
	//------------------------------------------------------------------------------
	this.getHttpStatusCode = function () 
	{
		return _iHttpStatusCode ;
	}
	//------------------------------------------------------------------------------
	this.getHttpStatusText = function () 
	{
		return _sHttpStatusText ;
	}
	//------------------------------------------------------------------------------
	this.getReturnCode = function () 
	{
		return _sReturnCode ;
	}
	//------------------------------------------------------------------------------
	this.getReturnMessage = function () 
	{
		return _sReturnMessage ;
	}
	//------------------------------------------------------------------------------
	this.dataNotFound = function ()
	{
		//var tagReturn = this.getXmlSingleTag("return") ;
		var tagReturn = _getXmlSingleTag("return") ;
		if ( tagReturn != null )
		{
			//--- Get the given TAG  
			var tagNotFound = null ;
			try {
				tagNotFound = tagReturn.getElementsByTagName("not-found").item(0);
			} catch (e) {
				tagNotFound = null ;
			}
			return ( tagNotFound != null ? true : false ) ;
		}
		return false ;
	}
	//------------------------------------------------------------------------------
	this.dataFound = function ()
	{
		var tagReturn = _getXmlSingleTag("return") ;
		if ( tagReturn != null )
		{
			//--- Get the given TAG  
			var tagFound = null ;
			try {
				tagFound = tagReturn.getElementsByTagName("found").item(0);
			} catch (e) {
				tagFound = null ;
			}
			return ( tagFound != null ? true : false ) ;
		}
		return false ;
	}
	//------------------------------------------------------------------------------
	// Return true if <response> <data void="true" /> </response> 
	//------------------------------------------------------------------------------
	this.dataVoid = function ()
	{
		//alert( "ScreenResponse.dataVoid()..." );
		var bRet = false ;
		var tagData = _getXmlSingleTag("data") ;
		if ( tagData != null )
		{
			//--- Get the "void" attribute  
			var attribVoid = null ;
			try {
				attribVoid = tagData.getAttribute("void");
			} catch (e) {
				attribVoid = null ;
			}
			bRet = ( attribVoid != null ? true : false ) ;
		}
		else
		{
			//--- No tag <data> => VOID
			bRet = true ;
		}
		//alert( "ScreenResponse.dataVoid() : return = " + bRet  );
		return bRet ;
	}
	
	//------------------------------------------------------------------------------
	this.show = function () 
	{
		var sMsg = "RESPONSE : \n" ;
		sMsg += "Request state : " + _iReadyState + " ( 4 = complete ) \n" ;
		sMsg += "Http Status : " + _iHttpStatusCode + " - " + _sHttpStatusText + " \n" ;
		sMsg += "Response text : \n" + _sResponseText  ;
		/**
		for ( p	in this.oParameters )
		{
			sParams += p + "=" + this.oParameters[p] + "\n" ;
		}
		**/
		alert( sMsg ); 
	}
	//------------------------------------------------------------------------------
	this.getXmlTagData = function () 
	{
		var tagData = null ;
		try {
			var allTagsData = _oResponseXML.getElementsByTagName( "data" ) ;
			if ( allTagsData != null )
			{
				//--- There is a single tag <data> so we take the first one
				tagData = allTagsData.item(0) ;
			}
		} catch (e) {
			_error("getXmlTagData() \nException : " + e.message );
		}
		return tagData ;
	}
	//------------------------------------------------------------------------------
	this.getXmlValue = function ( sTag, sAttr ) 
	{
		if ( _oResponseXML == null )
		{
			_error("Xml response is null !");
			return null ;
		}
		//--- Get the tag <data>
		var tagData = _getXmlSingleTag("data") ;
		if ( tagData == null )
		{
			_error("No <data> in the response !");
			return null ;
		}		
		//--- Get the given TAG  
		var xmlTag = null ;
		try {
			xmlTag = tagData.getElementsByTagName(sTag).item(0);
		} catch (e) {
			xmlTag = null ;
		}
		if ( xmlTag == null )
		{
			//_trace(sTag+"." + sAttr + "\nTag not found.");
			return null ;
		}
		//--- Get the given ATTRIBUTE  
		try {
			var sRet = xmlTag.getAttribute(sAttr) ;
			//_trace(sTag+"." + sAttr + " = [" + sRet + "]" );
			/**
			if ( sRet != null )
			{
				_trace(sTag+"." + sAttr + "\nOK, found : [" + sRet + "]" );
			}
			else
			{
				_trace(sTag+"." + sAttr + "\nAttribute not found." );
			}
			**/
			return sRet ;
		} catch (e) {
			_error(sTag+"." + sAttr + "\nError on getAttribute() !\nException : " + e.message );
			return null ;
		}
		return null ;
	}

	//------------------------------------------------------------------------------
	this.getMessages = function () 
	{
		var tagReturn = _getXmlSingleTag("return") ;
		if ( tagReturn != null )
		{
			//--- Get the TAG  
			var tagMessages = null ;
			try {
				tagMessages = tagReturn.getElementsByTagName("messages").item(0);
			} catch (e) {
				tagMessages = null ;
			}
			if ( tagMessages != null )
			{
				//alert ( "<messages> found " ) ;
				messages = tagMessages.getElementsByTagName("message");
				if ( messages != null )
				{
					var msgList = [] ;
					var i = 0 ;
					var n = messages.length ;
					var msg = null ;
					//alert ( "nb messages = " + n ) ;
					var code = null ;
					var text = null ;
					var type = null ;
					for ( i = 0 ; i < n ; i++ )
					{
						msg = messages.item(i) ;
						try {
							code = msg.getAttribute("code");							
							text = msg.getAttribute("text");
							type = msg.getAttribute("type");
						} catch (e) {
							_error("getMessages() \nError on getAttribute() !\nException : " + e.message );
							return null ;
						}
						msgList[i] = new ScreenMessage(code, text, type);
						//alert ( "message " + i + " : " + msgList[i].toString() ) ;
					} 
					return msgList;
				}
			}
		}
		return null ;
	}
	//------------------------------------------------------------------------------
	// Return the value of the given value name
	// <values> <value-name value="xxx" /> </values>
	//------------------------------------------------------------------------------
	this.getValue = function ( sName ) 
	{
		var htValues = this.getValues();
		if ( htValues != null )
		{
			return htValues[sName];
		}
		return null ;
	}
	//------------------------------------------------------------------------------
	// Return all the values of the response ( hashtable )
	// <values> <value-name value="xxx" /> </values>
	//------------------------------------------------------------------------------
	this.getValues = function () 
	{
		if ( _htValues != null ) // if already loaded
		{
			return _htValues ; // return the existing hashtable
		}
		//--- Get the TAG  
		var tagValues = _getXmlSingleTag("values") ;
		if ( tagValues != null )
		{
			//alert ( "<values> found " ) ;
			try {
				var values = tagValues.getElementsByTagName("value");
				if ( values != null )
				{
					var htValues = new Object() ;
					var value = null ;
					var i = 0 ;
					var n = values.length ;
					//alert ( "nb values = " + n ) ;
					for ( i = 0 ; i < n ; i++ ) // for each tag <value>
					{
						value = values.item(i) ;
						if ( value.attributes != null )
						{
							if ( value.attributes.length > 0 )
							{
								// store in the Values HashTable 
								htValues[value.attributes[0].name] = value.attributes[0].value ;
							}								
						}
					} 
					if ( _htValues == null )
					{
						_htValues = htValues ;
					}
					return htValues;
				}
			} catch (ex) {
				_error("getValues() \n Exception : " + ex.message );
				return null ;
			}
		}
		return null ;
	}	
	//------------------------------------------------------------------------------
	this.showValues = function () 
	{
		var htValues = this.getValues();
		if ( htValues != null )
		{
			var s = "" ;
			for ( name in htValues )
			{
				s = s + "\n . " + name + " = " + htValues[name] ;
			}
			alert ( "Values : " + s ) ;
		}
		else
		{
			_error("No values in the response !");
		}	
	}
	//------------------------------------------------------------------------------
/***
	this.getViewContent = function ()
	{
		var elements = _oResponseXML.getElementsByTagName( "view" );
		if ( elements == null )
		{
			//_error( "Tag <view> not found ( getElementsByTagName return null ) !" ) ; 
			return null ;
		}
		
		_trace ( elements.length + " <view> element(s)" );
		if ( elements.length < 1 )
		{
			//_error( "Tag <view> not found ( 0 elements )!" ) ; 
			return null ;
		}

		//--- There is a single tag for this name => take the first one
		var elementView = elements.item(0) ;
		if ( ! elementView.hasChildNodes() )
		{
			//_error( "Tag <view> is void ( no child nodes )!" ) ; 
			return null ;
		}
		
		_trace("Element <view> has " + elementView.childNodes.length + " child node(s)" );
		//--- Search the first CDATA-SECTION in the child nodes 
		for ( i = 0 ; i < elementView.childNodes.length ; i++ )
		{
			node = elementView.childNodes[i];
			//_trace ("Node [ " + i + " ] : name = " + node.nodeName + " type = " + node.nodeType + " value = " + node.nodeValue );
			if ( node.nodeType == 4 ) // Type 4 = "#cdata-section" / Type 3 = "#text"
			{
				_trace ("Node Content found : name = " + node.nodeName + " type = " + node.nodeType + " value = " + node.nodeValue );
				//--- Return the CDATA-SECTION content
				return node.nodeValue ;
			}
		}
		_error( "No CDATA section in the tag <view> !" ) ; 
	}
***/
	//------------------------------------------------------------------------------
	this.getViewContent = function ( sViewTagName )
	{
		var elements = _oResponseXML.getElementsByTagName( "view" );
		if ( elements == null )
		{
			//_error( "Tag <view> not found ( getElementsByTagName return null ) !" ) ; 
			return null ;
		}
		
		_trace ( elements.length + " <view> element(s)" );
		if ( elements.length < 1 )
		{
			//_error( "Tag <view> not found ( 0 elements )!" ) ; 
			return null ;
		}

		//--- There is a single tag for this name => take the first one
		var elementView = elements.item(0) ;
		if ( ! elementView.hasChildNodes() )
		{
			//_error( "Tag <view> is void ( no child nodes )!" ) ; 
			return null ;
		}
		_trace("Element <view> has " + elementView.childNodes.length + " child node(s)" );
		
		alert("getViewContent : view.getElementsByTagName(" + sViewTagName + ")" );
		var oNodeList = elementView.getElementsByTagName( sViewTagName );
		if ( oNodeList == null )
		{
			return null ;
		}
		if ( oNodeList.length < 1 )
		{
			return null ;
		}
		// Unique => Get the fist element of the list 
		elementView = oNodeList.item(0) ;
		
		//--- Search the first CDATA-SECTION in the child nodes 
		for ( i = 0 ; i < elementView.childNodes.length ; i++ )
		{
			node = elementView.childNodes[i];
			//_trace ("Node [ " + i + " ] : name = " + node.nodeName + " type = " + node.nodeType + " value = " + node.nodeValue );
			if ( node.nodeType == 4 ) // Type 4 = "#cdata-section" / Type 3 = "#text"
			{
				_trace ("Node Content found : name = " + node.nodeName + " type = " + node.nodeType + " value = " + node.nodeValue );
				//--- Return the CDATA-SECTION content
				return node.nodeValue ;
			}
		}
		_error( "No CDATA section in the tag <view> !" ) ; 
	}
}
//------------------------------------------------------------------------------
//                                  F  I  N
//------------------------------------------------------------------------------
