package org.objectweb.telosys.auth;

import java.io.IOException;

import javax.servlet.Filter;
import javax.servlet.FilterChain;
import javax.servlet.FilterConfig;
import javax.servlet.ServletException;
import javax.servlet.ServletRequest;
import javax.servlet.ServletResponse;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.objectweb.telosys.common.TelosysObject;

/**
 * Filter to restrict access for authenticated users.
 * Authenticated users can pass through the filter
 * Non authenticated users can acces only to the authentication servlet
 * 
 * @author Laurent GUERIN 
 * 
 */ 
public class AuthFilter extends TelosysObject implements Filter
{
	private String _sLoginUrl = null ; // "http://localhost:8080/paie/login/auth";

	/**
	 * @see javax.servlet.Filter#init(javax.servlet.FilterConfig)
	 */
	public void init(FilterConfig config) throws ServletException
	{
		//setFlagTrace(true);
		String sParam = config.getInitParameter("LoginUrl");
		if ( sParam != null )
		{
			_sLoginUrl = sParam ;
		}
		info("init() : Login URL = " + _sLoginUrl );
	}

	/**
	 * @see javax.servlet.Filter#destroy()
	 */
	public void destroy()
	{
		trace("destroy()");
	}

	/**
	 * @see javax.servlet.Filter#doFilter(javax.servlet.ServletRequest, javax.servlet.ServletResponse, javax.servlet.FilterChain)
	 */
	public void doFilter(final ServletRequest request, final ServletResponse response, final FilterChain chain) throws IOException, ServletException
	{
		trace("doFilter()...");

		HttpServletRequest httpRequest = (HttpServletRequest) request;
		HttpServletResponse httpResponse = (HttpServletResponse) response;
		
		//--- Get the full URL of the request
		String sQueryString = httpRequest.getQueryString() ;
		String sFullURL = null ; 
		if ( sQueryString != null )
		{
		    sFullURL = httpRequest.getRequestURL().toString() + "?" + sQueryString ;
		}
		else
		{
		    sFullURL = httpRequest.getRequestURL().toString() ;		    
		}
		trace("doFilter() : URL = " + sFullURL );
		
		//--- Check if user is already authenticated  
		if ( Auth.isAuthenticated(httpRequest) ) 
		{
			trace("doFilter() : User authenticated.");			
			info("Authentication Filter : URL = " + sFullURL + " : OK.");
			//--- Authenticated user => access permitted => chain
			chain.doFilter(request, response);
		}
		else
		{
			//--- Not authenticated user : => Force authentication ( BASIC or with login URL )
			trace("doFilter() : User not authenticated.");
			if ( _sLoginUrl != null )
			{
				String sRequestedUrl = httpRequest.getRequestURL().toString() ;
				if (sRequestedUrl.endsWith(_sLoginUrl))
				{
					//--- login URL => permitted
					trace("chain ( access permitted to Login URL )" );
					info("Authentication Filter : URL = " + sFullURL + " - User not authenticated - Access permitted to Login URL.");
					chain.doFilter(request, response);
				}
				else
				{
					//--- other URL => redirect to login URL
					String sLoginUrl = null ;
					String sContextPath = httpRequest.getContextPath();
					if ( _sLoginUrl.startsWith(sContextPath) )
					{
						sLoginUrl = _sLoginUrl ;
					}
					else
					{
						sLoginUrl = sContextPath + _sLoginUrl ;						
					}
					trace("sendRedirect to Login URL -> " + sLoginUrl);
					info("Authentication Filter : URL = " + sFullURL + " - User not authenticated - Redirected to Login URL.");
					httpResponse.sendRedirect(sLoginUrl);
				}				
			}
			else
			{
			    //--- No Login URL => force authentication ( Basic Http Authentication ) 
				info("Authentication Filter : URL = " + sFullURL + " - User not authenticated - Http authentication requested.");
				//WebUtil.msgPage(httpResponse, "Authentication error", "No login URL !");
				Auth.checkAuthentication(httpRequest,httpResponse);				
			}
		}
	}
}
