package org.objectweb.telosys.common;

import java.io.IOException;

import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.objectweb.telosys.uil.TelosysUIL;
import org.objectweb.telosys.uil.screenmap.ScreenDefinitions;
import org.objectweb.telosys.uil.screenmap.ScreenMap;
import org.objectweb.telosys.uil.screenmap.ScreenMapConst;
import org.objectweb.telosys.util.web.WebUtil;

/**
 * ScreenMap implementation of the View interface<br>
 * To use a screenmap as a standard "MVC view"
 *  
 * @author Laurent GUERIN
 *  
 */
public class ScreenView extends TelosysObject implements View
{

    private String    _sName      = null;

    //private String    _sId        = null;

    private int       _iContextId = 0;

    private String    _sType      = null;

    private String    _sAction    = null;

    private ScreenMap _screenMap  = null;

    private String    _sTarget    = null;

    private ScreenView() // no default constructor
    {

    }

    public ScreenView(String sName, int iId) throws TelosysException
    {
        _sName = sName;
        _iContextId = iId;
        _sType = TelosysUIL.getDefaultScreenType();
        initView();
    }

    public ScreenView(String sName, int iId, String sType) throws TelosysException
    {
        _sName = sName;
        _iContextId = iId;
        _sType = sType;
        initView();
    }

    private void initView() throws TelosysException
    {
        //--- Get the screen definitions
        ScreenDefinitions screenDefinitions = TelosysUIL.getScreenDefinitions();
        if (screenDefinitions == null)
        {
            throw new TelosysException("Screens definitions are not initialized ! ");
        }

        //--- Get the screen map for the current screen
        _screenMap = screenDefinitions.getScreenMap(_sName, _sType, _iContextId);
        if (_screenMap == null)
        {
            String sMsg = "Cannot get the screen map : getScreenMap(" + _sName + "," + _sType + "," + _iContextId + ")";
            throw new TelosysException(sMsg);
        }

        //--- Retrieve the main target for the screen map
        _sTarget = _screenMap.getTargetURL();
        if (_sTarget == null)
        {
            String sMsg = "No target for the screen : " + _sName + " ( type = " + _sType + " )";
            throw new TelosysException(sMsg);
        }
    }

    public String getAction()
    {
        return _sAction;
    }

    public int getId()
    {
        return _iContextId ;
    }

    public String getName()
    {
        return _sName;
    }

    public String getType()
    {
        return _sType;
    }

    public ScreenMap getScreenMap()
    {
        return _screenMap;
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.objectweb.telosys.common.View#getTarget()
     */
    public String getTarget()
    {
        return _sTarget;
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.objectweb.telosys.common.View#generate()
     */
    public void generate(HttpServletRequest httpRequest, HttpServletResponse httpResponse) throws TelosysException
    {
        if (_screenMap == null)
        {
            String sMsg = "No screen map for this view : " + _sName + " ( id = " + _iContextId + " type = " + _sType + " )";
            throw new TelosysException(sMsg);
        }
        if (_sTarget == null)
        {
            String sMsg = "No target for this view : " + _sName + " ( type = " + _sType + " )";
            throw new TelosysException(sMsg);
        }

        //--- Set the Screen Map object as request attribute
        httpRequest.setAttribute(ScreenMapConst.SCREEN_MAP, _screenMap);
        trace("Screen map set as request attribute");

        //--- Try to forward to the Screen Map Target
        trace("Forward to " + _sTarget + " ... ");
        boolean bForwardOk = false;
        String sMsgForwardError = "Cannot forward to target '" + _sTarget + "' ";
        try
        {
            bForwardOk = WebUtil.forward(_sTarget, httpRequest, httpResponse);
        } catch (ServletException e)
        {
            throw new TelosysException(sMsgForwardError + " (ServletException) ", e);
        } catch (IOException e)
        {
            throw new TelosysException(sMsgForwardError + " (IOException) ", e);
        }
        if (!bForwardOk)
        {
            throw new TelosysException(sMsgForwardError + " (invalid target) ");
        }
    }

}