package org.objectweb.telosys.common;

import java.io.File;
import java.util.Properties;

import javax.servlet.ServletConfig;
import javax.servlet.ServletContext;
import javax.xml.parsers.DocumentBuilderFactory;

/**
 * @author Laurent GUERIN
 *  
 */
public final class Telosys
{
	private static final String   SAX_XERCES  = "org.apache.xerces.parsers.SAXParser";
	
	private static final String   SAX_CRIMSON = "org.apache.crimson.parser.XMLReaderImpl";

    private static final TelosysClassLogger log                = new TelosysClassLogger(Telosys.class);

    //--- Telosys config file ( absolute path )
    private static String                   $TelosysPropFile   = null;

    //--- Telosys application root directory
    private static String                   $TelosysRootDir    = null;

    //--- Telosys config directory (directory where "telosys.properties" was found)
    private static String                   $TelosysConfDir    = null;

    //--- The properties loaded from "telosys.properties"
    private static Properties               $TelosysProperties = null;

    //--- Instance of the "Framework Internal Factory"
    private static TelosysFactory           $telosysFactory    = null;

    //--- Trace configuration file 
    private static String                   $sTraceConfFile    = null;

    //--- Authentication required to create ScreenSession 
    private static boolean                  $bAuthenticationRequired = true ;

    //--- XML SAX Parser Class Name ( for SAX 2 XMLReaderFactory.createXMLReader() )
    private static String                   $sSAXParserClassName     = null;
    
    private static LogWriter                $TelosysLogger     = null;

    //-----------------------------------------------------------------------------
    private static void initTraceConf(Properties prop)
    {
        if (prop != null)
        {
            //--- Nom de domaine pour l'authentification "basic HTTP"
            $sTraceConfFile = prop.getProperty("TraceConfFile");
            if ($sTraceConfFile == null)
            {
                log.info("initParameters : No trace config file ('TraceConfFile' not found) ");
            }
            else
            {
                $sTraceConfFile = $sTraceConfFile.trim() ;
                $sTraceConfFile = Telosys.getConfigFileName($sTraceConfFile);
                if ( Tool.fileExists( $sTraceConfFile ) )
                {
                    TraceConfig.init($sTraceConfFile);
                }
                else
                {
                    Telosys.error("Telosys.initTraceConf() : File " + $sTraceConfFile + " not found !");                                        
                }        		                    
            }
        }
        else
        {
            log.error("initParameters : properties = null !");
        }
    }

    //-----------------------------------------------------------------------------
    private static void createLogWriter(Properties prop)
    {
        if (prop != null)
        {
            log.info("Create LogWriter with properties. ");
            $TelosysLogger = new LogWriter(prop);
        }
        else
        {
            log.error("initParameters : properties = null !");
            log.info("Create LogWriter without properties.  ");
            $TelosysLogger = new LogWriter();
        }
    }

    //-----------------------------------------------------------------------------
    //    private static void createExternalMainFactory()
    //    {
    //        IExternalMainFactory mainFactory = null;
    //        if ( $sExternalMainFactoryClassName != null )
    //        {
    //            try {
    //                log.info ( "createExternalMainFactory() : class name = " +
    // $sExternalMainFactoryClassName ) ;
    //                Object obj =
    // ObjUtil.createObjectInstance($sExternalMainFactoryClassName);
    //                if ( obj != null )
    //                {
    //                    log.trace ( "createExternalMainFactory() : object created " ) ;
    //                    mainFactory = (IExternalMainFactory) obj ;
    //                }
    //                else
    //                {
    //                    log.error ( "createExternalMainFactory() : cannot create object " ) ;
    //                }
    //            }
    //            catch ( Exception ex )
    //            {
    //                log.error ( "createExternalMainFactory() : Exception : " + ex.toString()
    // + " ( " + ex.getMessage() + " ) " ) ;
    //            }
    //        }
    //        else
    //        {
    //            error ( " createExternalMainFactory() : Class name is null !");
    //        }
    //        $externalMainFactory = mainFactory ;
    //    }

    //-----------------------------------------------------------------------------
    private static boolean checkNotInitialized()
    {
        if (initialized())
        {
            //--- Already initialized
            log.info("INIT : Telosys.init(properties) : WARNING - Already initialized => No change !");
            return false; // Do not init again
        }
        else
        {
            return true; // OK to init
        }
    }

    //-----------------------------------------------------------------------------
    private static void initParamAuthentication(Properties prop)
    {
        //--- Authentication required ?
        $bAuthenticationRequired = true ;                    
        String sProperty = prop.getProperty("AuthenticationRequired");
        //log.info("INIT : Property Authentication Required = " + sAuthenticationRequired );
        if ( sProperty != null )
        {
            String sPropertyTrimLC = sProperty.trim().toLowerCase();
            //log.info("INIT : toLowerCase = '" + sPropTrimLC + "'");
            if ( sPropertyTrimLC.equals("false") )
            {
                //log.info("INIT : Flag is false" );
                $bAuthenticationRequired = false ; 
            }
            else
            {
                //log.info("INIT : Flag is NOT false" );                    
            }
        }        
    }
    //-----------------------------------------------------------------------------
    private static void initParamXmlSaxParser(Properties prop)
    {
        //--- Authentication required ?
        $sSAXParserClassName = null ;                    
        String sProperty = prop.getProperty("SAXParserClassName");
        if ( sProperty != null )
        {
            $sSAXParserClassName = sProperty.trim();
        }        
        else
        {
            $sSAXParserClassName = SAX_CRIMSON ;
        }
    }
    //-----------------------------------------------------------------------------
//    private static void initParamXmlDomParser(Properties prop)
//    {
//        //--- Authentication required ?
//        $sXmlDomParser = null ;                    
//        String sProperty = prop.getProperty("XmlDomParser");
//        if ( sProperty != null )
//        {
//            $sXmlDomParser = sProperty.trim();
//        }        
//    }
    //-----------------------------------------------------------------------------
    /**
     * Telosys initialization from properties
     * 
     * @param extFactory
     * @param prop
     * @return true if Ok / false if Error
     */
    private static boolean initTelosys(IExternalMainFactory extFactory, Properties prop)
    {
        if (checkNotInitialized())
        {
            //--- Check arguments
            if (extFactory == null)
            {
                log.error("initTelosys() : No external factory ( extFactory == NULL ) !");
                return false;
            }

            if (prop == null)
            {
                log.error("initTelosys() : properties = null !");
                return false;
            }

            //--- Build the Telosys factory with the "external factory"
            //$externalMainFactory = extFactory;
            $telosysFactory = new TelosysFactory(extFactory);
            log.info("INIT : Telosys factory created. ");

            //--- Store the properties
            $TelosysProperties = prop;
            log.info("INIT : Telosys properties set. ");

            //--- Authentication required ?
            initParamAuthentication(prop);
            log.info("INIT : Flag Authentication Required = " + $bAuthenticationRequired );

            //--- XML SAX parser
            initParamXmlSaxParser(prop);
            log.info("INIT : XML SAX parser class = '" + $sSAXParserClassName + "'" );
            
            //--- XML DOM parser
//            initParamXmlDomParser(prop);
//            log.info("INIT : XML DOM parser class = '" + $sXmlDomParser + "'" );
            
            //--- Init trace config & logger
            initTraceConf($TelosysProperties);
            createLogWriter($TelosysProperties);
            log.info("INIT : Telosys log writer created. ");
        }
        return true;
    }

    //-----------------------------------------------------------------------------
    /**
     * Telosys initialization from a properties file
     * 
     * @param extFactory
     * @param sPropFile
     * @return true if Ok / false if Error
     */
    private static boolean initTelosys(IExternalMainFactory extFactory, String sPropFile)
    {
        log.info("INIT : Telosys.init(" + sPropFile + ") ... ");
        Properties propLoaded = Tool.loadProperties(sPropFile);
        if (propLoaded != null)
        {
            //--- Store the file name
            $TelosysPropFile = sPropFile ;
            if ( $TelosysPropFile != null )
            {
                //--- Store the directory
                File file = new File($TelosysPropFile); 
                $TelosysConfDir = file.getParent();
            }
            //--- Initialize Telosys with properties loaded
            return initTelosys(extFactory, propLoaded);
        }
        else
        {
            log.error("initTelosys() : cannot load properties from '" + sPropFile + "' ");
            return false;
        }
    }

    //-----------------------------------------------------------------------------
    // PUBLIC METHODS
    //-----------------------------------------------------------------------------
    //-----------------------------------------------------------------------------
    public static boolean initialized()
    {
        if ($TelosysProperties != null && $telosysFactory != null)
        {
            return true;
        }
        return false;
    }

    //-----------------------------------------------------------------------------
//    /**
//     * Initialize Telosys with a set of properties
//     * 
//     * @param prop
//     *            (properties to use for initialization)
//     */
//    public static void init(IExternalMainFactory extFactory, Properties prop)
//    {
//        initTelosys(extFactory, prop);
//    }

    //-----------------------------------------------------------------------------
    /**
     * Try to initialize the Telosys properties : try to find 'telosys.properties' using the classpath if the file is find, the properties are loaded and Telosys is initialized.
     * 
     * @return the properties
     */
    public static Properties init(IExternalMainFactory extFactory)
    {
        if (checkNotInitialized())
        {
            log.info("INIT : Telosys.init(extFactory) ... ");
            //--- Try to find the Telosys Properties file
            String sPropFile = Tool.findTelosysPropertiesFile();
            if (sPropFile != null)
            {
                initTelosys(extFactory, sPropFile);
            }
            else
            {
                log.error("Telosys.init(extFactory) : cannot found 'telosys.properties' ");
            }

//            log.info("INIT : Telosys.init() : Properties loaded via classpath ... ");
//            Properties propLoaded = Tool.findAndLoadProperties("telosys.properties");
//            if (propLoaded != null)
//            {
//                initTelosys(extFactory, propLoaded);
//            }
//            else
//            {
//                log.error("Telosys.init() : cannot found 'telosys.properties' ");
//            }
        }
        return $TelosysProperties;
    }

    //-----------------------------------------------------------------------------
    /**
     * Initialize the Telosys properties. <br>
     * Try to find a properties file :<br>
     * 1/ specific file name in 'web.xml' ( servlet <init-param>TelosysProperties <init-param>)<br>
     * 2/ 'telosys.properties' located in 'WEB-INF/conf' <br>
     * 3/ 'telosys.properties' located via CLASSPATH <br>
     * 
     * @param extFactory
     * @param config
     * @return
     */
    public static Properties init(IExternalMainFactory extFactory, ServletConfig config)
    {
        if (checkNotInitialized())
        {
            log.info("INIT : Telosys.init(extFactory, config) ... ");
            //--- Try to find the Telosys Properties file
            String sPropFile = Tool.findTelosysPropertiesFile(config);
            if (sPropFile != null)
            {
                initTelosys(extFactory, sPropFile);
            }
            else
            {
                log.error("Telosys.init(extFactory, config) : cannot found 'telosys.properties' ");
            }

            //--- Set the default Telosys application root directory from the ServletContext
            ServletContext context = config.getServletContext();
            if (context != null)
            {
                $TelosysRootDir = context.getRealPath("/");
            }    
        }
        return $TelosysProperties;
    }

    //-----------------------------------------------------------------------------
    /**
     * Initialize the Telosys properties from a properties file
     * 
     * @param extFactory
     * @param sPropFile
     * @return
     */
    public static Properties init(IExternalMainFactory extFactory, String sPropFile)
    {
        if (checkNotInitialized())
        {
            log.info("INIT : Telosys.init(extFactory, file) ... ");
            if (sPropFile != null)
            {
                initTelosys(extFactory, sPropFile);
            }
            else
            {
                log.error("Telosys.init(extFactory, file) : properties file is null !");
            }
        }
        return $TelosysProperties;
    }

    //-----------------------------------------------------------------------------

    //-----------------------------------------------------------------------------
    // Rcupration des "properties" gnrales de TELOSYS
    //-----------------------------------------------------------------------------
    //-----------------------------------------------------------------------------
    //--- PUBLIC GETTERS/SETTERS
    //-----------------------------------------------------------------------------
    public static String getPropertiesFile()
    {
        return $TelosysPropFile ;
    }
    //-----------------------------------------------------------------------------
    public static String getConfigDirectory()
    {
        return $TelosysConfDir ;
    }
    //-----------------------------------------------------------------------------
    public static String getRootDirectory()
    {
        return $TelosysRootDir ;
    }
    //-----------------------------------------------------------------------------
    public static String getSAXParserClassName()
    {
        return $sSAXParserClassName ;
    }
    //-----------------------------------------------------------------------------
    /**
     * Return the full path of given relative file<br> 
     * ( ie : "myfile.txt" return "C:/dir/myfile.txt" ) <br>
     * This method concats the "Application Root Dir" with the file name<br>
     * if the "Application Root Dir" is not set, it returns the file as is.<br>
     * @param sFile
     * @return 
     */
    public static String getFullPath (String sFile )
    {
        if ( $TelosysRootDir != null )
        {
            String sRoot2 = $TelosysRootDir ;
            if ( $TelosysRootDir.endsWith("/") || $TelosysRootDir.endsWith("\\") )
            {
                //--- Cut the last char
                sRoot2 = $TelosysRootDir.substring(0, $TelosysRootDir.length()-1 ) ;
            }

            if ( sFile != null )
            {
                String sFile2 = sFile ;
                if ( sFile.startsWith("/") )
                {
                    //--- Cut the first char
                    sFile2 = sFile.substring(1); 
                }
                return sRoot2 + "/" + sFile2 ;                                
            }
            else
            {
                return sRoot2 + "/" ;
            }        
            
        }
        else
        {
            return sFile ; // No change 
        }
    }
    //-----------------------------------------------------------------------------
    public static String getConfigFileName(String sFile)
    {
        if ( sFile == null )
        {
            return null ;
        }
        if ( sFile.length() == 0 )
        {
            return sFile ;
        }
        //--- If the file name starts with "." => it's a relative path 
        if ( sFile.charAt(0) == '.' )
        {
            // ie : "./foo" => "Dir/foo"
            return $TelosysConfDir + sFile.substring(1) ;
        }
        else
        {
            //--- Not relative path => return "as is"
            return sFile ;
        }
    }

    //-----------------------------------------------------------------------------
    public static Properties getProperties()
    {
        return $TelosysProperties;
    }
    //-----------------------------------------------------------------------------
    public static TelosysFactory getFactory()
    {
        return $telosysFactory;
    }
    //-----------------------------------------------------------------------------
    public static boolean isAuthenticationRequired()
    {
        return $bAuthenticationRequired ;
    }
    //-----------------------------------------------------------------------------
    public static DocumentBuilderFactory getXmlDomParserFactory()
    {
//        // NB: An implementation of the DocumentBuilderFactory class is NOT guaranteed to be thread safe.
//        //     => a factory is created for each call 
//        String sParserClass = null ;
//        if ( $sXmlDomParser != null )
//        {
//            sParserClass = $sXmlDomParser ;
//        }
//        else
//        {
//            //--- Use CRIMSON by default
//            sParserClass = DOM_CRIMSON ;            
//        }
//        //--- Set the JAXP system propety 
//        System.setProperty( DOM_PARSER, sParserClass );
        //--- Create a new factory instance via JAXP
        DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
        //--- Configuring the Factory ...
        //factory.setValidating(true);
        //factory.setNamespaceAware(true);        

        log.info("Telosys.getXmlDomParserFactory() : JAXP factory class = " + factory.getClass().getName() );
        
        return factory ;
    }
    
    //-----------------------------------------------------------------------------
    //--- PUBLIC LOG METHODS
    //-----------------------------------------------------------------------------
    public static void error(String sClassName, String sMsg)
    {
        if ($TelosysLogger != null)
        {
            $TelosysLogger.error(sClassName, sMsg);
        }
        else
        {
            Console.error(sClassName, sMsg);
        }
    }

    public static void error(String sMsg)
    {
        if ($TelosysLogger != null)
        {
            $TelosysLogger.error(sMsg);
        }
        else
        {
            Console.error(sMsg);
        }
    }

    //public static String getMsg(Exception ex)
    public static String getMsg(Throwable ex)
    {
        StackTraceElement t[] = ex.getStackTrace();
        StackTraceElement loc = t[0];
        return "Exception " + ex.toString() + " in file '" + loc.getFileName() + "', method '" + loc.getMethodName() + "', line " + loc.getLineNumber();
    }

    public static String getCaller(StackTraceElement elt)
    {
        return elt.getClassName() + "." + elt.getMethodName() + "():" + elt.getLineNumber();
    }

    public static String getCallers(Throwable ex)
    //public static String getCallers(Exception ex)
    {
        StackTraceElement t[] = ex.getStackTrace();
        StackTraceElement loc = null;
        StringBuffer sbMsg = new StringBuffer(300);
        sbMsg.append("Exception : callers : ");
        for ( int i = 1 ; i < t.length && i < 3 ; i++ )
        {
            loc = t[i];
            if (i > 1)
                sbMsg.append(" <- ");
            //sbMsg.append(loc.toString());
            sbMsg.append(getCaller(loc));
        }
        return sbMsg.toString();
    }

    public static void error(String sClassName, Throwable ex)
    {
        if ($TelosysLogger != null)
        {
            $TelosysLogger.error(sClassName, getMsg(ex));
            $TelosysLogger.error(sClassName, getCallers(ex));
        }
        else
        {
            Console.error(sClassName, getMsg(ex));
            Console.error(sClassName, getCallers(ex));
        }
    }

    public static void error(Throwable ex)
    {
        if ($TelosysLogger != null)
        {
            $TelosysLogger.error(getMsg(ex));
            $TelosysLogger.error(getCallers(ex));
        }
        else
        {
            Console.error(getMsg(ex));
            Console.error(getCallers(ex));
        }
    }

    //-----------------------------------------------------------------------------
    public static void info(String sClassName, String sMsg)
    {
        if ($TelosysLogger != null)
        {
            $TelosysLogger.info(sClassName, sMsg);
        }
        else
        {
            Console.info(sClassName, sMsg);
        }
    }

    public static void info(String sMsg)
    {
        if ($TelosysLogger != null)
        {
            $TelosysLogger.info(sMsg);
        }
        else
        {
            Console.info(sMsg);
        }
    }

    //-----------------------------------------------------------------------------
    public static void trace(String sClassName, String sMsg)
    {
        if ($TelosysLogger != null)
        {
            $TelosysLogger.trace(sClassName, sMsg);
        }
        else
        {
            Console.trace(sClassName, sMsg);
        }
    }

    //-----------------------------------------------------------------------------

    public static void trace(String sMsg)
    {
        if ($TelosysLogger != null)
        {
            $TelosysLogger.trace(sMsg);
        }
        else
        {
            Console.trace(sMsg);
        }
    }
    //-----------------------------------------------------------------------------
}