package org.objectweb.telosys.common;

/**
 * Standard servlet ancestor used by other TELOSYS servlets. <br>
 * Provides standard log methods (info, trace, error)
 * @author Laurent GUERIN
 *  
 */

import javax.servlet.http.HttpServlet;

public abstract class TelosysServlet extends HttpServlet
{
    protected final static String XML_CONTENT_TYPE       = "text/xml";

    protected int                 _iLastTraceConfigState = -1;

    protected boolean             _bTrace                = false;

    //-----------------------------------------------------------------------------
    /**
     * Set the flag "trace" (specific flag for each instance)
     * @param bFlag
     */
    public void setFlagTrace(boolean bFlag)
    {
        TraceConfig.setFlag(this.getClass().getName(), bFlag);
    }

    //-----------------------------------------------------------------------------
    /**
     * Get the flag "trace" of the instance
     * @return
     */
    public boolean getFlagTrace()
    {
        return TraceConfig.getFlag(this.getClass().getName());
    }

    //-----------------------------------------------------------------------------
    /**
     * Print a standard trace message, using the active loggers (if the trace flag is "true")
     * @param s
     */
    public void trace(String s)
    {
        String sClassName = this.getClass().getName();
        //--- If the configuration has changed since the last acces ...
        if (TraceConfig.getCurrentState() != _iLastTraceConfigState)
        {
            //--- ScreenTitle the flag from the configuration
            _bTrace = TraceConfig.flagIsOn(sClassName);
            //--- ScreenTitle the configuration state
            _iLastTraceConfigState = TraceConfig.getCurrentState();
        }
        if (_bTrace == true)
        {
            Telosys.trace(sClassName, s);
        }
    }

    //-----------------------------------------------------------------------------
    /**
     * Print a standard error message, using the active loggers
     * @param sMsg
     */
    public void error(String sMsg)
    {
        Telosys.error(this.getClass().getName(), sMsg);
    }
    
    /**
     * Print an error message, using the active loggers
     * 
     * @param exception : the error cause
     */
    public void error(Throwable exception)
    {
        Telosys.error(this.getClass().getName(), exception);
    }

    /**
     * Print error messages, using the active loggers
     * @param sMsg : the error message 
     * @param exception : the error cause 
     */
    public void error(String sMsg, Throwable exception)
    {
        error(sMsg);
        error(exception);
    }

    //-----------------------------------------------------------------------------
    /**
     * Print a standard info message, using the active loggers
     * @param s
     */
    public void info(String s)
    {
        Telosys.info(this.getClass().getName(), s);
    }
}