package org.objectweb.telosys.common.data;

import java.util.Iterator;
import java.util.Vector;

import org.objectweb.telosys.common.TelosysObject;
import org.objectweb.telosys.util.XmlUtil;

/**
 * @author Laurent GUERIN
 *  
 */
public abstract class DataSet extends TelosysObject
{
    //-----------------------------------------------------------------------------
    //--- PRIVATE ATTRIBUTES
    //-----------------------------------------------------------------------------
    protected Vector _vDataRows = null;

    protected int    _iRowCount = 0;

    protected int    _iColCount = 0;

    /**
     *  
     */
    public DataSet()
    {
        super();
        _vDataRows = new Vector(10, 10); // initial capacity = 10, increment = 10
        _vDataRows.add(0, null); // Position 0 : void
    }

    //-----------------------------------------------------------------------------
    //--- PUBLIC METHODS
    //-----------------------------------------------------------------------------
    public abstract int load() throws Exception;

    //-----------------------------------------------------------------------------
    /**
     * Returns the number of rows 
     * @return
     */
    public int getRowCount()
    {
        return _iRowCount;
    }
    
    //-----------------------------------------------------------------------------
    /**
     * Returns 'true' if the DataSet contains no elements.
     * @return
     */
    public boolean isEmpty()
    {
        return ( _iRowCount < 1 ? true : false ) ;
    }
    
    //-----------------------------------------------------------------------------
    /**
     * Returns an iterator over the 'datarows' in this DataSet
     * @return
     */
    public Iterator iterator()
    {
        return new DataSetIterator(this);
    }
    
    //-----------------------------------------------------------------------------
    public DataRow getDataRow(int iRow)
    {
        if (iRow < 1 && iRow > _iRowCount)
        {
            return null;
        }
        else
        {
            return (DataRow) _vDataRows.get(iRow);
        }
    }

    //-----------------------------------------------------------------------------
    public void toXml(StringBuffer sb)
    {
        toXml(sb, "list");
    }

    //-----------------------------------------------------------------------------
    public void toXml(StringBuffer sb, String sTagList)
    {
        String sCRLF = "\n";

        sb.append("<" + sTagList + ">" + sCRLF);
        DataRow dr = null;
        int iSize = 0;
        for ( int iRow = 1 ; iRow <= _iRowCount ; iRow++ )
        {
            dr = (DataRow) _vDataRows.get(iRow);
            if (dr != null)
            {
                iSize = dr.getSize();
                Object obj = null;
                sb.append("<row");
                //--- For each column in the DataRow
                for ( int iCol = 1 ; iCol <= iSize ; iCol++ )
                {
                    obj = dr.getObject(iCol);
                    if (obj != null)
                    {
                        sb.append(" c" + iCol + "=\"" + XmlUtil.xmlString(obj.toString()) + "\"");
                    }
                }
                sb.append(" />" + sCRLF);
            }
        }
        sb.append("</" + sTagList + ">" + sCRLF);
    }

    //-----------------------------------------------------------------------------
    // Get Cell value with different types
    //-----------------------------------------------------------------------------
    public Object getObject(int iRow, int iCol)
    {
        DataRow dr = getDataRow(iRow);
        if (dr != null)
        {
            return dr.getObject(iCol);
        }
        return null;
    }

    //-----------------------------------------------------------------------------
    public String getString(int iRow, int iCol)
    {
        DataRow dr = getDataRow(iRow);
        return (dr != null ? dr.getString(iCol) : null);
    }

    //-----------------------------------------------------------------------------
    public String getString(int iRow, int iCol, String sNullValue)
    {
        DataRow dr = getDataRow(iRow);
        return (dr != null ? dr.getString(iCol, sNullValue) : null);
    }

    //-----------------------------------------------------------------------------
    public java.util.Date getDate(int iRow, int iCol) //throws TelosysException
    {
        DataRow dr = getDataRow(iRow); 
        return (dr != null ? dr.getDate(iCol) : null);
    }

    //-----------------------------------------------------------------------------
    public java.sql.Date getSqlDate(int iRow, int iCol) //throws TelosysException
    {
        DataRow dr = getDataRow(iRow);
        return (dr != null ? dr.getSqlDate(iCol) : null);
    }

    //-----------------------------------------------------------------------------
    public boolean getBoolean(int iRow, int iCol) //throws TelosysException
    {
        DataRow dr = getDataRow(iRow);
        return (dr != null ? dr.getBoolean(iCol) : false );
    }
    //-----------------------------------------------------------------------------
    public byte getByte(int iRow, int iCol) //throws TelosysException
    {
        DataRow dr = getDataRow(iRow);
        return (dr != null ? dr.getByte(iCol) : 0);
    }
    //-----------------------------------------------------------------------------
    public int getInt(int iRow, int iCol) //throws TelosysException
    {
        DataRow dr = getDataRow(iRow);
        return (dr != null ? dr.getInt(iCol) : 0);
    }
    //-----------------------------------------------------------------------------
    public short getShort(int iRow, int iCol) //throws TelosysException
    {
        DataRow dr = getDataRow(iRow);
        return (dr != null ? dr.getShort(iCol) : 0);
    }
    //-----------------------------------------------------------------------------
    public long getLong(int iRow, int iCol) //throws TelosysException
    {
        DataRow dr = getDataRow(iRow);
        return (dr != null ? dr.getLong(iCol) : 0);
    }
    //-----------------------------------------------------------------------------
    public float getFloat(int iRow, int iCol) //throws TelosysException
    {
        DataRow dr = getDataRow(iRow);
        return (dr != null ? dr.getFloat(iCol) : 0);
    }
    //-----------------------------------------------------------------------------
    public double getDouble(int iRow, int iCol) //throws TelosysException
    {
        DataRow dr = getDataRow(iRow);
        return (dr != null ? dr.getDouble(iCol) : 0);
    }
    //-----------------------------------------------------------------------------
}