package org.objectweb.telosys.common.data;



/**
 * @author Laurent GUERIN
 *  
 */
public abstract class UpdatableDataSet extends DataSet
{
    //-----------------------------------------------------------------------------
    //--- PRIVATE ATTRIBUTES
    //-----------------------------------------------------------------------------

    /**
     *  
     */
    public UpdatableDataSet()
    {
        super();
    }

    //-----------------------------------------------------------------------------
    //--- PUBLIC METHODS
    //-----------------------------------------------------------------------------
    /**
     * Replace the DataRow at the given position by a new one
     * @param iRow
     * @param dr
     * @return
     */
    public boolean replaceRow(int iRow, DataRow dr)
    {
        //--- Is the row position valid ?
        if (iRow < 1 && iRow > _iRowCount)
        {
            return false;
        }
        if ( dr == null )
        {
            return false;            
        }
        //--- Is the column count compatible with the DataSet ?
        if ( dr.getSize() != _iColCount )
        {
            return false;            
        }
        //--- Set the row at the given position 
        _vDataRows.set(iRow, dr);
        return true;
    }
    
    /**
     * Add a DataRow at the end of the DataSet
     * @param dr the DataRow to add
     * @return the number of rows in the DataSet with the new DataRow added
     */
    public int addRow(DataRow dr)
    {
        _vDataRows.add(dr);
        _iRowCount++;
        return _iRowCount;
    }
    /**
     * Add a new void DataRow at the end of the DataSet
     * @return the number of rows in the DataSet with the new DataRow added
     */
    public int addRow()
    {
        return addRow( new DataRow(_iColCount)) ;
    }
    
    //-----------------------------------------------------------------------------
    /**
     * Replace the object at the given position by a new one
     * @param iRow
     * @param iCol
     * @param newObject
     * @return true is OK, false if the object cannot be replace (ie the DataRow is null)
     */
    public boolean replaceObject(int iRow, int iCol, Object newObject)
    {
        DataRow dr = getDataRow(iRow);
        if ( dr == null )
        {
            return false ;
        } 
        dr.setObject(iCol, newObject);
        return true ;
    }

    //-----------------------------------------------------------------------------
}