package org.objectweb.telosys.common.vo;

import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.ListIterator;

import org.objectweb.telosys.common.TelosysObject;

/**
 * For future use
 * 
 * @author Laurent GUERIN
 *  
 */
public abstract class GenericVOList extends TelosysObject
{
    //-----------------------------------------------------------------------------
    // ATTRIBUTES(S)
    //-----------------------------------------------------------------------------

    private LinkedList _list          = null;

    private int        _iCurrentIndex = 0;

    //-----------------------------------------------------------------------------
    // CONSTRUCTOR(S)
    //-----------------------------------------------------------------------------
    protected GenericVOList()
    {
        _list = new LinkedList();
        _iCurrentIndex = 0;
    }

    //-----------------------------------------------------------------------------
    // PROTECTED METHOD(S)
    //-----------------------------------------------------------------------------
    // Appends the specified element to the end of the list
    //-----------------------------------------------------------------------------
    protected void addElement(Object newElement)
    {
        _list.addLast(newElement);
    }

    //-----------------------------------------------------------------------------
    // Inserts the specified element at the specified position in the list.
    //-----------------------------------------------------------------------------
    protected void insertElement(int index, Object newElement)
    {
        _list.add(index, newElement);
    }

    //-----------------------------------------------------------------------------
    // Replaces the element at the specified position in the list with the
    // specified element.
    //-----------------------------------------------------------------------------
    protected Object replaceElement(int index, Object newElement)
    {
        return (Object) _list.set(index, newElement);
    }

    //-----------------------------------------------------------------------------
    // Removes the first occurrence of the specified element in the list.
    //-----------------------------------------------------------------------------
    protected boolean removeElement(Object element)
    {
        return _list.remove(element);
    }

    //-----------------------------------------------------------------------------
    // Removes the element at the specified position in the list.
    //-----------------------------------------------------------------------------
    protected Object removeElement(int index)
    {
        return (Object) _list.remove(index);
    }

    //-----------------------------------------------------------------------------
    // Returns the element at the specified position in this list.
    //-----------------------------------------------------------------------------
    protected Object getElement(int index)
    {
        if (_list.isEmpty())
        {
            return null;
        }
        else
        {
            return (Object) _list.get(index);
        }
    }

    //-----------------------------------------------------------------------------
    // Returns the first element in the list.
    //-----------------------------------------------------------------------------
    protected Object getFirstElement()
    {
        if (_list.isEmpty())
        {
            _iCurrentIndex = 0;
            return null;
        }
        else
        {
            _iCurrentIndex = 0;
            return (Object) _list.get(_iCurrentIndex);
        }
    }

    //-----------------------------------------------------------------------------
    // Returns the next element in the list.
    //-----------------------------------------------------------------------------
    protected Object getNextElement()
    {
        if (_list.isEmpty())
        {
            _iCurrentIndex = 0;
            return null;
        }
        else
        {
            if (_iCurrentIndex + 1 < _list.size())
            {
                _iCurrentIndex++;
                return (Object) _list.get(_iCurrentIndex);
            }
            else
            {
                return null;
            }
        }
    }

    //-----------------------------------------------------------------------------
    // PUBLIC METHOD(S)
    //-----------------------------------------------------------------------------
    /**
     * Returns the encapsulated List object
     * 
     * @return
     */
    public List getList()
    {
        return _list;
    }

    /**
     * Returns an iterator over the elements in this list in proper sequence
     * 
     * @return the iterator
     */
    public Iterator iterator()
    {
        return _list.iterator();
    }

    /**
     * Returns a list iterator of the elements in this list (in proper sequence)
     * 
     * @return the list iterator
     */
    public ListIterator listIterator()
    {
        return _list.listIterator();
    }

    //-----------------------------------------------------------------------------
    /**
     * Returns the number of elements in this list.
     * 
     * @return
     */
    public int size()
    {
        return _list.size();
    }

    //-----------------------------------------------------------------------------
    /**
     * Returns 'true' if this list contains no elements
     * 
     * @return
     */
    public boolean isEmpty()
    {
        return _list.isEmpty();
    }

    //-----------------------------------------------------------------------------
    /**
     * Removes all of the elements from this list
     */
    public void clear()
    {
        _list.clear();
    }

    //-----------------------------------------------------------------------------------------------
    public String toString()
    {
        StringBuffer sb = new StringBuffer(512);
        //sb.append(_list.getClass().getName() + " : \n");
        sb.append(this.getClass().getName() + " : \n");
        Object obj = getFirstElement();
        while (obj != null)
        {
            sb.append(" . " + obj.toString() + "\n");
            obj = getNextElement();
        }
        return sb.toString();
    }
}