package org.objectweb.telosys.common.vo;

import java.io.PrintWriter;
import java.util.List;

import org.objectweb.telosys.common.TelosysObject;
import org.objectweb.telosys.rpl.xml.IXmlWrapper;

/**
 * For future use
 * 
 * @author Laurent GUERIN
 *  
 */
public class GenericVOListXmlWrapper extends TelosysObject implements IXmlWrapper
{
    //private String        _sTagName    = null;

    /**
     * <code>_list</code> Generic list of Entity Value Objects
     */
    private GenericVOList _list        = null;

    private IXmlWrapper   _beanWrapper = null;

    //-----------------------------------------------------------------------------------
    /**
     * Create a new instance of Generic V.O. List wrapper
     * 
     * @param sTagName :
     *            the XML tag name
     * @param listClass :
     *            the class of the V.O. List wrapped
     * @param beanWrapper :
     *            the XML wrapper for the type of beans hold by the VO List
     */
//    public GenericVOListXmlWrapper(String sTagName, Class theClass, IXmlWrapper beanWrapper)
    public GenericVOListXmlWrapper( Class listClass, IXmlWrapper beanWrapper)
    {
        super();
        //_sTagName = sTagName;
        _list = createVOList(listClass);
        _beanWrapper = beanWrapper;
    }

    //-----------------------------------------------------------------------------------
    private GenericVOList createVOList(Class theClass)
    {
        Object obj = null;
        try
        {
            obj = theClass.newInstance();
            if (obj instanceof GenericVOList)
            {
                return (GenericVOList) obj;
            }
        } catch (InstantiationException e)
        {
            error("createDataList(" + theClass.getName() + ") : InstantiationException ");
        } catch (IllegalAccessException e)
        {
            error("createDataList(" + theClass.getName() + ") : IllegalAccessException ");
        }
        return null;
    }

    //-----------------------------------------------------------------------------------
//    public String getXmlTagName()
//    {
//        return _sTagName;
//    }

    //-----------------------------------------------------------------------------------
    public void newXmlChild(String sTagName, IXmlWrapper obj)
    {
        if (_list != null)
        {
            //_list.addElement(obj.getObject());
            Object element = obj.getObject();
            _list.addElement(element);
            //            if (element instanceof GenericVO)
            //            {
            //            }
            //            else
            //            {
            //                error("newChild(" + sTagName + ", " + obj.getClass() + ") : this object is not an instance of GenericVO ");
            //            }
        }
        else
        {
            error("newChild(" + sTagName + ", " + obj.getClass() + ") : _list is NULL ");
        }
    }

    //-----------------------------------------------------------------------------------
    public Object getObject()
    {
        return _list;
    }

    //-----------------------------------------------------------------------------------

    //-----------------------------------------------------------------------------------
    // Pour les listes, pas d'attributs XML
    //-----------------------------------------------------------------------------------
    /*
     * (non-Javadoc)
     * 
     * @see org.telosys.xml.IXmlWrapper#setXmlFieldValue(java.lang.String, java.lang.String)
     */
    //-----------------------------------------------------------------------------------
    public void setXmlFieldValue(final String sName, final String sValue)
    {
    }

//    //-----------------------------------------------------------------------------------
//    public void toXml(PrintWriter out)
//    {
//        toXml(out, _list);
//    }

    //-----------------------------------------------------------------------------------
    public void toXml(PrintWriter out, Object obj, String sElementName)
    {
        if (obj == null)
        {
            out.println("<!-- toXml(null) -->");
            return;
        }
        if (!(obj instanceof GenericVOList))
        {
            out.println("<!-- ERROR : toXml(.,.) : object argument is not an instance of GenericVOList -->");
            error("toXml(.,.) : object argument is not an instance of GenericVOList");
            return;
        }
        GenericVOList genericVOList = (GenericVOList) obj;
        //out.println("<" + _sTagName + ">");
        out.println("<" + sElementName + ">");
        List list = genericVOList.getList();
        if (list != null)
        {
            if (_beanWrapper != null)
            {
                Object element = null;
                int iSize = list.size();
                for ( int i = 0 ; i < iSize ; i++ )
                {
                    element = list.get(i);
                    if (element != null)
                    {
                        _beanWrapper.toXml(out, element, sElementName);
                    }
                    else
                    {
                        out.println("<!-- Element [" + i + "] of the list is NULL -->");
                    }
                }
            }
            else
            {
                out.println("<!-- ERROR : No bean wrapper -->");
                error("toXml() : the bean XML wrapper is null ");
            }
        }
        else
        {
            out.println("<!-- List in GenericVOList is NULL -->");
        }
        //out.println("</" + _sTagName + ">");
        out.println("</" + sElementName + ">");
    }
}