package org.objectweb.telosys.dal;

import java.util.Properties;

import org.objectweb.telosys.common.Telosys;
import org.objectweb.telosys.common.Tool;
import org.objectweb.telosys.dal.sql.ConnectionManager;

/**
 * @author Laurent GUERIN
 * 
 */
public final class TelosysDAL
{
    //--- Fichier de configuration des bases de donnes
    private static String                   $sDbConfigFileName             = null;

    //-----------------------------------------------------------------------------
    private static boolean initConnectionManager()
    {
        boolean bRet = false ;
        if ( $sDbConfigFileName != null )
        {
            bRet = ConnectionManager.init($sDbConfigFileName);            
            Telosys.info ( "Connection Manager initialized (conf. file = " + $sDbConfigFileName + ")" ) ;
        }
        else
        {
            Telosys.error ( "initConnectionManager() : No configuration file name => Cannot initialize Connection Manager  !" ) ;                                            
        }
        return bRet ;
    }
    
    //-----------------------------------------------------------------------------
    // PUBLIC METHODS
    //-----------------------------------------------------------------------------
    //-----------------------------------------------------------------------------
    public static boolean init()
    {
        Telosys.info("TelosysDAL : INIT without properties ..." );
        //--- Get the Telosys properties ...
        Properties prop = Telosys.getProperties();
        if ( prop == null )
        {
            Telosys.error("TelosysDAL.init() : cannot get Telosys properties !");
            return false ;
        }
        else
        {
            return init(prop);
        }
    }
    //-----------------------------------------------------------------------------
    private static boolean init(Properties prop)
    {
        Telosys.info("TelosysDAL : INIT with properties ..." );
        boolean bRet = false;
        if (prop != null)
        {
            //--- Fichier de configuration des bases de donnes
            $sDbConfigFileName = prop.getProperty("DbConfFile");
            if ($sDbConfigFileName == null)
            {
                Telosys.error("TelosysDAL.init() : DbConfFile not found in properties ! ");
            }
            else
            {
                $sDbConfigFileName = $sDbConfigFileName.trim() ;
                //--- Build full path from relative path (if any)
                $sDbConfigFileName = Telosys.getConfigFileName($sDbConfigFileName);
                if ( Tool.fileExists( $sDbConfigFileName ) )
                {
	                //--- Connection Manager initialization ( use the DbConfFile )
	        		bRet = initConnectionManager();
                }
                else
                {
                    Telosys.error("TelosysDAL.init(Properties) : File " + $sDbConfigFileName + " not found !");                                        
                }        		
            }
        }
        else
        {
            Telosys.error("TelosysDAL.init() : properties = null !");
        }
        return bRet;
    }
    //-----------------------------------------------------------------------------
    /**
     * Telosys DAL init with DbConfig file name<br>
     * (to init DAL without full Telosys environment)
     * @param sDbConfigFileName
     * @return
     */
    public static boolean init(String sDbConfigFileName)
    {
        Telosys.info("TelosysDAL : INIT with DbConfig file '" + sDbConfigFileName + "'" );
        boolean bRet = false;
        if (sDbConfigFileName == null)
        {
            Telosys.error("TelosysDAL.init(sDbConfigFileName) : parameter is null ");
        }
        else
        {
            sDbConfigFileName = sDbConfigFileName.trim() ;
            if ( Tool.fileExists( sDbConfigFileName ) )
            {
                $sDbConfigFileName = sDbConfigFileName ;
                //--- Connection Manager initialization ( use the DbConfFile )
        		bRet = initConnectionManager();
            }
            else
            {
                Telosys.error("TelosysDAL.init(sDbConfigFileName) : File " + $sDbConfigFileName + " not found !");                                        
            }        		
        }
        return bRet;
    }
    //-----------------------------------------------------------------------------
    public static String getDbConfig()
    {
        return $sDbConfigFileName ;
    }
}