package org.objectweb.telosys.dal.dao;

import java.sql.Connection;
import java.sql.SQLException;

import org.objectweb.telosys.common.TelosysException;
import org.objectweb.telosys.common.TelosysObject;

/**
 * @author Laurent GUERIN
 *  
 */
public class CopyOfDatabaseSession extends TelosysObject
{
    int        _iDatabaseId         = 0;

    Connection _con                 = null;

    boolean    _bOriginalAutoCommit = false;

    /**
     *  
     */
    public CopyOfDatabaseSession(int iDatabaseId, Connection con) throws TelosysException
    {
        super();
        // In a session, the connection must not be in "AutoCommit mode"
        try
        {
            //--- Keep the original AutoCommit state
            _bOriginalAutoCommit = con.getAutoCommit();
            //--- Set the Programmatic Transaction control
            con.setAutoCommit(false);
        } catch (SQLException ex)
        {
            throw new TelosysException("DatabaseSession constructor : cannot get/set AutoCommit", ex);
        }
        _iDatabaseId = iDatabaseId;
        _con = con;
    }

    public int getDatabaseId()
    {
        return _iDatabaseId;
    }

    /* package */Connection getConnection()
    {
        return _con;
    }

    public void commit() throws TelosysException
    {
        if (_con == null)
        {
            throw new TelosysException("DatabaseSession.commit() : connection is null");
        }
        else
        {
            try
            {
                _con.commit();
            } catch (SQLException e)
            {
                throw new TelosysException("DatabaseSession.commit() : Cannot commit", e);
            }
        }
    }

    public void rollback() throws TelosysException
    {
        if (_con == null)
        {
            throw new TelosysException("DatabaseSession.rollback() : connection is null");
        }
        else
        {
            try
            {
                _con.rollback();
            } catch (SQLException e)
            {
                throw new TelosysException("DatabaseSession.rollback() : Cannot rollback", e);
            }
        }
    }

    public void close() throws TelosysException
    {
        if (_con == null)
        {
            throw new TelosysException("DatabaseSession.close() : connection is null");
        }
        else
        {
            try
            {
                //--- Reset the original AutoCommit state
                _con.setAutoCommit( _bOriginalAutoCommit );
                //--- Release the connection 
                _con.close();
            } catch (SQLException e)
            {
                throw new TelosysException("DatabaseSession.close() : connection error on setAutoCommit or close ", e);
            }
        }
    }
}