package org.objectweb.telosys.dal.dataset;

import org.objectweb.telosys.common.TelosysException;
import org.objectweb.telosys.common.data.DataSet;
import org.objectweb.telosys.common.data.DataSetProvider;

/**
 * DataSet for stable data in application scope<br>
 * The dataset is load when the instance is created<br>
 * NB : the dataset is shared by all the sessions
 * 
 * @author Laurent GUERIN
 *  
 */
public class ApplicationSQLDataSet implements DataSetProvider
{
    private SQLDataSetDefinition _dsDefinition = null;

    //private SQLDataSet _dataSet = null;
    private DataSet _dataSet = null;

    /**
     * Build a default ApplicationDataSet  <br>
     * No data loading 
     */
    public ApplicationSQLDataSet() //throws TelosysException
    {
    }
    
    /**
     * Build an ApplicationDataSet from a SQLDataSetDefinition <br>
     * ( build a SQLDataSet and load it ). 
     * @param def
     * @throws TelosysException
     */
    public ApplicationSQLDataSet(SQLDataSetDefinition def) throws TelosysException
    {
//        _dsDefinition = def ;
//        _dataSet = new SQLDataSet(_dsDefinition);
//        _dataSet.load();
        
        if ( def != null )
        {
            _dsDefinition = def ;
            SQLDataSet sqlDataSet = new SQLDataSet(_dsDefinition);
            sqlDataSet.load();
            _dataSet = sqlDataSet ;
        }
        else
        {
            _dsDefinition = null ;
            _dataSet = null ;            
        }
    }
    
    /**
     * @return the DataSet held by this ApplicationDataSet
     */
    public DataSet getDataSet()
    {
        return _dataSet ;
    }
    
    /**
     * @return the number of rows of the DataSet held by this ApplicationDataSet
     */
    public int getRowCount()
    {
        if ( _dataSet != null )
        {
            return _dataSet.getRowCount() ;            
        }
        else
        {
            return 0 ;
        }
    }
    
}