package org.objectweb.telosys.dal.sql;

import java.sql.Connection;
import java.sql.Driver;
import java.sql.DriverManager;
import java.util.Properties;

import org.objectweb.telosys.common.TelosysObject;

/**
 * @author Laurent GUERIN
 *  
 */
public abstract class SqlConnectionProvider extends TelosysObject implements ConnectionProvider
{
    private String     _sName              = null;

    private boolean    _bDriverInitialized = false;

    private String     _sDriverName        = null;

    private String     _sUrlJdbc           = null;

    private Properties _propCon            = null;                                 // Properties pour la connexion JDBC

    private int        _iIsolationLevel    = Connection.TRANSACTION_READ_COMMITTED;

    //-----------------------------------------------------------------------------
    // Constructeur(s)
    //-----------------------------------------------------------------------------
    public SqlConnectionProvider(String sName, String sDriver, String sUrl, Properties prop, int iIsolationLevel)
    {
        trace("Constructor ...");

        _sName = sName;
        _sDriverName = sDriver;
        _sUrlJdbc = sUrl;
        _propCon = prop;
        _iIsolationLevel = iIsolationLevel;

        if (!_bDriverInitialized)
        {
            try
            {
                Class c = Class.forName(_sDriverName);
                DriverManager.registerDriver((Driver) c.newInstance());
                _bDriverInitialized = true;
            } catch (Exception e)
            {
                error("Cannot initialize JDBC driver ! (driver=" + _sDriverName + ") - Exception : " + e.toString());
                _bDriverInitialized = false;
            }
        }
    }

    //-----------------------------------------------------------------------------
    // Cration d'une connexion JDBC standard
    //-----------------------------------------------------------------------------
    protected Connection createStandardJdbcConnection() throws Exception
    {
        trace("createConnection() ...");
        //--- Si le driver JDBC n'est pas initialis
        if (!_bDriverInitialized)
            throw new Exception("JDBC driver is not initialized");
        if (_propCon == null)
        {
            error("createConnection: no connection properties !");
            return null;
        }

        Connection stdCon = null;

        try
        {
            stdCon = DriverManager.getConnection(_sUrlJdbc, _propCon);
            //--- Transaction Isolation Level
            stdCon.setTransactionIsolation(_iIsolationLevel);
        } catch (Throwable t)
        {
            throw new Exception(t.getMessage());
        }

        return stdCon;
    }

    //-----------------------------------------------------------------------------
    // Accesseurs
    //-----------------------------------------------------------------------------
    public String getName()
    {
        return _sName;
    }

    //public abstract SqlConnection getConnection() throws Exception;
    public abstract Connection getConnection() throws Exception;

    public abstract boolean isPool();

}