package org.objectweb.telosys.dal.sql;

import java.sql.Connection;

import javax.naming.Context;
import javax.naming.InitialContext;
import javax.naming.NamingException;
import javax.sql.DataSource;

import org.objectweb.telosys.common.TelosysException;
import org.objectweb.telosys.common.TelosysObject;

/**
 * Connection provider implementation based on a standard JDBC DataSource
 * 
 * @author Laurent GUERIN
 *  
 */

//-----------------------------------------------------------------------------
//
//-----------------------------------------------------------------------------
public class SqlDataSource extends TelosysObject implements ConnectionProvider //extends SqlConnectionProvider
{
    private String  _sProviderName = null ;
    
    private String  _sDataSourceName = null ;
    
    private DataSource _dataSource = null ;
    
    //-----------------------------------------------------------------------------
    public SqlDataSource( String sName, String sDataSourceName ) throws TelosysException
    {
        _sProviderName = sName ;
        _sDataSourceName = sDataSourceName ;
        
        //--- Try to find the DataSource via JNDI 
        Context initialContext = null ;
        Object obj = null ;

        //--- 1) Create the "Initial Context"
        try
        {
            initialContext = new InitialContext();
        } catch (NamingException e)
        {
            throw new TelosysException ("Cannot create InitialContext !");
        }
        
        //--- 2) Lookup the given DataSource name
        try
        {
            obj = initialContext.lookup(sDataSourceName);
        } catch (NamingException e1)
        {
            //error ("Cannot find the DataSource : lookup('" + sDataSourceName + "') !");
            throw new TelosysException ("Cannot find the DataSource : lookup('" + sDataSourceName + "') !");
        }
        
        //--- 3) Cast the object to DataSource
        if ( obj instanceof DataSource)
        {
            _sDataSourceName = sDataSourceName ;
            _dataSource = (DataSource) obj ;
        }
        else
        {
            //error ("objet got with lookup('" + sDataSourceName + "') is not a DataSource instance ");
            throw new TelosysException ("Objet got with lookup('" + sDataSourceName + "') is not a DataSource instance !");
        }
        
        trace("SqlDataSource created.");
    }

    //-----------------------------------------------------------------------------
    /* (non-Javadoc)
     * @see org.objectweb.telosys.dal.sql.ConnectionProvider#getConnection()
     */
    public Connection getConnection() throws Exception
    {
        trace("getConnection()");
        if ( _dataSource != null )
        {
            //--- Get the connection from the JDBC DataSource
            return _dataSource.getConnection();
        }
        else
        {
            throw new TelosysException("SqlDataSource.getConnection() : the DataSource is not set (_dataSource == null) ");
        }
    }
    
    /* (non-Javadoc)
     * @see org.objectweb.telosys.dal.sql.ConnectionProvider#getName()
     */
    public String getName() 
    {
        return _sProviderName ;
    }

    /**
     * Returns the DataSource resource name ( JNDI name )
     * @return
     */
    public String getDataSourceName() 
    {
        return _sDataSourceName ;
    }

}