package org.objectweb.telosys.rpl;

import org.objectweb.telosys.common.TelosysException;
import org.objectweb.telosys.common.TelosysObject;
import org.objectweb.telosys.screen.core.ScreenDataElement;
import org.objectweb.telosys.screen.core.ScreenDataElements;

/**
 * @author Laurent GUERIN
 *  
 */
public class ScreenRequest extends TelosysObject
{
    private final static String[]   VOID_ELEMENTS_LIST = new String[0];

    private String                  _sScreenName       = null;

    private String                  _sScreenId         = null;

    private String                  _sAction           = null;

    private boolean                 _bFlagSet          = true;

    private boolean                 _bFlagGet          = true;

    private boolean                 _bFlagClear        = true;

    private String                  _sProcName         = null;

    private String                  _sElements         = null;

    private String[]                _sElementsArray    = null;

    //private ScreenData _Data = null;

    private ScreenDataElements      _dataElements      = null;         // Data elements from the XML <data> section

    private ScreenRequestParameters _parameters        = null;

    /**
     *  
     */
    public ScreenRequest()
    {
        _parameters = new ScreenRequestParameters();
    }

    //-----------------------------------------------------------------------------------------------
    /**
     * @return Returns the _sAction.
     */
    public String getAction()
    {
        return _sAction;
    }

    /**
     * @param action
     *            The _sAction to set.
     */
    public void setAction(String action)
    {
        _sAction = action;
    }

    //-----------------------------------------------------------------------------------------------
    /**
     * @return Returns the _sProcName.
     */
    public String getProcName()
    {
        return _sProcName;
    }

    /**
     * @param procName
     *            The _sProcName to set.
     */
    public void setProcName(String procName)
    {
        _sProcName = procName;
    }

    //-----------------------------------------------------------------------------------------------
    /**
     * @return Returns the _sScreenId.
     */
    public String getScreenId()
    {
        return _sScreenId;
    }

    /**
     * @param screenId
     *            The _sScreenId to set.
     */
    public void setScreenId(String screenId)
    {
        _sScreenId = screenId;
    }

    //-----------------------------------------------------------------------------------------------
    /**
     * @param screenName
     *            The _sScreenName to set.
     */
    public void setScreenName(String screenName)
    {
        _sScreenName = screenName;
    }

    /**
     * @return Returns the _sScreenName.
     */
    public String getScreenName()
    {
        return _sScreenName;
    }

    //-----------------------------------------------------------------------------------------------
    /**
     * Builds the elements array and returns it
     * 
     * @return
     */
    private String[] buildElements()
    {
        if (_sElements != null)
        {
            String sElement = null;
            String[] elements1 = _sElements.split(",");
            String[] elements2 = new String[elements1.length];
            for ( int i = 0 ; i < elements2.length ; i++ )
            {
                sElement = elements1[i];
                if (sElement != null)
                {
                    elements2[i] = sElement.trim();
                }
                else
                {
                    elements2[i] = "";
                }
            }
            return elements2;
        }
        else
        {
            //--- No elements list => void array
            return VOID_ELEMENTS_LIST;
        }
    }

    /**
     * Set the "elements" attribute of the request
     * @param v
     */
    public void setElements(String v)
    {
        _sElements = v;
    }

    /**
     * Returns the "elements" attribute of the request
     * @return
     */
    public String getElements()
    {
        return _sElements;
    }

    /**
     * Returns true if the request has an "element" attribute
     * @return
     */
    public boolean hasElementsAttribute()
    {
        return (_sElements != null ? true : false);
    }

    /**
     * Returns an array of elements name containing each element of the "elements" attribute
     * @return
     */
    public String[] getElementsArray()
    {
        if (_sElementsArray == null)
        {
            //--- Array not yet build => build the array
            _sElementsArray = buildElements();
        }
        //--- Return a copy of the existing array
        return (String[]) _sElementsArray.clone();
    }

    /**
     * Returns true if the "elements" attribute of the request contains the given element name
     * @param sArgElementName
     * @return true if the element is in the list (else false)
     */
    public boolean elementsAttributeContains(String sArgElementName)
    {
        if (sArgElementName != null)
        {
            String sElementName = sArgElementName.trim();
            String[] elements = getElementsArray();
            for ( int i = 0 ; i < elements.length ; i++ )
            {
                if (sElementName.equals(elements[i]))
                {
                    return true;
                }
            }
        }
        return false;
    }

    //-----------------------------------------------------------------------------------------------
    /**
     * @return Returns the Request Parameters .
     */
    public ScreenRequestParameters getParameters()
    {
        return _parameters;
    }

    //-----------------------------------------------------------------------------------------------
    public boolean getFlagSetData()
    {
        return _bFlagSet;
    }

    public void setFlagSetData(String v)
    {
        if (v.equals("true"))
        {
            _bFlagSet = true;
        }
        else
        {
            _bFlagSet = false;
        }
    }

    //-----------------------------------------------------------------------------------------------
    public boolean getFlagGetData()
    {
        return _bFlagGet;
    }

    public void setFlagGetData(String v)
    {
        if (v.equals("true"))
        {
            _bFlagGet = true;
        }
        else
        {
            _bFlagGet = false;
        }
    }

    //-----------------------------------------------------------------------------------------------
    public boolean getFlagClearData()
    {
        return _bFlagClear;
    }

    public void setFlagClearData(String v)
    {
        if (v.equals("true"))
        {
            _bFlagClear = true;
        }
        else
        {
            _bFlagClear = false;
        }
    }

    //-----------------------------------------------------------------------------------------------
    public void setParameter(String sParamName, String sParamValue)
    {
        _parameters.setParameter(sParamName, sParamValue);
    }

    //-----------------------------------------------------------------------------------------------
    public String getParameter(String sParamName)
    {
        return _parameters.getParameter(sParamName);
    }

    //-----------------------------------------------------------------------------------------------
    public int getIntParameter(String sParamName) throws TelosysException
    {
        return _parameters.getIntParameter(sParamName);
    }

    //-----------------------------------------------------------------------------------------------
    //  <data ... >
    //-----------------------------------------------------------------------------------------------
    //    public ScreenData createData()
    //    {
    //        trace("createData()...");
    //        //TelosysFactory factory = TelosysScreen.getFactory();
    //        TelosysFactory factory = Telosys.getFactory();
    //        if (factory != null)
    //        {
    //            try
    //            {
    //                _Data = factory.createScreenData(_sScreenName);
    //            } catch (TelosysException e)
    //            {
    //                error("createData() : factory cannot create screen data (TelosysException)", e);
    //            }
    //        }
    //        else
    //        {
    //            error("createData() : factory is null");
    //            _Data = null;
    //        }
    //        return _Data;
    //    }

    //-----------------------------------------------------------------------------------------------
    //public ScreenData getData()
    //    public ScreenData getData()
    //    {
    //        return _Data;
    //    }
    public ScreenDataElements getDataElements()
    {
        return _dataElements;
    }

    //-----------------------------------------------------------------------------------------------
    //    public void storeDataElement(String sTagName, Object objData)
    //    {
    //        if (_Data != null)
    //        {
    //            trace("storeDataElement(" + objData.getClass() + ")");
    //            //_Data.store(sTagName, objData);
    //            _Data.setElement(sTagName, objData);
    //        }
    //        else
    //        {
    //            trace("storeDataElement() : data is NULL ! ");
    //        }
    //    }
    /**
     * Adds a new element in the request elements list
     * 
     * @param sTagName
     * @param objData
     */
    public void storeDataElement(String sTagName, Object objData)
    {
        if (_dataElements == null)
        {
            _dataElements = new ScreenDataElements();
        }
        //ScreenDataElement oElement = new ScreenDataElement(sTagName, objData.getClass(), objData);
        //--- Creates a void element
        ScreenDataElement oElement = new ScreenDataElement(sTagName, objData.getClass());
        //--- Set the element instance
        oElement.setInstance(objData);
        //--- Add the element to the request elements list
        _dataElements.add(oElement);
    }

    //-----------------------------------------------------------------------------------------------
    public String toString()
    {
        //--- Attributes
        String sReqHeader = _sScreenName + "|" + _sScreenId + "|" + _sAction + "|" + _sProcName;

        //--- Parameters
        String sParameters = "(no parameter)";
        if (_parameters != null)
        {
            sParameters = _parameters.toString();
        }

        //--- Data
        String sData = "(no data)";
        //        if (_Data != null)
        //        {
        //            sData = _Data.toString();
        //        }

        return "ScreenRequest : " + sReqHeader + "\n Parameters = " + sParameters + " \n Data = " + sData;
    }
    //-----------------------------------------------------------------------------------------------
}