package org.objectweb.telosys.rpl;

import java.io.IOException;
import java.io.InputStream;
import java.io.PrintWriter;

import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.objectweb.telosys.common.TelosysException;
import org.objectweb.telosys.common.TelosysRuntimeException;
import org.objectweb.telosys.common.TelosysServlet;
import org.objectweb.telosys.rpl.xml.XmlOutput;
import org.objectweb.telosys.screen.core.ScreenContext;
import org.objectweb.telosys.screen.env.ScreenSession;

/**
 * @author Laurent GUERIN
 *  
 */
public class ScreenRequestServlet extends TelosysServlet // HttpServlet
{
   // private final static String XML_CONTENT_TYPE = "text/xml" ;
    
    private boolean bSynchronizeActionOnScreenSession = true ;
    
    //---------------------------------------------------------------------------
    public void init() throws ServletException
    {
        trace("init()");
    }

    //---------------------------------------------------------------------------
    public void doGet(HttpServletRequest httpRequest, HttpServletResponse httpResponse) throws ServletException, IOException
    {
        trace("========================================");
        trace("doGet()...");
        //--- The response is always XML
        httpResponse.setContentType(XML_CONTENT_TYPE);
        //--- "GET" is not processed ( only "POST" )
        sendErrorResponse(httpResponse, 1001, "HTTP GET method is not supported ! ");
    }

    //---------------------------------------------------------------------------
    public void doPost(HttpServletRequest httpRequest, HttpServletResponse httpResponse) throws ServletException, IOException
    {
        trace("========================================");
        trace("doPost()");
        //--- The response is always XML
        httpResponse.setContentType(XML_CONTENT_TYPE);
        //--- Process the input request
        process(httpRequest, httpResponse);
    }

    //---------------------------------------------------------------------------
    public void destroy()
    {
        trace("destroy()");
    }

    //---------------------------------------------------------------------------
    private void process(HttpServletRequest httpRequest, HttpServletResponse httpResponse) throws ServletException, IOException
    {
        //--- Servlet output stream
        PrintWriter output = httpResponse.getWriter();
        if (output == null)
        {
            // Cannot build Http Response Error => just log
            error("Output stream is null ");
            return;
        }

        //--- Servlet input stream
        InputStream input = httpRequest.getInputStream();
        if (input == null)
        {
            sendErrorResponse(httpResponse, 1002, "Input stream is null ");
            return;
        }

        try
        {
            //--- Get a ScreenRequest from the XML input stream ( XML parsing )
            ScreenRequest screenRequest = ScreenRequestProcessor.parseInputStream(input);

            //--- Get a ScreenResponse
            ScreenResponse screenResponse = ScreenRequestProcessor.getScreenResponse(screenRequest);

            //--- Get the current "Screen Session" ( create a "dummy user" session if Authentication Not Requiered )
            ScreenSession screenSession = ScreenRequestProcessor.getScreenSession(httpRequest);

            //--- Get the ScreenContext associated with the request Screen ID (can be null if not yet opened and action = "open")
            ScreenContext screenContext = ScreenRequestProcessor.getScreenContext(screenRequest, screenSession);

            //--- Execute the action
            if ( bSynchronizeActionOnScreenSession )
            {
                //--- Execute the action in synchronized mode to avoid concurrency problems
                ScreenRequestProcessor.executeSynchronizedAction(screenSession, screenContext, screenRequest, screenResponse);
            }
            else
            {
                //--- Execute the action without synchronization
                ScreenRequestProcessor.executeAction(screenSession, screenContext, screenRequest, screenResponse);                                
            }

            //--- OUTPUT : Generate the XML response
            XmlOutput.generate(screenResponse, output, httpRequest, httpResponse );

            output.close();

        } catch (TelosysException ex)
        {
            sendErrorResponse(httpResponse, 1003, "TelosysException : " + ex.getMessage());
            ex.printStackTrace();
        } catch (TelosysRuntimeException ex)
        {
            sendErrorResponse(httpResponse, 1004, "TelosysRuntimeException : " + ex.getMessage());
            ex.printStackTrace();
        } catch (Throwable t)
        {
            sendErrorResponse(httpResponse, 1005, "Throwable Exception : " + t.getMessage());
            t.printStackTrace();
        }
    }

    //---------------------------------------------------------------------------
    private void sendErrorResponse(HttpServletResponse response, int iReturnCode, String sMsg) //throws ServletException, IOException
    {
        //--- Log the error
        error(sMsg);
        
        //--- Create a ScreenResponse object 
        ScreenResponse screenResponse = new ScreenResponse("", "", "", "");
        //--- Set the return code and message 
        screenResponse.setReturnCode(iReturnCode);
        screenResponse.setReturnMessage(sMsg);

        //--- Send the HTTP Response
        try
        {
            //trace("error - step 1");
            PrintWriter out = response.getWriter();
            if (out == null)
            {
                error("Servlet output stream is null ");
                return;
            }
            //trace("error - step 1");
            //--- Generate the XML response
            XmlOutput.generateError(screenResponse, out);
            
            //trace("error - step 3");
            out.close();
            //trace("error - step 4");

        } catch (Exception e)
        {
            //--- Log the error
            error("Error : " + e.getMessage());
        }
        //trace("error - end ");
    }
    //---------------------------------------------------------------------------
}