package org.objectweb.telosys.rpl.xml;

import java.io.PrintWriter;

/**
 * Interface to give XML mapping capabilities to a Java object. <br>
 * It provides transformations :<br>
 * from XML to java object <br>
 * from Java object to XML <br>
 * This interface can be implemented by the Java object itself or by a separated XML wrapper.
 * 
 * @author Laurent GUERIN
 *  
 */
public interface IXmlWrapper
{
    //-------------------------------------------------------------------------------
    /**
     * Method to set a Java object attribute from a XML tag attribute. <br>
     * In the RPL, this method is called by the XML parser for each attribute of the tag.
     * 
     * @param sFieldName :
     *            XML attribute name
     * @param sFieldValue :
     *            XML attribute value
     */
    public void setXmlFieldValue(String sFieldName, String sFieldValue);

    //-------------------------------------------------------------------------------
    /**
     * Notify a "parent wrapper" that it has a new "child wrapper" ( find in the XML tree )
     * 
     * @param sTagName :
     *            XML tag name of the child
     * @param obj :
     *            instance of the XML wrapper for this child
     */
    public void newXmlChild(String sTagName, IXmlWrapper obj);

    //-------------------------------------------------------------------------------
    /**
     * Method to retrieve the object managed by the XML wrapper<br>
     * (object instance populated from the XML stream)
     * 
     * @return reference on the object managed by the XML wrapper
     */
    public Object getObject();

//    //-------------------------------------------------------------------------------
//    /**
//     * Method to retrieve the XML tag name associated with the XML wrapper
//     * 
//     * @return the XML tag name
//     */
//    public String getXmlTagName();

//    //-------------------------------------------------------------------------------
//    /**
//     * Print the bean hold by the wrapper as XML tag on the given output
//     * 
//     * @param out
//     */
//    public void toXml(PrintWriter out);

    //-------------------------------------------------------------------------------
    /**
     * Print the given bean as XML tag on the given output
     * @param out
     * @param bean
     * @param sTagName
     */
    public void toXml(PrintWriter out, Object bean, String sTagName );

    //-------------------------------------------------------------------------------
}