package org.objectweb.telosys.rpl.xml;

import java.util.HashMap;

import org.objectweb.telosys.common.TelosysClassLogger;
import org.objectweb.telosys.common.TelosysException;
import org.objectweb.telosys.common.TelosysObject;
import org.objectweb.telosys.common.TelosysRuntimeException;
import org.objectweb.telosys.common.vo.GenericVOListXmlWrapper;
import org.objectweb.telosys.util.ClassUtil;

/**
 * @author Laurent GUERIN
 *  
 */
public class StandardXmlWrapperRegistry extends TelosysObject implements IXmlWrapperProvider
{
    //-----------------------------------------------------------------------------------
    // Inner Class
    //-----------------------------------------------------------------------------------
    private static class GenericListWrapper
    {
        //private String      _sTagName      = null;

        private Class       _dlwClass      = null;

        //private String      _sChildTagName = null;

        private IXmlWrapper _beanWrapper   = null;

        //public GenericListWrapper(String sTagName, Class dlwClass, String sChildTagName, IXmlWrapper beanWrapper)
        public GenericListWrapper( Class dlwClass, IXmlWrapper beanWrapper)
        {
            //_sTagName = sTagName;
            _dlwClass = dlwClass;
            //_sChildTagName = sChildTagName;
            _beanWrapper = beanWrapper;
        }

        public Class getDataListWrapperClass()
        {
            return _dlwClass;
        }

//        public String getChildTagName()
//        {
//            return _sChildTagName;
//        }

//        public String getListTagName()
//        {
//            return _sTagName;
//        }

        public IXmlWrapper getBeanWrapper()
        {
            return _beanWrapper;
        }
    }

    //-----------------------------------------------------------------------------------
    private static final TelosysClassLogger log                           = new TelosysClassLogger(StandardXmlWrapperRegistry.class);

    //--- SPECIFIC WRAPPERS ( for VO beans )
    private final static int                NB_SPECIFIC                   = 40;

    //private final static HashMap            $hmSpecificWrappersByTag      = new HashMap(NB_SPECIFIC);

    private final static HashMap            $hmSpecificWrappersByClass    = new HashMap(NB_SPECIFIC);

    //--- GENERIC WRAPPERS ( for VO lists )
    private final static int                NB_GENERIC                    = 30;

    //private final static HashMap            $hmGenericListWrappersByTag   = new HashMap(NB_GENERIC);

    private final static HashMap            $hmGenericListWrappersByClass = new HashMap(NB_GENERIC);

    //-----------------------------------------------------------------------------------
    /**
     * Register a specific XML wrapper class for a specific VO bean <br>
     * (the xml tag is given by the static field 'XML_TAG')
     * 
     * @param beanWrapperClass :
     *            the wrapper class
     */
    //protected static String register(Class beanWrapperClass)
    protected static void register(Class beanWrapperClass)
    {
        //Object oValue = null;
        if (IXmlWrapper.class.isAssignableFrom(beanWrapperClass) != true)
        {
            log.error("register(" + beanWrapperClass.getName() + ") : this class does not implements IXmlWrapper !");
            //return null;
            return ;
        }

//        //--- Try to get the xml tag of the bean
//        String sTagName = null;
//        try
//        {
//            Object oValue = ClassUtil.getFieldValue(beanWrapperClass, "XML_TAG");
//            if (oValue instanceof String)
//            {
//                sTagName = (String) oValue;
//            }
//        } catch (TelosysException e)
//        {
//            sTagName = null;
//        }
//
//        if (sTagName != null)
//        {
//            //--- Register the wrapper by XML TAG NAME ( the key is the tag name )
//            $hmSpecificWrappersByTag.put(sTagName, beanWrapperClass);
//        }
//        else
//        {
//            log.error("register(" + beanWrapperClass.getName() + ") : Cannot get field XML_TAG !");
//        }

        //--- Try to get class of the bean
        Class beanClass = null;
        try
        {
            Object oValue = ClassUtil.getFieldValue(beanWrapperClass, "BEAN_CLASS");
            if (oValue instanceof Class)
            {
                beanClass = (Class) oValue;
            }
        } catch (TelosysException e)
        {
            beanClass = null;
        }

        if (beanClass != null)
        {
            //--- Register the wrapper by bean class
            $hmSpecificWrappersByClass.put(beanClass.getName(), beanWrapperClass);
        }
        else
        {
            log.error("register(" + beanWrapperClass.getName() + ") : Cannot get field BEAN_CLASS !");
        }
        //return sTagName;
        return ;
    }

    //-----------------------------------------------------------------------------------
    /**
     * Register a specific XML wrapper class for a specific VO bean with a VO list <br>
     * (the xml tag is given by the static field 'XML_TAG')
     * @param beanWrapperClass : the class of the XML wrapper
     * @param sListTagName : the tag name of the VO List 
     * @param listClass : the VO List class
     */
    //protected static void register(Class beanWrapperClass, String sListTagName, Class listClass)
    protected static void register(Class beanWrapperClass, Class listClass)
    {
        //--- Check arguments
        if (beanWrapperClass == null)
        {
            log.error("register(.,.,.) : The bean wrapper class is null !");
            return;
        }
//        if (sListTagName == null)
//        {
//            log.error("register(.,.,.) : The VOList tag name is null !");
//            return;
//        }
        if (listClass == null)
        {
            log.error("register(.,.,.) : The VOList class is null !");
            return;
        }

        //-----------------------------------------------------------------------------------
        //--- 1) Register the bean xml wrapper 
        //-----------------------------------------------------------------------------------
        //String sElementTagName = register(beanWrapperClass);
        register(beanWrapperClass);

        //-----------------------------------------------------------------------------------
        //--- 2) Register the VOList XML wrapper
        //-----------------------------------------------------------------------------------
        //--- Create the bean (list element) wrapper
        IXmlWrapper beanWrapper = createWrapperInstance(beanWrapperClass);
        //--- Register the generic wrapper definition
//        GenericListWrapper glw = new GenericListWrapper(sListTagName, listClass, sElementTagName, beanWrapper);
        GenericListWrapper glw = new GenericListWrapper( listClass, beanWrapper);
//        //--- For access by Tag Name : the key is the list tag name
//        $hmGenericListWrappersByTag.put(sListTagName, glw);
        //--- For access by Class (the key is the name of the VOList(DataListWrapper) class)
        $hmGenericListWrappersByClass.put(listClass.getName(), glw);
    }

    //-----------------------------------------------------------------------------------
    //    protected static void declareGenericListWrapper(String sListTagName, Class listClass, String sElementTagName)
    //    {
    //        GenericListWrapper glw = new GenericListWrapper(sListTagName, listClass, sElementTagName);
    //        //--- For access by Tag Name : the key is the list tag name
    //        $hmGenericListWrappersByTag.put(sListTagName, glw);
    //        //--- For access by Class (the key is the name of the
    //        // VOList(DataListWrapper) class)
    //        $hmGenericListWrappersByClass.put(listClass.getName(), glw);
    //    }

    //-----------------------------------------------------------------------------------
    /**
     * Creates a wrapper instance with the given class
     * 
     * @param objClass
     *            class to use to create a wrapper instance
     * @return the wrapper instance
     */
    private static IXmlWrapper createWrapperInstance(Object objClass)
    {
        log.trace("createWrapperInstance(" + objClass + ")...");
        if (objClass != null)
        {
            if (objClass instanceof Class)
            {
                Class wrapperClass = (Class) objClass;
                try
                {
                    //--- Create the wrapper instance with the given class
                    Object objWrapper = wrapperClass.newInstance();
                    if (objWrapper instanceof IXmlWrapper)
                    {
                        log.trace("createWrapperInstance(" + objClass + ") : OK, wrapper inctance created");
                        return (IXmlWrapper) objWrapper;
                    }
                } catch (InstantiationException e)
                {
                    log.error("createWrapperInstance(" + objClass + ") : InstantiationException ");
                } catch (IllegalAccessException e)
                {
                    log.error("createWrapperInstance(" + objClass + ") : IllegalAccessException ");
                }
            }
            else
            {
                log.error("createWrapperInstance(" + objClass + ") : argument is not an instance of 'Class' ");
            }
        }
        return null;
    }

    //-----------------------------------------------------------------------------------
    // Try to find and return the specific XML wrapper associated with the tag
    // name
    //-----------------------------------------------------------------------------------
//    private IXmlWrapper getSpecificWrapper(String sTagName)
//    {
//        trace("getSpecificWrapper(" + sTagName + ")...");
//        Object objClass = $hmSpecificWrappersByTag.get(sTagName);
//        if (objClass != null)
//        {
//            return createWrapperInstance(objClass);
//        }
//        else
//        {
//            trace("getSpecificWrapper(" + sTagName + ") : wrapper not found");
//            return null;
//        }
//    }

    private IXmlWrapper getSpecificWrapper(Class cl)
    {
        if (cl == null)
        {
            error("getSpecificWrapper() : class argument is null ");
            return null;
        }
        String sClassName = cl.getName();
        trace("getSpecificWrapper(" + sClassName + ")...");
        Object objClass = $hmSpecificWrappersByClass.get(sClassName);
        if (objClass != null)
        {
            return createWrapperInstance(objClass);
        }
        else
        {
            trace("getSpecificWrapper(" + sClassName + ") : wrapper not found");
            return null;
        }
    }

    //-----------------------------------------------------------------------------------
    // Try to find and return the generic list XML wrapper associated with the tag name
    //-----------------------------------------------------------------------------------
    private IXmlWrapper createGenericWrapperInstance(GenericListWrapper glw, String sKey)
    {
        trace("createGenericWrapperInstance(x) ...");
        if (glw != null)
        {
            //--- Create the list wrapper
            //IXmlWrapper theListWrapper = new GenericVOListXmlWrapper(glw.getListTagName(), glw.getDataListWrapperClass(), glw.getBeanWrapper());
            IXmlWrapper theListWrapper = new GenericVOListXmlWrapper( glw.getDataListWrapperClass(), glw.getBeanWrapper());
            trace("getGenericListWrapper(" + sKey + ") : OK, wrapper found ");
            return theListWrapper;
        }
        else
        {
            trace("getGenericListWrapper(" + sKey + ") : wrapper definition not found ");
            return null;
        }
    }

//    private IXmlWrapper getGenericListWrapper(String sTagName)
//    {
//        if (sTagName == null)
//        {
//            error("getGenericListWrapper() : tag argument is null ");
//            return null;
//        }
//        trace("getGenericListWrapper(" + sTagName + ")...");
//        //--- Search a Generic List Wrapper for this Tag Name
//        GenericListWrapper glw = (GenericListWrapper) $hmGenericListWrappersByTag.get(sTagName);
//        return createGenericWrapperInstance(glw, sTagName);
//    }

    private IXmlWrapper getGenericListWrapper(Class cl)
    {
        if (cl == null)
        {
            error("getGenericListWrapper() : class argument is null ");
            return null;
        }
        String sClassName = cl.getName();
        trace("getGenericListWrapper(" + sClassName + ")...");
        //--- Search a Generic List Wrapper for this Class Name
        GenericListWrapper glw = (GenericListWrapper) $hmGenericListWrappersByClass.get(sClassName);
        return createGenericWrapperInstance(glw, sClassName);
    }

    //-----------------------------------------------------------------------------------
//    /* (non-Javadoc)
//     * @see org.objectweb.telosys.rpl.xml.IXmlWrapperProvider#getWrapper(java.lang.String, int)
//     */
//    //public IXmlWrapper getWrapper(String sTagName, int iLevel)
//    public IXmlWrapper getWrapper(String sTagName)
//    {
//        trace("getWrapper(" + sTagName + ")");
//        IXmlWrapper wrapper = null;
//        //--- 1st : try to find a Specific Wrapper
//        wrapper = getSpecificWrapper(sTagName);
//        if (wrapper == null)
//        {
//            //--- 2nd : try to find an Generic List Wrapper
//            wrapper = getGenericListWrapper(sTagName);
//            if (wrapper == null)
//            {
//                trace("wrapper not found");
//            }
//        }
//        return wrapper;
//    }

    //-----------------------------------------------------------------------------------
    public IXmlWrapper getWrapper(Class cl) //throws TelosysException
    {
        if (cl == null)
        {
            throw new TelosysRuntimeException("getWrapper(Class c) : class argument is null ");
        }
        trace("getWrapper( class [" + cl.getName() + "] )");
        IXmlWrapper wrapper = null;
        //--- 1st : try to find a Specific Wrapper
        wrapper = getSpecificWrapper(cl);
        if (wrapper == null)
        {
            //--- 2nd : try to find an Generic List Wrapper
            wrapper = getGenericListWrapper(cl);
            if (wrapper == null)
            {
                trace("wrapper not found");
            }
        }
        return wrapper;
    }

    //-----------------------------------------------------------------------------------
    /**
     * @param bean
     * @return
     * @throws TelosysException
     */
    public IXmlWrapper getWrapper(Object bean) //throws TelosysException
    {
        trace("getWrapper(bean)");
        if (bean == null)
        {
            throw new TelosysRuntimeException("getWrapper(Object o) : object argument is null ");
        }
        Class cl = bean.getClass();
        return getWrapper(cl);
    }

    //-----------------------------------------------------------------------------------
//    public String getTagName(Class theClass)
//    {
//        String sClassName = theClass.getName();
//        trace("getTagName(" + theClass.getName() + ")");
//        //--- Search by class
//        GenericListWrapper glw = (GenericListWrapper) $hmGenericListWrappersByClass.get(sClassName);
//        if (glw != null)
//        {
//            return glw.getListTagName();
//        }
//        else
//        {
//            error("getTagName(" + theClass.getName() + ") : Cannot found tag name for this class");
//        }
//        return null;
//    }
    //-----------------------------------------------------------------------------------
}