package org.objectweb.telosys.rpl.xml;

import java.io.PrintWriter;
import java.util.Enumeration;
import java.util.Iterator;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.objectweb.telosys.common.TelosysClassLogger;
import org.objectweb.telosys.common.TelosysException;
import org.objectweb.telosys.common.TelosysRuntimeException;
import org.objectweb.telosys.rpl.ScreenResponse;
import org.objectweb.telosys.screen.core.Renderer;
import org.objectweb.telosys.screen.core.ScreenDataElement;
import org.objectweb.telosys.screen.core.ScreenDataElements;
import org.objectweb.telosys.screen.core.ScreenError;
import org.objectweb.telosys.screen.core.ScreenMessage;
import org.objectweb.telosys.util.XmlUtil;

/**
 * @author Laurent GUERIN
 *
 */
public class XmlOutput //extends TelosysStaticClass
{
    private static final TelosysClassLogger log = new TelosysClassLogger(XmlOutput.class);
    
	//--------------------------------------------------------------------------------------------------
	// Mthodes de vrification des paramtres
	//--------------------------------------------------------------------------------------------------
	private static boolean checkPrintWriter(PrintWriter out)
	{
		if ( out == null )
		{
			log.error( "out parameter is null !" );
			return false;
		}
		else
		{
			return true ;
		}
	}
	//--------------------------------------------------------------------------------------------------
    

	//--------------------------------------------------------------------------------------------------
	private static void beginningOfXml ( ScreenResponse resp, PrintWriter out)
	{
	    log.trace ( "beginningOfXml()" );
		if ( checkPrintWriter(out) == false ) return ;
		
		out.println("<?xml  version=\"1.0\" encoding=\"ISO-8859-1\" ?>");
		
		out.print("<response");
		out.print(" screen-name=\"" + resp.getScreenName() + "\"");
		out.print(" screen-id=\"" + resp.getScreenId() + "\"");
		out.print(" action=\"" + resp.getAction() + "\"" );
		out.println(" >");

	}

	//--------------------------------------------------------------------------------------------------
	// <return> ... </return>
	//--------------------------------------------------------------------------------------------------
	private static void genReturn ( ScreenResponse resp, PrintWriter out)
	{
	    log.trace ( "genReturn()" );
		int iCount = 0 ;
		if ( checkPrintWriter(out) == false ) return ;

		out.println("<return code=\"" + resp.getReturnCode() + "\" message=\"" + XmlUtil.xmlString( resp.getReturnMessage() ) + "\" >" );		            
//		out.println("<return>" );
//		out.println(" <code>" + resp.getReturnCode() + "</code>" );
//		out.println(" <message>" + XmlUtil.xmlString( resp.getReturnMessage() ) + "</message>" );

		//--- Flag "FOUND" or "NOT-FOUND" (or nothing)
		if ( resp.isFlagNotFound() == true )
		{
			out.println(" <not-found/>" );
		}
		if ( resp.isFlagFound() == true )
		{
			out.println(" <found/>" );
		}
		
		//--- List of exceptions 
		iCount = resp.getNumberOfExceptions() ;
		if ( iCount > 0 )
		{
		    Exception ex = null ;
			out.println("<exceptions>" );
		    for ( int i = 0 ; i < iCount ; i++ )
		    {
		        ex = resp.getException(i);
		        if ( ex != null )
		        {
					out.println(" <exception num=\"" + (i+1) +"\" class=\"" + ex.getClass().getName() + "\" message=\"" + XmlUtil.xmlString( ex.getMessage() ) + "\" />" );		            
		        }
		    }
			out.println("</exceptions>" );
		}

		//--- List of errors 
		iCount = resp.getNumberOfErrors() ;
		if ( iCount > 0 )
		{
		    ScreenError error = null ;
			out.println("<errors>" );
		    for ( int i = 0 ; i < iCount ; i++ )
		    {
		        error = resp.getError(i);
		        if ( error != null )
		        {
					out.println(" <error num=\"" + (i+1) +"\" code=\"" + error.getCode() + "\" text=\"" + XmlUtil.xmlString( error.getText() ) + "\" />" );		            
		        }
		    }
			out.println("</errors>" );
		}

		//--- List of messages 
		iCount = resp.getNumberOfMessages() ;
		if ( iCount > 0 )
		{
		    ScreenMessage message = null ;
			out.println("<messages>" );
		    for ( int i = 0 ; i < iCount ; i++ )
		    {
		        message = resp.getMessage(i);
		        if ( message != null )
		        {
					out.println(" <message num=\"" + (i+1) 
					        + "\" type=\"" + message.getType()
					        + "\" code=\"" + message.getCode()
					        + "\" text=\"" + XmlUtil.xmlString( message.getText() ) + "\" />" );		            
		        }
		    }
			out.println("</messages>" );
		}

		out.println("</return>" );
	}
	
	//--------------------------------------------------------------------------------------------------
	// <values> ... </values>
	//--------------------------------------------------------------------------------------------------
	private static void genValues ( ScreenResponse resp, PrintWriter out)
	{
	    log.trace ( "genValues()" );
		if ( checkPrintWriter(out) == false ) return ;
		if ( resp.getNumberOfValues() > 0 )
		{
		    String sValueName = null ;
			Enumeration eNames = resp.getValueNames();
			if ( eNames != null )
			{
				out.println("<values>" );
			    while ( eNames.hasMoreElements() )
			    {
			        sValueName = (String) eNames.nextElement() ;
					//OLD: out.println(" <" + sValueName + " value=\"" + XmlUtil.xmlString(resp.getValue(sValueName))+ "\" />" );
					out.println("<value " + sValueName + "=\"" + XmlUtil.xmlString(resp.getValue(sValueName))+ "\" />" );
			    }
				out.println("</values>" );			    
			}
		}
	}

	//--------------------------------------------------------------------------------------------------
	// <data> ... </data>
	//--------------------------------------------------------------------------------------------------
//	private static void genData ( ScreenResponse resp, PrintWriter out) throws TelosysException
//	{
//	    log.trace ( "genData()" );
//		if ( checkPrintWriter(out) == false ) return ;
//        ScreenData data = resp.getData();
//        if ( data == null )
//        {
//    		out.println("<data void=\"true\" />" );
//    		//out.println("<!-- No data in the response object -->" );            
//        }
//        else
//        {
//    		out.println("<data>" );
//            data.toXml(out);            
//    		out.println("</data>" );
//        }
//	}
	
	//--------------------------------------------------------------------------------------------------
	/**
	 * Generates a tag in the <data> section for each response element without "view renderer"
	 * @param screenResponse
	 * @param out
	 * @param wrapperProvider
	 * @throws TelosysException
	 */
	private static void genDataElements ( ScreenResponse screenResponse, PrintWriter out, IXmlWrapperProvider wrapperProvider) throws TelosysException
	{
	    log.trace ( "genDataElements()" );
		if ( checkPrintWriter(out) == false ) return ;
		//--- Get the elements list 
        ScreenDataElements elements = screenResponse.getDataElements(); 
        if ( elements != null ) 
        {
            //--- Generate a tag in the "<data> section" for each element without "view renderer"
            if ( elements.size() > 0 )
            {
                ScreenDataElement elem = null ;
        		out.println("<data>" );
        		Iterator iter = elements.iterator() ;
        		while ( iter.hasNext() )
        		{
        		    elem = (ScreenDataElement) iter.next();
        		    if ( elem != null )
        		    {
        		        Renderer renderer = elem.getRenderer();
        		        if ( renderer == null )
        		        {
        		            //--- No renderer => generate DATA
            		        elem.toXmlData(out, wrapperProvider);        		            
        		        }
        		    }
        		}
        		out.println("</data>" );         
        		return ;
            }
        }
        //--- No data elements 
		out.println("<data void=\"true\" />" );
	}
	
	//--------------------------------------------------------------------------------------------------
	/**
	 * Generates a tag in the <view> section for each response element associated with a "view renderer"
	 * @param screenResponse
	 * @param out
	 * @param httpRequest
	 * @param httpResponse
	 */
	private static void genViewElements ( ScreenResponse screenResponse, PrintWriter out, HttpServletRequest httpRequest, HttpServletResponse httpResponse) //throws TelosysException
	{
	    log.trace ( "genViewElements()" );
		if ( checkPrintWriter(out) == false ) return ;
		//--- Get the elements list 
        ScreenDataElements elements = screenResponse.getDataElements();
        if ( elements != null ) 
        {
            //--- Generate a tag in the "<view> section" for each element with a renderer
            if ( elements.size() > 0 )
            {
                ScreenDataElement elem = null ;
        		out.println("<view>" );
        		Iterator iter = elements.iterator() ;
        		while ( iter.hasNext() )
        		{
        		    elem = (ScreenDataElement) iter.next();
        		    if ( elem != null )
        		    {
        		        //--- Is there a view renderer for this element ?
        		        Renderer renderer = elem.getRenderer();
        		        if ( renderer != null )
        		        {
        		            //--- There's a renderer => use it !
        		            String sName = elem.getName();
        		            if ( sName == null )
        		            {
        		                throw new TelosysRuntimeException("No element name");
        		            }
        		    		out.println("<"+ sName +">");    		
        		    		out.println("<![CDATA[");    	
        		    		//--- Render the view content 
                		    renderer.render(elem, httpRequest, httpResponse);            		            
            	    		out.println("]]>");    		
        		    		out.println("</"+ sName +">");    		
        		        }
        		    }
        		}
        		out.println("</view>" );         
        		return ;
            }
        }
        //--- No data elements 
		//out.println("<view void=\"true\" />" );
	}
	
	//--------------------------------------------------------------------------------------------------
	private static void endOfXml ( PrintWriter out ) 
	{
	    log.trace ( "endOfXml()" );
		if ( checkPrintWriter(out) == false ) return ;
		out.println("</response>");
	}
	//--------------------------------------------------------------------------------------------------
    
    public final static void generate ( ScreenResponse response, PrintWriter out, HttpServletRequest httpRequest, HttpServletResponse httpResponse ) throws TelosysException
    {
        log.trace ( "generate()" );
        IXmlWrapperProvider wrapperProvider = null ;
        try
        {
            //--- Get the wrapper provider (throws Exception if not found)
            wrapperProvider = XmlWrapperManager.getXmlWrapperProvider();
        } catch (TelosysException e)
        {
            throw new TelosysRuntimeException("toXml : cannot get XmlWrapperProvider ");
        }

        beginningOfXml ( response, out);
        genReturn ( response, out);
        genValues ( response, out);
        
        //genData ( response, out);        

        genDataElements ( response, out, wrapperProvider);        
        genViewElements ( response, out, httpRequest, httpResponse);        
        
        endOfXml (out);
        out.flush();
    }
    
    public final static void generateError ( ScreenResponse response, PrintWriter out ) 
    {
        beginningOfXml ( response, out);
        genReturn ( response, out);        
        endOfXml (out);
        out.flush();        
    }
}
