package org.objectweb.telosys.rpl.xml;

import java.io.ByteArrayInputStream;
import java.io.InputStream;
import java.io.UnsupportedEncodingException;

import org.objectweb.telosys.common.Telosys;
import org.objectweb.telosys.common.TelosysException;
import org.objectweb.telosys.common.TelosysObject;
import org.xml.sax.ContentHandler;
import org.xml.sax.InputSource;
import org.xml.sax.XMLReader;
import org.xml.sax.helpers.XMLReaderFactory;

/**
 * @author Laurent GUERIN
 *  
 */
public final class XmlSaxParser extends TelosysObject
{

    /** */
    private String         _sParserClass = null;

    /** */
    //private DefaultHandler _saxHandler = null;
    private ContentHandler _saxHandler   = null;

    //---------------------------------------------------------------------------
    /**
     * @param handler :
     */
    public XmlSaxParser(final ContentHandler handler)
    {
        _saxHandler = handler;
        _sParserClass = Telosys.getSAXParserClassName();
    }

    //---------------------------------------------------------------------------
    /**
     * @param handler :
     * @param sClass :
     */
    public XmlSaxParser(final ContentHandler handler, final String sClass)
    {
        _saxHandler = handler;
        _sParserClass = sClass;
    }

    //---------------------------------------------------------------------------
    /**
     * @param stream :
     */
    public void parse(final InputStream stream) throws TelosysException
    {
        trace("parse(InputStream)...");
        try
        {
            //--- Try to create an XML reader (SAX 2 parser)
            // NB: SAX parser instances are not thread safe
            XMLReader myXmlReader = null;
            if (_sParserClass != null)
            {
                //--- Attempt to create an XML reader from the given class name
                myXmlReader = XMLReaderFactory.createXMLReader(_sParserClass);
            }
            else
            {
                //--- Attempt to create an XML reader from a system property
                myXmlReader = XMLReaderFactory.createXMLReader();
            }

            if (myXmlReader == null)
            {
                throw new TelosysException("Cannot get SAX parser : XMLReaderFactory.createXMLReader(" + _sParserClass + ")");
            }
            trace("XMLReader created.");
            
            if (_saxHandler == null)
            {
                throw new TelosysException("SAX handler is null ");
            }
            //--- Set the SAX handler SAX
            myXmlReader.setContentHandler(_saxHandler);
            trace("SAX handler set.");
            //myXmlReader.setErrorHandler(ErrorHandler h);
            //myXmlReader.setProperty("xx", obj);

            //--- Build an InputSource from the given XML InputStream 
            InputSource source = new InputSource(stream);

            //--- Parse the XML InputSource
            //PrivateTrace("XML parsing...");
            trace("XMLReader.parse(InputSource)...");
            myXmlReader.parse(source);

        } catch (Exception e)
        {
            //System.out.println("Erreur dans parse( InputStream stream ) : " +
            // e.getMessage());
            throw new TelosysException("Error in parsing XML", e);
        }
    }

    //---------------------------------------------------------------------------
    /**
     * @param stringXml :
     */
    public void parseString(final String stringXml) throws TelosysException
    {
        try
        {
            //--- Flux XML et creation de l'InputSource
            byte[] bytesXml = stringXml.getBytes("UTF8");
            if (bytesXml == null)
            {
                throw new TelosysException("Error in parseString() : stringXml.getBytes('UTF8') returns NULL ");
            }
            InputStream streamXml = new ByteArrayInputStream(bytesXml);
            //            if ( streamXml == null )
            //            {
            //                throw new TelosysException("Error in parseString() : new ByteArrayInputStream(..) returns NULL " );
            //            }
            this.parse(streamXml);
        } catch (UnsupportedEncodingException e)
        {
            throw new TelosysException("Error in parseString() : UnsupportedEncodingException ", e);
        } catch (TelosysException e)
        {
            // Just re-throw
            throw e;
        } catch (Exception e)
        {
            throw new TelosysException("Error in parseString()", e);
        }
    }
    //---------------------------------------------------------------------------
}