package org.objectweb.telosys.rpl.xml;

import org.objectweb.telosys.common.TelosysObject;
import org.xml.sax.Attributes;
import org.xml.sax.ContentHandler;
import org.xml.sax.Locator;
import org.xml.sax.SAXException;

/**
 * Event Handler for parsing XML with a SAX 2 parser (can be used with JAXP 1.2) <br>
 * NB: This handler is not "thread safe" <br>
 * 
 * @author Laurent Gurin
 * @version 1.0
 */
// Changements SAX 2 (par rapport  SAX 1)
//  . La classe "DefaultHandler" remplace "HandlerBase"
//  . L'interface "Attributes" remplace "AttributeList"
public final class XmlToObjectSaxHandler extends TelosysObject implements ContentHandler
{
    //============================================================================
    // Attributs
    //============================================================================
    /** */
    private int               _indentLevel        = 0;

    /** */
    private XmlWrapperManager _oXmlWrapperManager = null;

    //============================================================================
    // CONSTRUCTOR(S)
    //============================================================================
    /**
     * @param fact :
     */
    public XmlToObjectSaxHandler(final XmlWrapperManager fact)
    {
        _oXmlWrapperManager = fact;
    }

    //============================================================================
    // METHOD(S)
    //============================================================================
    //============================================================================
    // SAX "Callback" methods (SAX events)
    //============================================================================

    //---------------------------------------------------------------------------
    // Methods without processing
    //---------------------------------------------------------------------------
    public void endPrefixMapping(String s)
    {
    }

    public void startPrefixMapping(String prefix, String uri)
    {
    }

    public void ignorableWhitespace(char[] ch, int start, int length)
    {
    }

    public void processingInstruction(String target, String data)
    {
    }

    public void characters(char buf[], int offset, int len)
    {
    }

    public void setDocumentLocator(Locator locator)
    {
    }

    public void skippedEntity(String name)
    {
    }

    //---------------------------------------------------------------------------
    /**
     * Beginning of XML document (before root tag opening)
     * @throws SAXException :
     */
    public void startDocument() // throws SAXException
    {
        trace("-> START DOCUMENT");
        //--- Init :
        _indentLevel = 0;
    }

    //---------------------------------------------------------------------------
    /**
     * End of XML document (after root tag closing)
     * @throws SAXException :
     */
    public void endDocument()
    {
        trace("-> END DOCUMENT");
        if (_indentLevel > 0)
        {
            error("Invalid XML document structure ! ( indentLevel >0 ) **");
        }
    }

    //---------------------------------------------------------------------------
    /**
     * New TAG (ie : "<tag ..." ) with its attributes
     * @param uri :
     * @param sEltLocalName :
     * @param qName :
     * @param attrs :
     * @throws SAXException :
     */
    public void startElement(final String uri, final String sEltLocalName, final String qName, final Attributes attrs)
    {
        trace("startElement(" + uri + ", " + sEltLocalName + ", " + qName + ")");
        _indentLevel++;
        String sTagName = sEltLocalName;
        if (sTagName == null)
        {
            error("Tag name is null !");
        }
        else
        {
            //--- Demande de cration d'un nouvel objet correspondant  ce tag
            IXmlWrapper wrapper = (IXmlWrapper) _oXmlWrapperManager.openTag(sTagName, _indentLevel);
            if (wrapper != null)
            {
                //--- Valorisation des champs de l'objet  partir des attributs du tag
                if (attrs != null)
                {
                    for ( int i = 0 ; i < attrs.getLength() ; i++ ) // Pour chaque attribut
                    {
                        //--- Nom de l'attribut
                        String sAttrName = attrs.getLocalName(i);
                        //--- Valeur de l'attribut
                        String sAttrValue = attrs.getValue(i);
                        //--- Trace
                        trace("  Attribut " + i + " : " + sAttrName + " = \"" + sAttrValue + "\"");
                        wrapper.setXmlFieldValue(sAttrName, sAttrValue);
                    }
                }
            }
            else
            {
                // It's not an error
                // some tags do not have wraper ! ("data", "params", ... )
                //error("startElement() : No XML Wrapper for the tag <"+sTagName+">");
            }
        }
    }

    //---------------------------------------------------------------------------
    /**
     * End of TAG ( ie : "...</tag>" or ".../>" in the current tag )
     * @param uri :
     * @param sEltLocalName :
     * @param qName :
     * @throws SAXException :
     */
    public void endElement(final String uri, final String sEltLocalName, final String qName)
    {
        String sTagName = sEltLocalName;

        trace("endElement(" + uri + ", " + sEltLocalName + ", " + qName + ")");
        _oXmlWrapperManager.closeTag(sTagName, _indentLevel);

        _indentLevel--; // Must be at the end of the method
    }
}