package org.objectweb.telosys.screen.core;

import java.io.IOException;

import javax.servlet.RequestDispatcher;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.objectweb.telosys.common.TelosysConst;
import org.objectweb.telosys.common.TelosysObject;
import org.objectweb.telosys.common.TelosysRuntimeException;

/**
 * @author Laurent GUERIN
 *  
 */
public class Renderer extends TelosysObject
{
    private String _sRendererPath = null;

    public Renderer(String sRendererPath)
    {
        super();
        _sRendererPath = sRendererPath;
        if (sRendererPath == null)
        {
            throw new TelosysRuntimeException("Constructor : renderer path is null ");
        }
    }
    
    public String getPath()
    {
        return _sRendererPath ;
    }

    public String getElementRequestName(ScreenDataElement element)
    {
        return "TELOSYS_ELEMENT_" + element.getName() ;
    }
    
    public final void render(ScreenDataElement element, HttpServletRequest httpRequest, HttpServletResponse httpResponse)
    {
        if (httpRequest == null)
        {
            throw new TelosysRuntimeException("httpRequest is null ");
        }
        if (httpResponse == null)
        {
            throw new TelosysRuntimeException("httpResponse is null ");
        }

        trace("render() : getRequestDispatcher(" + _sRendererPath + ")");
        RequestDispatcher rd = httpRequest.getRequestDispatcher(_sRendererPath);
        if ( rd != null )
        {
            //--- 1) Set the RENDERER BEAN in the Request
            //httpRequest.setAttribute( getElementRequestName(element), element.getInstance() );
            httpRequest.setAttribute( TelosysConst.RENDERER_BEAN, element.getInstance() );            
            //--- 2) Call the RENDERER (via "include")
            try
            {
                trace("render() : rd.include(..,..)");
                rd.include(httpRequest, httpResponse);
            } catch (ServletException e)
            {
                throw new TelosysRuntimeException("render(): ServletException ", e);
            } catch (IOException e)
            {
                throw new TelosysRuntimeException("render(): IOException ", e);
            }            
            //--- 3) Remove the RENDERER BEAN from the Request
            httpRequest.removeAttribute( TelosysConst.RENDERER_BEAN );            
        }
        else
        {
            throw new TelosysRuntimeException("render(): cannot get RequestDispatcher for path '" + _sRendererPath + "'" );            
        }
    }
}