package org.objectweb.telosys.screen.core;

import java.io.PrintWriter;

import org.objectweb.telosys.common.TelosysObject;
import org.objectweb.telosys.common.TelosysRuntimeException;
import org.objectweb.telosys.rpl.xml.IXmlWrapper;
import org.objectweb.telosys.rpl.xml.IXmlWrapperProvider;

/**
 * @author xx
 *  
 */
public class ScreenDataElement extends TelosysObject
{
    private String _sElementName     = null;

    private Class  _oElementClass    = null;

    private Object _oElementInstance = null;

    private Renderer _oRenderer     = null;
    
    private void init(String sElementName, Class oElementClass) //throws TelosysException
    {
        if (sElementName != null)
        {
            _sElementName = sElementName;
        }
        else
        {
            throw new TelosysRuntimeException("Cannot create ScreenDataElement : name is null");
        }
        if (oElementClass != null)
        {
            _oElementClass = oElementClass;
        }
        else
        {
            throw new TelosysRuntimeException("Cannot create ScreenDataElement : class is null");
        }
        _oElementInstance = null;
    }

    /**
     * Creates a new element (without instance)
     * 
     * @param sElementName
     * @param oElementClass
     */
    public ScreenDataElement(String sElementName, Class oElementClass) 
    {
        super();
        init(sElementName, oElementClass);
    }

    /**
     * Creates a new element with an associated renderer (without instance)
     * @param sElementName
     * @param oElementClass
     * @param sRendererPath
     */
    public ScreenDataElement(String sElementName, Class oElementClass, String sRendererPath)
    {
        super();
        init(sElementName, oElementClass);
        //--- Creates the renderer associated with this new element
        _oRenderer = new Renderer(sRendererPath);
    }

//    /**
//     * Creates an new element with initial instance
//     * 
//     * @param sElementName
//     * @param oElementClass
//     * @param oElementInstance
//     */
//    public ScreenDataElement(String sElementName, Class oElementClass, Object oElementInstance)
//    {
//        super();
//        init(sElementName, oElementClass);
//        if (oElementInstance != null)
//        {
//            //--- Check class compatibility
//            if (_oElementClass.isAssignableFrom(oElementInstance.getClass()))
//            {
//                _oElementInstance = oElementInstance;
//            }
//            else
//            {
//                throw new TelosysRuntimeException("Cannot create ScreenDataElement : instance and class are incompatible");
//            }
//        }
//    }

    /**
     * Returns the current element class
     * 
     * @return
     */
    public Class getInstanceClass()
    {
        return _oElementClass;
    }

    /**
     * Returns the current element name
     * 
     * @return
     */
    public String getName()
    {
        return _sElementName;
    }

    /**
     * Returns the current element instance
     * 
     * @return
     */
    public Object getInstance()
    {
        return _oElementInstance;
    }

    /**
     * Set the new element instance
     * 
     * @param oInstance
     */
    public void setInstance(Object oInstance) //throws TelosysRuntimeException
    {
        if (oInstance != null)
        {
            if (_oElementClass.isAssignableFrom(oInstance.getClass()))
            {
                _oElementInstance = oInstance;
            }
            else
            {
                throw new TelosysRuntimeException("Cannot set element instance : instance and class are incompatible");
            }
        }
        else
        {
            _oElementInstance = null;
        }
    }

    /**
     * Create a new element instance
     * 
     */
    public void newInstance() 
    {
        try
        {
            _oElementInstance = _oElementClass.newInstance();
        } catch (InstantiationException ex)
        {
            throw new TelosysRuntimeException("Cannot create element instance : InstantiationException ", ex);
        } catch (IllegalAccessException ex)
        {
            throw new TelosysRuntimeException("Cannot create element instance : IllegalAccessException ", ex);
        }
    }
    
//    public int getXmlOutputType()
//    {
//        if ( _oRenderer != null )
//        {
//            return TelosysConst.XML_VIEW ; 
//        }
//        return TelosysConst.XML_DATA ; // Default standard type
//    }
    
    public void toXmlData(PrintWriter out, IXmlWrapperProvider wrapperProvider)
    {
        trace("toXmlData(out, oElementInstance, '" + _sElementName + "' )... ");
        if (out == null)
        {
            throw new TelosysRuntimeException("toXmlData : output stream is null (PrintWriter 'out' argument) ");
        }
        if (wrapperProvider == null)
        {
            throw new TelosysRuntimeException("toXmlData : wrapperProvider is null ( IXmlWrapperProvider argument) ");
        }
        if (_sElementName == null)
        {
            throw new TelosysRuntimeException("toXmlData : no element name (String '_sElementName' attribute) ");
        }
        if (_oElementInstance == null)
        {
            out.println("<!-- Element '" + _sElementName + "' instance is null -->");
            return;
        }

        //--- Get the XML Wrapper for the given object
        IXmlWrapper wrapper = null;
        try
        {
            wrapper = wrapperProvider.getWrapper(_oElementClass);
        } catch (TelosysRuntimeException e)
        {
            wrapper = null;
        }
        if (wrapper != null)
        {
            //--- Print the XML tag
            wrapper.toXml(out, _oElementInstance, _sElementName);
        }
        else
        {
            throw new TelosysRuntimeException("toXmlData : cannot get XmlWrapper for element '" + _sElementName + "'");
        }
    }

    public Renderer getRenderer()
    {
        return _oRenderer ;
    }
    
//    public void toXmlView(PrintWriter out, HttpServletRequest httpRequest, HttpServletResponse httpResponse)
//    {
//        trace("toXmlView('" + _sElementName + "' )... ");
//        
//        if (_sElementName == null)
//        {
//            throw new TelosysRuntimeException("toXmlData : no element name (String '_sElementName' attribute) ");
//        }
//        if (_oElementInstance == null)
//        {
//            out.println("<!-- Element '" + _sElementName + "' instance is null -->");
//            return;
//        }
//
//        if ( _oRenderer == null )
//        {
//            out.println("<!-- Element '" + _sElementName + "' : no view renderer -->");
//            return;            
//        }
//        
//        _oRenderer.render( _oElementInstance, httpRequest, httpResponse );
//    }

    
}