package org.objectweb.telosys.screen.core;

import java.sql.SQLException;

import org.objectweb.telosys.common.TelosysException;
import org.objectweb.telosys.common.TelosysObject;
import org.objectweb.telosys.dal.dao.DatabaseSession;
import org.objectweb.telosys.rpl.ScreenRequest;
import org.objectweb.telosys.rpl.ScreenResponse;

/**
 * @author Laurent GUERIN
 *  
 * This class is the default implementation of the ScreenActions interface.<br>
 * This is the standard behavior of a screen context.
 *  
 */
public class StandardScreenActions extends TelosysObject implements ScreenActions
{
    /**
     * @author Laurent GUERIN
     *
     * This inner class is a void implementation of the ScreenTriggers interface
     * In this implementation, all the triggers do "nothing" and always return "true"
     */
/***
    private static final class VoidTrigger implements ScreenTriggers
    {
        public boolean beforeSet(ScreenData data, ScreenContext screen, ScreenRequest request, ScreenResponse response )
        {
            return true;
        }

        public boolean afterSet(ScreenData data, ScreenContext screen, ScreenRequest request, ScreenResponse response )
        {
            return true;            
        }        
        
        public boolean beforeGet( ScreenContext screen, ScreenRequest request, ScreenResponse response )
        {
            return true;
        }

        public boolean afterGet( ScreenContext screen, ScreenRequest request, ScreenResponse response )
        {
            return true;            
        }        
        
        public boolean beforeClear( ScreenContext screen, ScreenRequest request, ScreenResponse response )
        {
            return true;
        }

        public boolean afterClear( ScreenContext screen, ScreenRequest request, ScreenResponse response )
        {
            return true;            
        }        
        
        public boolean beforeLoad(ScreenData data, ScreenContext screen, ScreenRequest req, ScreenResponse response, DatabaseSession dbSession)
        {
            return true;
        }

        public boolean afterLoad(ScreenData data, ScreenContext screen, ScreenRequest req, ScreenResponse response, DatabaseSession dbSession)
        {
            return true;
        }

        public boolean beforeSave(ScreenData data, ScreenContext screen, ScreenRequest req, ScreenResponse response, DatabaseSession dbSession)
        {
            return true;
        }

        public boolean afterSave(ScreenData data, ScreenContext screen, ScreenRequest req, ScreenResponse response, DatabaseSession dbSession)
        {
            return true;
        }

        public boolean beforeInsert(ScreenData data, ScreenContext screen, ScreenRequest req, ScreenResponse response, DatabaseSession dbSession)
        {
            return true;
        }

        public boolean afterInsert(ScreenData data, ScreenContext screen, ScreenRequest req, ScreenResponse response, DatabaseSession dbSession)
        {
            return true;
        }

        public boolean beforeUpdate(ScreenData data, ScreenContext screen, ScreenRequest req, ScreenResponse response, DatabaseSession dbSession)
        {
            return true;
        }

        public boolean afterUpdate(ScreenData data, ScreenContext screen, ScreenRequest req, ScreenResponse response, DatabaseSession dbSession)
        {
            return true;
        }

        public boolean beforeDelete(ScreenData data, ScreenContext screen, ScreenRequest req, ScreenResponse response, DatabaseSession dbSession)
        {
            return true;
        }

        public boolean afterDelete(ScreenData data, ScreenContext screen, ScreenRequest req, ScreenResponse response, DatabaseSession dbSession)
        {
            return true;
        }

    }
**/
    
    /**
     * Provide a ScreenTriggers implementation 
     * Return the specific Screen Triggers if it exists 
     * or a void implementation 
     * 
     * @return the ScreenTriggers implementation
     */
    private ScreenTriggers getScreenTriggers(ScreenContext screen)
    {
        trace("getScreenTriggers()");
        ScreenTriggers triggers = screen.getScreenTriggers();
        if (triggers != null)
        {
            return triggers;
        }
        else
        {
            //return new VoidTrigger();
            return new StandardScreenTriggers();
        }
    }

//    private ScreenDAO getScreenDAO(ScreenContext screen) throws TelosysException
//    {
//        trace("getScreenDAO()");
//        ScreenDAO dao = screen.getScreenDAO();
//        if (dao == null)
//        {
//            throw new TelosysException("StandardScreenActions - getScreenDAO() : DAO is null");
//        }
//        return dao;
//    }
    private ScreenDataAccessor getScreenDataAccessor(ScreenContext screen) throws TelosysException
    {
        trace("getScreenDataAccessor()");
        ScreenDataAccessor accessor = screen.getScreenDataAccessor();
        if (accessor == null)
        {
            throw new TelosysException("StandardScreenActions - getScreenDAO() : DataAccessor is null");
        }
        return accessor;
    }

//    Since v 0.9.4 the method "getDatabaseSession" is provided by "ScreenDAO"
//    private DatabaseSession getDatabaseSession(ScreenContext screen) throws TelosysException
//    {
//        trace("getDatabaseSession()");
//        DatabaseSession dbSession = screen.getDatabaseSession();
//        if (dbSession == null)
//        {
//            throw new TelosysException("StandardScreenActions - getDatabaseSession() : dbSession is null");
//        }
//        return dbSession;
//    }

    private void close( DatabaseSession dbSession )
    {
        if ( dbSession != null )
        {
            try
            {
                dbSession.close();
            } catch (TelosysException e)
            {
                error("close(DatabaseSession)", e);
            }
        }
    }
    
    private void rollback( DatabaseSession dbSession )
    {
        if ( dbSession != null )
        {
            try
            {
                dbSession.rollback();
            } catch (TelosysException e)
            {
                error("rollback(DatabaseSession)", e);
            }
        }
    }
    
//    private void commit( DatabaseSession dbSession )
//    {
//        if ( dbSession != null )
//        {
//            try
//            {
//                dbSession.commit();
//            } catch (TelosysException e)
//            {
//                error(e);
//            }
//        }
//    }
    
    //===========================================================================
    // Actions without persistence
    //===========================================================================
    private void setElements(ScreenData screenData, ScreenRequest request) 
    {
        //ScreenData currentScreenData = screen.getCurrentData();
        
        ScreenDataElements dataElements = request.getDataElements() ;
        int n = dataElements.size() ;
        for ( int i=0 ; i < n ; i++ ) // for each data element of the request 
        {
            ScreenDataElement e = dataElements.get(i);                
            if ( request.hasElementsAttribute() ) // the request has an "elements" attribute
            {
                //--- Set the element only if present in the "elements" list
                if ( request.elementsAttributeContains(e.getName()) )
                {
                    //--- Set the element instance
                    screenData.setElement(e.getName(), e.getInstance() );                    
                }
            }
            else
            {
                //--- No "elements" attribute => always set the element instance
                screenData.setElement(e.getName(), e.getInstance() );                    
            }
        }
    }
    
    //---------------------------------------------------------------------------
    // Perform action "SET"
    //---------------------------------------------------------------------------
    public void actionSet(ScreenContext screen, ScreenRequest request, ScreenResponse response) throws TelosysException
    {
        trace("actionSet()");
        //--- Get the triggers for this screen
        ScreenTriggers triggers = getScreenTriggers(screen);

        /**
        //--- Get the data received in the request to do the SET operation (with parameters set) 
        ScreenData newData = request.getData();
        **/
        //ScreenData newData = screen.getDataForAction();
        //ScreenData oldData = screen.getCurrentData();
        ScreenData newData = screen.beginAction();
        
        setElements( newData, request) ;
        boolean bCanceled = false ;
        //--- Check Business RULES "before SET "
        if (triggers.beforeSet(newData, screen, request, response) == false ) 
        {
            //--- Action canceled by Trigger  
            bCanceled = true ;
        }
        else
        {                
            screen.setCurrentData(newData);
            /***
            //--- Set the new data in the screen context ( all or by elements )
            if ( request.getElements() != null ) // the request has an "element" attribute
            {
                ScreenData requestScreenData = request.getData();
                ScreenData currentScreenData = screen.getCurrentData();
                //--- Attribute "elements" in request => get only those elements
                String[] reqElements = request.getElementsArray(); // Requested elements names
                //ScreenDataElements responseElements = new ScreenDataElements(); // Void list
                for ( int i = 0 ; i < reqElements.length ; i++ )
                {
                    //--- Get the element name (from request parameters)
                    String sElementName = reqElements[i];                    
                    //--- Get the request data element 
                    Object oInputElement = requestScreenData.getElement(sElementName);
                    //--- Set the element instance
                    currentScreenData.setElement(sElementName, oInputElement );                    
                }
                
            }
            else
            {
                ScreenData requestScreenData = request.getData();
                ScreenData currentScreenData = screen.getCurrentData();
                
                //ScreenDataElements responseElements = new ScreenDataElements(); // Void list
                for ( int i = 0 ; i < reqElements.length ; i++ )
                {
                    //--- Get the element name (from request parameters)
                    String sElementName = reqElements[i];                    
                    //--- Get the request data element 
                    Object oInputElement = requestScreenData.getElement(sElementName);
                    //--- Set the element instance
                    currentScreenData.setElement(sElementName, oInputElement );                    
                }
                //--- No "elements" in request => Set all the request data as the current screen data
                screen.setCurrentData(newData);
                
            }            
            ***/
            
            //--- Check Business RULES "after SET"
            if ( triggers.afterSet(newData, screen, request, response) == false )
            {
                //--- Action canceled by Trigger  
                bCanceled = true ;
                //--- Restore the original data in the screen context
                /*
                screen.setCurrentData(oldData);
                */
                screen.cancelAction();
            }                
        }
        
        //--------------------------------------------------
        //--- Operation result ( CANCELED or NOT )
        //--------------------------------------------------
        if ( bCanceled )
        {
            //--- Action RETURN : "canceled" (by Trigger) 
            response.setActionReturn( ScreenResponse.ACTION_CANCELED );
        }
        else
        {
            //--- Action RETURN : OK  
            response.setActionReturn( ScreenResponse.ACTION_OK );
        }        
    }

    //---------------------------------------------------------------------------
    // Perform action "GET"
    //---------------------------------------------------------------------------
    public void actionGet(ScreenContext screen, ScreenRequest request, ScreenResponse response) throws TelosysException
    {
        trace("actionGet()");
        //--- Get the triggers for this screen
        ScreenTriggers triggers = getScreenTriggers(screen);

        boolean bCanceled = false ;
        //--- Check Business RULES "before GET"
        if (triggers.beforeGet( screen, request, response) == false ) 
        {
            //--- Action canceled by Trigger  
            bCanceled = true ;
        }
        else
        {                
            //--- Get the current screen data and put it in the response
            //response.setData(screen.getCurrentData());
            ScreenData currentScreenData = screen.getCurrentData();
            if ( request.getElements() != null ) // the request has an "element" attribute
            {
                //--- Attribute "elements" in request => get only those elements
                String[] reqElements = request.getElementsArray(); // Requested elements names
                ScreenDataElements responseElements = new ScreenDataElements(); // Void list
                for ( int i = 0 ; i < reqElements.length ; i++ )
                {
                    //--- Get the element name (from request parameters)
                    String sElementName = reqElements[i];
                    //--- Get the screen data element associated with this name 
                    ScreenDataElement oScreenDataElement = currentScreenData.getScreenDataElement(sElementName);
                    if ( oScreenDataElement != null)
                    {
                        //--- Add the screen data element to the response elements list 
                        responseElements.add(oScreenDataElement);                        
                    }
                }
                //--- Set the selected elements as the response elements list 
                response.setDataElements(responseElements); 
            }
            else
            {
                //--- No "elements" in request => get all elements
                response.setDataElements(currentScreenData);                
            }

            //--- Check Business RULES "after GET"
            if ( triggers.afterGet( screen, request, response) == false )
            {
                //--- Action canceled by Trigger  
                bCanceled = true ;
                //--- No data returned in the response 
                //response.setData(null); 
                response.setDataElements((ScreenDataElements)null); 
            }                
        }

        //--------------------------------------------------
        //--- Operation result ( CANCELED or NOT )
        //--------------------------------------------------
        if ( bCanceled )
        {
            //--- Action RETURN : "canceled" (by Trigger) 
            response.setActionReturn( ScreenResponse.ACTION_CANCELED );
        }
        else
        {
            //--- Action RETURN : OK  
            response.setActionReturn( ScreenResponse.ACTION_OK );
        }        
        
    }

    //---------------------------------------------------------------------------
    // Perform action "CLEAR"
    //---------------------------------------------------------------------------
    public void actionClear(ScreenContext screen, ScreenRequest request, ScreenResponse response) throws TelosysException
    {
        trace("actionClear()");
        //--- Get the triggers for this screen
        ScreenTriggers triggers = getScreenTriggers(screen);
        
        //--- Keep a reference on the original screen data (for restore if necesary)
        ScreenData originalData = screen.getCurrentData();

        boolean bCanceled = false ;
        //--- Check Business RULES "before CLEAR"
        if (triggers.beforeClear( screen, request, response) == false ) 
        {
            //--- Action canceled by Trigger  
            bCanceled = true ;
        }
        else
        {
            //--- CLEAR the screnn context data
            screen.clearCurrentData();
            
            //--- Check Business RULES "after CLEAR"
            if ( triggers.afterClear( screen, request, response) == false )
            {
                //--- Action canceled by Trigger  
                bCanceled = true ;
                //--- Restore the original data in the screen context
                screen.setCurrentData(originalData);
            }                            
        }
        
        //--------------------------------------------------
        //--- Operation result ( CANCELED or NOT )
        //--------------------------------------------------
        if ( bCanceled )
        {
            //--- Action RETURN : "canceled" (by Trigger) 
            response.setActionReturn( ScreenResponse.ACTION_CANCELED );
        }
        else
        {
            //--- Action RETURN : OK  
            response.setActionReturn( ScreenResponse.ACTION_OK );
        }        
    }

    //===========================================================================
    // Actions with persistence
    //===========================================================================
    //---------------------------------------------------------------------------
    // Perform action "LOAD"
    //---------------------------------------------------------------------------
    public void actionLoad(ScreenContext screen, ScreenRequest request, ScreenResponse response) throws TelosysException
    {
        trace("actionLoad()");
        //--- Get a Database session
        //DatabaseSession dbSession = getDatabaseSession(screen);
        DatabaseSession dbSession = null ; // v 0.9.4
        try
        {            
            boolean bCanceled = false ;
            //--------------------------------------------------
            //--- Try to LOAD
            //--------------------------------------------------                        
            //--- Get the DAO for this screen
//            ScreenDAO screenDAO = getScreenDAO(screen);         
            ScreenDataAccessor screenDataAccessor = getScreenDataAccessor(screen); // v 0.9.5        
            //--- Get a Database session
//            dbSession = screenDAO.getDatabaseSession(); // v 0.9.4
            dbSession = screenDataAccessor.getDatabaseSession(); // v 0.9.5
            //--- Get the triggers for this screen
            ScreenTriggers triggers = getScreenTriggers(screen);
            //--- Get the data to use for load operation  
            ScreenData newData = screen.getDataForLoad();
//            //--- Set the Key Parameters from the REQUEST ( to do the LOAD operation )
//            screenDAO.prepareDataForLoad(newData, request); 

            //--- Check Business RULES "before LOAD"
            if (triggers.beforeLoad(newData, screen, request, response, dbSession) == false ) 
            {
                //--- Action canceled by Trigger  
                bCanceled = true ;
            }
            else
            {                
                //--- LOAD OPERATION (use the DAO to load data from the Database) 
//                screenDAO.doLoad(newData, dbSession, request);
                screenDataAccessor.doLoad(newData, screen, request, response, dbSession);
                
                //--- Check Business RULES "after LOAD"
                if ( triggers.afterLoad(newData, screen, request, response, dbSession) == false )
                {
                    //--- Action canceled by Trigger  
                    bCanceled = true ;
                }                
            }
            
            //--------------------------------------------------
            //--- Operation result ( CANCELED or NOT )
            //--------------------------------------------------
            if ( bCanceled )
            {
                //--- Action RETURN : "canceled" (by Trigger) 
                response.setActionReturn( ScreenResponse.ACTION_CANCELED );
            }
            else
            {
                //--- Action RETURN : OK  
                response.setActionReturn( ScreenResponse.ACTION_OK );

                //--- Use the data loaded (if found)
                if (newData.isFound())
                {
                    //--- Set the current screen data if the request flag SET is TRUE
                    if (request.getFlagSetData() == true) //-- SET="true"
                    {
                        //--- Set the data loaded as the current screen
                        screen.setCurrentData(newData);
                    }
                    //--- Set the response flag "FOUND"
                    response.setFlagFound(); 
                    //--- Return the data in the response if flag "GET" is TRUE
                    if (request.getFlagGetData() == true) //-- GET="true"
                    {
                        //--- Return the data loaded in the response
                        //response.setData(newData);
                        response.setDataElements(newData);                        
                    }
                }
                else
                {
                    //--- Set the response flag "NOT FOUND"
                    response.setFlagNotFound(); 
                }
            }
            
        } catch (TelosysException tex)
        {
            // dbSession.rollback(); // No rollback on LOAD operation
            error("actionLoad() : TelosysException ", tex );
            response.setActionReturn( ScreenResponse.ACTION_ERROR );
            response.addError(tex);
        } catch (Exception ex)
        {
            // dbSession.rollback(); // No rollback on LOAD operation
            error("actionLoad() : Exception ", ex );
            response.setActionReturn( ScreenResponse.ACTION_ERROR );
            response.addError(ex);
        } catch (Throwable th)
        {
            // dbSession.rollback(); // No rollback on LOAD operation
            error("actionLoad() : Throwable ", th );
            response.setActionReturn( ScreenResponse.ACTION_ERROR );
            response.addError(th);
        } finally
        {
            //--- Close (release) the dbSession
            //dbSession.close();                
            close(dbSession); // v 0.9.4
        }
    }

    //---------------------------------------------------------------------------
    // Perform action "SAVE"
    //---------------------------------------------------------------------------
    public void actionSave(ScreenContext screen, ScreenRequest request, ScreenResponse response) throws TelosysException
    {
        trace("actionSave()");
        ScreenData screenData = screen.beginAction();
        setElements(screenData, request);
        if (request.getFlagSetData() == true)
        {
            //--- "SET Flag" is "TRUE" => set the current screen data
            /*
            screen.setCurrentData(request.getData());
            */
            screen.setCurrentData(screenData);
        }
        //--- Save the Data provided by the request
        /*
        ScreenData dataToSave = request.getData();
        save(screen, request, response, dataToSave);
        */
        boolean bOK = save(screen, request, response, screenData) ;
        if ( bOK != true )
        {
            screen.cancelAction();
        }
    }

    //---------------------------------------------------------------------------
    // Perform action "SAVE_CURRENT"
    //---------------------------------------------------------------------------
    public void actionSaveCurrent(ScreenContext screen, ScreenRequest request, ScreenResponse response) throws TelosysException
    {
        trace("actionSaveCurrent()");
        // No request flag "SET" to managed
        //--- Save the current data of the screen
        ScreenData dataToSave = screen.getCurrentData();
        save(screen, request, response, dataToSave);
    }

    //---------------------------------------------------------------------------
    // Generic behavior for "SAVE and "SAVE_CURRENT"
    //---------------------------------------------------------------------------
    private boolean save(ScreenContext screen, ScreenRequest request, ScreenResponse response, ScreenData dataToSave) throws TelosysException
    {
        trace("save()");
        //--- Get a Database session
        //DatabaseSession dbSession = getDatabaseSession(screen);
        DatabaseSession dbSession = null ; // v 0.9.4
        boolean bCanceled = false ;
        try
        {
            //--------------------------------------------------
            //--- Try to SAVE
            //--------------------------------------------------
            //--- Get the DAO for this screen
//            ScreenDAO dao = getScreenDAO(screen);
            ScreenDataAccessor screenDataAccessor = getScreenDataAccessor(screen); // v 0.9.5        
            //--- Get a Database session
//            dbSession = dao.getDatabaseSession(); // v 0.9.4
            dbSession = screenDataAccessor.getDatabaseSession(); // v 0.9.5
            //--- Get the triggers for this screen
            ScreenTriggers triggers = getScreenTriggers(screen);
            //--- Check Business RULES "before SAVE"
            if ( triggers.beforeSave(dataToSave, screen, request, response, dbSession) == false )
            {
                //--- Action canceled by Trigger  
                bCanceled = true ;
            }
            else
            {
                //--- SAVE OPERATION (use the DAO to save the data in database)
//                dao.doSave(dataToSave, dbSession);
                screenDataAccessor.doSave(dataToSave, screen, request, response, dbSession);
                //--- Check Business RULES "after SAVE"
                if ( triggers.afterSave(dataToSave, screen, request, response, dbSession) == false )
                {
                    //--- Action canceled by Trigger  
                    bCanceled = true ;
                }
            }
            
            //--------------------------------------------------
            //--- Operation result ( CANCELED or NOT )
            //--------------------------------------------------
            if ( bCanceled )
            {
                //--- Rollback all operations ( Before / Save / After )
                dbSession.rollback();                         
                //--- Action RETURN : "canceled" (by Trigger) 
                response.setActionReturn( ScreenResponse.ACTION_CANCELED );
            }
            else
            {
                //--- Commit all operations ( Before / Save / After )
                dbSession.commit();                
                //--- Action RETURN : OK  
                response.setActionReturn( ScreenResponse.ACTION_OK );
            }

        } catch (TelosysException tex)
        {
            //dbSession.rollback();
            rollback(dbSession); // v 0.9.4
            error("save() : TelosysException", tex);
            response.setActionReturn( ScreenResponse.ACTION_ERROR );
            response.addError(tex);
        } catch (Exception ex)
        {
            //dbSession.rollback();
            rollback(dbSession); // v 0.9.4
            error("save() : Exception", ex);
            response.setActionReturn( ScreenResponse.ACTION_ERROR );
            response.addError(ex);
        } catch (Throwable th)
        {
            //dbSession.rollback();
            rollback(dbSession); // v 0.9.4
            error("save() : Throwable", th);
            response.setActionReturn( ScreenResponse.ACTION_ERROR );
            response.addError(th);
        } finally
        {
            //--- Close (release) the dbSession
//            dbSession.close();                
            close(dbSession); // v 0.9.4
        }
        return (! bCanceled);
    }
    
    //---------------------------------------------------------------------------
    // Perform action "INSERT"
    //---------------------------------------------------------------------------
    public void actionInsert(ScreenContext screen, ScreenRequest request, ScreenResponse response) throws TelosysException
    {
        trace("actionInsert()");
        ScreenData screenData = screen.beginAction();
        setElements(screenData, request);
        if (request.getFlagSetData() == true)
        {
            //--- "SET Flag" is "TRUE" => set the current screen data
            /*
            screen.setCurrentData(request.getData());
            */
            screen.setCurrentData(screenData);
        }
        //--- Insert the Data provided by the request
        /*
        ScreenData dataToInsert = request.getData();
        insert(screen, request, response, dataToInsert);
        */
        boolean bOK = insert(screen, request, response, screenData) ;
        if ( bOK != true )
        {
            screen.cancelAction();
        }
    }


    //---------------------------------------------------------------------------
    // Generic behavior for "INSERT" and "INSERT_CURRENT"
    //---------------------------------------------------------------------------
    private boolean insert(ScreenContext screen, ScreenRequest request, ScreenResponse response, ScreenData dataToInsert) throws TelosysException
    {
        trace("insert()");
        DatabaseSession dbSession = null ; // v 0.9.4
        boolean bCanceled = false ;
        try
        {
            //--------------------------------------------------
            //--- Try to INSERT 
            //--------------------------------------------------
            //--- Get the DAO for this screen
//            ScreenDAO dao = getScreenDAO(screen);
            ScreenDataAccessor screenDataAccessor = getScreenDataAccessor(screen); // v 0.9.5        
            //--- Get a Database session
//            dbSession = dao.getDatabaseSession(); // v 0.9.4           
            dbSession = screenDataAccessor.getDatabaseSession(); // v 0.9.5
            //--- Get the triggers for this screen
            ScreenTriggers triggers = getScreenTriggers(screen);
            //--- Check Business RULES "before INSERT"
            if ( triggers.beforeInsert(dataToInsert, screen, request, response, dbSession) == false )
            {
                //--- Action canceled by Trigger  
                bCanceled = true ;
            }
            else
            {
                //--- INSERT OPERATION (use the DAO to insert the data in database)
//                dao.doInsert(dataToInsert, dbSession);
                screenDataAccessor.doInsert(dataToInsert, screen, request, response, dbSession);
                //--- Check Business RULES "after INSERT"
                if ( triggers.afterInsert(dataToInsert, screen, request, response, dbSession) == false )
                {
                    //--- Action canceled by Trigger  
                    bCanceled = true ;
                }
            }
            
            //--------------------------------------------------
            //--- Operation result ( CANCELED or NOT )
            //--------------------------------------------------
            if ( bCanceled )
            {
                //--- Rollback all operations ( Before / Save / After )
                dbSession.rollback();                         
                //--- Action RETURN : "canceled" (by Trigger) 
                response.setActionReturn( ScreenResponse.ACTION_CANCELED );
            }
            else
            {
                //--- Commit all operations ( Before / Save / After )
                dbSession.commit();                
                //--- Action RETURN : OK  
                response.setActionReturn( ScreenResponse.ACTION_OK );
            }

        } catch (TelosysException tex)
        {
            rollback(dbSession); // v 0.9.4            
            error("insert() : TelosysException ", tex);
            response.setActionReturn( ScreenResponse.ACTION_ERROR );
            response.addError(tex);
            if ( tex.getCause() != null )
            {
                Throwable t = tex.getCause();
                response.addError(t);
                if ( t instanceof SQLException )
                {
                    SQLException sqlEx = (SQLException) t ;
                    String sMsg = "SQL Error : SQL State = " + sqlEx.getSQLState() 
                    		+ " - Vendor-specific error code = " + sqlEx.getErrorCode() ;
                    ScreenError error = new ScreenError(0, sMsg);
                    response.addError(error);
                }
            }
        } catch (Exception ex)
        {
            rollback(dbSession); // v 0.9.4            
            error("insert() : Exception ", ex);
            response.setActionReturn( ScreenResponse.ACTION_ERROR );
            response.addError(ex);
        } catch (Throwable th)
        {
            rollback(dbSession); // v 0.9.4            
            error("insert() : Throwable ", th);
            response.setActionReturn( ScreenResponse.ACTION_ERROR );
            response.addError(th);
        } finally
        {
            //--- Close (release) the dbSession
            close(dbSession); // v 0.9.4            
        }
        return (! bCanceled);        
    }

    //---------------------------------------------------------------------------
    // Perform action "UPDATE"
    //---------------------------------------------------------------------------
    public void actionUpdate(ScreenContext screen, ScreenRequest request, ScreenResponse response) throws TelosysException
    {
        trace("actionUpdate()");
        ScreenData screenData = screen.beginAction();
        setElements(screenData, request);
        if (request.getFlagSetData() == true)
        {
            //--- "SET Flag" is "TRUE" => set the current screen data
            /*
            screen.setCurrentData(request.getData());
            */
            screen.setCurrentData(screenData);
        }
        //--- Update with the Data provided by the request
        /*
        ScreenData dataToInsert = request.getData();
        update(screen, request, response, dataToInsert);
        */
        boolean bOK = update(screen, request, response, screenData) ;
        if ( bOK != true )
        {
            screen.cancelAction();
        }
    }

    //---------------------------------------------------------------------------
    // Generic behavior for "UPDATE" and "UPDATE_CURRENT"
    //---------------------------------------------------------------------------
    private boolean update(ScreenContext screen, ScreenRequest request, ScreenResponse response, ScreenData dataToUpdate) throws TelosysException
    {
        trace("update()");
        DatabaseSession dbSession = null ; // v 0.9.4
        boolean bCanceled = false ;
        try
        {
            //--------------------------------------------------
            //--- Try to UPDATE 
            //--------------------------------------------------
            //--- Get the DAO for this screen
//            ScreenDAO dao = getScreenDAO(screen);
            ScreenDataAccessor screenDataAccessor = getScreenDataAccessor(screen); // v 0.9.5        
            //--- Get a Database session
//            dbSession = dao.getDatabaseSession(); // v 0.9.4      
            dbSession = screenDataAccessor.getDatabaseSession(); // v 0.9.5            
            //--- Get the triggers for this screen
            ScreenTriggers triggers = getScreenTriggers(screen);
            //--- Check Business RULES "before UPDATE"
            if ( triggers.beforeUpdate(dataToUpdate, screen, request, response, dbSession) == false )
            {
                //--- Action canceled by Trigger  
                bCanceled = true ;
            }
            else
            {
                //--- INSERT OPERATION (use the DAO to insert the data in database)
//                dao.doUpdate(dataToUpdate, dbSession);
                screenDataAccessor.doUpdate(dataToUpdate, screen, request, response, dbSession);
                //--- Check Business RULES "after UPDATE"
                if ( triggers.afterUpdate(dataToUpdate, screen, request, response, dbSession) == false )
                {
                    //--- Action canceled by Trigger  
                    bCanceled = true ;
                }
            }
            
            //--------------------------------------------------
            //--- Operation result ( CANCELED or NOT )
            //--------------------------------------------------
            if ( bCanceled )
            {
                //--- Rollback all operations ( Before / Save / After )
                dbSession.rollback();                         
                //--- Action RETURN : "canceled" (by Trigger) 
                response.setActionReturn( ScreenResponse.ACTION_CANCELED );
            }
            else
            {
                //--- Commit all operations ( Before / Save / After )
                dbSession.commit();                
                //--- Action RETURN : OK  
                response.setActionReturn( ScreenResponse.ACTION_OK );
            }

        } catch (TelosysException tex)
        {
            rollback(dbSession); // v 0.9.4            
            error("update() : TelosysException", tex);
            response.setActionReturn( ScreenResponse.ACTION_ERROR );
            response.addError(tex);
            if ( tex.getCause() != null )
            {
                Throwable t = tex.getCause();
                response.addError(t);
                if ( t instanceof SQLException )
                {
                    SQLException sqlEx = (SQLException) t ;
                    String sMsg = "SQL Error : SQL State = " + sqlEx.getSQLState() 
                    		+ " - Vendor-specific error code = " + sqlEx.getErrorCode() ;
                    ScreenError error = new ScreenError(0, sMsg);
                    response.addError(error);
                }
            }
        } catch (Exception ex)
        {
            rollback(dbSession); // v 0.9.4            
            error("update() : Exception", ex);
            response.setActionReturn( ScreenResponse.ACTION_ERROR );
            response.addError(ex);
        } catch (Throwable th)
        {
            rollback(dbSession); // v 0.9.4
            error("update() : Throwable", th);
            response.setActionReturn( ScreenResponse.ACTION_ERROR );
            response.addError(th);
        } finally
        {
            //--- Close (release) the dbSession
            close(dbSession); // v 0.9.4                        
        }
        return (! bCanceled);        
    }
    
    //---------------------------------------------------------------------------
    // Perform action "DELETE"
    //---------------------------------------------------------------------------
    public void actionDelete(ScreenContext screen, ScreenRequest request, ScreenResponse response) throws TelosysException
    {
        trace("actionDelete()");
        //--- Set the parameters for the SQL DELETE
        ScreenData dataToDelete = screen.getDataForDelete(); // Work on Temporay Data
        //--- Set the Key Parameters from the REQUEST ( to do the LOAD operation )
//        //--- Get the DAO for this screen
//        ScreenDAO dao = getScreenDAO(screen);
//        dao.prepareDataForDelete(dataToDelete, request); 
        //--- Call the common DELETE operation
        delete(screen, request, response, dataToDelete);
    }

    //---------------------------------------------------------------------------
    // Perform action "DELETE_CURRENT"
    //---------------------------------------------------------------------------
    public void actionDeleteCurrent(ScreenContext screen, ScreenRequest request, ScreenResponse response) throws TelosysException
    {
        trace("actionDeleteCurrent()");
        ScreenData dataToDelete = screen.getCurrentData();
        //--- Call the common DELETE operation
        delete(screen, request, response, dataToDelete);
    }

    //---------------------------------------------------------------------------
    // Generic behavior for "DELETE and "DELETE_CURRENT"
    //---------------------------------------------------------------------------
    private void delete(ScreenContext screen, ScreenRequest request, ScreenResponse response, ScreenData dataToDelete) throws TelosysException
    {
        trace("delete()");
        //--- Get a Database session
        //DatabaseSession dbSession = getDatabaseSession(screen);
        DatabaseSession dbSession = null ; // v 0.9.4
        
        try
        {
            boolean bCanceled = false ;
            //--------------------------------------------------
            //--- Try to DELETE
            //--------------------------------------------------            
            //--- Get the DAO for this screen
//            ScreenDAO dao = getScreenDAO(screen);
            ScreenDataAccessor screenDataAccessor = getScreenDataAccessor(screen); // v 0.9.5        
            //--- Get a Database session
//            dbSession = dao.getDatabaseSession(); // v 0.9.4      
            dbSession = screenDataAccessor.getDatabaseSession(); // v 0.9.5                        
            //--- Get the triggers for this screen
            ScreenTriggers triggers = getScreenTriggers(screen);

            //--- Check Business RULES "before DELETE"
            if (triggers.beforeDelete(dataToDelete, screen, request, response, dbSession) == false)
            {
                //--- Action canceled by Trigger  
                bCanceled = true ;
            }
            else
            {
                //--- DELETE OPERATION (use the DAO to delete data in the Database) 
//                dao.doDelete(dataToDelete, dbSession);
                screenDataAccessor.doDelete(dataToDelete, screen, request, response, dbSession);
                //--- Check Business RULES "after DELETE"
                if ( triggers.afterDelete(dataToDelete, screen, request, response, dbSession) == false )
                {
                    //--- Action canceled by Trigger  
                    bCanceled = true ;
                }
            }
            
            //--------------------------------------------------
            //--- Operation result ( CANCELED or NOT )
            //--------------------------------------------------
            if ( bCanceled )
            {
                //--- Rollback all operations ( Before / Save / After )
                dbSession.rollback();                         
                //--- Action RETURN : "canceled" (by Trigger) 
                response.setActionReturn( ScreenResponse.ACTION_CANCELED );
            }
            else
            {
                //--- Commit all operations ( Before / Save / After )
                dbSession.commit();                
                //--- Request Flag "clear=true/false"
                if (request.getFlagClearData() == true)
                {
                    //--- "CLEAR Flag" : clear the current screen data
                    screen.clearCurrentData();
                }
                //--- Action RETURN : OK  
                response.setActionReturn( ScreenResponse.ACTION_OK );
            }

        } catch (TelosysException tex)
        {
            rollback(dbSession); // v 0.9.4                        
            error("delete() : TelosysException ", tex );
            response.setActionReturn( ScreenResponse.ACTION_ERROR );
            response.addError(tex);
        } catch (Exception ex)
        {
            rollback(dbSession); // v 0.9.4            
            error("delete() : Exception ", ex );
            response.setActionReturn( ScreenResponse.ACTION_ERROR );
            response.addError(ex);
        } catch (Throwable th)
        {
            rollback(dbSession); // v 0.9.4            
            error("delete() : Throwable ", th );
            response.setActionReturn( ScreenResponse.ACTION_ERROR );
            response.addError(th);
        } finally
        {
            //--- Close (release) the dbSession
            close(dbSession); // v 0.9.4                        
        }
    }

    //---------------------------------------------------------------------------
    // Perform action "EXEC"
    //---------------------------------------------------------------------------
    public void actionExec(ScreenContext screen, ScreenRequest request, ScreenResponse response) throws TelosysException
    {   
        //--- Retrieve the Procedure Name from the request
        String sProcedureName = request.getProcName();
        trace("exec() : " + sProcedureName);
        
        if ( sProcedureName == null )
        {
            //--- Action RETURN : ERROR
            response.setActionReturn(ScreenResponse.ACTION_ERROR);
            ScreenMessage msg = new ScreenMessage(0, "StandardScreenActions : No procedure name (ProcName is null)", 
                    								ScreenMessage.ERROR );
            response.addMessage(msg);                                    
            return ;
        }
        
        try
        {
            //--------------------------------------------------
            //--- Try to EXECUTE the procedure
            //--------------------------------------------------            
            //--- Get the procedures for this screen
            ScreenProcedures procedures = screen.getScreenProcedures();
            if ( procedures != null )
            {
                boolean bProcRet = procedures.execute(sProcedureName, screen, request, response) ;
                if ( bProcRet == true )
                {
                    //--- Action RETURN : OK  
                    response.setActionReturn( ScreenResponse.ACTION_OK );
                }
                else
                {
                    //--- Action RETURN : ERROR
                    response.setActionReturn(ScreenResponse.ACTION_ERROR);
                    ScreenMessage msg = new ScreenMessage(0, "StandardScreenActions : Procedure execution return false ", 
                            								ScreenMessage.ERROR );
                    response.addMessage(msg);                                    
                }
            }
            else
            {
                //--- Action RETURN : ERROR
                response.setActionReturn(ScreenResponse.ACTION_ERROR);
                ScreenMessage msg = new ScreenMessage(0, "StandardScreenActions : Procedures not found for this screen (Screen class = " + screen.getClass().getName(), 
                        								ScreenMessage.ERROR );
                response.addMessage(msg);                
            }
        } catch (TelosysException tex)
        {
            error("actionExec() : TelosysException ", tex );
            response.setActionReturn( ScreenResponse.ACTION_ERROR );
            response.addError(tex);
        } catch (Exception ex)
        {
            error("actionExec() : Exception ", ex );
            response.setActionReturn( ScreenResponse.ACTION_ERROR );
            response.addError(ex);
        } catch (Throwable th)
        {
            error("actionExec() : Throwable ", th );
            response.setActionReturn( ScreenResponse.ACTION_ERROR );
            response.addError(th);
        } finally
        {
            //--- Nothing to do
            //dbSession.close();
        }
        
    }

}