package org.objectweb.telosys.screen.core;

import java.util.Hashtable;

import javax.servlet.http.HttpSession;

import org.objectweb.telosys.common.TelosysObject;
import org.objectweb.telosys.screen.env.ScreenApplication;
import org.objectweb.telosys.screen.env.ScreenSession;

/**
 * The standard ScreenContext implementation.
 * 
 * @author Laurent GUERIN
 *  
 */
public class StandardScreenContext extends TelosysObject implements ScreenContext
{
    private String             _sScreenName        = null;

    private int                _iScreenId          = 0;              // ADD v 0.9.4

    //private ScreenApplication _screenApplication = null;

    private ScreenSession      _screenSession      = null;

    //private Class _screenClass = null; // SUPPR v 0.9.4

    private ScreenData         _currentScreenData  = null;

    private ScreenData         _screenDataImage    = null;

    private ScreenActions      _screenActions      = null;

    private ScreenTriggers     _screenTriggers     = null;

    private ScreenProcedures   _screenProcedures   = null;

    //    private ScreenDAO _screenDAO = null;

    private ScreenDataAccessor _screenDataAccessor = null;

    private Class              _screenDataClass    = null;

    //--- Screen Data used for LOAD operation
    private ScreenData         _newDataLoaded      = null;

    //--- Reusable temporary Screen Data (used only for DELETE operations)
    private ScreenData         _tempData           = null;

    private Hashtable          _htData             = new Hashtable();

    //    public abstract void prepareDataForLoad(ScreenRequest request, ScreenData data) throws TelosysException;
    //
    //    public abstract void prepareDataForDelete(ScreenRequest request, ScreenData data) throws TelosysException;

    //    public StandardScreenContext(ScreenData currentData)
    //    {
    //        _currentData = currentData; // can be null
    //    }
    //    private StandardScreenContext()
    //    {
    //        _currentData = null; // can be null
    //    }

    //    protected void initScreen(String sScreenName, Class screenClass)
    //    {
    //        _sScreenName = sScreenName;
    //        _screenClass = screenClass;
    //    }
    //public void initScreen(String sScreenName, Class screenClass, Class screenDataClass, ScreenSession screenSession )
    /**
     * StandardScreenContext constructor
     * 
     * @param sScreenName :
     *            the symbolic screen name
     * @param screenDataClass :
     *            the class of the screen data
     * @param screenSession :
     *            the screen session
     * @param iScreenId :
     *            the screen ID ( 0 to N )
     */
    public StandardScreenContext(String sScreenName, Class screenDataClass, ScreenSession screenSession, int iScreenId) // v 0.9.4
    {
        _currentScreenData = null; // null : no ScreenData
        _sScreenName = sScreenName;
        //_screenClass = screenClass; // SUPPR v 0.9.4
        _screenDataClass = screenDataClass;
        _screenSession = screenSession;
        //_screenApplication = screenApplication ;
        //--- Set the Screen ID // ADD v 0.9.4
        if (_iScreenId >= 0)
        {
            _iScreenId = iScreenId;
        }
        else
        {
            error("ScreenContext creation : initScreen : invalid Screen ID : " + iScreenId);
            _iScreenId = 0;
        }
    }

    //---------------------------------------------------------------------------
    // get the screen name
    //---------------------------------------------------------------------------
    public String getScreenName()
    {
        return _sScreenName;
    }

    public int getScreenId()
    {
        return _iScreenId;
    }

    //---------------------------------------------------------------------------
    public ScreenApplication getScreenApplication()
    {
        if (_screenSession != null)
        {
            return _screenSession.getScreenApplication();
        }
        else
        {
            error("getScreenApplication() : ScreenSession instance is null !");
            return null;
        }
    }

    //---------------------------------------------------------------------------
    public ScreenSession getScreenSession()
    {
        return _screenSession;
    }

    //---------------------------------------------------------------------------
    public HttpSession getHttpSession()
    {
        if (_screenSession != null)
        {
            return _screenSession.getHttpSession();
        }
        else
        {
            error("getHttpSession() : ScreenSession instance is null !");
            return null;
        }
    }

    //---------------------------------------------------------------------------
    // get the screen class
    //---------------------------------------------------------------------------
    //    public Class getScreenClass() // SUPPR v 0.9.4
    //    {
    //        return _screenClass;
    //    }

    //---------------------------------------------------------------------------
    // get the screen class name
    //---------------------------------------------------------------------------
    //    public String getScreenClassName() // SUPPR v 0.9.4
    //    {
    //        if ( _screenClass != null )
    //        {
    //            return _screenClass.getName();
    //        }
    //        return null;
    //    }

    //---------------------------------------------------------------------------
    // get/set/clear the current data of the screen
    //---------------------------------------------------------------------------
    /*
     * (non-Javadoc)
     * 
     * @see org.objectweb.telosys.screen.core.ScreenContext#getCurrentData()
     */
    public ScreenData getCurrentData()
    {
        return _currentScreenData;
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.objectweb.telosys.screen.core.ScreenContext#getOrCreateCurrentData()
     */
    public ScreenData getOrCreateCurrentData()
    {
        if (_currentScreenData == null)
        {
            //--- No current data => create it
            _currentScreenData = createNewScreenData();
        }
        return _currentScreenData;
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.objectweb.telosys.screen.core.ScreenContext#setCurrentData(org.objectweb.telosys.screen.core.ScreenData)
     */
    public void setCurrentData(ScreenData data)
    {
        _currentScreenData = data;
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.objectweb.telosys.screen.core.ScreenContext#clearCurrentData()
     */
    public void clearCurrentData()
    {
        _currentScreenData = null;
    }

    /**
     * Inject DATA CLASS dependency
     * 
     * @param screenDataClass
     */
    /* package */void setScreenDataClass(Class screenDataClass)
    {
        _screenDataClass = screenDataClass;
    }

//    /**
//     * Inject DAO dependency
//     * 
//     * @param screenDAO
//     */
//    /* package */void setScreenDAO(ScreenDAO screenDAO)
//    {
//        _screenDAO = screenDAO;
//    }

    /**
     * Injects the ScreenDataAccessor dependency ( v 0.9.5 )
     * @param screenDataAccessor
     */
    /* package */ void setScreenDataAccessor(ScreenDataAccessor screenDataAccessor)
    {
        _screenDataAccessor = screenDataAccessor;
    }

    /**
     * Injects ACTIONS dependency
     * 
     * @param screenActions
     */
    /* package */void setScreenActions(ScreenActions screenActions)
    {
        _screenActions = screenActions;
    }

    /**
     * Inject TRIGGERS dependency
     * 
     * @param screenTriggers
     */
    /* package */void setScreenTriggers(ScreenTriggers screenTriggers)
    {
        _screenTriggers = screenTriggers;
    }

    /**
     * Inject PROCEDURES dependency
     * 
     * @param screenProcedures
     */
    /* package */void setScreenProcedures(ScreenProcedures screenProcedures)
    {
        _screenProcedures = screenProcedures;
    }

    //==============================================================================
    // ABSTRACT METHODS IMPLEMENTATIONS
    //==============================================================================

    /*
     * (non-Javadoc)
     * 
     * @see org.objectweb.telosys.screen.core.ScreenContext#getScreenActions()
     */
    public ScreenActions getScreenActions()
    {
        return _screenActions;
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.objectweb.telosys.screen.core.ScreenContext#getScreenTriggers()
     */
    public ScreenTriggers getScreenTriggers()
    {
        return _screenTriggers; // or null if no specific triggers
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.objectweb.telosys.screen.core.ScreenContext#getScreenProcedures()
     */
    public ScreenProcedures getScreenProcedures()
    {
        return _screenProcedures; // or null if no specific triggers
    }

//    /*
//     * (non-Javadoc)
//     * 
//     * @see org.objectweb.telosys.screen.core.ScreenContext#getScreenDAO()
//     */
//    public ScreenDAO getScreenDAO()
//    {
//        return _screenDAO;
//    }

    /* (non-Javadoc)
     * @see org.objectweb.telosys.screen.core.ScreenContext#getScreenDataAccessor()
     */
    public ScreenDataAccessor getScreenDataAccessor()
    {
        return _screenDataAccessor ;
    }

    // SUPPR. v 0.9.4
    //    public DatabaseSession getDatabaseSession() throws TelosysException
    //    {
    //        return ConnectionManager.getSession();
    //    }

    /*
     * (non-Javadoc)
     * 
     * @see org.objectweb.telosys.screen.core.ScreenContext#createNewScreenData()
     */
    public ScreenData createNewScreenData()
    {
        ScreenData screenData = null;
        //--- If the class to build screen data is set...
        if (_screenDataClass != null)
        {
            //--- Try to create a valid instance of ScreenData with the given class
            try
            {
                //--- Create the screenData instance with the class
                Object object = _screenDataClass.newInstance();
                if (object instanceof ScreenData)
                {
                    screenData = (ScreenData) object;
                }
            } catch (InstantiationException e)
            {
                screenData = null;
                error("createNewScreenData() : InstantiationException ! ", e);
            } catch (IllegalAccessException e)
            {
                screenData = null;
                error("createNewScreenData() : IllegalAccessException ! ", e);
            }
        }
        return screenData;
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.objectweb.telosys.screen.core.ScreenContext#beginAction()
     */
    public ScreenData beginAction()
    {
        if (_currentScreenData == null)
        {
            _screenDataImage = null;
            _currentScreenData = createNewScreenData();
        }
        else
        {
            if (_screenDataImage == null)
            {
                _screenDataImage = createNewScreenData();
            }
            _currentScreenData.copyElementsTo(_screenDataImage);
        }
        return _currentScreenData;
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.objectweb.telosys.screen.core.ScreenContext#cancelAction()
     */
    public void cancelAction()
    {
        //--- Restore the preserved ScreenData image
        _currentScreenData = _screenDataImage;
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.objectweb.telosys.screen.core.ScreenContext#getDataForLoad(org.objectweb.telosys.rpl.ScreenRequest)
     */
    //public ScreenData getDataForLoad(ScreenRequest request) throws TelosysException
    public ScreenData getDataForLoad() //throws TelosysException
    {
        trace("getDataForLoad");
        //--- Create a new ScreenData to store the data loaded
        _newDataLoaded = createNewScreenData();

        // v 0.9.4 : now done in the Screen DAO
        //--- Set the Key Parameters from the REQUEST ( to do the LOAD operation )
        //prepareDataForLoad(request, _newDataLoaded); // Call the concrete class method

        //--- Return the data
        return _newDataLoaded;
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.objectweb.telosys.screen.core.ScreenContext#getDataForDelete(org.objectweb.telosys.rpl.ScreenRequest)
     */
    //public ScreenData getDataForDelete(ScreenRequest request) throws TelosysException
    public ScreenData getDataForDelete() //throws TelosysException
    {
        trace("getDataForDelete");
        //--- Create a new ScreenData to store the data loaded
        if (_tempData == null)
        {
            _tempData = createNewScreenData();
        }

        // v 0.9.4 : now done in the Screen DAO
        //--- Set the Key Parameters from the REQUEST ( to do the DELETE operation )
        //prepareDataForDelete(request, _tempData); // Call the concrete class method

        //--- Return the data
        return _tempData;
    }

    //---------------------------------------------------------------------------
    // get/set/clear every types of data linked with the screen
    //---------------------------------------------------------------------------
    public Object put(Object key, Object value)
    {
        //--- Avoid NullPointerException if key is null
        if (key != null && value != null)
        {
            return _htData.put(key, value);
        }
        else
        {
            error("put(key,value) : key or value is null !");
            return null;
        }
    }

    public Object get(Object key)
    {
        //--- Avoid NullPointerException if key is null
        if (key != null)
        {
            return _htData.get(key);
        }
        else
        {
            error("get(key) : key is null !");
            return null;
        }
    }

    public Object remove(Object key)
    {
        //--- Avoid NullPointerException if key is null
        if (key != null)
        {
            return _htData.remove(key);
        }
        else
        {
            error("remove(key) : key is null !");
            return null;
        }
    }

}