package org.objectweb.telosys.screen.core;

import java.io.PrintWriter;
import java.util.Collection;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;
import java.util.Set;

import org.objectweb.telosys.common.TelosysException;
import org.objectweb.telosys.common.TelosysObject;
import org.objectweb.telosys.common.TelosysRuntimeException;
import org.objectweb.telosys.rpl.xml.IXmlWrapper;
import org.objectweb.telosys.rpl.xml.IXmlWrapperProvider;
import org.objectweb.telosys.rpl.xml.XmlWrapperManager;

/**
 * @author Laurent GUERIN
 *  
 */
public class StandardScreenData extends TelosysObject implements ScreenData
{
    private final HashMap       _hmElements      = new HashMap(4);

    private IXmlWrapperProvider _wrapperProvider = null;

    private boolean             _bFound          = false;

    //-----------------------------------------------------------------------------------------------
    /**
     * Defines a screen data element without renderer
     * @param sElementName
     * @param elementClass
     * @throws TelosysException
     */
    public void defineElement(String sElementName, Class elementClass) throws TelosysException
    {
        _hmElements.put(sElementName, new ScreenDataElement(sElementName, elementClass));
    }

    //-----------------------------------------------------------------------------------------------
    public void defineElement(String sElementName, Class elementClass, String sRendererPath) throws TelosysException
    {
        _hmElements.put(sElementName, new ScreenDataElement(sElementName, elementClass, sRendererPath));
    }

    //-----------------------------------------------------------------------------------------------
/**
    public void createElements() //throws TelosysException
    {
        Set entries = _hmElements.entrySet();
        if (entries != null)
        {
            Iterator iter = entries.iterator();
            if (iter != null)
            {
                //--- For each element defined : create a new instance
                while (iter.hasNext())
                {
                    //--- Next entry
                    Map.Entry entry = (Map.Entry) iter.next();
                    trace("createElements() : element key = '" + entry.getKey() + "' ");
                    //--- Get the element
                    ScreenDataElement element = (ScreenDataElement) entry.getValue();
                    if (element != null)
                    {
                        element.newInstance();
                    }
                    else
                    {
                        throw new TelosysRuntimeException("Invalid screen data definition (element entry is null) ");
                    }
                }
            }
        }
    }
**/
    //-----------------------------------------------------------------------------------------------
    public void copyElementsTo( ScreenData screenData) //throws TelosysException
    {
        Set entries = _hmElements.entrySet();
        if (entries != null)
        {
            Iterator iter = entries.iterator();
            if (iter != null)
            {
                //--- For each element defined : create a new instance
                while (iter.hasNext())
                {
                    //--- Next entry
                    Map.Entry entry = (Map.Entry) iter.next();
                    //--- Get the element
                    ScreenDataElement element = (ScreenDataElement) entry.getValue();
                    if (element != null)
                    {
                        //--- Set the same element instance in the ScreenData destiniation
                        screenData.setElement(element.getName(), element.getInstance());
                    }
                    else
                    {
                        throw new TelosysRuntimeException("Invalid screen data definition (element entry is null) ");
                    }
                }
            }
        }
    }

    //-----------------------------------------------------------------------------------------------
    /* (non-Javadoc)
     * @see org.objectweb.telosys.screen.core.ScreenData#setElement(java.lang.String, java.lang.Object)
     */
    public void setElement(String sElementName, Object oElementInstance)
    {
        if (sElementName == null)
        {
            throw new TelosysRuntimeException("setElement : element name is null");
        }
        ScreenDataElement element = (ScreenDataElement) _hmElements.get(sElementName);
        if (element != null)
        {
            element.setInstance(oElementInstance);
        }
        else
        {
            throw new TelosysRuntimeException("setElement : Unknown element name '" + sElementName + "'");
        }
    }

    //-----------------------------------------------------------------------------------------------
    /* (non-Javadoc)
     * @see org.objectweb.telosys.screen.core.ScreenData#getElement(java.lang.String)
     */
    public Object getElement(String sElementName)
    {
        if (sElementName == null)
        {
            throw new TelosysRuntimeException("getElement : element name is null");
        }
        ScreenDataElement element = (ScreenDataElement) _hmElements.get(sElementName);
        if (element != null)
        {
            return element.getInstance();
        }
        else
        {
            if (_hmElements.containsKey(sElementName))
            {
                return null;
            }
            else
            {
                throw new TelosysRuntimeException("getElement : Unknown element name '" + sElementName + "'");
            }
        }
    }

    //-----------------------------------------------------------------------------------------------
    /* (non-Javadoc)
     * @see org.objectweb.telosys.screen.core.ScreenData#getElementClass(java.lang.String)
     */
    public Class getElementClass(String sElementName)
    {
        if (sElementName == null)
        {
            throw new TelosysRuntimeException("getElementClass : element name is null");
        }
        ScreenDataElement element = (ScreenDataElement) _hmElements.get(sElementName);
        if (element != null)
        {
            return element.getInstanceClass();
        }
        else
        {
            throw new TelosysRuntimeException("getElementClass : Unknown element name '" + sElementName + "'");
        }
    }
    
    /* (non-Javadoc)
     * @see org.objectweb.telosys.screen.core.ScreenData#getScreenDataElement(java.lang.String)
     */
    public ScreenDataElement getScreenDataElement(String sElementName)
    {
        if (sElementName == null)
        {
            throw new TelosysRuntimeException("getScreenDataElement : element name is null");
        }
        ScreenDataElement element = (ScreenDataElement) _hmElements.get(sElementName);
        if (element != null)
        {
            return element;
        }
        else
        {
            throw new TelosysRuntimeException("getElementClass : Unknown element name '" + sElementName + "'");
        }        
    }
    //-----------------------------------------------------------------------------------------------
    /* (non-Javadoc)
     * @see org.objectweb.telosys.screen.core.ScreenData#getElements()
     */
    public ScreenDataElements getElements() 
    {
        Collection c =_hmElements.values();
        if ( c != null )
        {
            return new ScreenDataElements(c);            
        }
        return new ScreenDataElements();  
    }
    
    //-----------------------------------------------------------------------------------------------
    /*
     * (non-Javadoc)
     * 
     * @see org.objectweb.telosys.screen.core.ScreenData#toXml(java.io.PrintWriter)
     */
    public void toXml(PrintWriter out) //throws TelosysException
    {
        trace("toXml(out)...");
        Set entries = _hmElements.entrySet();
        if (entries != null)
        {
            Iterator iter = entries.iterator();
            if (iter != null)
            {
                //int i = 0;
                //--- For each element defined : create a new instance
                while (iter.hasNext())
                {
                    //--- Next entry
                    Map.Entry entry = (Map.Entry) iter.next();
                    //i++ ;
                    trace("toXml(out) : element key = '" + entry.getKey() + "' ");
                    //--- Get the element
                    ScreenDataElement element = (ScreenDataElement) entry.getValue();
                    Object oElementInstance = null;
                    String sElementName = null;
                    if (element != null)
                    {
                        oElementInstance = element.getInstance();
                        sElementName = element.getName();
                    }
                    //--- Generate the XML tag
                    //toXml(out, oElementInstance );
                    toXml(out, oElementInstance, sElementName);
                }
            }
        }
    }

    public void toXml(PrintWriter out, String sElementName) //throws TelosysException
    {
        //--- Get the element instance
        Object oElementInstance = getElement(sElementName);
        //--- Instance to XML
        toXml(out, oElementInstance, sElementName);
    }

    //-----------------------------------------------------------------------------------------------
    /* (non-Javadoc)
     * @see org.objectweb.telosys.screen.core.ScreenData#setFound()
     */
    public void setFound()
    {
        _bFound = true;
    }

    /* (non-Javadoc)
     * @see org.objectweb.telosys.screen.core.ScreenData#setNotFound()
     */
    public void setNotFound()
    {
        _bFound = false;
    }

    /* (non-Javadoc)
     * @see org.objectweb.telosys.screen.core.ScreenData#isFound()
     */
    public boolean isFound()
    {
        return _bFound;
    }

    /**
     * Print the given element instance as XML tag, using the associated XML wrapper
     * 
     * @param out
     * @param oElementInstance
     * @param sElementName
     */
    protected void toXml(PrintWriter out, Object oElementInstance, String sElementName)
    //protected void toXml(PrintWriter out, Object obj )
    {
        trace("toXml(out, oElementInstance, '" + sElementName + "' )... ");
        if (out == null)
        {
            throw new TelosysRuntimeException("toXml : output stream is null (PrintWriter 'out' argument) ");
            //            error("toXml(PrintWriter out, Object obj) : out is null ! ");
            //            return;
        }
        if (sElementName == null)
        {
            throw new TelosysRuntimeException("toXml : element name is null (String 'sElementName' argument) ");
        }
        if (oElementInstance == null)
        {
            out.println("<!-- Element '" + sElementName + "' instance is null -->");
            return;
        }

        if (_wrapperProvider == null)
        {
            try
            {
                //--- Get the wrapper provider (throws Exception if not found)
                _wrapperProvider = XmlWrapperManager.getXmlWrapperProvider();
            } catch (TelosysException e)
            {
                throw new TelosysRuntimeException("toXml : cannot get XmlWrapperProvider ");
                //                String sMsg = "toXml() : Cannot get XmlWrapperProvider";
                //                error(sMsg);
                //                out.println("<!-- " + sMsg + " -->");
                //                return;
            }
        }

        //--- Get the XML Wrapper for the given object
        IXmlWrapper wrapper = null;
        try
        {
            wrapper = _wrapperProvider.getWrapper(oElementInstance);
        } catch (TelosysRuntimeException e)
        {
            wrapper = null;
        }
        if (wrapper != null)
        {
            //--- Print the XML tag
            wrapper.toXml(out, oElementInstance, sElementName);
        }
        else
        {
            throw new TelosysRuntimeException("toXml : cannot get XmlWrapper for element '" + sElementName + "'");
            //            out.println("<!-- Cannot found XML wrapper for object ( class : " + oElementInstance.getClass().getName() + " ) -->");
        }
    }

    /**
     * Default toString method for a ScreenData 
     * @return
     */
    public String toString() 
    {
        StringBuffer sb = new StringBuffer(128);
        sb.append("ScreenData (" + this.getClass().getName() + ") : ");
        Set entries = _hmElements.entrySet();
        if (entries != null)
        {
            Iterator iter = entries.iterator();
            if (iter != null)
            {
                //--- For each element defined : create a new instance
                while (iter.hasNext())
                {
                    //--- Next entry
                    Map.Entry entry = (Map.Entry) iter.next();
                    //--- Get the element
                    ScreenDataElement element = (ScreenDataElement) entry.getValue();
                    Object oElementInstance = null;
                    String sElementName = null;
                    if (element != null)
                    {
                        sElementName = element.getName();
                        String sElementClass = "(null)";
                        if (element.getInstanceClass() != null)
                        {
                            sElementClass = element.getInstanceClass().getName();
                        }
                        String sElementToString = "(null)";
                        oElementInstance = element.getInstance();
                        if (oElementInstance != null)
                        {
                            sElementToString = oElementInstance.toString();
                            sElementClass = oElementInstance.getClass().getName();
                        }
                        //--- Add element informations
                        sb.append("\n . " + sElementName + "(" + sElementClass + ") : " + sElementToString);
                    }
                }
            }
        }
        else
        {
            sb.append("(no element)" );            
        }
        return sb.toString();
    }

}