package org.objectweb.telosys.screen.core;

import org.objectweb.telosys.common.TelosysException;
import org.objectweb.telosys.common.TelosysObject;
import org.objectweb.telosys.dal.dao.DatabaseSession;
import org.objectweb.telosys.dal.sql.ConnectionManager;
import org.objectweb.telosys.rpl.ScreenRequest;
import org.objectweb.telosys.rpl.ScreenResponse;

/**
 * @author Laurent GUERIN
 * 
 */
public abstract class StandardScreenDataAccessor extends TelosysObject implements ScreenDataAccessor
{
    private static final String ACTION_LOAD      = "load";

    private static final String ACTION_SAVE      = "save";

    private static final String ACTION_DELETE    = "delete";

    private static final String ACTION_INSERT    = "insert";

    private static final String ACTION_UPDATE    = "update";

    Class                       _screenDataClass = null;

    /**
     *  
     */
    public StandardScreenDataAccessor(Class screenDataClass)
    {
        super();
        _screenDataClass = screenDataClass;
    }

    //=========================================================================================================
    // Abstract methods ( to be implemented in the concrete Screen DAO )
    //=========================================================================================================
    //public abstract void prepareDataForLoad(ScreenData data, ScreenRequest request) throws TelosysException;

//    public abstract void prepareDataForDelete(ScreenData data, ScreenRequest request ) throws TelosysException;
    
    public abstract void load(ScreenData screenData, ScreenContext screenContext, ScreenRequest request, ScreenResponse response, DatabaseSession session) throws TelosysException;

    public abstract void save(ScreenData screenData, ScreenContext screenContext, ScreenRequest request, ScreenResponse response, DatabaseSession session) throws TelosysException;

    public abstract void insert(ScreenData screenData, ScreenContext screenContext, ScreenRequest request, ScreenResponse response, DatabaseSession session) throws TelosysException;

    public abstract void update(ScreenData screenData, ScreenContext screenContext, ScreenRequest request, ScreenResponse response, DatabaseSession session) throws TelosysException;

    public abstract void delete(ScreenData screenData, ScreenContext screenContext, ScreenRequest request, ScreenResponse response, DatabaseSession session) throws TelosysException;

    //=========================================================================================================
    // Check methods
    //=========================================================================================================
    private void checkDatabaseSession(DatabaseSession dbSession, String sAction) throws TelosysException
    {
        if (dbSession == null)
        {
            String sClassName = this.getClass().getName();
            throw new TelosysException("ScreenDAO (" + sClassName + ") : " + sAction + " : No database session (dbSession == NULL) !");
        }
    }

    private void checkScreenData(ScreenData screenData, String sAction) throws TelosysException
    {
        String sClassName = this.getClass().getName();
        if (screenData == null)
        {
            throw new TelosysException("ScreenDAO (" + sClassName + ") : " + sAction + " : No screen data (data == NULL) !");
        }
        else
        {
            if (_screenDataClass != null)
            {
                if (!_screenDataClass.isInstance(screenData))
                {
                    throw new TelosysException("ScreenDAO (" + sClassName + ") : " + sAction + " : Screen data is not an instance of '" + _screenDataClass.getName() + "' !");
                }
            }
            else
            {
                throw new TelosysException("ScreenDAO (" + sClassName + ") : " + sAction + " : Unknown ScreenData type ( not specified in the ScreenDAO constructor ) !");
            }
        }
    }

    //=========================================================================================================
    // Generic methods ( interface implementation )
    //=========================================================================================================
    public DatabaseSession getDatabaseSession() throws TelosysException
    {
        //--- Return a database session for the default database
        return ConnectionManager.getSession();
    }

    public void doLoad(ScreenData screenData, ScreenContext screenContext, ScreenRequest request, ScreenResponse response, DatabaseSession dbSession) throws TelosysException
    {
        checkScreenData(screenData, ACTION_LOAD);
        checkDatabaseSession(dbSession, ACTION_LOAD);
        load(screenData, screenContext, request, response, dbSession);
    }

    public void doSave(ScreenData screenData, ScreenContext screenContext, ScreenRequest request, ScreenResponse response, DatabaseSession dbSession) throws TelosysException
    {
        checkScreenData(screenData, ACTION_SAVE);
        checkDatabaseSession(dbSession, ACTION_SAVE);
        save(screenData, screenContext, request, response, dbSession);
    }

    public void doInsert(ScreenData screenData, ScreenContext screenContext, ScreenRequest request, ScreenResponse response, DatabaseSession dbSession) throws TelosysException
    {
        checkScreenData(screenData, ACTION_INSERT);
        checkDatabaseSession(dbSession, ACTION_INSERT);
        insert(screenData, screenContext, request, response, dbSession);
    }

    public void doUpdate(ScreenData screenData, ScreenContext screenContext, ScreenRequest request, ScreenResponse response, DatabaseSession dbSession) throws TelosysException
    {
        checkScreenData(screenData, ACTION_UPDATE);
        checkDatabaseSession(dbSession, ACTION_UPDATE);
        update(screenData, screenContext, request, response, dbSession);
    }

    public void doDelete(ScreenData screenData, ScreenContext screenContext, ScreenRequest request, ScreenResponse response, DatabaseSession dbSession) throws TelosysException
    {
        checkScreenData(screenData, ACTION_DELETE);
        checkDatabaseSession(dbSession, ACTION_DELETE);
        delete(screenData, screenContext, request, response, dbSession);
    }
}