package org.objectweb.telosys.screen.core;

import java.util.HashMap;

import org.objectweb.telosys.common.TelosysException;
import org.objectweb.telosys.common.TelosysObject;
import org.objectweb.telosys.screen.env.ScreenSession;

/**
 * @author Laurent GUERIN
 *  
 */
public class StandardScreenRegistry extends TelosysObject implements IScreenProvider, IScreenDataProvider
//public abstract class AbstractScreenRegistry implements IScreenProvider, IScreenDataProvider
{
    //=========================================================================================
    // INDEPENDENT (static) INNER CLASS : ResponseWrapper
    //=========================================================================================
    private static class ScreenComponents
    {
        private final static String ERR_SCREEN      = "ScreenProvider : cannot create screen ";

        private final static String ERR_SCREEN_DATA = "ScreenProvider : cannot create screen data ";

        private String              _screenName;

        //private Class               _screenClass; // v 0.9.4

        private Class               _screenDataClass;

        private Class               _screenDataAccessorClass;

        private Class               _screenActionsClass;

        private Class               _screenTriggersClass;

        private Class               _screenProceduresClass;

        /**
         * Constructor
         * 
         * @param screenName
         * @param screenDataClass
         * @param screenDaoClass
         * @param screenActionsClass
         * @param screenTriggersClass
         * @param screenProceduresClass
         */
        //public ScreenComponents(String screenName, Class screenClass, Class screenDataClass, Class screenDaoClass, Class screenActionsClass, Class screenTriggersClass, Class screenProceduresClass) throws TelosysException
        public ScreenComponents(String screenName, Class screenDataClass, Class screenDaoClass, Class screenActionsClass, Class screenTriggersClass, Class screenProceduresClass) throws TelosysException
        {
            final String sErrMsg = "Register Screen '" + screenName + "' : Invalid class for ";
          
            try
            {
                //--- Screen Name ------------------------------------
                _screenName = screenName;
                if (_screenName == null)
                {
                    throw new TelosysException("Register Screen : Screen name is NULL ");
                }

                //--- ScreenContext class ---// SUPPR in  v 0.9.4
//                _screenClass = screenClass;
//                if (_screenClass != null)
//                {
//                    if (ScreenContext.class.isAssignableFrom(_screenClass) != true)
//                    {
//                        throw new TelosysException(sErrMsg + "ScreenContext ");
//                    }
//                }

                //--- ScreenData class ------------------------------------
                _screenDataClass = screenDataClass;
                if (_screenDataClass != null)
                {
                    if (ScreenData.class.isAssignableFrom(_screenDataClass) != true)
                    {
                        throw new TelosysException(sErrMsg + "ScreenData ");
                    }
                }

                //--- ScreenDAO class ------------------------------------
                _screenDataAccessorClass = screenDaoClass;
                if (_screenDataAccessorClass != null)
                {
//                    if (ScreenDAO.class.isAssignableFrom(_screenDataAccessorClass) != true)
//                    {
//                        throw new TelosysException(sErrMsg + "ScreenDAO ");
//                    }
                    if (ScreenDataAccessor.class.isAssignableFrom(_screenDataAccessorClass) != true)
                    {
                        throw new TelosysException(sErrMsg + "ScreenDataAccessor ");
                    }
                }
                
                //--- ScreenActions class ------------------------------------
                _screenActionsClass = screenActionsClass;
                if (_screenActionsClass != null)
                {
                    if (ScreenActions.class.isAssignableFrom(_screenActionsClass) != true)
                    {
                        throw new TelosysException(sErrMsg + "ScreenActions ");
                    }
                }
                
                //--- ScreenTriggers class ------------------------------------                
                _screenTriggersClass = screenTriggersClass;
                if (_screenTriggersClass != null)
                {
                    if (ScreenTriggers.class.isAssignableFrom(_screenTriggersClass) != true)
                    {
                        throw new TelosysException(sErrMsg + "ScreenTriggers ");
                    }
                }
                
                //--- ScreenProcedures class ------------------------------------                
                _screenProceduresClass = screenProceduresClass;
                if (_screenProceduresClass != null)
                {
                    if (ScreenProcedures.class.isAssignableFrom(_screenProceduresClass) != true)
                    {
                        throw new TelosysException(sErrMsg + "ScreenProcedures ");
                    }
                }

            } catch (Throwable e)
            {
                throw new TelosysException("Register Screen '"+ _screenName + "' : Exception", e);
            }
        }

        /**
         * Create an instance of ScreenContext (without screen data)
         * 
         * @return
         * @throws TelosysException
         */
        protected ScreenContext createScreen(ScreenSession screenSession, int iScreenId ) throws TelosysException
        {
            try
            {
                //--- Create the screen instance with the class
                //StandardScreenContext screen = (StandardScreenContext) _screenClass.newInstance();// v 0.9.4
                StandardScreenContext screen = new StandardScreenContext(_screenName, _screenDataClass, screenSession, iScreenId ) ;// v 0.9.4
//                if (screen != null)
//                {
                    //--- Init the screen
                    //screen.initScreen(_screenName, _screenClass, _screenDataClass, screenSession); // v 0.9.4
                    //screen.initScreen(_screenName, _screenDataClass, screenSession, iScreenId ); // v 0.9.4
                    

//                    //--- Plug the Screen DAO
//                    if (_screenDaoClass != null)
//                    {
//                        screen.setScreenDAO((ScreenDAO) _screenDaoClass.newInstance());
//                    }
                    //--- Plug the Screen Data Accessor
                    if (_screenDataAccessorClass != null)
                    {
                        screen.setScreenDataAccessor((ScreenDataAccessor) _screenDataAccessorClass.newInstance());
                    }

                    //--- Plug the specific screen ACTIONS
                    if (_screenActionsClass != null)
                    {
                        screen.setScreenActions((ScreenActions) _screenActionsClass.newInstance());
                    }

                    //--- Plug the screen TRIGGERS
                    if (_screenTriggersClass != null)
                    {
                        screen.setScreenTriggers((ScreenTriggers) _screenTriggersClass.newInstance());
                    }

                    //--- Plug the screen PROCEDURES
                    if (_screenProceduresClass != null)
                    {
                        screen.setScreenProcedures((ScreenProcedures) _screenProceduresClass.newInstance());
                    }

                    return screen;
//                }
//                else
//                {
//                    throw new TelosysException(ERR_SCREEN + _screenName + " (class.newInstance return null)");
//                }
            } catch (InstantiationException e)
            {
                throw new TelosysException(ERR_SCREEN + _screenName, e);
            } catch (IllegalAccessException e)
            {
                throw new TelosysException(ERR_SCREEN + _screenName, e);
            } catch (Throwable e)
            {
                throw new TelosysException(ERR_SCREEN + _screenName, e);
            }
        }

        /**
         * Create an instance of ScreenData
         * 
         * @return
         * @throws TelosysException
         */
        protected ScreenData createScreenData() throws TelosysException
        {
            try
            {
                //--- Create the screen data instance with the class
                ScreenData screenData = (ScreenData) _screenDataClass.newInstance();
                if (screenData != null)
                {
                    return screenData;
                }
                else
                {
                    throw new TelosysException(ERR_SCREEN_DATA + _screenName + " (class.newInstance return null)");
                }
            } catch (InstantiationException e)
            {
                throw new TelosysException(ERR_SCREEN_DATA + _screenName, e);
            } catch (IllegalAccessException e)
            {
                throw new TelosysException(ERR_SCREEN_DATA + _screenName, e);
            } catch (Throwable e)
            {
                throw new TelosysException(ERR_SCREEN_DATA + _screenName, e);
            }
        }

    }

    //=========================================================================================

    private final HashMap hmRegistry = new HashMap(40);

    // v 0.9.4
    //protected final void register(String screenName, Class screenClass, Class screenDataClass, Class screenDaoClass, Class screenActionsClass, Class screenTriggersClass, Class screenProceduresClass)
    protected final void register(String screenName, Class screenDataClass, Class screenDaoClass, Class screenActionsClass, Class screenTriggersClass, Class screenProceduresClass)
    {
        trace("register screen " + screenName);
        try
        {
            // v 0.9.4
            //ScreenComponents sc = new ScreenComponents(screenName, screenClass, screenDataClass, screenDaoClass, screenActionsClass, screenTriggersClass, screenProceduresClass);
            ScreenComponents sc = new ScreenComponents(screenName, screenDataClass, screenDaoClass, screenActionsClass, screenTriggersClass, screenProceduresClass);
            hmRegistry.put(screenName, sc);
        } catch (TelosysException e)
        {
            error("Cannot register screen " + screenName);
            error(e.getMessage());
        }
    }

    private final ScreenComponents lookup(String sScreenName) throws TelosysException
    {
        ScreenComponents screenDef = (ScreenComponents) hmRegistry.get(sScreenName);
        if (screenDef != null)
        {
            return screenDef;
        }
        else
        {
            throw new TelosysException("Screen " + sScreenName + " not found in the registy ");
        }
    }

    //public ScreenContext getScreen(String sScreenName, ScreenSession screenSession ) throws TelosysException
    // v 0.9.4
    public ScreenContext getScreen( ScreenSession screenSession, String sScreenName, int iScreenId ) throws TelosysException
    {
        trace(" getScreen(" + sScreenName + ")");

        ScreenComponents screenDef = lookup(sScreenName);
        if (screenDef != null)
        {
            //return screenDef.createScreen(screenSession);
            return screenDef.createScreen(screenSession, iScreenId); // v 0.9.4
        }
        return null;
    }

    public ScreenData getScreenData(String sScreenName) throws TelosysException
    {
        trace(" getScreenData(" + sScreenName + ")");

        ScreenComponents screenDef = lookup(sScreenName);
        if (screenDef != null)
        {
            return screenDef.createScreenData();
        }
        return null;
    }
}