package org.objectweb.telosys.screen.env;

import javax.servlet.ServletContext;
import javax.servlet.http.HttpSession;

import org.objectweb.telosys.common.Telosys;

/**
 * @author Laurent GUERIN
 *
 */
public final class ScreenApplicationManager // extends TelosysObject
{
    private static final String TELOSYS_APPLICATION = "TELOSYS_APPLICATION" ;

    /**
     * No constructor for this class
     */
    private ScreenApplicationManager()
    {
    }
    
    //---------------------------------------------------------------------------
    public static ScreenApplication getScreenApplication( HttpSession httpSession )
    {
        if ( httpSession != null )
        {
            ServletContext servletContext = httpSession.getServletContext() ;
            if ( servletContext != null )
            {
        		Object obj = servletContext.getAttribute( TELOSYS_APPLICATION ) ; 
        		if ( obj != null )
        		{
        			if ( obj instanceof ScreenApplication ) 
        			{
        				return (ScreenApplication) obj ;
        			}
        			else 
        			{
        	            Telosys.error("getScreenApplication: ServletContext corrupted ! Attribute is not a ScreenApplication instance !");
        			}
        		}                
            }
        }
        return null ;
    }
    
    //---------------------------------------------------------------------------

    //---------------------------------------------------------------------------
    public static ScreenApplication openScreenApplication( HttpSession httpSession )
    {
        if (httpSession == null)
        {
            Telosys.error("openScreenApplication : HTTP session is null");
            return null ;
        }
        else
        {
            //--- Get the WebApp context
            ServletContext servletContext = httpSession.getServletContext() ;
            if ( servletContext != null )
            {
                //--- If not yet created ...
                if ( servletContext.getAttribute( TELOSYS_APPLICATION ) == null )
                {
                    //--- Create a screen application object 
                    ScreenApplication screenApplication = new ScreenApplication(servletContext);                    
                    //--- Store the screen application object in the servlet context
                    servletContext.setAttribute( TELOSYS_APPLICATION, screenApplication ) ;                        
            		return screenApplication ;
                }
            }
            else
            {
                Telosys.error("openScreenApplication : Cannot get ServletContext from HTTP session => Cannot create ScreenApplication !");
                return null ;                    
            }
        }
		return null ;
    }
    
}
