package org.objectweb.telosys.service;

import java.util.Map;
import java.util.Set;

import javax.servlet.http.HttpServletRequest;

import org.objectweb.telosys.common.TelosysConst;
import org.objectweb.telosys.common.TelosysException;
import org.objectweb.telosys.common.TelosysObject;
import org.objectweb.telosys.screen.env.ScreenSession;
import org.objectweb.telosys.uil.ScreenPostback;
import org.objectweb.telosys.uil.SessionPostback;

/**
 * @author Laurent GUERIN
 *  
 */
public class ServiceRequest extends TelosysObject
{
    //private static final String SCREEN_FORM_MARKER = "__TELOSYS_SCREEN_FORM_MARKER__";

    //private static final String POSTBACK_TYPE = "__TELOSYS_POSTBACK_TYPE__";

    private static final int   POSTBACK_NONE    = 0;

    private static final int   POSTBACK_REQUEST = 1;

    private static final int   POSTBACK_SESSION = 2;

    //-----------------------------------------------------------------------------------------------
    // Request parameters
    //-----------------------------------------------------------------------------------------------

    private String             _sServiceName    = null;

    private HttpServletRequest _httpRequest     = null;

    private ScreenSession      _screenSession   = null;

    //--- Optional attributes
    private int                _iPostBackType   = POSTBACK_NONE;

    private String             _sScreenName     = null;

    private String             _sScreenId       = null;

    private String             _sFormName       = null;

    private ScreenPostback     _screenPostback  = null;

    //-----------------------------------------------------------------------------------------------
    // Constructor(s)
    //-----------------------------------------------------------------------------------------------
    /**
     *  
     */
    public ServiceRequest(String sServiceName, HttpServletRequest httpRequest, ScreenSession screenSession)
    {
        _sServiceName = sServiceName;
        _httpRequest = httpRequest;
        _screenSession = screenSession;

        if (_httpRequest != null)
        {
            String sPostBackType = null;
            //--- Try to find the "Screen Marker" ( special hidden field )
            String sScreenMarker = _httpRequest.getParameter(TelosysConst.SCREEN_FORM_MARKER);
            if (sScreenMarker != null)
            {
                trace("Screen marker found : '" + sScreenMarker + "' ");
                //--- Split the marker ( "ScreenName;ScreenId;FormName;postback" )
                //String[] sParts = sScreenMarker.split(";");
                String[] sParts = parseMarker(sScreenMarker);
                if (sParts.length < 4)
                {
                    error("Invalid ScreenMarker '" + sScreenMarker + "' ( " + sParts.length + " parts, 4 expected )");
                    _sScreenName = null;
                    _sScreenId = null;
                    _sFormName = null;
                    sPostBackType = null;
                }
                else
                {
                    int i = 0;
                    _sScreenName = sParts[i++].trim();
                    _sScreenId = sParts[i++].trim();
                    _sFormName = sParts[i++].trim();
                    sPostBackType = sParts[i++].trim();
                }
            }

            //--- Set the "PostBack Type" from the screen form marker ( hidden field )
            _iPostBackType = POSTBACK_NONE;
            if (sPostBackType != null)
            {
                sPostBackType = sPostBackType.trim();
                if ("request".equalsIgnoreCase(sPostBackType))
                {
                    _iPostBackType = POSTBACK_REQUEST;
                }
                else if ("session".equalsIgnoreCase(sPostBackType))
                {
                    _iPostBackType = POSTBACK_SESSION;
                }
                try
                {
                    if (_iPostBackType != POSTBACK_NONE)
                    {
                        managePostBack();
                    }
                } catch (TelosysException e)
                {
                    error("Cannot managed postback values - TelosysException : " + e.getMessage());
                }
            }
        }
    }

    /**
     * Returns an array of 4 Strings (the 4 parts of the marker)
     * 
     * @param sMarker
     * @return : the 4 parts of the marker
     */
    private String[] parseMarker(String sMarker)
    {
        String sTokens[] = { "", "", "", "" };
        if (sMarker != null)
        {
            char chars[] = sMarker.toCharArray();
            int iToken = 0;
            for ( int i = 0 ; i < chars.length ; i++ )
            {
                if (chars[i] == ';')
                {
                    iToken++;
                }
                else
                {
                    if (iToken < sTokens.length)
                    {
                        sTokens[iToken] = sTokens[iToken] + chars[i];
                    }
                }
            }
        }
        return sTokens;
    }

    //-----------------------------------------------------------------------------------------------
    // Methods
    //-----------------------------------------------------------------------------------------------

    //-----------------------------------------------------------------------------------------------
    public String getServiceName()
    {
        return _sServiceName;
    }

    //-----------------------------------------------------------------------------------------------
    public HttpServletRequest getHttpRequest()
    {
        return _httpRequest;
    }

    //-----------------------------------------------------------------------------------------------
    public int getPostBackType()
    {
        return _iPostBackType;
    }

    //-----------------------------------------------------------------------------------------------
    public String getScreenName()
    {
        return _sScreenName;
    }

    //-----------------------------------------------------------------------------------------------
    public String getScreenId()
    {
        return _sScreenId;
    }

    //-----------------------------------------------------------------------------------------------
    //    public String getScreenKey()
    //    {
    //        return _sScreenKey;
    //    }

    //-----------------------------------------------------------------------------------------------
    public String getFormName()
    {
        return _sFormName;
    }

    //-----------------------------------------------------------------------------------------------
    public ScreenPostback getScreenPostback()
    {
        return _screenPostback;
    }

    //-----------------------------------------------------------------------------------------------

    /**
     * Returns the standard parameters map of the HttpServletRequest object <br>
     * NB : each value of the Map is a String array ( cf HttpServletRequest : getParameterMap() )
     * 
     * @return
     */
    public Map getParameterMap()
    {
        return _httpRequest.getParameterMap();
    }

    /**
     * Return the number of parameters
     * 
     * @return
     */
    public int getParameterCount()
    {
        Map params = _httpRequest.getParameterMap();
        if (params != null)
        {
            return params.size();
        }
        return 0;
    }

    /**
     * Returns all the parameter names
     * 
     * @return : Names (never null : size = 0 if no parameters )
     */
    public String[] getParameterNames()
    {
        Map params = _httpRequest.getParameterMap();
        if (params != null)
        {
            Set keys = params.keySet();
            if (keys != null)
            {
                //Object[] arrayKeys = keys.toArray();
                try
                {
                    // NB: cannot cast directly
                    // ( ie: sKeys = (String[]) arrayKeys ; => Exception )
                    // => Cast from "Object[]" to "String[]" with "Collection.toArray(Object[])"
                    // ( returns a new String array with the good size )
                    String[] voidArrayModel = new String[0];
                    String[] sKeys = (String[]) keys.toArray(voidArrayModel);
                    return sKeys;
                } catch (ClassCastException e)
                {
                    error("Cannot cast parameters keys to String[] ");
                }
            }
        }
        return new String[0];
    }

    //-----------------------------------------------------------------------------------------------
    /**
     * Returns the value for the given parameter name
     * 
     * @param sParamName :
     *            parameter name
     * @return : parameter value (or null if not found)
     */
    public String getParameter(String sParamName)
    {
        //        Map requestParameters = getParametersMap();
        //        String[] val = (String[]) requestParameters.get(sParamName);
        //        if (val != null)
        //        {
        //            if (val.length > 0)
        //            {
        //                return val[0];
        //            }
        //        }
        //        return null;
        return _httpRequest.getParameter(sParamName);
    }

    //-----------------------------------------------------------------------------------------------
    /**
     * Returns the 'nth' parameter value
     * 
     * @param iParamNum :
     *            parameter index ( 1 for "p1", 2 for "p2", ... )
     * @return : parameter value (or null if not found)
     */
    public String getParameter(int iParamNum)
    {
        return getParameter("p" + iParamNum);
    }

    /**
     * Returns true if the submit element name is present in the request parameters <br>( useful for forms with multiple submit buttons and/or submit images )<br>
     * An HTML form can submit a request with <br>
     * a "button" ( input type="submit" name="abc" : parameter "abc" )<br>
     * an "image" ( input type="image" name="abc" : parameters "abc.x" and "abc.y" )<br>
     * 
     * @param sSubmitElementName
     * @return
     */
    public boolean submitBy(String sSubmitElementName)
    {
        //--- If the submit element is a standard button ( <input type="submit" name="" ... /> )
        if (getParameter(sSubmitElementName) != null)
        {
            return true;
        }
        //--- If the submit element is an image ( <input type="image" name="" ... /> )
        if (getParameter(sSubmitElementName + ".x") != null)
        {
            return true;
        }
        if (getParameter(sSubmitElementName + ".X") != null)
        {
            return true;
        }
        //--- Submit name not found
        return false;
    }

    /**
     * Manages the request input field values storage for future postback
     * 
     * @throws TelosysException
     */
    private void managePostBack() throws TelosysException
    {
        trace("managePostBack()...");
        Map requestParameters = this._httpRequest.getParameterMap();
        if (requestParameters == null)
        {
            return;
        }
        if (requestParameters.isEmpty())
        {
            return;
        }
        //--- There's field values posted in the current http request => store the field values
        if (_iPostBackType == POSTBACK_SESSION)
        {
            String sScreenKey = SessionPostback.composeScreenKey(_sScreenName, _sScreenId);
            trace("managePostBack() : type 'session' : screen key = '" + sScreenKey + "' ");
            //--- Store the field values in the ScreenSession
            if (sScreenKey == null)
            {
                error("Cannot managed postback values ( screen session key is null )");
                return;
            }
            else
            {
                ScreenPostback screenPostback = _screenSession.getScreenPostback(sScreenKey);
                screenPostback.initFieldValues(_sFormName, _httpRequest);
            }
        }
        else if (_iPostBackType == POSTBACK_REQUEST)
        {
            trace("managePostBack() : type 'request' ");
            //--- Store the field values in the current ServiceRequest
            _screenPostback = new ScreenPostback();
            _screenPostback.initFieldValues(_sFormName, _httpRequest);
            //--- Store the ScreenPostback in the http session (for taglib )
            _httpRequest.setAttribute(TelosysConst.POSTBACK_KEY, _screenPostback);
        }

        //OldScreensValues lOv = OldValuesManager.getScreenOldValue(screenSession, sScreenName, iScreenId);
        // TODO en fonction de l'id d'cran courant, du formulaire post etc ... on va stocker les donnes qui doivent l'tre
        //	        for (Iterator it=lParams.entrySet().iterator(); it.hasNext(); ) {
        //	            Map.Entry entrees = (Map.Entry)it.next();
        //	            Object cle = entrees.getKey();
        //	            Object valeur = entrees.getValue();
        //	            if (lOv.containsKey(cle)) {
        //	        		lOv.setFieldValue(cle, valeur);
        //	        	}
        //	        }
    }

    //-----------------------------------------------------------------------------------------------
    public String toString()
    {
        return "ServiceRequest : " + _sServiceName + " " + _httpRequest.getQueryString();
    }
    //-----------------------------------------------------------------------------------------------
}