package org.objectweb.telosys.service;

import java.util.Enumeration;
import java.util.Hashtable;
import java.util.LinkedList;

import org.objectweb.telosys.common.TelosysObject;
import org.objectweb.telosys.screen.core.ScreenError;
import org.objectweb.telosys.screen.core.ScreenMessage;

/**
 * @author Laurent GUERIN
 *  
 */
public class ServiceResponse extends TelosysObject
{
    //-----------------------------------------------------------------------------------------------
    // Request parameters
    //-----------------------------------------------------------------------------------------------

    private String          _sServiceName     = null;

    //-----------------------------------------------------------------------------------------------
    // Response
    //-----------------------------------------------------------------------------------------------

    private Hashtable       _htValues         = null; // Values to return in the response

    private LinkedList      _listExceptions   = null; // List of all the exceptions

    private LinkedList      _listErrors       = null; // List of all the errors

    private LinkedList      _listMessages     = null; // List of all the app. messages

    private int             _iReturnCode      = 0;   // default = "0" ( = OK )

    private String          _sReturnMessage   = "Ok"; // default = "Ok"

    private Object          _serviceResult    = null;

    /**
     *  
     */
    public ServiceResponse(String sServiceName)
    {
        _sServiceName = sServiceName;
    }

    //-----------------------------------------------------------------------------------------------
    // constructor(s)
    //-----------------------------------------------------------------------------------------------

    //-----------------------------------------------------------------------------------------------
    // methods
    //-----------------------------------------------------------------------------------------------

    //-----------------------------------------------------------------------------------------------
    public String getServiceName()
    {
        return _sServiceName;
    }

    //-----------------------------------------------------------------------------------------------
    //-----------------------------------------------------------------------------------------------
    public void setReturnCode(int iCode)
    {
        _iReturnCode = iCode;
    }

    //-----------------------------------------------------------------------------------------------
    public int getReturnCode()
    {
        return _iReturnCode;
    }

    //-----------------------------------------------------------------------------------------------
    public void setReturnMessage(String sMsg)
    {
        _sReturnMessage = sMsg;
    }

    //-----------------------------------------------------------------------------------------------
    public String getReturnMessage()
    {
        return _sReturnMessage;
    }

    //-----------------------------------------------------------------------------------------------

    //-----------------------------------------------------------------------------------------------
    // Messages
    //-----------------------------------------------------------------------------------------------
    public void addMessage(ScreenMessage message)
    {
        if (_listMessages == null)
        {
            _listMessages = new LinkedList();
        }

        if (message != null)
        {
            _listMessages.add(message);
        }
    }

    //-----------------------------------------------------------------------------------------------
    public int getNumberOfMessages()
    {
        if (_listMessages == null)
        {
            return 0;
        }
        else
        {
            return _listMessages.size();
        }
    }

    //-----------------------------------------------------------------------------------------------
    public ScreenMessage getMessage(int index)
    {
        if (_listMessages == null)
        {
            return null;
        }
        else
        {
            Object obj = _listMessages.get(index);
            if (obj instanceof ScreenMessage)
            {
                return (ScreenMessage) obj;
            }
            else
            {
                error("getMessage(" + index + ") : not a ScreenMessage instance ");
                return null;
            }
        }
    }

    //-----------------------------------------------------------------------------------------------
    // Errors
    //-----------------------------------------------------------------------------------------------
    public void addError(ScreenError error)
    {
        if (_listErrors == null)
        {
            _listErrors = new LinkedList();
        }

        if (error != null)
        {
            _listErrors.add(error);
        }
    }

    //-----------------------------------------------------------------------------------------------
    public int getNumberOfErrors()
    {
        if (_listErrors == null)
        {
            return 0;
        }
        else
        {
            return _listErrors.size();
        }
    }

    //-----------------------------------------------------------------------------------------------
    public ScreenError getError(int index)
    {
        if (_listErrors == null)
        {
            return null;
        }
        else
        {
            Object obj = _listErrors.get(index);
            if (obj instanceof ScreenError)
            {
                return (ScreenError) obj;
            }
            else
            {
                error("getError(" + index + ") : not a ScreenError instance ");
                return null;
            }
        }
    }

    //-----------------------------------------------------------------------------------------------
    // Exceptions
    //-----------------------------------------------------------------------------------------------
    public void addError(Exception ex)
    {
        if (_listExceptions == null)
        {
            _listExceptions = new LinkedList();
        }

        if (ex != null)
        {
            _listExceptions.add(ex);
        }
    }

    public void addError(Throwable th)
    {
        if (_listExceptions == null)
        {
            _listExceptions = new LinkedList();
        }

        if (th != null)
        {
            _listExceptions.add(th);
        }
    }

    //-----------------------------------------------------------------------------------------------
    public int getNumberOfExceptions()
    {
        if (_listExceptions == null)
        {
            return 0;
        }
        else
        {
            return _listExceptions.size();
        }
    }

    //-----------------------------------------------------------------------------------------------
    public Exception getException(int index)
    {
        if (_listExceptions == null)
        {
            return null;
        }
        else
        {
            Object obj = _listExceptions.get(index);
            if (obj instanceof Exception)
            {
                return (Exception) obj;
            }
            else
            {
                error("getException(" + index + ") : not an Exception instance ");
                return null;
            }
        }
    }

    //-----------------------------------------------------------------------------------------------
    // Data
    //-----------------------------------------------------------------------------------------------
    public Object getResult()
    {
        return _serviceResult;
    }

    //-----------------------------------------------------------------------------------------------
    public void setResult(Object v)
    {
        _serviceResult = v;
    }

    //-----------------------------------------------------------------------------------------------
    // Values
    //-----------------------------------------------------------------------------------------------
    public void setValue(String sName, String sValue)
    {
        if (_htValues == null)
        {
            _htValues = new Hashtable();
        }

        //--- NB : NullPointerException if the key or value is null.
        if (sName != null && sValue != null )
        {
            _htValues.put(sName, sValue);
        }
    }

    //-----------------------------------------------------------------------------------------------
    public String getValue(String sName)
    {
        if (_htValues == null)
        {
            return null;
        }
        else
        {
            return (String) _htValues.get(sName);
        }
    }

    //-----------------------------------------------------------------------------------------------
    public int getNumberOfValues()
    {
        if (_htValues == null)
        {
            return 0;
        }
        else
        {
            return _htValues.size();
        }
    }

    //-----------------------------------------------------------------------------------------------
    public Enumeration getValueNames()
    {
        if (_htValues == null)
        {
            return null;
        }
        else
        {
            return _htValues.keys();
        }
    }

    //-----------------------------------------------------------------------------------------------
    public String toString()
    {
        //--- Attributes
        //String sHeader = _sServiceName + "|" + _sServiceType;
        String sHeader = _sServiceName;
        /**
         * //--- Data String sData = "(no data)"; if (_Data != null) { sData = _Data.toString(); }
         */
        //return "ScreenResponse : " + sHeader + "\n Data = " + sData;
        return "ServiceResponse : " + sHeader;
    }
    //-----------------------------------------------------------------------------------------------
}