package org.objectweb.telosys.service;

import java.io.PrintWriter;
import java.util.Enumeration;

import javax.servlet.ServletException;

import org.objectweb.telosys.common.TelosysClassLogger;
import org.objectweb.telosys.common.TelosysException;
import org.objectweb.telosys.screen.core.ScreenError;
import org.objectweb.telosys.screen.core.ScreenMessage;
import org.objectweb.telosys.util.XmlUtil;

/**
 * @author Laurent GUERIN
 *
 */
public class XmlServiceOutput 
{
    private static final TelosysClassLogger log = new TelosysClassLogger(XmlServiceOutput.class);
    
	//--------------------------------------------------------------------------------------------------
	// Mthodes de vrification des paramtres
	//--------------------------------------------------------------------------------------------------
	private static boolean checkPrintWriter(PrintWriter out)
	{
		if ( out == null )
		{
			log.error( "out parameter is null !" );
			return false;
		}
		else
		{
			return true ;
		}
	}
	//--------------------------------------------------------------------------------------------------
    

	//--------------------------------------------------------------------------------------------------
	private static void beginningOfXml ( ServiceResponse resp, PrintWriter out)
	{
	    log.trace ( "beginningOfXml()" );
		if ( checkPrintWriter(out) == false ) return ;
		
		out.println("<?xml  version=\"1.0\" encoding=\"ISO-8859-1\" ?>");
		
		out.print("<response");
		out.print(" service-name=\"" + resp.getServiceName() + "\"");
		out.println(" >");

	}

	//--------------------------------------------------------------------------------------------------
	// <return> ... </return>
	//--------------------------------------------------------------------------------------------------
	private static void genReturn ( ServiceResponse resp, PrintWriter out)
	{
	    log.trace ( "genReturn()" );
		int iCount = 0 ;
		if ( checkPrintWriter(out) == false ) return ;

		out.println("<return code=\"" + resp.getReturnCode() + "\" message=\"" + XmlUtil.xmlString( resp.getReturnMessage() ) + "\" >" );		            
		
		//--- List of exceptions 
		iCount = resp.getNumberOfExceptions() ;
		if ( iCount > 0 )
		{
		    Exception ex = null ;
			out.println("<exceptions>" );
		    for ( int i = 0 ; i < iCount ; i++ )
		    {
		        ex = resp.getException(i);
		        if ( ex != null )
		        {
					out.println(" <exception num=\"" + (i+1) +"\" class=\"" + ex.getClass().getName() + "\" message=\"" + XmlUtil.xmlString( ex.getMessage() ) + "\" />" );		            
		        }
		    }
			out.println("</exceptions>" );
		}

		//--- List of errors 
		iCount = resp.getNumberOfErrors() ;
		if ( iCount > 0 )
		{
		    ScreenError error = null ;
			out.println("<errors>" );
		    for ( int i = 0 ; i < iCount ; i++ )
		    {
		        error = resp.getError(i);
		        if ( error != null )
		        {
					out.println(" <error num=\"" + (i+1) +"\" code=\"" + error.getCode() + "\" text=\"" + XmlUtil.xmlString( error.getText() ) + "\" />" );		            
		        }
		    }
			out.println("</errors>" );
		}

		//--- List of messages 
		iCount = resp.getNumberOfMessages() ;
		if ( iCount > 0 )
		{
		    ScreenMessage message = null ;
			out.println("<messages>" );
		    for ( int i = 0 ; i < iCount ; i++ )
		    {
		        message = resp.getMessage(i);
		        if ( message != null )
		        {
					out.println(" <message num=\"" + (i+1) 
					        + "\" type=\"" + message.getType()
					        + "\" code=\"" + message.getCode()
					        + "\" text=\"" + XmlUtil.xmlString( message.getText() ) + "\" />" );		            
		        }
		    }
			out.println("</messages>" );
		}

		out.println("</return>" );
	}
	
	//--------------------------------------------------------------------------------------------------
	// <values> ... </values>
	//--------------------------------------------------------------------------------------------------
	private static void genValues ( ServiceResponse resp, PrintWriter out)
	{
	    log.trace ( "genValues()" );
		if ( checkPrintWriter(out) == false ) return ;
		if ( resp.getNumberOfValues() > 0 )
		{
		    String sValueName = null ;
			Enumeration eNames = resp.getValueNames();
			if ( eNames != null )
			{
				out.println("<values>" );
			    while ( eNames.hasMoreElements() )
			    {
			        sValueName = (String) eNames.nextElement() ;
					//OLD: out.println(" <" + sValueName + " value=\"" + XmlUtil.xmlString(resp.getValue(sValueName))+ "\" />" );
					out.println("<value " + sValueName + "=\"" + XmlUtil.xmlString(resp.getValue(sValueName))+ "\" />" );
			    }
				out.println("</values>" );			    
			}
		}
	}

	private static void printError ( PrintWriter out, Throwable ex)
	{
        out.println("<b>Exception</b> : " + ex.getClass().getName() + " ( " + ex.getMessage() + " ) <br>");
        if ( ex.getCause() != null )
        {
            out.println("<b>Cause</b> : <br>");
            printError ( out, ex.getCause() );
        }
        else
        {
            if ( ex instanceof ServletException )
            {
                out.println("<b>Root Cause</b> : <br>");
                printError ( out, ((ServletException)ex).getRootCause() );                
            }
        }
	}
	
	//--------------------------------------------------------------------------------------------------
	private static void genView ( ServiceResponse response, PrintWriter out, ServiceRenderer renderer)// throws TelosysException
	{
		out.println("<view>");
		out.println("<service-view>");
		out.println("<![CDATA[");    		
        try
        {
            renderer.render(response);
        } catch (Throwable e)
        {
            out.println("<b>ERROR</b> in renderer <b>'" + renderer.getPath() + "'</b><br>");
            printError(out, e);
//            out.println("<b>Exception</b> : " + e.getClass().getName() + "<br>");
//            out.println(" . Message : " + e.getMessage() + "<br>" );
//            out.println(" . Cause : " + e.getCause() + "<br>" );
        }
        finally {
    		out.println("]]>");    		
    		out.println("</service-view>");
    		out.println("</view>");                            
        }
	}

	//--------------------------------------------------------------------------------------------------
	private static void endOfXml ( PrintWriter out ) 
	{
	    log.trace ( "endOfXml()" );
		if ( checkPrintWriter(out) == false ) return ;
		out.println("</response>");
	}
	//--------------------------------------------------------------------------------------------------
    
    public final static void generate ( ServiceResponse response, PrintWriter out, ServiceRenderer renderer ) throws TelosysException
    {
        log.trace ( "generate()" );
        beginningOfXml ( response, out);
        genReturn ( response, out);
        genValues ( response, out);
        
        if ( renderer != null )
        {
            genView(response, out, renderer );
//            try
//            {
//                genView(renderer, out);
//            } catch (TelosysException e)
//            {
//                endOfXml (out);
//                out.flush();            
//                throw e ;
//            }
        }
        
        endOfXml (out);
        out.flush();            
    }
}
