package org.objectweb.telosys.service.search;

import java.util.Map;

import org.objectweb.telosys.common.TelosysException;
import org.objectweb.telosys.common.View;
import org.objectweb.telosys.dal.dataset.SQLDataSet;
import org.objectweb.telosys.screen.core.ScreenMessage;
import org.objectweb.telosys.screen.env.ScreenSession;
import org.objectweb.telosys.service.ServiceRequest;
import org.objectweb.telosys.service.ServiceResponse;

/**
 * @author Laurent GUERIN
 *  
 */
public class SQLSearch extends SQLSearchAncestor
{
    //public View execute(ScreenSession screenSession, Map requestParameters, ServiceResponse serviceResponse) throws TelosysException
    public View execute(ScreenSession screenSession, ServiceRequest serviceRequest, ServiceResponse serviceResponse) throws TelosysException
    {
        Map requestParameters = serviceRequest.getParameterMap();
        String strParams[] = null;

        if (_bDynamicParameters)
        {
            //--- Set the useful criterias
            setCriterias(requestParameters);
            //--- Change the SQLDataSet definition
            _dsDefinition.alterWhere(_criterias);
            strParams = _criterias.getDynamicParameters();
            trace("WHERE  = " + _dsDefinition.getSqlWhere());
            trace("TYPES  = " + _dsDefinition.getParametersTypes());
            trace("PARAMS length = " + strParams.length);
            trace("PARAMS = " + strParams.toString());
        }
        else
        {
            //--- Get the SQL parameters values (as String)
            strParams = getStringParams(requestParameters);
            if (strParams.length < _dsDefinition.getNbParams())
            {
                throw new TelosysException(_dsDefinition.getNbParams() + " parameters expected (" + strParams.length + " received)");
            }
        }

        //--- Set the "MAX" limit 
        if (_iMaxi <= 0)
        {
            error("Maxi <= 0 : set the default value !");
            _iMaxi = 1000 ; // by default
        }
        _dsDefinition.setLinesPerPage(_iMaxi);
        _dsDefinition.setMaxRows(_iMaxi);

        //--- DataSet creation
        SQLDataSet ds = null;
        try
        {
            ds = new SQLDataSet(_dsDefinition, strParams);
        } catch (TelosysException e)
        {
            serviceResponse.addError(e);
            serviceResponse.addMessage(new ScreenMessage("Cannot create SQLDataSet ", ScreenMessage.ERROR));
            throw e;
        }
        trace("DataSet created");

        try
        {
            //--- Limited load ( only one page )
            trace(" DataSet.load() , maxi = " + _iMaxi);
            int iRet = ds.load();
            trace(" DataSet.load() : return code = " + iRet);
            trace(" DataSet.getRowCount() : " + ds.getRowCount() );
            trace(" DataSet.endOfResultSet() : " + ds.endOfResultSet() );

            if (ds.getRowCount() == 0)
            {
                //String sMsg = "No row";
                serviceResponse.addMessage(_messages.noRow());
            }
            else
            {
                if (ds.endOfResultSet())
                {
                    //--- The end of the ResultSet was reached : all the lines are loaded in the DataSet
//                    String sMsg = ds.getRowCount() + " rows ";
//                    serviceResponse.addMessage(new ScreenMessage(sMsg, ScreenMessage.INFORMATION));
                    serviceResponse.addMessage(_messages.numberOfRows(ds.getRowCount()));
                }
                else
                {
                    //--- The end of the ResultSet was not reached : there's more lines than DataSet capacity
//                    String sMsg = "Too much rows ( more than " + _iMaxi + " )";
//                    serviceResponse.addMessage(new ScreenMessage(sMsg, ScreenMessage.WARNING));
                    serviceResponse.addMessage(_messages.tooMuchRows(_iMaxi) );
                }
                //--- Store the service result in the response (for the renderer)
                serviceResponse.setResult(ds);
            }
        } catch (Exception ex)
        {
            if (ex.getCause() != null)
            {
                serviceResponse.addError(ex.getCause());
            }
            serviceResponse.addError(ex);
            throw new TelosysException("DataSet cannot execute SQL operation ", ex);
        }
        return null ;
    }
}