package org.objectweb.telosys.service.search;

import java.util.Map;

import org.objectweb.telosys.dal.dataset.SQLCriterias;
import org.objectweb.telosys.dal.dataset.SQLDataSetDefinition;
import org.objectweb.telosys.service.StandardScreenService;

/**
 * @author Laurent GUERIN
 *  
 */
public abstract class SQLSearchAncestor extends StandardScreenService //implements ScreenService
{
    private final static SearchMessages STD_MSG             = new StandardSearchMessages();

    protected boolean                   _bDynamicParameters = false;

    protected SQLCriterias              _criterias          = null;

    protected int                       _iMaxi              = 200;                         // = page size

    protected SQLDataSetDefinition      _dsDefinition       = null;

    protected SearchMessages            _messages           = STD_MSG;

    protected String                    _sSessionKey        = null;                        // Default : Not stored in session

    protected void init(SQLDataSetDefinition def)
    {
        _dsDefinition = def;
        _criterias = null;
        _bDynamicParameters = false;
    }

    protected void init(SQLDataSetDefinition def, SQLCriterias criterias)
    {
        _dsDefinition = def;
        _criterias = criterias;
        _bDynamicParameters = true;
    }

    protected void setMaxRows(int iMaxi)
    {
        _iMaxi = iMaxi;
    }

    protected void setMessages(SearchMessages messages)
    {
        _messages = messages;
    }

    protected void setSessionKey(String s)
    {
        _sSessionKey = s;
    }

    protected String getParamValue(Map requestParameters, String sParamName)
    {
        String[] val = (String[]) requestParameters.get(sParamName);
        if (val != null)
        {
            if (val.length > 0)
            {
                return val[0];
            }
        }
        return null;
    }

    protected String getParamValue(Map requestParameters, int iParamNum)
    {
        String[] val = (String[]) requestParameters.get("p" + iParamNum);
        if (val != null)
        {
            if (val.length > 0)
            {
                return val[0];
            }
        }
        return null;
    }

    protected int countParams(Map requestParameters)
    {
        int i = 1;
        while (requestParameters.get("p" + i) != null)
        {
            i++;
        }
        // "p4" == null => i == 4 ==> count == 3
        return i - 1;
    }

    /**
     * Return a String array of parameters
     * 
     * @param requestParameters
     * @return parameters array
     */
    protected String[] getStringParams(Map requestParameters)
    {
        int iSize = countParams(requestParameters);
        trace(" countParams : " + iSize);
        String strParams[] = new String[iSize];
        for ( int i = 1 ; i <= iSize ; i++ )
        {
            strParams[i - 1] = getParamValue(requestParameters, i);
        }
        return strParams;
    }

    /**
     * Set the dynamic criterias according with the input parameters
     * 
     * @param requestParameters
     * @return
     */
    protected void setCriterias(Map requestParameters)
    {
        int iSize = _criterias.getCount();
        for ( int i = 1 ; i <= iSize ; i++ )
        {
            //--- If there's a value for this parameter
            String sVal = getParamValue(requestParameters, i);
            if (sVal != null)
            {
                trace("VAL " + i + " = '" + sVal + "'");
                sVal = sVal.trim();
                if (sVal.length() > 0)
                {
                    if (sVal.endsWith("*"))
                    {
                        sVal = sVal.substring(0, sVal.length() - 1) + "%";
                    }
                    //--- Parameter set => use this criteria
                    _criterias.useWithValue(i, sVal);
                    trace("USE " + i + " : " + sVal);
                }
                else
                {
                    _criterias.doNotUse(i);
                    trace("DO NOT USE " + i);
                }
            }
            else
            {
                //--- No parameter => Criteria not used
                _criterias.doNotUse(i);
                trace("DO NOT USE " + i);
            }
        }
    }
}