package org.objectweb.telosys.service.search;

import java.util.Map;

import org.objectweb.telosys.common.TelosysException;
import org.objectweb.telosys.common.View;
import org.objectweb.telosys.dal.dataset.SQLDataSet;
import org.objectweb.telosys.screen.core.ScreenMessage;
import org.objectweb.telosys.screen.env.ScreenSession;
import org.objectweb.telosys.service.ServiceRequest;
import org.objectweb.telosys.service.ServiceResponse;

/**
 * @author Laurent GUERIN
 *  
 */
public class SQLSearchWithCount extends SQLSearchAncestor
{
    //public View execute(ScreenSession screenSession, Map requestParameters, ServiceResponse serviceResponse) throws TelosysException
    public View execute(ScreenSession screenSession, ServiceRequest serviceRequest, ServiceResponse serviceResponse) throws TelosysException
    {
        Map requestParameters = serviceRequest.getParameterMap();        
        String strParams[] = null;

        if (_bDynamicParameters)
        {
            //--- Set the useful criterias
            setCriterias(requestParameters);
            //--- Change the SQLDataSet definition
            _dsDefinition.alterWhere(_criterias);
            strParams = _criterias.getDynamicParameters();
            trace("WHERE  = " + _dsDefinition.getSqlWhere());
            trace("TYPES  = " + _dsDefinition.getParametersTypes());
            trace("PARAMS length = " + strParams.length);
            trace("PARAMS = " + strParams.toString());
        }
        else
        {
            //--- Get the SQL parameters values (as String)
            strParams = getStringParams(requestParameters);
            if (strParams.length < _dsDefinition.getNbParams())
            {
                throw new TelosysException(_dsDefinition.getNbParams() + " parameters expected (" + strParams.length + " received)");
            }
        }

        //--- If max is set => set the page size (limit) ( else "no limit" )
        if (_iMaxi > 0)
        {
            _dsDefinition.setLinesPerPage(_iMaxi);
        }

        //--- DataSet creation
        SQLDataSet ds = null;
        try
        {
            ds = new SQLDataSet(_dsDefinition, strParams);
        } catch (TelosysException e)
        {
            serviceResponse.addError(e);
            serviceResponse.addMessage(new ScreenMessage("Cannot create SQLDataSet ", ScreenMessage.ERROR));
            throw e;
        }
        trace("DataSet created");

        try
        {
            //--- Count the number of rows
            trace(" DataSet.count");
            int iCount = ds.count();

            if (iCount == 0)
            {
                //--- No row => no load
                serviceResponse.addMessage(_messages.noRow());
            }
            else
            {
                //--- Load the rows ...
                if (_iMaxi > 0)
                {
                    //--- Limited load ( only one page )
                    trace(" Load only one page : DataSet.loadPage(1) , maxi = " + _iMaxi);
                    int iRet = ds.loadPage(1);
                    trace(" DataSet.loadPage(1) : return code = " + iRet);
                }
                else
                {
                    //--- Not limited load
                    trace(" Load all : DataSet.load() , maxi = " + _iMaxi);
                    int iRet = ds.load();
                    trace(" DataSet.load() : return code = " + iRet);
                }
                if (ds.getRowCount() == 0) // It would never happened ( because _iCount > 0 )
                {
                    // Hum... something has changed in the database between the 2 requests !!!
                    serviceResponse.addMessage(_messages.noRow());                    
                }
                else
                {
                    if (_iMaxi > 0 && iCount > _iMaxi)
                    {
                        serviceResponse.addMessage(_messages.tooMuchRows(iCount, _iMaxi) );                        
                    }
                    else
                    {
                        serviceResponse.addMessage(_messages.numberOfRows(ds.getRowCount()));                        
                    }
                    //--- Store the service result in the response (for the renderer)
                    serviceResponse.setResult(ds);
                    
                }
            }
        } catch (Exception ex)
        {
            if (ex.getCause() != null)
            {
                serviceResponse.addError(ex.getCause());
            }
            serviceResponse.addError(ex);
            throw new TelosysException("DataSet cannot execute SQL operation ", ex);
        }
        return null ;
    }

}