package org.objectweb.telosys.tools;

import java.io.BufferedReader;
import java.io.CharArrayWriter;
import java.io.FileWriter;
import java.io.IOException;
import java.io.PrintWriter;
import java.io.Writer;
import java.util.Enumeration;
import java.util.HashMap;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.Set;

import javax.servlet.Filter;
import javax.servlet.FilterChain;
import javax.servlet.FilterConfig;
import javax.servlet.ServletException;
import javax.servlet.ServletRequest;
import javax.servlet.ServletResponse;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpServletResponseWrapper;

public class ScreenRequestLogger_OLD implements Filter
{

    //=========================================================================================
    // INDEPENDENT (static) INNER CLASS : ResponseWrapper
    //=========================================================================================
    /**
     * Servlet Response Wrapper class Wrap the original servlet response and
     * provide a "writer" which keeps the response output in a "CharArray"
     */
    private static class CharResponseWrapper extends HttpServletResponseWrapper
    {
        private int             iHttpStatusCode = SC_OK;        // 200 (by
                                                                // default)

        private HashMap         hmHeaders       = new HashMap();

        private CharArrayWriter output          = null;

        public CharResponseWrapper(HttpServletResponse response)
        {
            super(response);
            output = new CharArrayWriter();
        }

        /**
         * Return the response status
         * 
         * @return
         */
        public String getStatus()
        {
            return "HTTP Status Code = " + iHttpStatusCode;
        }

        /**
         * Return the response headers
         * 
         * @return
         */
        public String getHeaders()
        {
            StringBuffer sb = new StringBuffer(400);
            //Collection values = hmHeaders.values();
            Set keys = hmHeaders.keySet();
            Iterator iter = keys.iterator();
            while (iter.hasNext())
            {
                Object objKey = iter.next();
                Object objVal = hmHeaders.get(objKey);
                sb.append(objKey.toString() + ":");
                if (objVal instanceof String)
                {
                    sb.append(objVal.toString() + "\r\n");
                }
                else
                {
                    if (objVal instanceof LinkedList)
                    {
                        sb.append("LinkedList" + "\r\n");
                    }
                }
            }
            return sb.toString();
        }

        /**
         * Return the response body
         * 
         * @return
         */
        public String getBody()
        {
            return output.toString();
        }

        //----------------------------------------------------------------------------
        // Override methods which change the STATUS ( to catch the response status ).
        //----------------------------------------------------------------------------
        public void setStatus(int iStatus)
        {
            this.iHttpStatusCode = iStatus;
            super.setStatus(iStatus);
        }

        public void setStatus(int iStatus, String str)
        {
            this.iHttpStatusCode = iStatus;
            super.setStatus(iStatus, str);
        }

        public void sendError(int errorCode) throws IOException
        {
            this.iHttpStatusCode = errorCode;
            super.sendError(errorCode);
        }

        public void sendError(int errorCode, String errorMessage) throws IOException
        {
            this.iHttpStatusCode = errorCode;
            super.sendError(errorCode, errorMessage);
        }

        public void sendRedirect(String location) throws IOException
        {
            this.iHttpStatusCode = SC_MOVED_TEMPORARILY;
            super.sendRedirect(location);
        }

        //----------------------------------------------------------------------------
        // Override the response "writer provider".
        //----------------------------------------------------------------------------
        /*
         * Provide an specific writer instead of the standard writer
         * 
         * @see javax.servlet.ServletResponse#getWriter()
         */
        public PrintWriter getWriter()
        {
            return new PrintWriter(output);
        }

        /*
         * (non-Javadoc)
         * 
         * @see javax.servlet.http.HttpServletResponse#addHeader(java.lang.String,
         *      java.lang.String)
         */
        public void addHeader(String sName, String sValue)
        {
            Object obj = hmHeaders.get(sName);
            if (obj == null)
            {
                LinkedList list = new LinkedList();
                list.add(sValue);
                hmHeaders.put(sName, list);
            }
            else
            {
                if (obj instanceof LinkedList)
                {
                    LinkedList list = (LinkedList) obj;
                    list.add(sValue);
                }
                else
                {
                    if (obj instanceof String)
                    {
                        String sFirstValue = (String) obj;
                        LinkedList list = new LinkedList();
                        list.add(sFirstValue);
                        list.add(sValue);
                        hmHeaders.put(sName, list);
                    }
                    else
                    {
                        // ERROR !
                    }
                }
            }
            super.addHeader(sName, sValue);
        }

        /*
         * (non-Javadoc)
         * 
         * @see javax.servlet.http.HttpServletResponse#setHeader(java.lang.String,
         *      java.lang.String)
         */
        public void setHeader(String name, String value)
        {
            hmHeaders.put(name, value);
            super.addHeader(name, value);
        }
    }

    //=========================================================================================

    //private FilterConfig filterConfig = null;

    private String _sLogDirectory = null;

    private String _sFilterName   = null;

    public void init(FilterConfig filterConfig) throws ServletException
    {
        //this.filterConfig = filterConfig;
        _sLogDirectory = filterConfig.getInitParameter("directory");
        _sFilterName = filterConfig.getFilterName();
        System.out.println("Filter " + _sFilterName + " ready, log directory = " + _sLogDirectory);
    }

    public void destroy()
    {
        //this.filterConfig = null;

    }

    public void doFilter(ServletRequest request, ServletResponse response, FilterChain chain) throws IOException, ServletException
    {
        HttpServletRequest httpRequest = null;
        HttpServletResponse httpResponse = null;

        CharArrayWriter cawRequestQuery = new CharArrayWriter();
        CharArrayWriter cawRequestHeader = new CharArrayWriter();
        CharArrayWriter cawRequestBody = new CharArrayWriter();

        //String sChemin = filterConfig.getInitParameter("directory");

        //--- Cast the request object
        if (request instanceof HttpServletRequest)
        {
            httpRequest = (HttpServletRequest) request;
            httpResponse = (HttpServletResponse) response;
        }

        //--- Keep the REQUEST informations in "CharArray" buffers
        writeRequestQuery(cawRequestQuery, httpRequest);
        writeRequestHeader(cawRequestHeader, httpRequest);
        writeRequestBody(cawRequestBody, httpRequest);

        //--- Keep the original reponse output
        PrintWriter out = response.getWriter();

        //--- Create a response wrapper for interceptions
        CharResponseWrapper responseWrapper = new CharResponseWrapper(httpResponse);

        //--------------------------------------------------------------------------
        //--- Chain to next step ...
        chain.doFilter(request, responseWrapper);
        //--------------------------------------------------------------------------

        //--- Write the response body in the original servlet output
        out.write(responseWrapper.getBody());

        //--- After request processing : log the REQUEST & RESPONSE
        FileWriter fw = null;
        try
        {
            String sFileName = null;
            if (_sLogDirectory.endsWith("/"))
            {
                sFileName = _sLogDirectory + "log.txt";
            }
            else
            {
                sFileName = _sLogDirectory + "/log.txt";
            }
            fw = new FileWriter(sFileName, true);

            fw.write("\r\n");
            fw.write("==============================================\r\n");
            //---------- REQUEST
            //--- Write the header log in the file
            fw.write(cawRequestQuery.toString());
            fw.write("----------------------------------------------\r\n");
            fw.write(cawRequestHeader.toString());
            fw.write(cawRequestBody.toString());

            fw.write("----------------------------------------------\r\n");
            //---------- RESPONSE
            //--- Write the response status code in the log file
            fw.write(responseWrapper.getStatus() + "\r\n");

            //--- Write the response headers in the log file
            fw.write(responseWrapper.getHeaders() + "\r\n");

            //--- Write the response output body in the log file
            fw.write(responseWrapper.getBody() + "\r\n");

            fw.write("==============================================\r\n");
            fw.flush();

        } catch (IOException ex)
        {
            ex.printStackTrace();
        } finally
        {
            try
            {
                //--- FERMETURE LOG
                if (fw != null)
                {
                    fw.close();
                }
            } catch (IOException ex_close)
            {
                ex_close.printStackTrace();
            }
        }

    }

    /**
     * Write a string in the given "writer" (deal with null values)
     * 
     * @param writer
     * @param chaine
     * @throws IOException
     */
    private void writeLine(Writer writer, String chaine) throws IOException
    {
        if (chaine != null)
        {
            writer.write(chaine);
        }
        else
        {
            writer.write("");
        }
        writer.write("\r\n");
    }

    /**
     * Write the HTTP REQUEST HEADER in the given "writer"
     * 
     * @param writer
     * @param request
     */
    private void writeRequestQuery(Writer writer, HttpServletRequest request) throws IOException
    {
        //String sProtocol = request.getProtocol();
        writeLine(writer, "Protocol : " + request.getProtocol());

        //String sMethod = request.getMethod();
        writeLine(writer, "Method   : " + request.getMethod());

        String sURL = request.getRequestURL().toString();
        if (sURL == null)
            sURL = "";
        String sParams = request.getQueryString();
        String sFullURL = null;
        if (sParams != null)
        {
            sFullURL = sURL + "?" + sParams;
        }
        else
        {
            sFullURL = sURL;
        }
        writeLine(writer, "URL      : " + sFullURL);
    }

    /**
     * Write the HTTP REQUEST HEADER in the given "writer"
     * 
     * @param writer
     * @param request
     */
    private void writeRequestHeader(Writer writer, HttpServletRequest request) throws IOException
    {
        Enumeration names = request.getHeaderNames();
        while (names != null && names.hasMoreElements())
        {
            String name = (String) names.nextElement();
            Enumeration values = request.getHeaders(name);
            while (values != null && values.hasMoreElements())
            {
                String value = (String) values.nextElement();
                writeLine(writer, name + ":" + value);
            }
        }
    }

    /**
     * Write the HTTP REQUEST BODY in the given "writer"
     * 
     * @param writer
     * @param request
     */
    private void writeRequestBody(Writer writer, HttpServletRequest request) throws IOException
    {
        writeLine(writer, "Content : length = " + request.getContentLength() + " type = " + request.getContentType());
        BufferedReader br = request.getReader();
        String sLine = null;
        while ( ( sLine = br.readLine() ) != null )
        {
            writeLine(writer, sLine );
        }
    }

    /**
     * Write the HTTP REQUEST HEADER in the given "writer"
     * 
     * @param writer
     * @param request
     */
    private void writeRequestHeader_OLD(Writer writer, HttpServletRequest request)
    {
        int iServerPort = request.getServerPort();
        String sServerPort = String.valueOf(iServerPort);
        //int iRemotePort = request.getRemotePort();
        //String sfilterName = filterConfig.getFilterName();
        String sServerName = request.getServerName();
        String sHost = request.getRemoteHost();
        String sAdr = request.getRemoteAddr();
        String sUser = request.getRemoteUser();
        String sQuery = request.getQueryString();
        String sEncoding = request.getCharacterEncoding();
        // variables complexes (numration, tableau, flux binaire)
        Enumeration eParams = request.getParameterNames();
        Enumeration eHeaderNames = request.getHeaderNames();
        //Cookie[] tCookies = request.getCookies();

        try
        {
            // ############## criture des donnes requte ##############
            //writeString(writer, sfilterName);
            //writer.write(" ");
            //writer.write(sAdr);
            writer.write("\r\n##################################################\r\n");
            writeLine(writer, sQuery);
            writer.write("Port du serveur hte : ");
            writer.write(sServerPort);
            writer.write("\r\n");
            writer.write("Nom du serveur hte : ");
            writeLine(writer, sServerName);
            writer.write("\r\n");
            writer.write("Hte distant : ");
            writeLine(writer, sHost);
            writer.write("\r\n");
            writer.write("Adresse distante : ");
            writeLine(writer, sAdr);
            writer.write("\r\n");
            writer.write("Utilisateur distant : ");
            writeLine(writer, sUser);
            writer.write("\r\n");
            writer.write("Char Encoding : ");
            writeLine(writer, sEncoding);
            writer.write("\r\n");
            writer.write("### Paramtres de la requte ###\r\n");
            while (eParams.hasMoreElements())
            {
                String sParamName = (String) eParams.nextElement();
                String sParamValue = request.getParameter(sParamName);
                writeLine(writer, sParamName);
                writer.write(" : ");
                writeLine(writer, sParamValue);
                writer.write("\r\n");
            }
            writer.write("### Headers de la requte ###\r\n");
            while (eHeaderNames.hasMoreElements())
            {
                String sHeaderName = (String) eHeaderNames.nextElement();
                String sHeaderValue = request.getHeader(sHeaderName);
                //long lHeaderDate = httpRequest.getDateHeader(sHeaderName);
                writeLine(writer, sHeaderName);
                writer.write(" : ");
                writeLine(writer, sHeaderValue);
                //writer.write(" Date : ");
                //writer.write(lHeaderDate);
                writer.write("\r\n");
            }
            writer.write("\r\n--------------------------------------------------\r\n");
        } catch (IOException i2)
        {
            // TODO Auto-generated catch block
            i2.printStackTrace();
        }

    }

}