package org.objectweb.telosys.tools;

import java.io.IOException;
import java.util.Enumeration;

import javax.servlet.Filter;
import javax.servlet.FilterChain;
import javax.servlet.FilterConfig;
import javax.servlet.ServletException;
import javax.servlet.ServletRequest;
import javax.servlet.ServletResponse;
import javax.servlet.http.HttpServletRequest;

import org.objectweb.telosys.common.TelosysObject;

public class TraceFilter extends TelosysObject implements Filter
{
    private boolean _bDuration = false ;
    private boolean _bHeaders  = false ;
    
	/**
	 * @see javax.servlet.Filter#init(javax.servlet.FilterConfig)
	 */
	public void init(FilterConfig config) throws ServletException
	{
		info("init()");	
		String sParamVal = null ;
	    //--- Show duration ?
	    sParamVal = config.getInitParameter("Duration");
		if ( sParamVal != null )
		{
		    if ( sParamVal.equals("true") )
		    {
		        _bDuration = true ;
		    }
		}
	    //--- Show headers ?
	    sParamVal = config.getInitParameter("Headers");
		if ( sParamVal != null )
		{
		    if ( sParamVal.equals("true") )
		    {
		        _bHeaders = true ;
		    }
		}
	}

	/**
	 * @see javax.servlet.Filter#destroy()
	 */
	public void destroy()
	{
		trace("destroy()");
	}

	public void doFilter(final ServletRequest request, final ServletResponse response, final FilterChain chain) throws IOException, ServletException
	{
		trace("doFilter()");
		HttpServletRequest httpRequest = (HttpServletRequest) request;
		//HttpServletResponse httpResponse = (HttpServletResponse) response;

		String sRequestedPage = getRequestedPage(httpRequest);
		trace("Filter IN  (" + httpRequest.getRequestURI() + ") : Page = '" + sRequestedPage + "'");
		/***
		trace(" getContextPath = " + httpRequest.getContextPath() );
		trace(" getLocalAddr   = " + httpRequest.getLocalAddr() );
		trace(" getLocalName   = " + httpRequest.getLocalName() );
		trace(" getLocalPort   = " + httpRequest.getLocalPort() );
		trace(" getRootUrl = " + WebUtil.getRootUrl(httpRequest)) ;
		**/
		
		if ( _bHeaders )
		{
			trace(" HTTP Headers : " ) ;
		    Enumeration enu = httpRequest.getHeaderNames() ;
		    String sHeader = null ;
		    while (enu.hasMoreElements()) {
                sHeader = (String) enu.nextElement();
    			trace("  . " + sHeader + " : " + httpRequest.getHeader(sHeader)) ;                
            }
		}
		
		long start = System.currentTimeMillis();
		//--- Suite de la chaine
		chain.doFilter(request, response);
		//--- Retour de la chaine
		long end = System.currentTimeMillis();
		//System.out.println("Time elapsed: " + (end - start) + " milliseconds");
		String s = "Filter OUT (" + httpRequest.getRequestURI() + ") : " ;
		if ( _bDuration )
		{
		    s += " Duration = " + (end - start) + " milliseconds" ;
		}
		trace(s);
	}

	private String getRequestedPage(HttpServletRequest request)
	{
		String sUrl = request.getRequestURI();
		int iFirstSlash = sUrl.indexOf("/", 1);
		String sRequestedPage = null;
		if (iFirstSlash != -1)
		{
			sRequestedPage = sUrl.substring(iFirstSlash + 1, sUrl.length());
		}
		return sRequestedPage;
	}

}