package org.objectweb.telosys.uil;

import javax.servlet.ServletRequest;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpSession;
import javax.servlet.jsp.PageContext;

import org.objectweb.telosys.common.TelosysException;
import org.objectweb.telosys.common.TelosysObject;
import org.objectweb.telosys.screen.core.ScreenContext;
import org.objectweb.telosys.screen.core.ScreenData;
import org.objectweb.telosys.screen.env.ScreenApplication;
import org.objectweb.telosys.screen.env.ScreenApplicationManager;
import org.objectweb.telosys.screen.env.ScreenContextManager;
import org.objectweb.telosys.screen.env.ScreenSession;
import org.objectweb.telosys.screen.env.ScreenSessionManager;
import org.objectweb.telosys.uil.screenmap.ScreenMap;
import org.objectweb.telosys.uil.screenmap.ScreenMapConst;

/**
 * @author Laurent GUERIN
 *  
 */
public class ScreenEnv extends TelosysObject
{
    PageContext        _pageContext       = null;

    HttpServletRequest _httpRequest       = null;

    HttpSession        _httpSession       = null;

    ScreenSession      _screenSession     = null;

    ScreenApplication  _screenApplication = null;

    ScreenContext      _screenContext     = null;

    ScreenData         _screenData        = null;

    /**
     * Return the ScreenMap instance associated with the current pageContext <br>
     * 
     * @param pageContext
     * @return ScreenMap instance or null
     */
    private ScreenMap getScreenMap(PageContext pageContext)
    {
        if (pageContext == null)
        {
            return null;
        }
        ServletRequest request = pageContext.getRequest();
        if (request == null)
        {
            return null;
        }
        return (ScreenMap) request.getAttribute(ScreenMapConst.SCREEN_MAP);
    }

    /**
     * Init the screen context environment from the 'pageContext' instance.
     * 
     * @param pageContext
     */
    private void initEnvironment(PageContext pageContext, String sScreenName, int iScreenContextId)
    {
        _pageContext = pageContext;
        if (_pageContext == null)
        {
            error("Page Context is null ");
            return;
        }
        if (sScreenName == null)
        {
            error("Screen Name is null ");
            return;
        }
        if (iScreenContextId < 0)
        {
            error("Screen Context Id < 0 ");
            return;
        }
        _httpRequest = (HttpServletRequest) _pageContext.getRequest();
        try
        {
            _screenContext = ScreenContextManager.useScreenContext(_httpRequest, sScreenName, iScreenContextId);
        } catch (TelosysException e)
        {
            _screenContext = null ;
            error("Cannot use screen context : Exception : " + e.getMessage());
        }
        if ( _screenContext != null )
        {
            _screenData = _screenContext.getCurrentData();            
        }
        //--- After useScreenContext (in the case of a Session creation )
        _httpSession = _httpRequest.getSession(false);
        _screenSession = ScreenSessionManager.findScreenSession(_httpRequest);
        _screenApplication = ScreenApplicationManager.getScreenApplication(_httpSession);
    }

    /**
     * @param pageContext
     */
    public ScreenEnv(PageContext pageContext)
    {
        super();
        ScreenMap screenMap = getScreenMap(pageContext);
        if (screenMap != null)
        {
            initEnvironment(pageContext, screenMap.getName(), screenMap.getScreenContextId());
        }
    }

    /**
     * @param pageContext
     * @param sScreenName
     * @param iScreenContextId
     */
    public ScreenEnv(PageContext pageContext, String sScreenName, int iScreenContextId)
    {
        super();
        initEnvironment(pageContext, sScreenName, iScreenContextId);
    }

    public HttpServletRequest getHttpRequest()
    {
        return _httpRequest;
    }

    public HttpSession getHttpSession()
    {
        return _httpSession;
    }

    public ScreenApplication getScreenApplication()
    {
        return _screenApplication;
    }

    public ScreenSession getScreenSession()
    {
        return _screenSession;
    }

    public ScreenContext getScreenContext()
    {
        return _screenContext;
    }

    public ScreenData getScreenData()
    {
        return _screenData;
    }
}