package org.objectweb.telosys.uil;

import java.util.Properties;

import org.objectweb.telosys.common.Telosys;
import org.objectweb.telosys.common.Tool;
import org.objectweb.telosys.uil.screenmap.ScreenDefLoader;
import org.objectweb.telosys.uil.screenmap.ScreenDefinitions;
import org.objectweb.telosys.uil.screenmap.ScreenMapConst;
import org.objectweb.telosys.uil.taglib.widget.TagDefaultValues;

/**
 * @author Laurent GUERIN
 * 
 */
public final class TelosysUIL
{
    public  final static String      TELOSYS_CSS_FILE        = "telosys.css";
    
    private final static String      CLASS_NAME              = "TelosysUIL";

    //--- Fichiers de configuration
    private static String            $sScreensConfigFileName = null;

    private static String            $sTagsConfigFileName    = null;

    private static ScreenDefinitions $screenDefinitions      = null;

    //-----------------------------------------------------------------------------
    private static ScreenDefinitions initScreenMapDefinitions()
    {
        if ($sScreensConfigFileName != null)
        {
            Telosys.info(CLASS_NAME + " init Screen Definitions from file " + $sScreensConfigFileName );
            try {
	            ScreenDefLoader loader = new ScreenDefLoader();
	            return loader.load($sScreensConfigFileName);
            }
            catch ( Exception ex ) 
            {
                return null ;
            }
        }
        else
        {
            Telosys.error(CLASS_NAME + ".initScreenMapDefinitions() : No configuration file name => Cannot initialize ScreenMapDefinitions !");
            return null;
        }
    }

    //-----------------------------------------------------------------------------
    // PUBLIC METHODS
    //-----------------------------------------------------------------------------
    public static boolean init()
    {
        Telosys.trace(CLASS_NAME + ".init()");
        //--- Get the Telosys properties ...
        Properties prop = Telosys.getProperties();
//        if (prop == null)
//        {
//            //--- Try to initialize Telosys ...
//            Telosys.trace(CLASS_NAME + " Telosys is not initialized => call Telosys.init()");
//            prop = Telosys.init();
//        }
        if (prop == null)
        {
            Telosys.error(CLASS_NAME + ".init() : cannot get Telosys properties !");
            return false;
        }
        else
        {
            return init(prop);
        }
    }

    //-----------------------------------------------------------------------------
    private static boolean init(Properties prop)
    {
        Telosys.trace(CLASS_NAME + ".init(Properties)");
        boolean bRet1 = false;
        boolean bRet2 = false;
        if (prop != null)
        {
            //-------------------------------------------------------------------
            //--- SCREENS configuration file
            //-------------------------------------------------------------------
            $sScreensConfigFileName = prop.getProperty("ScreensConfFile");
            if ($sScreensConfigFileName == null)
            {
                Telosys.error(CLASS_NAME + ".init(Properties) : Screens Config File not found ");
            }
            else
            {
                $sScreensConfigFileName = $sScreensConfigFileName.trim();
                $sScreensConfigFileName = Telosys.getConfigFileName($sScreensConfigFileName);
                if ( Tool.fileExists( $sScreensConfigFileName ) )
                {
	                //--- Init ScreenMap Definitions
	                $screenDefinitions = initScreenMapDefinitions();
	                if ( $screenDefinitions != null )
	                {
	                    Telosys.info(CLASS_NAME + ".init(Properties) : screen definitions loaded.");                    
	                    bRet1 = true;                    
	                }
	                else
	                {
	                    Telosys.error(CLASS_NAME + ".init(Properties) : Cannot init screen definitions ! ");                    
	                    bRet1 = false;                    
	                }
                }
                else
                {
                    Telosys.error(CLASS_NAME + ".init(Properties) : File " + $sScreensConfigFileName + " not found !");                                        
                }
            }
            Telosys.trace(CLASS_NAME + ".init(Properties) : Screens def. file = " + $sScreensConfigFileName);

            //-------------------------------------------------------------------
            //--- TAGS configuration file
            //-------------------------------------------------------------------
            $sTagsConfigFileName = prop.getProperty("TagsConfFile");
            if ($sTagsConfigFileName == null)
            {
                Telosys.error(CLASS_NAME + ".init(Properties) : Tags Config File not found ");
            }
            else
            {
                $sTagsConfigFileName = $sTagsConfigFileName.trim();
                $sTagsConfigFileName = Telosys.getConfigFileName($sTagsConfigFileName);
                if ( Tool.fileExists( $sTagsConfigFileName ) )
                {
	                //--- Init Tags Default Values
	                bRet2 = TagDefaultValues.init($sTagsConfigFileName);
                }
                else
                {
                    Telosys.error(CLASS_NAME + ".init(Properties) : File " + $sTagsConfigFileName + " not found !");                                        
                }
                
            }
            Telosys.trace(CLASS_NAME + ".init(Properties) : Tags config. file = " + $sTagsConfigFileName);

            //-------------------------------------------------------------------
        }
        else
        {
            Telosys.error(CLASS_NAME + ".init(Properties) : properties = null !");
        }
        return bRet1 && bRet2;
    }

    //-----------------------------------------------------------------------------
    public static String getScreensConfigFileName()
    {
        return $sScreensConfigFileName;
    }

    //-----------------------------------------------------------------------------
    public static String getTagsConfigFileName()
    {
        return $sTagsConfigFileName;
    }

    //-----------------------------------------------------------------------------
    public static ScreenDefinitions getScreenDefinitions()
    {
        return $screenDefinitions;
    }

    //-----------------------------------------------------------------------------
    /**
     * Returns the default screen type ( "html", "xul", ... )
     * @return
     */
    public static String getDefaultScreenType()
    {
        // TODO : load from screens.xml
        return ScreenMapConst.HTML_SCREEN_TYPE;
    }
    //-----------------------------------------------------------------------------
    /**
     * Returns the default screen id 
     * @return
     */
    public static int getDefaultScreenId()
    {
        // TODO : load from screens.xml
        return 0 ;
    }
    //-----------------------------------------------------------------------------
}