package org.objectweb.telosys.uil.screenmap;

import java.io.File;

import org.objectweb.telosys.common.Telosys;
import org.objectweb.telosys.common.TelosysObject;
import org.objectweb.telosys.uil.TelosysUIL;

/**
 * @author Laurent GUERIN
 *  
 */
public class ScreenDefConfig extends TelosysObject
{
    private String _sDefaultAction        = "none";

    // <config telosys_jar_file
    //    private String _sTelosysJarFile = "./WEB-INF/lib/telosys.jar";
    private String _sTelosysJarFile       = "/WEB-INF/lib/telosys.jar";

    // <config telosys_framework_file
    //    private String _sTelosysFrameworkFile = "./js/telosys.js";
    private String _sTelosysFrameworkFile = "/telosys/telosys.js";

    //    private String _sTelosysStyleDir = "./css/telosys";
    private String _sFrameworkCSSDir      = "/telosys";

    // <config telosys_framework_url
    //private String _sTelosysFrameworkUrl  = "/js/telosys.js";

    // <config ajax_request_url
    private String _sAjaxRequestUrl       = null;

    // <config screen_map_url
    private String _sScreenMapUrl         = null;

    //---------------------------------------------------------------------------
    private String getFullPathFile(String sFile, boolean bMustExist)
    {
        if (sFile != null)
        {
            //--- Determine Full Path
            String sFullPath = null;
            //            if ( sFile.startsWith(".") ) // RELATIVE
            //            {
            //                //--- Cut the first char
            //                sFile = sFile.substring(1);
            //--- Full path = Application root dir + relative file name
            sFullPath = Telosys.getFullPath(sFile);
            //            }
            //            else // ABSOLUTE
            //            {
            //                sFullPath = sFile;
            //            }

            //--- Check existence (if needed)
            if (bMustExist)
            {
                File file = new File(sFullPath);
                if (file.exists())
                {
                    return sFullPath;
                }
            }
            else
            {
                return sFullPath;
            }
        }
        return null;
    }

    //---------------------------------------------------------------------------

    //---------------------------------------------------------------------------
    // Context : Auto-Open Context mode
    //---------------------------------------------------------------------------
    /* package */void setDefaultAction(String s)
    {
        _sDefaultAction = s;
    }

    //---------------------------------------------------------------------------
    public String getDefaultAction()
    {
        return _sDefaultAction;
    }

    //---------------------------------------------------------------------------

    //---------------------------------------------------------------------------
    // Telosys Framework Javascript File
    //---------------------------------------------------------------------------
    /**
     * Set the web context path for the telosys javascript framework file
     * 
     * @param s ( ie: "/js/telosys.js" )
     */
    /* package */void setFrameworkJavascriptFile(String s)
    {
        //_sTelosysFrameworkFile = getFullPathFile(s, false);
        // Set if not null and not void ( else keep default value )
        if ( s != null ) // else keep default value
        {
            if ( s.trim().length() > 0 )
            {
                _sTelosysFrameworkFile = s ;  
            }
        }
    }

    //---------------------------------------------------------------------------
    /**
     * Returns the web context path of the javascript framework file
     * @return ( ie: "/js/telosys.js" )
     */
    public String getFrameworkJavascriptFile()
    {
        return _sTelosysFrameworkFile;
    }
    //---------------------------------------------------------------------------
    /**
     * Returns the full filesystem path of the Telosys Framework Javascript file
     * 
     * @return the path ( ie : "/filesystem_dir/web_dir/context_dir/js/telosys.js" )
     */
    public String getFrameworkJavascriptFilePath()
    {
        return getFullPathFile(_sTelosysFrameworkFile, false);
    }
    //---------------------------------------------------------------------------
    // Telosys Javascript framework URL
    //---------------------------------------------------------------------------
//    /* package */void setTelosysFrameworkUrl(String s)
//    {
//        _sTelosysFrameworkUrl = s;
//    }
//    //---------------------------------------------------------------------------
//    public String getTelosysFrameworkUrl()
//    {
//        return _sTelosysFrameworkUrl;
//    }
//    public String getFrameworkJavascriptFilePath()
//    {
//        return _sTelosysFrameworkFile ;
//    }

    //---------------------------------------------------------------------------


    //---------------------------------------------------------------------------
    // Telosys CSS files
    //---------------------------------------------------------------------------
    /**
     * Set the web context path for the framework CSS directory 
     * @param s the directory ( ie: "/css/telosys" )
     */
    /* package */ void setFrameworkCSSDir(String s)
    {
        // Set if not null and not void ( else keep default value )
        if (s != null) 
        {
            if ( s.trim().length() > 0 )
            {
                _sFrameworkCSSDir = s.trim();                
            }
        }
    }

    //---------------------------------------------------------------------------
    /**
     * Returns the web context path of the framework CSS directory 
     * @return the directory ( ie: "/css/telosys" )
     */
    public String getFrameworkCSSDir()
    {
        return _sFrameworkCSSDir ;
    }

    //---------------------------------------------------------------------------
    /**
     * Returns the web context path of a given framework CSS file 
     * @param sFileName ( ie : "image.png" )
     * @return the directory ( ie: "/css/telosys/image.png" )
     */
    public String getFrameworkCSSFile(String sFileName)
    {
        String sDir = _sFrameworkCSSDir;
        if (!sDir.endsWith("/"))
        {
            sDir = sDir + "/";
        }
        return sDir + sFileName ;
    }

    //---------------------------------------------------------------------------
    /**
     * Returns the web context path of the framework CSS file 
     * @return the directory ( ie: "/css/telosys/telosys.css" )
     */
    public String getFrameworkCSSFile()
    {
        return getFrameworkCSSFile( TelosysUIL.TELOSYS_CSS_FILE );
    }

    //---------------------------------------------------------------------------
    /**
     * Returns the full filesystem path of the given CSS file name
     * @param sFileName ( ie : "image.png" )
     * @return the path ( ie : "/filesystem_dir/web_dir/context_dir/css/image.png" )
     */
    public String getFrameworkCSSFilePath(String sFileName)
    {
//        String sDir = _sFrameworkCSSDir;
//        if (!sDir.endsWith("/"))
//        {
//            sDir = sDir + "/";
//        }
        return getFullPathFile( getFrameworkCSSFile(sFileName), false);
    }
    
    //---------------------------------------------------------------------------
//    /**
//     * Returns the full filesystem path of the Telosys Framework CSS file
//     * 
//     * @return the path ( ie : "/filesystem_dir/web_dir/context_dir/css/telosys.css" )
//     */
//    public String getFrameworkCSSFilePath()
//    {
//        return getFrameworkCSSFilePath( TelosysUIL.TELOSYS_CSS_FILE ) ;
//    }

    //---------------------------------------------------------------------------
    // Telosys JAR File
    //---------------------------------------------------------------------------
    /**
     * Set the full file name of the telosys jar file <br>
     * The file name can be absolute, or relative ( added to application root dir )<br>
     * The file is set only if it exists or if it's null
     * 
     * @param s
     */
    /* package */void setTelosysJarFile(String s)
    {
        _sTelosysJarFile = getFullPathFile(s, true);
    }

    //---------------------------------------------------------------------------
    public String getTelosysJarFile()
    {
        return _sTelosysJarFile;
    }

    //---------------------------------------------------------------------------

    //---------------------------------------------------------------------------
    // AJAX request URL
    //---------------------------------------------------------------------------
    /* package */void setAjaxRequestUrl(String s)
    {
        _sAjaxRequestUrl = s;
    }

    //---------------------------------------------------------------------------
    public String getAjaxRequestUrl()
    {
        return _sAjaxRequestUrl;
    }

    //---------------------------------------------------------------------------

    //---------------------------------------------------------------------------
    // Screen Map URL
    //---------------------------------------------------------------------------
    /* package */void setScreenMapUrl(String s)
    {
        _sScreenMapUrl = s;
    }

    //---------------------------------------------------------------------------
    public String getScreenMapUrl()
    {
        return _sScreenMapUrl;
    }
    //---------------------------------------------------------------------------

}