package org.objectweb.telosys.uil.screenmap;

import org.objectweb.telosys.common.TelosysObject;
import org.objectweb.telosys.util.XmlUtil;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

/**
 * @author Laurent GUERIN
 *  
 */
public class ScreenDefLoader extends TelosysObject
{
    //private final static String HTML = "html";
    private final static String HTML = ScreenMapConst.HTML_SCREEN_TYPE ;

    //private final static String XUL  = "xul";
    private final static String XUL  = ScreenMapConst.XUL_SCREEN_TYPE ;

    //---------------------------------------------------------------------------
    private Element toXmlElement(Node node)
    {
        if (node instanceof Element)
        {
            return (Element) node;
        }
        else
        {
            return null;
        }
    }

    //---------------------------------------------------------------------------
    // tag <xxx> <html ...> <xul ...> </xxx>
    //---------------------------------------------------------------------------
    private String getRootDir(Node tag, String sChildName)
    {
        trace("getRootDir : tag = " + tag.getNodeName() + "  child name = " + sChildName);
        NodeList childNodes = tag.getChildNodes(); // <html ...> or <xul ...>
        if (childNodes != null)
        {
            for ( int i = 0 ; i < childNodes.getLength() ; i++ )
            {
                Node node = childNodes.item(i);
                if (node.getNodeType() == Node.ELEMENT_NODE)
                {
                    Element elem = toXmlElement(node);
                    if (elem != null)
                    {
                        if (elem.getNodeName().equals(sChildName))
                        {
                            String sRet = elem.getAttribute("root");
                            trace(" -> " + sRet);
                            return sRet;
                        }
                    }
                }
            }
        }
        trace(" -> not found ! ");
        return null;
    }

    //---------------------------------------------------------------------------
    // tag <rootdir>
    //---------------------------------------------------------------------------
    private void processRootDir(ScreenDefinitions screenDefinitions, NodeList list)
    {
        trace("processRootDir()...");
        if (screenDefinitions == null)
        {
            error("processRootDir : screenDefinitions = null !");
            return;
        }
        if (list == null)
        {
            error("processRootDir : no node ( list = null ) !");
            return;
        }
        
        //--- Get the HTML root dir config
        ScreenDefRootDir htmlRootDir = screenDefinitions.getHtmlRootDir();
        if (htmlRootDir == null)
        {
            error("processRootDir : No HTML root dir instance in screen definitions !");
            return;
        }
        //--- Get the XUL root dir config
        ScreenDefRootDir xulRootDir = screenDefinitions.getXulRootDir();
        if (xulRootDir == null)
        {
            error("processRootDir : No XUL root dir instance in screen definitions !");
            return;
        }
        //--- for each "rootdir" node
        for ( int iRootDir = 0 ; iRootDir < list.getLength() ; iRootDir++ )
        {
            Node tagRootdir = list.item(iRootDir);
            //trace("processRootDir() : node name = " + tagRootdir.getNodeName() + " / i = " + iRootDir );
            NodeList childNodes = tagRootdir.getChildNodes();
            if (childNodes != null)
            {
                //trace("processRootDir() : childNodes.getLength() = " + childNodes.getLength() );
                //--- for each child : "template", "body", "page", ....
                for ( int iChild = 0 ; iChild < childNodes.getLength() ; iChild++ )
                {
                    Node node = childNodes.item(iChild);
                    //trace("processRootDir() : node.getNodeType() = " + node.getNodeType() );
                    if (node.getNodeType() == Node.ELEMENT_NODE)
                    {
                        trace("processRootDir() : <rootdir> <" + node.getNodeName() + "> ...");
                        if (node.getNodeName().equals("template"))
                        {
                            htmlRootDir.setTemplateRootDir(getRootDir(node, HTML));
                            xulRootDir.setTemplateRootDir(getRootDir(node, XUL));
                        }
                        else if (node.getNodeName().equals("body"))
                        {
                            htmlRootDir.setBodyRootDir(getRootDir(node, HTML));
                            xulRootDir.setBodyRootDir(getRootDir(node, XUL));
                        }
                        else if (node.getNodeName().equals("css"))
                        {
                            htmlRootDir.setCssRootDir(getRootDir(node, HTML));
                            xulRootDir.setCssRootDir(getRootDir(node, XUL));
                        }
                        else if (node.getNodeName().equals("images"))
                        {
                            htmlRootDir.setImagesRootDir(getRootDir(node, HTML));
                            xulRootDir.setImagesRootDir(getRootDir(node, XUL));
                        }
                        else if (node.getNodeName().equals("script"))
                        {
                            htmlRootDir.setScriptRootDir(getRootDir(node, HTML));
                            xulRootDir.setScriptRootDir(getRootDir(node, XUL));
                        }
                        else if (node.getNodeName().equals("page"))
                        {
                            htmlRootDir.setPageRootDir(getRootDir(node, HTML));
                            xulRootDir.setPageRootDir(getRootDir(node, XUL));
                        }
                        else
                        {
                            error("processRootDir : unknown tag <" + node.getNodeName() + ">");
                        }
                    }
                }
            }
            else
            {
                error("processRootDir : no childs for tag 'rootdir' (" + iRootDir + ") ");
            }
        }
    }

    //---------------------------------------------------------------------------
    // tag <screen>
    //---------------------------------------------------------------------------
    private void processScreen(ScreenDefinitions screenDefinitions, NodeList list)
    {
        trace("processScreen()...");

        if (screenDefinitions == null)
        {
            error("processScreen : screenDefinitions = null !");
            return;
        }
        if (list == null)
        {
            error("processScreen : no node ( list = null ) !");
            return;
        }

        ScreenDefinition screen = null;
        String sScreenName = null;
        String sScreenBody = null;
        String sScreenTemplate = null;

        for ( int i = 0 ; i < list.getLength() ; i++ )
        {
            //--- New screen
            Element screenTag = toXmlElement(list.item(i));
            if (screenTag != null)
            {
                //--- screen attributes
                sScreenName = screenTag.getAttribute("name");
                if (sScreenName != null)
                {
                    //--- New instance
                    ScreenDefLocation screenLoc = new ScreenDefLocation();
                    screenLoc.setScreenBody(screenTag.getAttribute("body"));
                    screenLoc.setScreenTemplate(screenTag.getAttribute("template"));
                    screenLoc.setScreenPage(screenTag.getAttribute("page"));
                    screenLoc.setScreenUrl(screenTag.getAttribute("url"));
                    screenLoc.setScreenCss(screenTag.getAttribute("css"));
                    screenLoc.setScreenScript(screenTag.getAttribute("script"));

                    screen = new ScreenDefinition(sScreenName, screenLoc);

                    //--- screen childs : <html ..> <xul ..> <element ..>
                    NodeList childNodes = screenTag.getChildNodes();
                    if (childNodes != null)
                    {
                        for ( int iChild = 0 ; iChild < childNodes.getLength() ; iChild++ )
                        {
                            Node node = childNodes.item(iChild);
                            if (node.getNodeType() == Node.ELEMENT_NODE)
                            {
                                Element tag = toXmlElement(node);
                                String sNodeName = node.getNodeName();
                                if ((sNodeName.equals(HTML)) || (sNodeName.equals(XUL)))
                                {
                                    //--- Tags <html ...> or <xul ...>
                                    trace("new ScreenDefLocation("+sNodeName+")");
                                    ScreenDefLocation loc = new ScreenDefLocation(sNodeName);
                                    loc.setScreenBody(tag.getAttribute("body"));
                                    loc.setScreenTemplate(tag.getAttribute("template"));
                                    loc.setScreenPage(tag.getAttribute("page"));
                                    loc.setScreenUrl(tag.getAttribute("url"));
                                    loc.setScreenCss(tag.getAttribute("css"));
                                    loc.setScreenScript(tag.getAttribute("script"));
                                    screen.setScreenDefLocation(loc);
                                }
                                else if (sNodeName.equals("element"))
                                {
                                    //--- Tag <element ...>
                                    String sName = tag.getAttribute("name");
                                    String sValue = tag.getAttribute("value");
                                    String sContent = tag.getAttribute("content");
                                    trace("new ScreenMapElement("+sName+","+sValue+","+sContent+")");
                                    screen.addElement(new ScreenMapElement(sName, sValue, sContent));
                                }
                            }
                        } // for each child node
                    } // if (childNodes != null)
                    //---
                    trace("addScreenDefinition : name "+screen.getScreenName() + " - " + screen.getElemCount() );
                    screenDefinitions.addScreenDefinition(screen);
                }
            }
        }
    }
    //---------------------------------------------------------------------------
    // tag <config>
    //---------------------------------------------------------------------------
    private void processConfig(ScreenDefinitions screenDefinitions, NodeList list)
    {
        trace("processConfig()...");
        if (screenDefinitions == null)
        {
            error("processConfig : screenDefinitions = null !");
            return;
        }
        ScreenDefConfig config = screenDefinitions.getConfig();
        if (config == null)
        {
            error("processConfig : config = null !");
            return;
        }
        if (list == null)
        {
            error("processConfig : no node ( list = null ) !");
            return;
        }
        
        String sValue = null ;        
        //--- for each "config" node ( ONLY ONE EXPECTED )
        for ( int i = 0 ; i < list.getLength() ; i++ )
        {
            Node node = list.item(i);
            Element element = toXmlElement(node);
            if ( element != null )
            {
	            sValue  = element.getAttribute("default_action") ;
	            if ( sValue != null )
	            {
	                config.setDefaultAction(sValue);	                
	            }
	            sValue  = element.getAttribute("telosys_jar_file") ;
	            if ( sValue != null )
	            {
	                config.setTelosysJarFile(sValue);	                
	            }
                sValue = element.getAttribute("framework_javascript_file") ;
	            if ( sValue != null )
	            {
	                config.setFrameworkJavascriptFile(sValue);	                
	            }
                sValue = element.getAttribute("framework_css_dir") ;
	            if ( sValue != null )
	            {
	                config.setFrameworkCSSDir(sValue);	                
	            }
//	            sValue  = element.getAttribute("telosys_framework_url") ;
//	            if ( sValue != null )
//	            {
//	                config.setTelosysFrameworkUrl(sValue);	                
//	            }
	            sValue  = element.getAttribute("screen_map_url") ;
	            if ( sValue != null )
	            {
	                config.setScreenMapUrl(sValue);	                
	            }
	            sValue  = element.getAttribute("ajax_request_url") ;
	            if ( sValue != null )
	            {
	                config.setAjaxRequestUrl(sValue);	                
	            }
            }
        }
    }
    
    //---------------------------------------------------------------------------
    public ScreenDefinitions load(String sFileName)
    {
        trace("load(" + sFileName + ")");

        //--- The screens definitions container
        ScreenDefinitions screenDefinitions = null ;

        //--- Initialize the screens definitions from the XML file
        Document document = XmlUtil.loadFile(sFileName);
        if (document != null)
        {
            //--- Create the screens definitions container
            screenDefinitions = new ScreenDefinitions();

            //--- Process the tag <config> ( only one tag expected )
            NodeList configNodes = document.getElementsByTagName("config");
            processConfig(screenDefinitions, configNodes);

            //--- Process the tags <rootdir> ( only one tag expected )
            NodeList rootdirNodes = document.getElementsByTagName("rootdir");
            processRootDir(screenDefinitions, rootdirNodes);

            //--- Process the tags <screen> ( N tags )
            NodeList screenNodes = document.getElementsByTagName("screen");
            processScreen(screenDefinitions, screenNodes);

        }
        return screenDefinitions;
    }

}