package org.objectweb.telosys.uil.screenmap;

import java.util.HashMap;

import org.objectweb.telosys.common.TelosysObject;
import org.objectweb.telosys.uil.TelosysUIL;

/**
 * @author Laurent GUERIN
 *  
 */
public class ScreenMap extends TelosysObject
{
    private String  _sType                 = null; // HTML or XUL

    private String  _sScreenName           = null;

    private int     _iScreenContextId      = 0;   // The screen context used by this Screen Map

    //--- Screen Definitions Config ( from "screens.xml" )
//    private String  _sTelosysFrameworkPath = null;

//    private String  _sAjaxRequestPath      = null;

//    private String  _sScreenMapPath        = null;

    //--- The ressource names for the specific Screen Type ( HTML, XUL, ... )
    private String  _sTarget               = null;

    private String  _sBody                 = null; // This screen Body

    private String  _sCss                  = null; // This screen CSS file

    private String  _sScript               = null; // This screen JS file (behavior)

    //--- The URLs for the specific Screen Type ( HTML, XUL, ... )
    private String  _sTargetURL            = null;

    private String  _sBodyURL              = null; // This screen Body

    private String  _sCssURL               = null; // This screen CSS file

    private String  _sScriptURL            = null; // This screen JS file (behavior)

    private String  _sImagesRootDir        = null;

    private HashMap _hmElements            = null;

    //---------------------------------------------------------------------------
    private void initConfig()
    {
        ScreenDefinitions definitions = TelosysUIL.getScreenDefinitions();
        if (definitions != null)
        {
            ScreenDefConfig config = definitions.getConfig();
            if (config != null)
            {
//                _sTelosysFrameworkPath = config.getTelosysFrameworkUrl();
//                _sAjaxRequestPath = config.getAjaxRequestUrl();
//                _sScreenMapPath = config.getScreenMapUrl();
            }
            else
            {
                error("No config in screen definitions : definitions.getConfig() return null ");
            }
        }
        else
        {
            error("No screen definitions : TelosysUIL.getScreenDefinitions() return null ");
        }
    }

    //---------------------------------------------------------------------------
    public ScreenMap(ScreenDefRootDir rootDir, ScreenDefinition def, String sScreenType, int iScreenId)
    {
        super();
        _sType = sScreenType;
        _iScreenContextId = iScreenId;

        initConfig();

        if (def != null)
        {
            _sScreenName = def.getScreenName();

            _sTarget = def.getScreenTarget(sScreenType);
            _sBody = def.getScreenBody(sScreenType);
            _sCss = def.getScreenCss(sScreenType);
            _sScript = def.getScreenScript(sScreenType);

            //--- Target URL
            if (def.isBodyTarget(sScreenType))
            {
                //--- Target is "Body with template" => target URL = template URL
                //_sTargetURL = buildURL(_sTarget, rootDir.getTemplateRootDir());
                _sTargetURL = rootDir.getTemplatePath(_sTarget);
            }
            else if (def.isPageTarget(sScreenType))
            {
                //--- Target is "Simple Page" => target URL = page URL
                //_sTargetURL = buildURL(_sTarget, rootDir.getPageRootDir());
                _sTargetURL = rootDir.getPagePath(_sTarget);
            }
            else
            {
                _sTargetURL = _sTarget; // Target "as is"
            }

            //--- Body URL
            //_sBodyURL = buildURL(_sBody, rootDir.getBodyRootDir());
            _sBodyURL = rootDir.getBodyPath(_sBody);

            //--- CSS URL
            //_sCssURL = buildURL(_sCss, rootDir.getCssRootDir());
            _sCssURL = rootDir.getCssPath(_sCss);

            //--- JavaScript URL
            //_sScriptURL = buildURL(_sScript, rootDir.getScriptRootDir());
            _sScriptURL = rootDir.getScriptPath(_sScript);

            //--- Images root directory
            _sImagesRootDir = rootDir.getImagesRootDir();

            //--- Templates elements ( 0 .. N )
            _hmElements = def.getElements();
        }
    }

    //---------------------------------------------------------------------------
//    public String getTelosysFrameworkPath()
//    {
//        return _sTelosysFrameworkPath;
//    }

    //---------------------------------------------------------------------------
//    public String getAjaxRequestPath()
//    {
//        return _sAjaxRequestPath;
//    }

    //---------------------------------------------------------------------------
//    public String getScreenMapPath()
//    {
//        return _sScreenMapPath ;
//    }

    //---------------------------------------------------------------------------
    public String getTargetURL()
    {
        return _sTargetURL;
    }

    //---------------------------------------------------------------------------
    public String getBodyURL()
    {
        return _sBodyURL;
    }

    //---------------------------------------------------------------------------
    public String getCssURL()
    {
        return _sCssURL;
    }

    //---------------------------------------------------------------------------
    public String getScriptURL()
    {
        return _sScriptURL;
    }

    //---------------------------------------------------------------------------
    public String getType()
    {
        return _sType;
    }

    //---------------------------------------------------------------------------
    public int getScreenContextId()
    {
        return _iScreenContextId;
    }

    //---------------------------------------------------------------------------
    public boolean isHtmlType()
    {
        return (_sType.equals(ScreenMapConst.HTML_SCREEN_TYPE));
    }

    //---------------------------------------------------------------------------
    public boolean isXulType()
    {
        return (_sType.equals(ScreenMapConst.XUL_SCREEN_TYPE));
    }

    //---------------------------------------------------------------------------
    public String getImagesRootDir()
    {
        return _sImagesRootDir;
    }

    //---------------------------------------------------------------------------
    public String getName()
    {
        return _sScreenName;
    }

    //---------------------------------------------------------------------------
    public ScreenMapElement getElement(String sName)
    {
        if (_hmElements != null)
        {
            return (ScreenMapElement) _hmElements.get(sName);
        }
        else
        {
            return null;
        }
    }

    //---------------------------------------------------------------------------
    public int getElemCount()
    {
        if (_hmElements == null)
        {
            return 0;
        }
        else
        {
            return _hmElements.size();
        }
    }

    //---------------------------------------------------------------------------
    public String describe()
    {
        return "name=" + _sScreenName + " - type=" + _sType + "\n" + "target : " + _sTarget + " -> URL : " + _sTargetURL + "\n" + "body   : " + _sBody + " -> URL : " + _sBodyURL + "\n" + "css    : " + _sCss + " -> URL : " + _sCssURL + "\n" + "script : " + _sScript + " -> URL : " + _sScriptURL + "\n" + "images dir = " + _sImagesRootDir + "\n" + "elements.size() = " + getElemCount() + "\n";
    }
    //---------------------------------------------------------------------------
}