package org.objectweb.telosys.uil.screenmap;

import java.io.IOException;
import java.io.PrintWriter;

import javax.servlet.RequestDispatcher;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.objectweb.telosys.common.Telosys;
import org.objectweb.telosys.common.TelosysException;
import org.objectweb.telosys.common.TelosysServlet;
import org.objectweb.telosys.screen.core.ScreenContext;
import org.objectweb.telosys.screen.env.ScreenContextManager;
import org.objectweb.telosys.uil.TelosysUIL;
import org.objectweb.telosys.util.JarUtil;

/**
 * @author Laurent GUERIN
 *  
 */
public class ScreenMapServlet extends TelosysServlet // HttpServlet
{
    private final static String      SCREEN_NAME        = "name";

    private final static String      SCREEN_TYPE        = "type";

    private final static String      SCREEN_ID          = "id";

    private final static String      SCREEN_ACTION      = "action";

    private final static String      ACTION_OPEN        = "open";

    private final static String      ACTION_USE         = "use";

    private final static String      ACTION_NONE        = "none";

    private static ScreenDefinitions _screenDefinitions = null;

    //private static boolean           _bAutoOpenContext  = false;
    private static String            _sDefaultAction    = null ;

    //---------------------------------------------------------------------------
    private void traceRootDir( String sType, ScreenDefRootDir rootDir ) throws ServletException
    {
        trace( sType + " Root Dir : ");

        trace(". Template : " + rootDir.getTemplateRootDir());
        trace(". Body     : " + rootDir.getBodyRootDir());
        trace(". Page     : " + rootDir.getPageRootDir());
        trace(". Script   : " + rootDir.getScriptRootDir());
        trace(". CSS      : " + rootDir.getCssRootDir());
        trace(". Images   : " + rootDir.getImagesRootDir());        
    }
    //---------------------------------------------------------------------------
    public void init() throws ServletException
    {
        info("ScreenMap servlet init ...");

        info(" - Retrieve ScreenMap definitions ...");
        _screenDefinitions = TelosysUIL.getScreenDefinitions();
        //--- If UIL is not yet initialized
        if (_screenDefinitions == null)
        {
            trace("init UIL ...");
            TelosysUIL.init();
            trace("get ScreenMap definitions...");
            _screenDefinitions = TelosysUIL.getScreenDefinitions();
            if (_screenDefinitions == null)
            {
                error("Cannot get the screen definitions !");
                return;
            }
        }

        info("  " + _screenDefinitions.getScreenDefCount() + " screen definitions loaded ");
        info("  Telosys application root dir = " + Telosys.getRootDirectory());

        ScreenDefConfig config = _screenDefinitions.getConfig();
        if (config != null)
        {
            //_bAutoOpenContext = config.getAutoOpenContext();
            String s = config.getDefaultAction() ;
            if ( s != null )
            {
                _sDefaultAction = s.trim().toLowerCase() ;                
            }
            trace("Config : ");
            trace(". Telosys JAR file          : " + config.getTelosysJarFile());
            trace(". Framework javascript file : " + config.getFrameworkJavascriptFile());
            trace(". Framework CSS directory   : " + config.getFrameworkCSSDir());
            trace(". Screen Map URL            : " + config.getScreenMapUrl());
            trace(". AJAX request URL          : " + config.getAjaxRequestUrl());
            trace(". Default action            : " + _sDefaultAction);
        }
        else
        {
            error("Cannot get config from screen definitions !");
        }

        ScreenDefRootDir rootDir = _screenDefinitions.getHtmlRootDir();
        if (rootDir != null)
        {
            traceRootDir("HTML", rootDir);
            /***
            trace("HTML Root Dir : ");
            trace(". Template : " + rootDir.getTemplateRootDir());
            trace(". Body     : " + rootDir.getBodyRootDir());
            trace(". Page     : " + rootDir.getPageRootDir());
            trace(". Script   : " + rootDir.getScriptRootDir());
            trace(". CSS      : " + rootDir.getCssRootDir());
            trace(". Images   : " + rootDir.getImagesRootDir());
            ***/
        }
        else
        {
            error("Cannot get 'HTML root dir' from screen definitions !");
        }
        
        rootDir = _screenDefinitions.getXulRootDir();
        if (rootDir != null)
        {
            traceRootDir("XUL", rootDir);
        }
        else
        {
            error("Cannot get 'XUL root dir' from screen definitions !");
        }

        //--- Extract the file "telosys.js" from "telosys.jar"
        if (config != null)
        {
            String sJarFile = config.getTelosysJarFile();
            info("Extracting framework files from JAR (" + sJarFile + ") ...");
            
            //--- Extract the Javascript framework file 
            extract(sJarFile, "js/telosys.js", config.getFrameworkJavascriptFilePath() );
//            String sJarEntry = "js/telosys.js";
//            String sTelosysFwkFile = config.getTelosysFrameworkFile();
//            info(" - Extracting the JavaScript Telosys framework (jar=" + sJarFile + " dest=" + sTelosysFwkFile + ") ...");
//            try
//            {
//                JarUtil.extract(sJarFile, sJarEntry, sTelosysFwkFile);
//                info("  " + sTelosysFwkFile + " extracted from " + sJarFile);
//            } catch (TelosysException e)
//            {
//                error("Cannot set '" + sTelosysFwkFile + "' from '" + sJarFile + "' !");
//                e.printStackTrace();
//            }
            
            //--- Extract the STYLE files ( css & images )
            extract(sJarFile, "css/telosys.css",         config.getFrameworkCSSFilePath(TelosysUIL.TELOSYS_CSS_FILE) ); // "telosys.css" 
            extract(sJarFile, "css/telosys_tab_off.png", config.getFrameworkCSSFilePath("telosys_tab_off.png") );
            extract(sJarFile, "css/telosys_tab_on.png",  config.getFrameworkCSSFilePath("telosys_tab_on.png") );
        }
        else
        {
            error("Config is null => Cannot extract framework files from JAR !");
        }
    }

    //---------------------------------------------------------------------------
    private void extract(String sJarFile, String sJarEntry, String sDestFile) 
    {
        info(" . Extracting file '" + sJarEntry + "' to '" + sDestFile + "' ...");
        try
        {
            JarUtil.extract(sJarFile, sJarEntry, sDestFile);
        } catch (TelosysException e)
        {
            error("Cannot set '" + sDestFile + "' from '" + sJarFile + "' !");
            e.printStackTrace();
        }
    }

    //---------------------------------------------------------------------------
    public void doGet(HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException
    {
        trace("========================================");
        trace("doGet()");
        process(request, response);
    }

    //---------------------------------------------------------------------------
    public void doPost(HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException
    {
        trace("========================================");
        trace("doPost()");
        process(request, response);
    }

    //---------------------------------------------------------------------------
    public void destroy()
    {
        trace("destroy()");
    }

    //---------------------------------------------------------------------------
    private void process(HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException
    {
        trace("process()...");

        if (_screenDefinitions == null)
        {
            String sMsg = "Screens definitions are not initialized ! ";
            error(sMsg);
            response_error(response, sMsg);
            return;
        }

        //--- Get the Screen Name ( request parameter : mandatory )
        String sScreenName = request.getParameter(SCREEN_NAME);
        if (sScreenName == null)
        {
            String sMsg = "No screen name parameter !";
            error(sMsg);
            response_error(response, sMsg);
            return;
        }
        trace(". Screen name = " + sScreenName);

        //--- Get the Screen Type from request parameter ( optional )
        String sScreenType = request.getParameter(SCREEN_TYPE);
        if (sScreenType == null)
        {
            //sScreenType = ScreenMapConst.HTML_SCREEN_TYPE;
            sScreenType = TelosysUIL.getDefaultScreenType() ;
        }
        trace(". Screen type = " + sScreenType);

        //--- Get the Screen Context ID from request parameter ( optional )
        //int iScreenContextId = 0; // ZERO = Default Screen Context Id
        int iScreenContextId = TelosysUIL.getDefaultScreenId() ; 
        String sScreenContextId = request.getParameter(SCREEN_ID);
        if (sScreenContextId != null)
        {
            try
            {
                iScreenContextId = Integer.parseInt(sScreenContextId);
            } catch (NumberFormatException e)
            {
                String sMsg = "Invalid Screen Context Id : '" + sScreenContextId + "'";
                error(sMsg);
                response_error(response, sMsg);
                return;
            }
        }
        trace(". Screen context id = " + sScreenContextId + "(" + iScreenContextId + ")");

        //--- Get the Screen Action ( request parameter : optional )
        String sScreenAction = request.getParameter(SCREEN_ACTION);
        if (sScreenAction != null)
        {
            sScreenAction = sScreenAction.toLowerCase();
        }
        trace(". Screen action = " + sScreenAction);

        //--- Get a Screen Map instance from the Screen Definitions
        trace("getScreenMap(" + sScreenName + "," + sScreenType + "," + iScreenContextId + ")...");
        ScreenMap screenMap = _screenDefinitions.getScreenMap(sScreenName, sScreenType, iScreenContextId);
        if (screenMap == null)
        {
            String sMsg = "Cannot get the screen map : getScreenMap(" + sScreenName + "," + sScreenType + "," + iScreenContextId + ")";
            error(sMsg);
            response_error(response, sMsg);
            return;
        }
        trace("ScreenMap found : URL = " + screenMap.getTargetURL() + " Body URL = " + screenMap.getBodyURL());
        //trace("SCREEN : " + screenMap.describe());

        //--- Retrieve the main target URL of the screen map
        String sScreenMapUrl = screenMap.getTargetURL();
        if (sScreenMapUrl == null)
        {
            String sMsg = "No URL for the screen : " + sScreenName + " ( type = " + sScreenType + " )";
            error(sMsg);
            response_error(response, sMsg);
            return;
        }

        //--- Open or Use a ScreenContext ? ( action="open|use" or "default_action" )
        ScreenContext screenContext = null;
        if (sScreenAction == null)
        {
            if ( _sDefaultAction != null )
            {
                sScreenAction = _sDefaultAction ;                
                trace("No specific action => use default action ( " + _sDefaultAction + " )");
            }
        }
        
        if (sScreenAction != null)
        {
            try
            {
                if (sScreenAction.equals(ACTION_OPEN))
                {
                    trace("Open a new screen context (action=" + sScreenAction + ")");
                    screenContext = ScreenContextManager.openScreenContext(request, sScreenName, iScreenContextId);
                }
                else if (sScreenAction.equals(ACTION_USE))
                {
                    trace("Reuse or open screen contex (action=" + sScreenAction + ")");
                    screenContext = ScreenContextManager.useScreenContext(request, sScreenName, iScreenContextId);
                }
                else if (sScreenAction.equals(ACTION_NONE))
                {
                    trace("No action on the screen contex (action=" + sScreenAction + ")");
                }
                else
                {
                    String sMsg = "Invalid action '" + sScreenAction + "'";
                    error(sMsg);
                    response_error(response, sMsg);
                    return;
                }
            } catch (TelosysException e)
            {
                String sMsg = "Cannot " + sScreenAction + " Screen Context [" + iScreenContextId + "-" + sScreenName + "] - Exception :" + e.getMessage();
                error(sMsg);
                response_error(response, sMsg);
                return;
            }
        }
        else
        {
            trace("No action in request and no default action.");            
        }

        if (screenContext != null)
        {
            //trace("Screen context ready ( ClassName = " + screenContext.getScreenClassName() + " )");
            trace("Screen context ready ( screen name = '" + screenContext.getScreenName() + "' )");
        }
        else
        {
            trace("No screen context.");
        }

        /***********************************************************************************************************************************************************************************************
         * if ( sScreenContextId != null || _bAutoOpen ) { trace("Screen context required"); try { screenContext = ScreenContextManager.useScreenContext(request, sScreenName, iScreenContextId); }
         * catch (TelosysException e) { String sMsg = "Cannot use Screen Context [" + iScreenContextId + "-" + sScreenName + "] - Exception :" + e.getMessage(); error(sMsg); response_error(response,
         * sMsg); return; } trace("OK : Screen context ready. ClassName = " + screenContext.getScreenClassName()); //--- Set the screen context for future use by the Tags //TODO
         * screenMap.setScreenContext(screenContext); } else { trace("No screen context required"); }
         **********************************************************************************************************************************************************************************************/

        //--- Set the Screen Map object as request attribute
        request.setAttribute(ScreenMapConst.SCREEN_MAP, screenMap);
        trace("Screen map set as request attribute");

        //--- Forward to the Screen Map Target URL
        trace("Forward to " + sScreenMapUrl);
        boolean bForwardOk = forward(sScreenMapUrl, request, response);
        if (!bForwardOk)
        {
            String sMsg = "Cannot forward to " + sScreenMapUrl;
            error(sMsg);
            response_error(response, sMsg);
            return;
        }
    }

    //---------------------------------------------------------------------------
    private boolean forward(final String sUrl, final HttpServletRequest request, final HttpServletResponse response) throws ServletException, IOException
    {
        trace("forward(" + sUrl + ")...");
        RequestDispatcher rd = request.getRequestDispatcher(sUrl);
        if (rd != null)
        {
            rd.forward(request, response);
            return true;
        }
        return false;
    }

    //---------------------------------------------------------------------------
    private void response_error(HttpServletResponse response, String sMsg) throws ServletException, IOException
    {
        trace("response_error()...");
        try
        {
            response.setContentType("text/html");
            // Servlet output stream
            PrintWriter out = response.getWriter();
            if (out == null)
            {
                error("Output stream is null ");
                return;
            }
            out.println("<html>");
            out.println("<body>");
            out.println("<h2> ERROR : " + sMsg + "</h2>");
            out.println("</body>");
            out.println("</html>");
            out.close();
        } catch (Exception e)
        {
            error("Exception : " + e.getMessage());
        }
    }
}