package org.objectweb.telosys.uil.taglib;

import java.io.IOException;

import javax.servlet.RequestDispatcher;
import javax.servlet.ServletContext;
import javax.servlet.ServletException;
import javax.servlet.ServletRequest;
import javax.servlet.ServletResponse;
import javax.servlet.jsp.JspWriter;
import javax.servlet.jsp.tagext.TagSupport;

import org.objectweb.telosys.uil.TelosysUIL;
import org.objectweb.telosys.uil.screenmap.ScreenDefConfig;
import org.objectweb.telosys.uil.screenmap.ScreenDefinitions;
import org.objectweb.telosys.uil.screenmap.ScreenMap;
import org.objectweb.telosys.uil.screenmap.ScreenMapConst;

/**
 * @author Laurent GUERIN
 * 
 */
// /* package */class CommonTagSupport extends TagSupport
public class CommonTagSupport extends TagSupport
{
    protected final static boolean SILENT    = true ;

    protected final static boolean NOT_SILENT = false ;

    //----------------------------------------------------------------
    protected void print(JspWriter out, String sMsg)
    {
        if (out != null && sMsg != null)
        {
            try
            {
                out.print(sMsg);
            } catch (IOException ioex)
            {
                // Nothing to do
            }
        }
    }
    //----------------------------------------------------------------
    protected void println(JspWriter out, String sMsg)
    {
        if (out != null && sMsg != null)
        {
            try
            {
                out.println(sMsg);
            } catch (IOException ioex)
            {
                // Nothing to do
            }
        }
    }

    //----------------------------------------------------------------
    //----------------------------------------------------------------
    protected void tagLibError ( JspWriter out, String sMsg )
    {
        println(out, "");
        println(out, "<!-- ### TAGLIB ERROR : " + sMsg + " ### -->");        
    }
    //----------------------------------------------------------------
    protected void tagLibError ( String sMsg )
    {
        tagLibError ( pageContext.getOut(), sMsg );
    }
    
    //----------------------------------------------------------------
    //----------------------------------------------------------------
    protected void tagLibMessage ( JspWriter out, String sMsg )
    {
        println(out, "");
        println(out, "<!-- ### TAGLIB MESSAGE : " + sMsg + " ### --> ");
    }
    //----------------------------------------------------------------
    protected void tagLibMessage ( String sMsg )
    {
        tagLibMessage(pageContext.getOut(), sMsg );
    }

    //----------------------------------------------------------------
    // Exemple : "http://localhost:8080/contextname"
    protected String getRootContextURL()
    {
        /***
        HttpServletRequest request = (HttpServletRequest) pageContext.getRequest();
        if (request == null)
        {
            return "";
        }

        //        String sURL = request.getRequestURL().toString();
        //        String sURI = request.getRequestURI();
        //        if (sURL == null || sURI == null)
        //        {
        //            return null;
        //        }
        //        int i = sURL.indexOf(sURI);
        //        return sURL.substring(0, i) + request.getContextPath();

        String sProtocol   = request.getScheme();
        String sHostName   = request.getServerName();
        int    iHostPort   = request.getServerPort();
        String sContexPath = request.getContextPath();
        if (sProtocol == null || sHostName == null || sContexPath == null )
        {
            return "";
        }
        
        return sProtocol + "://" + sHostName + ":" + iHostPort + sContexPath ;
        ***/
        return TagCommons.getRootContextURL(pageContext);
    }
    //----------------------------------------------------------------
    // Exemple : "http://localhost:8080/contextname/" + sPath
    protected String getFullHttpURL(String sPath)
    {
        String sRootContextURL = getRootContextURL();
        if ( sRootContextURL == null )
        {
            tagLibError("Cannot get Context URL");
            return null ;
        }
        
        String sRoot2 = sRootContextURL ;
        if ( sRootContextURL.endsWith("/") )
        {
            //--- Cut the last char
            sRoot2 = sRootContextURL.substring(0, sRootContextURL.length()-1 ) ;
        }

        if ( sPath != null )
        {
            String sPath2 = sPath ;
            if ( sPath.startsWith("/") )
            {
                //--- Cut the first char
                sPath2 = sPath.substring(1); 
            }
            return sRoot2 + "/" + sPath2 ;                                
        }
        else
        {
            return sRoot2 + "/" ;
        }        
    }

    //----------------------------------------------------------------
    protected ScreenDefConfig getConfig()
    {
        ScreenDefinitions screenDefinitions = TelosysUIL.getScreenDefinitions();
        if (screenDefinitions == null)
        {
            tagLibError( "Cannot get 'ScreenDefinitions' from 'TelosysUIL' ");
            return null;
        }
        ScreenDefConfig config = screenDefinitions.getConfig();
        if (config == null)
        {
            tagLibError( "Cannot get config from 'ScreenDefinitions' ");
        }
        return config ;
    }

    //----------------------------------------------------------------
    protected ScreenMap getScreenMap()
    {
        return getScreenMap(NOT_SILENT);
    }
    //----------------------------------------------------------------
    protected ScreenMap getScreenMap(boolean bSilentMode )
    {
        ServletRequest request = pageContext.getRequest();
        if (request == null)
        {
            if ( bSilentMode != SILENT )
            {
                tagLibError( "Cannot get the ServletRequest object from the pageContext");                
            }
            return null;
        }

        ScreenMap screen = (ScreenMap) request.getAttribute(ScreenMapConst.SCREEN_MAP);
        if (screen == null)
        {
            if ( bSilentMode != SILENT )
            {
                tagLibMessage( "Cannot get the ScreenMap object from the request");
            }
            return null;
        }
        return screen;
    }

    //----------------------------------------------------------------
    protected boolean include(String sURL)
    {
        JspWriter out = pageContext.getOut();

        ServletRequest request = pageContext.getRequest();
        if (request == null)
        {
            tagLibError("Cannot get the ServletRequest object from the pageContext");
            return false;
        }

        ServletResponse response = pageContext.getResponse();
        if (response == null)
        {
            tagLibError("Cannot get the ServletResponse object from the pageContext");
            return false;
        }

        ServletContext ctx = pageContext.getServletContext();
        if (ctx == null)
        {
            tagLibError("Cannot get the ServletContext object from the pageContext");
            return false;
        }

        RequestDispatcher rd = ctx.getRequestDispatcher(sURL);
        if (rd == null)
        {
            tagLibError("Cannot get the RequestDispatcher object from the ServletContext");
            return false;
        }

        boolean bRet = false;
        try
        {
            out.flush();
            rd.include(request, response);
            out.flush();
            bRet = true;
        } catch (ServletException e)
        {
            println(out, "");
            println(out, "<!-- INCLUDE ERROR : ServletException -->");
            println(out, "<!-- Message  = " + e.getMessage() + " -->");
            println(out, "<!-- toString = " + e.toString() + " -->");
            //e.printStackTrace();
        } catch (IOException e)
        {
            println(out, "");
            println(out, "<!-- INCLUDE ERROR : IOException -->");
            println(out, "<!-- Message  = " + e.getMessage() + " -->");
            println(out, "<!-- toString = " + e.toString() + " -->");
        }
        return bRet;
    }
}