package org.objectweb.telosys.uil.taglib;

import javax.servlet.ServletContext;
import javax.servlet.ServletRequest;
import javax.servlet.http.HttpSession;
import javax.servlet.jsp.PageContext;

import org.objectweb.telosys.common.TelosysConst;
import org.objectweb.telosys.common.TelosysException;
import org.objectweb.telosys.common.TelosysRuntimeException;

/**
 * Static methods to find a bean with a symbolic name
 * 
 * @author Laurent GUERIN
 *  
 */
public class Finder
{
    /**
     * Try to find an object with the given name in the given scope
     * 
     * @param pageContext
     * @param sName
     * @param sScope
     * @return
     */
    public static Object find(PageContext pageContext, String sName, String sScope) throws TelosysException
    {
        if (pageContext != null)
        {
            String sBeanName = sName.trim();
            if (sBeanName.equalsIgnoreCase("RENDERER-BEAN") || sBeanName.equalsIgnoreCase("RENDERER_BEAN"))
            {
                return findInRequest(pageContext, TelosysConst.RENDERER_BEAN);
            }
            else
            {
                if (sScope != null)
                {
                    return findWithScope(pageContext, sBeanName, sScope.trim());
                }
                else
                {
                    return findWithoutScope(pageContext, sBeanName);
                }
            }
        }
        else
        {
            throw new TelosysRuntimeException("Finder : cannot get ServletRequest");
        }
    }

    public static Object findWithScope(PageContext pageContext, String sBeanName, String sScope) throws TelosysException
    {
        if (sScope.equals("request"))
        {
            return findInRequest(pageContext, sBeanName);
        }
        else if (sScope.equals("session"))
        {
            return findInSession(pageContext, sBeanName);
        }
        else if (sScope.equals("application"))
        {
            return findInApplication(pageContext, sBeanName);
        }
        else if (sScope.equals("screen"))
        {
            // TODO
            // 1) "screen-data"
            // 2) "screen-attributes"
            return null ;
        }
        else if (sScope.equals("screen-data"))
        {
            // TODO 
            // findInScreenData(pageContext, sBeanName);
            return null ;
        }
        else if (sScope.equals("screen-attributes"))
        {
            // TODO
            // findInScreenAttributes(pageContext, sBeanName);
            return null ;
        }
        else if (sScope.equals("screen-session"))
        {
            // TODO
            return null ;
        }
        else if (sScope.equals("screen-application"))
        {
            // TODO
            return null ;
        }
        else
        {
            // Invalid scope
            throw new TelosysException("Invalid scope '" + sScope + "'");
        }
    }

    public static Object findWithoutScope(PageContext pageContext, String sBeanName)
    {
        Object oBean = null;

        //--- 1) Search in the screen context
        // TODO

        //--- Search in "request" scope
        if ((oBean = findInRequest(pageContext, sBeanName)) != null)
        {
            return oBean;
        }

        //--- Search in the screen session
        // TODO

        //--- Search in "session" scope
        if ((oBean = findInSession(pageContext, sBeanName)) != null)
        {
            return oBean;
        }

        //--- Search in "application" scope
        if ((oBean = findInApplication(pageContext, sBeanName)) != null)
        {
            return oBean;
        }
        //--- Not found
        return null;
    }

    public static Object findInRequest(PageContext pageContext, String sBeanName)
    {
        ServletRequest servletRequest = pageContext.getRequest();
        if (servletRequest != null)
        {
            return servletRequest.getAttribute(sBeanName);
        }
        else
        {
            throw new TelosysRuntimeException("Finder (request) : cannot get ServletRequest");
        }
    }

    public static Object findInSession(PageContext pageContext, String sBeanName)
    {
        HttpSession httpSession = pageContext.getSession();
        if (httpSession != null)
        {
            return httpSession.getAttribute(sBeanName);
        }
        else
        {
            throw new TelosysRuntimeException("Finder (session) : cannot get HttpSession");
        }
    }

    public static Object findInApplication(PageContext pageContext, String sBeanName)
    {
        ServletContext servletContext = pageContext.getServletContext();
        if (servletContext != null)
        {
            return servletContext.getAttribute(sBeanName);
        }
        else
        {
            throw new TelosysRuntimeException("Finder (application) : cannot get ServletContext");
        }
    }

}