package org.objectweb.telosys.uil.taglib;

import java.io.IOException;

import javax.servlet.ServletRequest;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.jsp.JspWriter;
import javax.servlet.jsp.PageContext;

import org.objectweb.telosys.common.TelosysClassLogger;
import org.objectweb.telosys.uil.TelosysUIL;
import org.objectweb.telosys.uil.screenmap.ScreenDefConfig;
import org.objectweb.telosys.uil.screenmap.ScreenDefRootDir;
import org.objectweb.telosys.uil.screenmap.ScreenDefinitions;
import org.objectweb.telosys.uil.screenmap.ScreenMap;
import org.objectweb.telosys.uil.screenmap.ScreenMapConst;
import org.objectweb.telosys.uil.taglib.widget.TagScreenEnv;

/**
 * @author Laurent GUERIN
 *  
 */
public class TagCommons
{
    private final static String             TELOSYS_SCREEN_ENV = "TELOSYS_SCREEN_ENV";

    private static final TelosysClassLogger log                = new TelosysClassLogger(TagCommons.class);

    //    public static String styleAlign(String align)
    //    {
    //        if ("l".equals(align))
    //            align = "left";
    //        else if ("c".equals(align))
    //            align = "center";
    //        else if ("r".equals(align))
    //            align = "right";
    //
    //        return "text-align:" + align + ";";
    //    }

    //    public static String styleXY(String x, String y, PageContext p)
    //    {
    //        String res = "";
    //        if (x == null && y == null) // No (X,Y) coord.
    //        {
    //            return "";
    //        }
    //        else
    //        {
    //            if (isHTML(p))
    //                res = " position:absolute;";
    //            if (x != null)
    //                res += "left:" + x.trim() + "px;";
    //            if (y != null)
    //                res += "top:" + y.trim() + "px; ";
    //        }
    //        return res;
    //    }

    //    public static String styleXYHW(String x, String y, String h, String w, PageContext p)
    //    {
    //        String res = "";
    //        if (x != null || y != null || h != null || w != null)
    //        {
    //            if (isHTML(p))
    //                res = " position:absolute;";
    //            if (x != null)
    //                res += "left:" + new Integer(x).toString() + "px;";
    //            if (y != null)
    //                res += "top:" + new Integer(y).toString() + "px; ";
    //            if (h != null)
    //                res += "height:" + new Integer(h).toString() + "px; ";
    //            if (w != null)
    //                res += "width:" + new Integer(w).toString() + "px; ";
    //        }
    //        return res;
    //
    //    }

    //    public static String styleHW(String h, String w)
    //    {
    //        String res = "";
    //        if (h != null || w != null)
    //        {
    //            if (h != null)
    //                res += "height:" + new Integer(h).toString() + "px; ";
    //            if (w != null)
    //                res += "width:" + new Integer(w).toString() + "px; ";
    //        }
    //        return res;
    //    }

    //    public static boolean isComboParentValue(GenericTag tag) throws TelosysException
    //    {
    //        String sTagValue = null ;
    //        if ( tag instanceof ComboItem )
    //        {
    //            ComboItem comboItem = (ComboItem) tag;
    //            sTagValue = comboItem.getValue();
    //        }
    //        else
    //        {
    //            throw new TelosysException("isComboParentValue() : tag is not a ComboItem ");
    //        }
    //        Tag tagParent = tag.getParent();
    //        if (tagParent != null)
    //        {
    //            if (tagParent instanceof ComboBox )
    //            {
    //                ComboBox comboBox = (ComboBox) tagParent;
    //                String sParentValue = comboBox.getValue() ;
    //                if ( sParentValue != null && sTagValue != null )
    //                {
    //                    return sParentValue.equals(sTagValue) ;
    //                }
    //            }
    //            else
    //            {
    //                throw new TelosysException("isComboParentValue() : parent tag is not a ComboBox ");
    //            }
    //        }
    //        else
    //        {
    //            throw new TelosysException("isComboParentValue() : no parent tag for this ComboItem ");
    //        }
    //        return false ;
    //    }

    /**
     * Create a screen environment for the current page context
     * 
     * @param pageContext
     * @return
     */
    public static TagScreenEnv initScreenEnv(PageContext pageContext)
    {
        //System.out.println("TagCommons : initScreenEnv " );
        //TagScreenEnv screenEnv = new TagScreenEnv(pageContext);
        TagScreenEnv screenEnv = new TagScreenEnv();
        pageContext.setAttribute(TELOSYS_SCREEN_ENV, screenEnv);
        return screenEnv;
    }

    /**
     * Returns the screen map instance associated with the current page <br>
     * The screen map can be null if the JSP is invoked directly (without the screenmap servlet)
     * 
     * @param pageContext
     * @return the screen map (or null)
     */
    private static ScreenMap getScreenMap(PageContext pageContext)
    {
        ScreenMap screenMap = null;
        ServletRequest request = pageContext.getRequest();
        if (request != null)
        {
            screenMap = (ScreenMap) request.getAttribute(ScreenMapConst.SCREEN_MAP);
            if (screenMap != null)
            {
                return screenMap;
            }
            // ScreenMap not found : It's not an error => do not log 
//            else
//            {                
//                log.error("getScreenMap() : Screen map attribute ('" + ScreenMapConst.SCREEN_MAP + "') not found in request ");
//            }
        }
        else
        {
            log.error("getScreenMap() : Cannot get request object from 'pageContext' ");
        }
        return null;
    }

    /**
     * Return the screen environment for the current page context
     * 
     * @param pageContext
     * @return
     */
    public static TagScreenEnv getScreenEnv(PageContext pageContext)
    {
        TagScreenEnv screenEnv = (TagScreenEnv) pageContext.getAttribute(TELOSYS_SCREEN_ENV);
        if (screenEnv == null)
        {
            screenEnv = initScreenEnv(pageContext);
        }
        return screenEnv;
    }

    /**
     * Return the screen map type
     * 
     * @param p
     * @return true if HTML, else (XUL) false
     */
    public static boolean isHTML(PageContext p)
    {
        //        TagScreenEnv screenEnv = getScreenEnv(p);
        //        return screenEnv.isHTML();
        ScreenMap screenMap = getScreenMap(p);
        if (screenMap != null)
        {
            return screenMap.isHtmlType();
        }
        return true; // HTML by default
    }

    /**
     * Returns the full URL for the context root <br>( ie : "http://server:port/contextname/" )
     * 
     * @param p
     * @return
     */
    public static String getRootContextURL(PageContext pageContext)
    {
        HttpServletRequest request = (HttpServletRequest) pageContext.getRequest();
        if (request == null)
        {
            log.error("getRootContextURL() : request is null ");
            return "xxx(error_1)xxx";
        }

        String sProtocol = request.getScheme();
        if (sProtocol == null)
        {
            log.error("getRootContextURL() : cannot get protocol ");
            return "xxx(error_2)xxx";
        }
        String sHostName = request.getServerName();
        if (sHostName == null)
        {
            log.error("getRootContextURL() : cannot get server name ");
            return "xxx(error_3)xxx";
        }
        String sContexPath = request.getContextPath();
        if (sContexPath == null)
        {
            log.error("getRootContextURL() : cannot get context path");
            return "xxx(error_4)xxx";
        }
        int iHostPort = request.getServerPort();

        //--- Build the full http URL
        return sProtocol + "://" + sHostName + ":" + iHostPort + sContexPath;

    }

    //----------------------------------------------------------------
    /**
     * Returns the full "client side" http URL for the given resource path
     * 
     * @param pageContext
     * @param sResourcePath :
     *            the server side resource path ( ie : "/folder/file" )
     * @return : the full URL ( ie : "http://myhost:8080/contextname/folder/file")
     */
    public static String getFullHttpURL(PageContext pageContext, String sResourcePath)
    {
        String sRootContextURL = getRootContextURL(pageContext);
        if (sRootContextURL == null)
        {
            log.error("getFullHttpURL() : cannot get root context URL ");
            return null;
        }

        String sRoot2 = sRootContextURL;
        if (sRootContextURL.endsWith("/"))
        {
            //--- Cut the last char
            sRoot2 = sRootContextURL.substring(0, sRootContextURL.length() - 1);
        }

        if (sResourcePath != null)
        {
            String sPath2 = sResourcePath;
            if (sResourcePath.startsWith("/"))
            {
                //--- Cut the first char
                sPath2 = sResourcePath.substring(1);
            }
            return sRoot2 + "/" + sPath2;
        }
        else
        {
            return sRoot2 + "/";
        }
    }

    //----------------------------------------------------------------
    /**
     * Returns the global configuration of the screens definitions <br>( cf "config" tag in the screens XML file )
     * 
     * @return : the screens definition congig object
     */
    public static ScreenDefConfig getScreenDefConfig()
    {
        ScreenDefinitions screenDefinitions = TelosysUIL.getScreenDefinitions();
        if (screenDefinitions == null)
        {
            log.error("getConfig() : Cannot get 'ScreenDefinitions' from 'TelosysUIL' ");
            return null;
        }
        return screenDefinitions.getConfig();
    }

    //----------------------------------------------------------------
    /**
     * Returns the "server side" screen map servlet path <br>
     * defined in the "config" tag of the screens XML file
     * 
     * @return
     */
    public static String getScreenMapResourcePath()
    {
        ScreenDefConfig config = getScreenDefConfig();
        if (config != null)
        {
            return config.getScreenMapUrl();
        }
        else
        {
            return null;
        }
    }

    /**
     * Returns the "server side" screen action servlet path <br>
     * defined in the "config" tag of the screens XML file
     * 
     * @return
     */
    public static String getScreenActionResourcePath()
    {
        ScreenDefConfig config = getScreenDefConfig();
        if (config != null)
        {
            return config.getAjaxRequestUrl();
        }
        else
        {
            return null;
        }
    }

    /**
     * Returns the full "client side" URL of the screen map servlet
     * 
     * @param pageContext
     * @return : the URL ( ie : "http://myhost:8080/contextname/screenmap" )
     */
    public static String getScreenMapURL(PageContext pageContext)
    {
        String sServerSideScreenMapPath = TagCommons.getScreenMapResourcePath();
        return getFullHttpURL(pageContext, sServerSideScreenMapPath);
    }

    /**
     * Returns the full "client side" URL of the AJAX screen actions servlet
     * 
     * @param pageContext
     * @return : the URL ( ie : "http://myhost:8080/contextname/screenaction" )
     */
    public static String getScreenActionURL(PageContext pageContext)
    {
        String sServerSideScreenActionPath = TagCommons.getScreenActionResourcePath();
        return getFullHttpURL(pageContext, sServerSideScreenActionPath);
    }

    /**
     * Returns the full URL for the images directory <br>
     * ie : "http://server:port/contextname/images_dir/"
     * 
     * @param pageContext
     * @return
     */
    public static String getImgDirURL(PageContext pageContext)
    {
        return getRootContextURL(pageContext) + getImgDir(pageContext) + "/";
    }

    /**
     * Returns the images directory for the current type of screen map <br>
     * or for the default screen type if there's no screen map (direct call to JSP)
     * 
     * @param p
     * @return
     */
    public static String getImgDir(PageContext p)
    {
        ScreenMap screenMap = getScreenMap(p);
        if (screenMap != null)
        {
            //--- There's a ScreenMap object associated with the request => use it
            String sImgRootDir = screenMap.getImagesRootDir();
            if (sImgRootDir != null)
            {
                return sImgRootDir;
            }
            else
            {
                log.error("Images root dir not set for type " + screenMap.getType());
                return "xxx(error_1)xxx";
            }
        }
        else
        {
            //--- No ScreenMap object associated with the request => use default type image directory
            ScreenDefinitions screenDefinitions = TelosysUIL.getScreenDefinitions();
            if (screenDefinitions != null)
            {
                //--- Get the "root dir" definitions for the default type
                String sScreenType = TelosysUIL.getDefaultScreenType();
                ScreenDefRootDir rootDir = screenDefinitions.getRootDir(sScreenType);
                String sImgRootDir = rootDir.getImagesRootDir();
                if (sImgRootDir != null)
                {
                    return sImgRootDir;
                }
                else
                {
                    log.error("Images root dir not set for the default type " + sScreenType);
                    return "xxx(error_2)xxx";
                }
            }
            else
            {
                log.error("ScreenDefinitions not initialized !");
                return "xxx(error_3)xxx";
            }
        }
    }

    /**
     * Returns true if the postback argument is "session"
     * 
     * @param sPostback
     * @return
     */
    public static boolean isSessionPostback(String sPostback)
    {
        if (sPostback == null)
        {
            return false;
        }
        else
        {
            return "session".equalsIgnoreCase(sPostback.trim());
        }
    }

    /**
     * Returns true if the postback argument is "request"
     * 
     * @param sPostback
     * @return
     */
    public static boolean isRequestPostback(String sPostback)
    {
        if (sPostback == null)
        {
            return false;
        }
        else
        {
            return "request".equalsIgnoreCase(sPostback.trim());
        }
    }

    /**
     * Returns true if the postback argument is "none"
     * 
     * @param sPostback
     * @return
     */
    public static boolean isNoPostback(String sPostback)
    {
        if (sPostback == null)
        {
            return false;
        }
        else
        {
            return "none".equalsIgnoreCase(sPostback.trim());
        }
    }

    /**
     * Prints a trace message (as a comment tag) in the generated page <br>
     * 
     * @param out
     * @param sMessage
     */
    public static void printTrace(JspWriter out, String sMessage)
    {
        try
        {
            out.println("<!-- TRACE : " + sMessage + " -->");
        } catch (IOException ioex)
        {
            ioex.printStackTrace();
        }
    }

    /**
     * Prints a trace message (as a comment tag) in the generated page <br>
     * 
     * @param pageContext
     * @param sMessage
     */
    public static void printTrace(PageContext pageContext, String sMessage)
    {
        printTrace(pageContext.getOut(), sMessage);
    }

    /**
     * Prints an error message (as a comment tag) in the generated page <br>
     * 
     * @param out
     * @param sMessage
     */
    public static void printError(JspWriter out, String sMessage)
    {
        try
        {
            out.println("<!-- ERROR : " + sMessage + " -->");
        } catch (IOException ioex)
        {
            ioex.printStackTrace();
        }
    }

    /**
     * Prints an error message (as a comment tag) in the generated page <br>
     * 
     * @param pageContext
     * @param sMessage
     */
    public static void printError(PageContext pageContext, String sMessage)
    {
        printError(pageContext.getOut(), sMessage);
    }

}