package org.objectweb.telosys.uil.taglib.util;

import java.util.Collection;
import java.util.Iterator;

import org.objectweb.telosys.common.TelosysException;
import org.objectweb.telosys.common.data.DataSet;
import org.objectweb.telosys.common.vo.GenericVOList;
import org.objectweb.telosys.uil.taglib.CommonTagSupport;
import org.objectweb.telosys.uil.taglib.Finder;
import org.objectweb.telosys.uil.taglib.Page;

/**
 * @author Laurent GUERIN
 *  
 */
public class Iterate extends CommonTagSupport // TagSupport
{
    //----------------------------------------------------------------
    // Private variables
    //----------------------------------------------------------------
    private int      _iCount   = 0;

    private int      _iIndex   = 0;

    private Iterator _iterator = null;

    //----------------------------------------------------------------
    // Tag attributes
    //----------------------------------------------------------------
    private String   _sList    = null;

    private String   _sScope   = null;

    public void setList(String v)
    {
        this._sList = v;
    }

    public void setScope(String v)
    {
        this._sScope = v;
    }

    //----------------------------------------------------------------
    private Iterator getIterator()
    {
        if (_sList == null)
        {
            tagLibMessage("No list attribute : cannot get iterator ");
            return null;
        }
        else
        {
            Object oList = null;
            try
            {
                oList = Finder.find(pageContext, _sList, _sScope);
            } catch (TelosysException e)
            {
                tagLibMessage("Cannot get list bean : RuntimeException : " + e.getMessage());
            }

            //            ServletRequest request = pageContext.getRequest();
            //            Object oList = request.getAttribute(_sList);
            if (oList != null)
            {
                if (oList instanceof GenericVOList)
                {
                    return ((GenericVOList) oList).iterator();
                }
                else if (oList instanceof DataSet)
                {
                    return ((DataSet) oList).iterator();
                }
                else if (oList instanceof Collection) 
                {
                    return ((Collection) oList).iterator();
                }
                else
                {
                    tagLibMessage("Invalid list type : class = " + oList.getClass().getName());
                }
            }
            else
            {
                tagLibMessage("List attribute '" + _sList + "' not found");
            }
        }
        return null;
    }

    /**
     * Get the next item and set it in the page context
     * 
     * @return true if there's a next item, else false
     */
    private boolean nextItem()
    {
        if (_iterator != null)
        {
            if (_iterator.hasNext())
            {
                _iCount++; // 1 to N
                Page.setIterationCount(pageContext, _iCount);
                Page.setIterationIndex(pageContext, _iIndex);
                Page.setCurrentBean(pageContext, _iterator.next());
                _iIndex++; // 0 to N
                return true;
            }
        }
        return false;
    }

    //----------------------------------------------------------------
    //----------------------------------------------------------------
    public int doStartTag()
    {
        _iCount = 0;
        _iIndex = 0;
        _iterator = getIterator();
        if (nextItem())
        {
            return EVAL_BODY_INCLUDE;
        }
        return SKIP_BODY;
    }

    //----------------------------------------------------------------
    public int doAfterBody()
    {
        if (nextItem())
        {
            return EVAL_BODY_AGAIN;
        }
        return SKIP_BODY;
    }
    //----------------------------------------------------------------
}