package org.objectweb.telosys.uil.taglib.util;

import javax.servlet.jsp.JspWriter;

import org.objectweb.telosys.common.TelosysException;
import org.objectweb.telosys.common.data.DataRow;
import org.objectweb.telosys.uil.taglib.CommonTagSupport;
import org.objectweb.telosys.uil.taglib.Page;
import org.objectweb.telosys.util.BeanUtil;
import org.objectweb.telosys.util.ObjUtil;

/**
 * @author Laurent GUERIN
 *  
 */
public class Out extends CommonTagSupport // TagSupport
{
    //----------------------------------------------------------------
    // Private variables
    //----------------------------------------------------------------

    //----------------------------------------------------------------
    // Tag attributes
    //----------------------------------------------------------------
    private String             _sData = null ;

    private String             _sCol = null ;

    private String             _sDefault = null ;

    private String             _sFormat = null ;

    public void setData(String v)
    {
        if ( v!= null )
        {
            this._sData = v.trim() ;            
        }
        else
        {
            this._sData = null ;
        }
    }

    public void setCol(String v)
    {
        if ( v!= null )
        {
            this._sCol = v.trim() ;            
        }
        else
        {
            this._sCol = null ;
        }
    }

    public void setDefault(String v)
    {
        this._sDefault = v ; // keep as is            
    }

    public void setFormat(String v)
    {
        this._sFormat = v ; // keep as is            
    }

    //----------------------------------------------------------------
    private String format( Object oValue, String sFormat )
    {
        return ObjUtil.format(oValue, sFormat);
    }
    //----------------------------------------------------------------
    private void printObjectValue( JspWriter out, Object oValue )
    {
        if ( oValue != null )
        {
            if ( _sFormat != null )
            {
                print(out, format (oValue, _sFormat) );
            }
            else
            {
                print(out, oValue.toString());  // No format
            }            
        }
        else
        {
            if ( _sDefault != null )
            {
                print(out, _sDefault);
            }
            else
            {
                print(out, "");                                                
            }
        }        
    }
    
    //----------------------------------------------------------------
    private void printFieldValue( JspWriter out, Object oBean, String sFieldName )
    {        
        if (oBean == null)
        {
            print(out, "(no bean)");
        }
        if (sFieldName == null)
        {
            print(out, "(no field name)");
        }
        
        try
        {
            Object value = BeanUtil.getFieldValue(oBean, sFieldName );
            printObjectValue(out, value);
//            if ( value != null )
//            {
//                print(out, value.toString());  // TODO : format(value)              
//            }
//            else
//            {
//                print(out, ""); // TODO : default value ?                               
//            }
        } catch (TelosysException e)
        {
            print(out, "(unknown field "+sFieldName+")");
            }
    }
    
    private void printData(JspWriter out)
    {
        Object oBean = null;
        String sFieldName = null ;
        int iDotIndex = _sData.indexOf('.') ;
        if ( iDotIndex > 0 )
        {
            String[] tokens = _sData.split("\\.");
            if ( tokens.length == 2 )
            {
                //--- data="bean.property"
//                String sBeanName  = tokens[0] ;                
//                sFieldName = tokens[1] ;                
//                oBean = null ; // TODO Page.getBean(pageContext, sBean); ????
            }
            else
            {
                //--- data="aaa.bbb.ccc.xxx"
                print(out, "(data='" + _sData + "':invalid)" );
            }
        }
        else
        {
            // No '.' or '.' is at the first position => no bean name
            if ( iDotIndex == 0 ) 
            {
                //--- Remove the '.' at the 1rst position
                sFieldName = _sData.substring(1) ;                
            }
            else
            {
                sFieldName = _sData ;
            }
            oBean = Page.getCurrentBean(pageContext);
            printFieldValue(out, oBean, sFieldName);
        }
    }

    private int getColumn(String sCol, DataRow dr)
    {
        int iCol = 0 ;
        try
        {
            iCol = Integer.parseInt(_sCol);
            if ( iCol <= 0 )
            {
                return -1 ;
            }
            if ( iCol > dr.getSize() )
            {
                return -2 ;
            }
        } catch (NumberFormatException e)
        {
            return -3 ;
        }
        return iCol ;
    }
    
    private void printColumn(JspWriter out)
    {
        Object oBean = Page.getCurrentBean(pageContext);
        if ( oBean instanceof DataRow )
        {
            DataRow dr = (DataRow) oBean ;
            int iCol = getColumn(_sCol, dr) ;
            if ( iCol > 0 && iCol <= dr.getSize() )
            {
                printObjectValue(out, dr.getObject(iCol));
            }
            else
            {
                print(out, "(col='" + _sCol+"':invalid)" );                
            }
        }
        else
        {
            print(out, "(no DataRow)" );
        }
    }
    
    //----------------------------------------------------------------
    // Tag open
    //----------------------------------------------------------------
    public int doStartTag()
    {
        JspWriter out = pageContext.getOut();

        if ( _sData != null )
        {
            printData(out);            
        }
        else if ( _sCol != null )
        {
            printColumn(out);
        }
        else
        {
            print(out, "(?)");            
        }
        
        return (SKIP_BODY);
    }

    //----------------------------------------------------------------
}