package org.objectweb.telosys.uil.taglib.widget;

import org.objectweb.telosys.uil.taglib.widget.html.CheckBoxHTML;
import org.objectweb.telosys.uil.taglib.widget.xul.CheckBoxXUL;

/**
 * @author Laurent GUERIN
 *  
 */
public class CheckBox extends GenericPostbackTag // GenericTag 
{
    private static final String  TAG_NAME   = "checkbox";

    //----------------------------------------------------------------
    // TAG generators ( thread safe => static )
    //----------------------------------------------------------------
    private final static IWidget $htmlGen   = new CheckBoxHTML();

    private final static IWidget $xulGen    = new CheckBoxXUL();

    //----------------------------------------------------------------
    // TAG attributes
    //----------------------------------------------------------------
    private String               _sChecked  = "false";

    //private String               _sValue    = "1";

    private String               _sValueOff = "0";

    private String               _sTxt      = null;

    private String               _sTxtPos   = "right";           // "r", "right", "l", "left"

    /**
     * Constructor
     */
    public CheckBox()
    {
        super(TAG_NAME, $htmlGen, $xulGen);
        setIdGeneration(true); // Automatic ID generation (if ID not set by user)
    }

    //----------------------------------------------------------------
    public String getValue() // Overrides the standard method to use POSTBACK
    {
        trace("getValue()...");
        String sValue = getValueWithPostback();
        trace("getValue() : return " + sValue );
        return sValue;
    }

    //----------------------------------------------------------------
    public void setValueoff(String v)
    {
        _sValueOff = v;
    }

    public String getValueoff()
    {
        return _sValueOff;
    }

    //----------------------------------------------------------------
    public void setTxt(String v)
    {
        _sTxt = v;
    }

    public String getTxt()
    {
        return _sTxt;
    }

    //----------------------------------------------------------------
    public void setTxtpos(String v)
    {
        _sTxtPos = v;
    }

    public String getTxtpos()
    {
        return _sTxtPos;
    }
    //----------------------------------------------------------------
    public void setChecked(String v)
    {
        _sChecked = v;
    }

    public void setChecked(boolean bFlag)
    {
        _sChecked = (bFlag == true ? "true" : "false" ) ;
    }

    public String getChecked()
    {
        return _sChecked;
    }

    public boolean isChecked()
    {
        //--- If postback is enabled AND there's is already a ScreenPostback in request or session scope
        if ( isPostbackEnabled() && ( getScreenPostback() != null ) ) 
        {
            if ( getPostbackValue() != null )
            {
                //--- There is a postback value for this checkbox => transmitted => checked
                return true ;
            }
            else
            {
                return false ;
            }
        }
        else
        {
            //--- The postback management is disabled or not yet initialized => Just use the "checked" attribute 
            if (_sChecked != null)
            {
                if (_sChecked.trim().equalsIgnoreCase("true"))
                {
                    return true;
                }
            }
            return false ;
        }
        
//        if (_sChecked != null)
//        {
//            if (_sChecked.trim().equalsIgnoreCase("true"))
//            {
//                return true;
//            }
//        }
//        return false;
    }


    //----------------------------------------------------------------
    /*
     * (non-Javadoc)
     * 
     * @see javax.servlet.jsp.tagext.Tag#doStartTag()
     */
    public int doStartTag()
    {
        startTag();
        return (SKIP_BODY);
    }
    
    /* (non-Javadoc)
     * @see javax.servlet.jsp.tagext.Tag#doEndTag()
     */
    public int doEndTag()
    {
        endTag();
        return (EVAL_PAGE); //  EVAL_PAGE = continue , SKIP_PAGE = abort the processing
    }

}