package org.objectweb.telosys.uil.taglib.widget;

import org.objectweb.telosys.uil.taglib.widget.html.FieldHTML;
import org.objectweb.telosys.uil.taglib.widget.xul.FieldXUL;

/**
 * @author Laurent GUERIN 
 *  
 */
public class Field extends GenericPostbackTag // GenericTag
{
    private static final String  TAG_NAME       = "field";

    //----------------------------------------------------------------
    // Internal field type
    //----------------------------------------------------------------
    public static final int      TYPE_INVALID   = -1;

    public static final int      TYPE_UNDEFINED = 0;

    public static final int      TYPE_INT       = 1;

    public static final int      TYPE_NUM       = 2;

    public static final int      TYPE_DATE      = 3;

    public static final int      TYPE_ALPHA     = 4;

    public static final int      TYPE_ALPHANUM  = 5;

    public static final int      TYPE_SELECT    = 6;

    //----------------------------------------------------------------
    // TAG generators ( thread safe => static )
    //----------------------------------------------------------------
    private final static IWidget $htmlGen       = new FieldHTML();

    private final static IWidget $xulGen        = new FieldXUL();

    //----------------------------------------------------------------
    // TAG attributes
    //----------------------------------------------------------------

    //private String _sValue = null;

    private String               _sMaxLength    = null;

    private String               _sSize         = null;

    //    private boolean _bDisabled = false;

    private boolean              _bReadOnly     = false;

    private boolean              _bRequired     = false;

    private boolean              _bPassword     = false;

    private String               _sType         = null;

    private String               _sFormat       = null;

    private String               _sMin          = null;

    private String               _sMax          = null;

    private String               _sRegexp       = null;

    private String               _sFilter       = null;

    //private String _sPostBack = null;

    //----------------------------------------------------------------
    private String               _sPicto        = null;

    private String               _sPictocl      = null;

    private String               _sOnpicto      = null;

    //----------------------------------------------------------------
    private int                  _iInternalType = TYPE_UNDEFINED;

    /**
     * Constructor
     */
    public Field()
    {
        super(TAG_NAME, $htmlGen, $xulGen);
    }

    //----------------------------------------------------------------
    public String getValue() // Overrides the standard method to use POSTBACK
    {
        trace("getValue()...");
        String sValue = getValueWithPostback();
        trace("getValue() : return " + sValue);
        return sValue;
    }

    //----------------------------------------------------------------
    public void setSize(String v)
    {
        _sSize = v;
    }

    public void setSize(int v)
    {
        _sSize = "" + v;
    }

    public String getSize()
    {
        return _sSize;
    }

    //----------------------------------------------------------------
    public void setMaxlength(String v)
    {
        _sMaxLength = v;
    }

    public void setMaxlength(int v)
    {
        _sMaxLength = "" + v;
    }

    public String getMaxlength()
    {
        return _sMaxLength;
    }

    //    //----------------------------------------------------------------
    //    public void setDisabled(String v)
    //    {
    //        _bDisabled = isTrue(v);
    //    }
    //
    //    public void setDisabled(boolean v)
    //    {
    //        _bDisabled = v;
    //    }
    //
    //    public boolean isDisabled()
    //    {
    //        return _bDisabled;
    //    }

    //----------------------------------------------------------------
    public void setReadonly(String v)
    {
        _bReadOnly = isTrue(v);
    }

    public void setReadonly(boolean v)
    {
        _bReadOnly = v;
    }

    public boolean isReadonly()
    {
        return _bReadOnly;
    }

    //----------------------------------------------------------------
    //    public void setAlign(String v) // Attribut "align"
    //    {
    //        _sAlign = v;
    //    }

    //----------------------------------------------------------------
    public void setRequired(String v)
    {
        _bRequired = isTrue(v);
    }

    public void setRequired(boolean v)
    {
        _bRequired = v;
    }

    public boolean isRequired()
    {
        return _bRequired;
    }

    //----------------------------------------------------------------
    public void setPassword(String v)
    {
        _bPassword = isTrue(v);
    }

    public void setPassword(boolean v)
    {
        _bPassword = v;
    }

    public boolean isPassword()
    {
        return _bPassword;
    }

    //----------------------------------------------------------------
    public void setType(String v)
    {
        _sType = v;
        setInternalType(v);
    }

    private void setInternalType(String sOriginalType)
    {
        _iInternalType = TYPE_UNDEFINED;
        if (sOriginalType != null)
        {
            String sType = sOriginalType.trim().toLowerCase();
            if (sType.equals("date"))
            {
                _iInternalType = TYPE_DATE;
            }
            else if (sType.startsWith("int"))
            {
                _iInternalType = TYPE_INT;
            }
            else if (sType.startsWith("num"))
            {
                _iInternalType = TYPE_NUM;
            }
            else if (sType.startsWith("alpha"))
            {
                _iInternalType = TYPE_ALPHA;
            }
            else if (sType.startsWith("alphanum"))
            {
                _iInternalType = TYPE_ALPHANUM;
            }
            else if (sType.startsWith("select"))
            {
                _iInternalType = TYPE_SELECT;
            }
            else
            {
                error("attribute 'type' : invalid value '" + sOriginalType + "' ");
            }
        }
    }

    public String getType()
    {
        return _sType;
    }

    public int getInternaType()
    {
        return _iInternalType;
    }

    //----------------------------------------------------------------
    public void setFormat(String v)
    {
        _sFormat = v;
    }

    public String getFormat()
    {
        return _sFormat;
    }

    //----------------------------------------------------------------
    public void setMin(String v)
    {
        _sMin = v;
    }

    public void setMin(int v)
    {
        _sMin = "" + v;
    }

    public void setMin(double v)
    {
        _sMin = "" + v;
    }

    public String getMin()
    {
        return _sMin;
    }

    //----------------------------------------------------------------
    public void setMax(String v)
    {
        _sMax = v;
    }

    public void setMax(int v)
    {
        _sMax = "" + v;
    }

    public void setMax(double v)
    {
        _sMax = "" + v;
    }

    public String getMax()
    {
        return _sMax;
    }

    //----------------------------------------------------------------
    public void setRegexp(String v)
    {
        _sRegexp = v;
    }

    public String getRegexp()
    {
        return _sRegexp;
    }

    //----------------------------------------------------------------
    public void setFilter(String v)
    {
        _sFilter = v;
    }

    public String getFilter()
    {
        return _sFilter;
    }

    //----------------------------------------------------------------
    public void setPicto(String v)
    {
        _sPicto = v;
    }

    public String getPicto()
    {
        return _sPicto;
    }

    //----------------------------------------------------------------
    public void setPictocl(String v)
    {
        _sPictocl = v;
    }

    public String getPictocl()
    {
        return _sPictocl;
    }

    //----------------------------------------------------------------
    public void setOnpicto(String v)
    {
        _sOnpicto = v;
    }

    public String getOnpicto()
    {
        return _sOnpicto;
    }

    //----------------------------------------------------------------
    //----------------------------------------------------------------
    /* (non-Javadoc)
     * @see javax.servlet.jsp.tagext.Tag#doStartTag()
     */
    public int doStartTag()
    {
        startTag();
        return (SKIP_BODY);
    }

    //----------------------------------------------------------------
    /* (non-Javadoc)
     * @see javax.servlet.jsp.tagext.Tag#doEndTag()
     */
    public int doEndTag()
    {
        endTag();
        return (EVAL_PAGE); //  EVAL_PAGE = continue , SKIP_PAGE = abort the processing
    }

    //----------------------------------------------------------------

    /**
     * Builds and returns the JavaScript string for 'OnBlur' event
     * 
     * @return
     */
    public String getOnBlurEvent()
    {
        String sCanBeNull = (_bRequired == true ? "false" : "true");
        if (_sType != null)
        {
            String sFormat = (_sFormat == null ? "null" : "'" + _sFormat + "'"); // null or 'format'
            String sMin = (_sMin == null ? "null" : _sMin);
            String sMax = (_sMax == null ? "null" : _sMax);
            return "fwkCheckField(this," + sCanBeNull + ",'" + _sType + "'," + sMin + "," + sMax + "," + sFormat + ")";
        }
        else if (_sRegexp != null)
        {
            return "fwkCheckRegExp(this," + sCanBeNull + "," + _sRegexp + ")";
        }
        else if (_bRequired)
        {
            return "fwkCheckNotNull(this)";
        }
        return null;
    }

    /**
     * Builds and returns the JavaScript string for 'OnKeyPress' event
     * 
     * @return
     */
    public String getOnKeyPressEvent()
    {
        if (_sType != null)
        {
            //--- Apply the standard filter associated with the type
            return "return fwkFilter(event,this,'" + _sType + "')";
        }
        else
        {
            if (_sFilter != null)
            {
                //--- Apply the specific filter
                return "return fwkFilter(event,this,'" + _sFilter + "')";
            }
        }
        return null;
    }

    public String getFullPictoImageSrc()
    {
        return getFullImageSrc(_sPicto);
    }

}