package org.objectweb.telosys.uil.taglib.widget;

import org.objectweb.telosys.uil.taglib.TagCommons;
import org.objectweb.telosys.uil.taglib.widget.html.FormHTML;


/**
 * @author Laurent GUERIN
 *  
 */

public class Form extends GenericTag
{
    private static final String  TAG_NAME = "form";

    //----------------------------------------------------------------
    // TAG generators ( thread safe => static )
    //----------------------------------------------------------------
    private final static IWidget $htmlGen = new FormHTML();

    private final static IWidget $xulGen  = null ; // No FORM tag in XUL 

    //----------------------------------------------------------------
    // TAG attributes
    //----------------------------------------------------------------

    private String               _sAction    = null;

    private String               _sEnctype    = null ;

    private String               _sMethod   = null;

    private String               _sTarget   = null;

    private String               _sAccept   = null;

    private String               _sAcceptCharset   = null;

    private String               _sPostBack  = "none";

    /**
     * Constructor
     */
    public Form()
    {
        super(TAG_NAME, $htmlGen, $xulGen);
    }

    //----------------------------------------------------------------
    public void setAction(String v)
    {
        _sAction = v;
    }

    public String getAction()
    {
        return _sAction;
    }

    //----------------------------------------------------------------
    public void setEnctype(String v)
    {
        _sEnctype = v;
    }

    public String getEnctype()
    {
        return _sEnctype;
    }

    //----------------------------------------------------------------
    public void setMethod(String v)
    {
        _sMethod = v;
    }

    public String getMethod()
    {
        return _sMethod;
    }

    //----------------------------------------------------------------
    public void setTarget(String v)
    {
        _sTarget = v;
    }

    public String getTarget()
    {
        return _sTarget;
    }

    //----------------------------------------------------------------
    public void setAccept(String v)
    {
        _sAccept = v;
    }

    public String getAccept()
    {
        return _sAccept;
    }

    //----------------------------------------------------------------
    public void setAcceptCharset(String v)
    {
        _sAcceptCharset = v;
    }

    public String getAcceptCharset()
    {
        return _sAcceptCharset;
    }
    
    //----------------------------------------------------------------
    public void setPostback(String postBack)
    {
        _sPostBack = postBack;
    }

    public String getPostback()
    {
        return _sPostBack;
    }

    public boolean isSessionPostback()
    {
        //--- The postback type is inherited from the parent "form" tag
        return TagCommons.isSessionPostback( _sPostBack );
    }

    public boolean isRequestPostback()
    {
        //--- The postback type is inherited from the parent "form" tag
        return TagCommons.isRequestPostback( _sPostBack );
    }

    //----------------------------------------------------------------
    //----------------------------------------------------------------
    public int doStartTag()
    {
        //--- Set the current form name (if any)
        String sFormName = this.getName();
        if ( sFormName != null )
        {
            this.setCurrentFormName(sFormName);
        }
        //--- Set the current form postback
        this.setCurrentFormPostback(_sPostBack);
        //--- Open the tag
        startTag();
        return (EVAL_BODY_INCLUDE);
    }

    //----------------------------------------------------------------
    //----------------------------------------------------------------
    public int doEndTag()
    {
        //--- Close the tag
        endTag();
        //--- Unset the current form postback
        this.removeCurrentFormPostback();
        //--- Unset the current form name (if any) 
        String sFormName = this.getName();
        if ( sFormName != null )
        {
            this.removeCurrentFormName();
        }
        return (EVAL_PAGE); //  EVAL_PAGE = continue , SKIP_PAGE = abort the processing
    }
}