package org.objectweb.telosys.uil.taglib.widget;

import org.objectweb.telosys.uil.taglib.widget.GenericTag;

/**
 * Generic ancestor class for all tags generators (HTML, XUL, ...)
 * 
 * @author Laurent GUERIN
 *  
 */
public abstract class GeneratorAncestor
{
    protected final static int     POS_NONE = 1;

    protected final static int     POS_XY   = 2;

    protected final static boolean USE_VAR  = true;

    //protected final static boolean NOT_USE_VAR = false ;

    /**
     * Returns true if the string is not null and not void
     * 
     * @param sTxt
     * @return
     */
    protected boolean notVoid(String sTxt)
    {
        if (sTxt != null)
        {
            if (sTxt.trim().length() > 0)
            {
                return true; // not void
            }
        }
        return false;
    }

    /**
     * Returns true if the string is null or void
     * 
     * @param sTxt
     * @return
     */
    protected boolean voidOrNull(String sTxt)
    {
        return ! notVoid(sTxt);
    }


    /**
     * Returns the html "id" attribute ( ie : 'id="xx"' )
     * 
     * @param tag
     * @return 'id="xx"' or ''
     */
    protected String getIdAttr(GenericTag tag)
    {
        String sId = tag.getId();
        if (sId != null)
        {
            return "id=\"" + sId + "\"";
        }
        else
        {
            return "";
        }
    }
    
    /**
     * Returns the html "id" attribute concat with index( ie : 'id="xx_nn"' )
     * @param tag
     * @param iIndex to concat 
     * @return
     */
    protected String getIdAttr(GenericTag tag, int iIndex)
    {
        String sId = tag.getId();
        if (sId != null)
        {
            return "id=\"" + sId + "_" + iIndex + "\"";
        }
        else
        {
            return "";
        }
    }

    /**
     * Returns the html "class" attribute ( ie : 'class="xx"' )
     * 
     * @param tag
     * @return
     */
    protected String getClassAttr(GenericTag tag)
    {
        String sClass = tag.getCl();
        if (sClass != null)
        {
            return "class=\"" + sClass + "\"";
        }
        else
        {
            return "";
        }
    }

    /**
     * Returns the html "tabindex" attribute ( ie : 'tabindex="xx"' )
     * 
     * @param tag
     * @return
     */
    protected String getTabindexAttr(GenericTag tag)
    {
        String sTabIndex = tag.getTabindex();
        if (sTabIndex != null)
        {
            return "tabindex=\"" + sTabIndex + "\"";
        }
        else
        {
            return "";
        }
    }

}