package org.objectweb.telosys.uil.taglib.widget;

import javax.servlet.jsp.tagext.Tag;

/**
 * Nested "item" tags ancestor (for item tags with a parent tag )<br>
 * Subclasses : ComboItem, ListItem, RadioItem
 * 
 * @author Laurent GUERIN
 *  
 */
public class GenericItemTag extends GenericTag
{
    //----------------------------------------------------------------
    // Nested Item tags common attributes
    //----------------------------------------------------------------

    private String _sValue    = "";

    private String _sTxt      = "";

    private String _sSelected = "false";

    /**
     * Constructor
     * 
     * @param sTagName
     * @param htmlGen
     * @param xulGen
     */
    public GenericItemTag(String sTagName, IWidget htmlGen, IWidget xulGen)
    {
        super(sTagName, htmlGen, xulGen);
    }

    //----------------------------------------------------------------
    public void setValue(String v)
    {
        _sValue = v;
    }

    public String getValue()
    {
        return _sValue;
    }

    //----------------------------------------------------------------
    public void setTxt(String v)
    {
        _sTxt = v;
    }

    public String getTxt()
    {
        return _sTxt;
    }

    //----------------------------------------------------------------
    public void setSelected(String v)
    {
        _sSelected = v;
    }

    public void setSelected(boolean bFlag)
    {
        _sSelected = (bFlag == true ? "true" : "false");
    }

    public String getSelected()
    {
        return _sSelected;
    }

    public boolean isSelected()
    {
        if (_sSelected != null)
        {
            if (_sSelected.trim().equalsIgnoreCase("true"))
            {
                return true;
            }
        }
        //--- Is it the current parent value ?
        if (sameValueAsParent())
        {
            return true; // Select this item
        }
        return false;
    }

    //----------------------------------------------------------------

    public ComboBox getParentComboBox()
    {
        Tag tagParent = this.getParent();
        if (tagParent != null)
        {
            if (tagParent instanceof ComboBox)
            {
                return (ComboBox) tagParent;
            }
        }
        return null;
    }

    /**
     * Compares the Item value with the Parent value
     * 
     * @return true if same values, false if different values
     */
    public boolean sameValueAsParent()
    {
        //--- Get the current item tag value
        String sTagValue = getValue();
        if (sTagValue == null)
        {
            return false;
        }

        //--- Get the parent tag value
        String sParentValue = null;
        Tag tagParent = this.getParent();
        if (tagParent != null)
        {
            //--- There is only 3 parent tags with items :
            if (tagParent instanceof ComboBox )
            {
                sParentValue = ((ComboBox) tagParent).getValue();
            }
            else if (tagParent instanceof ListBox)
            {
                sParentValue = ((ListBox) tagParent).getValue();
            }
            else if (tagParent instanceof RadioGroup)
            {
                sParentValue = ((RadioGroup) tagParent).getValue();
            }
            else
            {
                error("invalid parent tag ( parent class = " + tagParent.getClass().getName() + " )" );
                return false;
            }
        }
        else
        {
            error("no parent tag !" );
            return false;
        }
        //--- Compare the 2 values ...
        if (sParentValue != null && sTagValue != null)
        {
            return sParentValue.equals(sTagValue);
        }
        return false;
    }

}