package org.objectweb.telosys.uil.taglib.widget;

import java.io.IOException;
import java.util.HashMap;
import java.util.Iterator;
import java.util.LinkedList;

import javax.servlet.ServletRequest;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpSession;
import javax.servlet.jsp.JspWriter;
import javax.servlet.jsp.PageContext;
import javax.servlet.jsp.tagext.BodyTagSupport;

import org.objectweb.telosys.common.Telosys;
import org.objectweb.telosys.common.TraceConfig;
import org.objectweb.telosys.screen.env.ScreenApplication;
import org.objectweb.telosys.screen.env.ScreenApplicationManager;
import org.objectweb.telosys.screen.env.ScreenSession;
import org.objectweb.telosys.screen.env.ScreenSessionManager;
import org.objectweb.telosys.uil.screenmap.ScreenMap;
import org.objectweb.telosys.uil.screenmap.ScreenMapConst;
import org.objectweb.telosys.uil.taglib.Page;
import org.objectweb.telosys.uil.taglib.TagCommons;

/**
 * @author Laurent GUERIN
 *  
 */
public class GenericTag extends BodyTagSupport
{
//    public static final int     UNKNOWN_OUTPUT         = 0;
//
//    public static final int     HTML_OUTPUT            = 1;
//
//    public static final int     XUL_OUTPUT             = 2;

//    private static final String VARIABLES              = "SCREEN_MAP_VARIABLES";

    //----------------------------------------------------------------
    // Trace management ( just like TelosysObject )
    //----------------------------------------------------------------
    protected int               _iLastTraceConfigState = -1;

    protected boolean           _bTrace                = false;

    //----------------------------------------------------------------
    // Widget generator implementations : HTML, XUL
    //----------------------------------------------------------------
    private IWidget             _htmlGenerator         = null;

    private IWidget             _xulGenerator          = null;

    //----------------------------------------------------------------
    // Widget tag name
    //----------------------------------------------------------------
    private String              _sTagName              = null;

    //----------------------------------------------------------------
    // Style values (CSS)
    //----------------------------------------------------------------
    private HashMap             _hmStyle               = new HashMap();

    //----------------------------------------------------------------
    // Standard tags attributes
    //----------------------------------------------------------------

    private String              _sId                   = null;

    private String              _sClass                = null;

    private String              _sStyle                = null;

    private String              _sTitle                = null;

    //----------------------------------------------------------------

    private String              _sX                    = null;

    private String              _sY                    = null;

    private String              _sWidth                = null;

    private String              _sHeight               = null;

    private String              _sTabIndex             = null;

    private String              _sZone                 = null;

    private String              _sDisabled             = null;

    private String              _sName                 = null;

    private String              _sData                 = null;

    private String              _sDataElement          = null;

    private String              _sDataAttribute        = null;

    //----------------------------------------------------------------
    // Events
    //----------------------------------------------------------------
    private HashMap             _hmEvents              = new HashMap();

    //----------------------------------------------------------------
    // Other instance variables
    //----------------------------------------------------------------

    //--- Automatic ID generation
    //--- NB : the same instance can be used by different tags in the JSP ( cf. tag pool )
    //private String _sInitialId = null;

    private String              _sGeneratedId          = null;

    private boolean             _bIdGeneration         = false; // No automatic ID generation by default

    /**
     *  
     */
    //    public GenericTag(String sTagName)
    //    {
    //        super();
    //        _sTagName = sTagName;
    //    }
    public GenericTag(String sTagName, IWidget htmlGen, IWidget xulGen)
    {
        super();
        _sTagName = sTagName;
        _htmlGenerator = htmlGen;
        _xulGenerator = xulGen;
    }

    //===============================================================================
    // Generic methods called from "doStartTag" and "doEndTag"
    //===============================================================================
    /**
     * Called by the standard "doStartTag" method
     */
    public void startTag()
    {
        trace("startTag()...");

        //--- Force generated ID as "not yet generated"
        _sGeneratedId = null;

        //        //--- Keep initial "id" value
        //        _sInitialId = _sId;

        //--- Process (call the generators) ...
        JspWriter out = pageContext.getOut();
        try
        {
            if (TagCommons.isHTML(pageContext))
            {
                //--- Start HTML tag
                if (_htmlGenerator != null)
                {
                    _htmlGenerator.startTag(out, this);
                }
                else
                {
                    error("No HTML generator ( startTag )");
                }
            }
            else
            {
                //--- Start XUL tag
                if (_xulGenerator != null)
                {
                    _xulGenerator.startTag(out, this);
                }
                else
                {
                    error("No XUL generator ( startTag )");
                }
            }
        } catch (IOException ioex)
        {
            ioex.printStackTrace();
            error("IOException ( startTag ) : " + ioex.getMessage());
        }
    }

    /**
     * Called by the standard "doEndTag" method NB: not always called ( see doEndTag of each tag )
     */
    public void endTag()
    {
        trace("endTag()...");

        JspWriter out = pageContext.getOut();
        try
        {
            if (TagCommons.isHTML(pageContext))
            {
                //--- Start HTML tag
                if (_htmlGenerator != null)
                {
                    _htmlGenerator.endTag(out, this);
                }
                else
                {
                    error("No HTML generator ( endTag )");
                }
            }
            else
            {
                //--- Start XUL tag
                if (_xulGenerator != null)
                {
                    _xulGenerator.endTag(out, this);
                }
                else
                {
                    error("No XUL generator ( endTag )");
                }
            }
        } catch (IOException ioex)
        {
            ioex.printStackTrace();
            error("IOException ( endTag ) : " + ioex.getMessage());
        }

        //        //--- Restore initial "id" value
        //        _sId = _sInitialId;
    }

    /**
     * Returns the name of the tag (ie: "field" for <x:field ..> )
     * 
     * @return : the tag name
     */
    public String getTagName()
    {
        return _sTagName;
    }

    /**
     * Returns the current page context
     * 
     * @return
     */
    public PageContext getPageContext()
    {
        return pageContext;
    }

    /**
     * Return the servlet class name of the current JSP
     * 
     * @return : servlet class name
     */
    public String getServletClassName()
    {
        Object oPage = pageContext.getPage();
        if (oPage != null)
        {
            return oPage.getClass().getName();
        }
        else
        {
            return null;
        }
    }

    //===============================================================================
    // "id"
    //===============================================================================
    /*
     * (non-Javadoc)
     * 
     * @see javax.servlet.jsp.tagext.TagSupport#setId(java.lang.String)
     */
    public void setId(String v)
    {
        _sId = v;
        if (v != null)
        {
            _sId = v.trim();
        }
    }

    /*
     * (non-Javadoc)
     * 
     * @see javax.servlet.jsp.tagext.TagSupport#getId()
     */
    public String getId()
    {
        if (_sId != null) // Id set by user
        {
            return _sId;
        }
        else
        // null : no id set by user
        {
            if (_bIdGeneration)
            {
                return getGeneratedId();
            }
            else
            {
                return null;
            }
        }

        //        if (_sId == null)
        //        {
        //            //--- Generate an ID
        //            _sId = Page.generateId(pageContext);
        //            
        //// TagScreenEnv screenEnv = TagCommons.getScreenEnv(pageContext);
        //// if (screenEnv != null)
        //// {
        //// _sId = screenEnv.generateId();
        //// }
        //// else
        //// {
        //// _sId = "no_fwk_id";
        //// }
        //        }
        //        return _sId;
    }

    protected boolean getIdGeneration()
    {
        return _bIdGeneration;
    }

    protected void setIdGeneration(boolean bFlag)
    {
        _bIdGeneration = bFlag;
    }

    /**
     * Returns the generated ID for the current tag
     * 
     * @return
     */
    private String getGeneratedId()
    {
        if (_sGeneratedId == null) // ID not yet generated
        {
            //--- Generate an ID
            _sGeneratedId = Page.generateId(pageContext);
        }
        return _sGeneratedId;
    }

    /**
     * Returns true if the tag has a valid ID (set manualy or generated)
     * 
     * @return
     */
    public boolean hasId()
    {
        if (_sId != null) // ID set manualy
        {
            if (_sId.length() > 0)
            {
                return true;
            }
        }
        if (_sGeneratedId != null) // Generated ID (always valid)
        {
            return true;
        }
        return false;
    }

    //    public String getInitialId()
    //    {
    //        return _sId;
    //    }

    //===============================================================================
    // "cl" ( "class" )
    //===============================================================================
    public void setCl(String v)
    {
        _sClass = v;
    }

    public String getCl()
    {
        return _sClass;
    }

    //===============================================================================
    // "style"
    //===============================================================================
    /**
     * @param v
     */
    public void setStyle(String v)
    {
        _sStyle = v;
    }

    /**
     * Returns the style set by the tag attribute "style"
     * 
     * @return a string ( never null, void if no style )
     */
    public String getStyle()
    {
        //--- Style set by the user
        String sUserStyle = "";
        if (_sStyle != null)
        {
            sUserStyle = _sStyle;
        }

        //--- Styles set programmaticaly
        String sProgStyle = "";
        if (_hmStyle.size() > 0)
        {
            String sKey = null;
            String sVal = null;
            Iterator it = _hmStyle.keySet().iterator();
            while (it.hasNext())
            {
                sKey = (String) it.next();
                sVal = (String) _hmStyle.get(sKey);
                sProgStyle = sProgStyle + sKey + ":" + sVal + ";";
            }
        }
        return sProgStyle + sUserStyle;
    }

    //===============================================================================
    // "title"
    //===============================================================================
    public void setTitle(String v)
    {
        _sTitle = v;
    }

    public String getTitle()
    {
        return _sTitle;
    }

    //===============================================================================
    // "x" coordinate
    //===============================================================================
    public void setX(String v)
    {
        _sX = v;
    }

    public void setX(int v) // for use with int Java variables ( "rtexprvalue" )
    {
        _sX = "" + v;
    }

    public String getX()
    {
        return _sX;
    }

    public String getXValue(boolean bUseVar)
    {
        if (notVoid(_sX))
        {
            //--- X attribute
            return _sX;
        }
        else if (bUseVar)
        {
            //--- X variable
            Variables variables = Variables.getVariables(pageContext);
            if (variables != null)
            {
                String sVarX = variables.get("x");
                return (notVoid(sVarX) ? sVarX : "0");
            }
        }
        return "0";
    }

    //===============================================================================
    // "y" coordinate
    //===============================================================================
    public void setY(String v)
    {
        _sY = v;
    }

    public void setY(int v) // for use with int Java variables ( "rtexprvalue" )
    {
        _sY = "" + v;
    }

    public String getY()
    {
        return _sY;
    }

    public String getYValue(boolean bUseVar)
    {
        if (notVoid(_sY))
        {
            //--- Y attribute
            return _sY;
        }
        else if (bUseVar)
        {
            //--- Y variable
            Variables variables = Variables.getVariables(pageContext);
            if (variables != null)
            {
                String sVarY = variables.get("y");
                return (notVoid(sVarY) ? sVarY : "0");
            }
        }
        return "0";
    }

    //===============================================================================
    // "width"
    //===============================================================================
    public void setWidth(String v)
    {
        _sWidth = v;
    }

    public void setWidth(int v) // for use with int Java variables ( "rtexprvalue" )
    {
        _sWidth = "" + v;
    }

    public String getWidth()
    {
        return _sWidth;
    }

    //===============================================================================
    // "height"
    //===============================================================================
    public void setHeight(String v)
    {
        _sHeight = v;
    }

    public void setHeight(int v) // for use with int Java variables ( "rtexprvalue" )
    {
        _sHeight = "" + v;
    }

    public String getHeight()
    {
        return _sHeight;
    }

    //===============================================================================
    // "tabindex"
    //===============================================================================
    public void setTabindex(String v)
    {
        _sTabIndex = v;
    }

    public void setTabindex(int v) // for use with int Java variables ( "rtexprvalue" )
    {
        _sTabIndex = "" + v;
    }

    public String getTabindex()
    {
        return _sTabIndex;
    }

    //===============================================================================
    // "name"
    //===============================================================================
    public void setName(String v)
    {
        _sName = v;
    }

    public String getName()
    {
        return _sName;
    }

    //===============================================================================
    // "zone"
    //===============================================================================
    public void setZone(String v)
    {
        _sZone = v;
        if (v != null)
        {
            _sZone = v.trim();
        }
    }

    public String getZone()
    {
        return _sZone;
    }

    public boolean hasZone()
    {
        if (_sZone != null)
        {
            if (_sZone.length() > 0)
            {
                return true;
            }
        }
        return false;
    }

    //===============================================================================
    // "disabled"
    //===============================================================================
    public void setDisabled(boolean v)
    {
        _sDisabled = "" + v;
    }

    public void setDisabled(String v)
    {
        _sDisabled = v;
    }

    public String getDisabled()
    {
        return _sDisabled;
    }

    public boolean isDisabled()
    {
        if (_sDisabled != null)
        {
            if (_sDisabled.trim().equalsIgnoreCase("true"))
            {
                return true;
            }
        }
        else
        {
            //--- Is there a default value for this attribute ?
            String sDefault = getDefaultAttributeValue(_sTagName, "disabled");
            if (sDefault != null)
            {
                if (sDefault.trim().equalsIgnoreCase("true"))
                {
                    return true;
                }
            }
        }
        return false;
    }

    //===============================================================================
    // "data binding"
    //===============================================================================
    public void setData(String s)
    {
        //--- 1) Keep the original data attribute
        _sData = s ; 
        
        //--- 2) Replace "#" by "id value"
        if (s != null)
        {
            if (s.trim().equals("#"))
            {
                //--- Use the field id as the "data" attribute
                s = this.getId();
            }
        }

        //--- 3) Split data in 2 parts ( element & attribute )
        if (s != null)
        {
            s = s.trim();
            //--- Is in "dot notation"
            int iPointPosition = s.lastIndexOf('.');
            if (iPointPosition >= 0)
            {
                //--- Syntaxe : "element.attribute" => split the String
                _sDataElement = s.substring(0, iPointPosition);
                _sDataAttribute = s.substring(iPointPosition + 1);
            }
            else
            {
                //--- Use the given name as the DataAttribute
                _sDataElement = Page.getDefaultDataElement(pageContext);
                _sDataAttribute = s;
            }
        }

        //--- Trim ...
        if (_sDataElement != null)
        {
            _sDataElement = _sDataElement.trim();
        }
        if (_sDataAttribute != null)
        {
            _sDataAttribute = _sDataAttribute.trim();
        }
    }

    public String getData()
    {
        return _sData;
    }

    public String getDataElement()
    {
        return _sDataElement;
    }

    public String getDataAttribute()
    {
        return _sDataAttribute;
    }

    public boolean hasDataBinding()
    {
        String sBindElement = getDataElement();
        String sBindAttribute = getDataAttribute();

        if (sBindElement != null && sBindAttribute != null)
        {
            if (sBindElement.length() > 0 && sBindAttribute.length() > 0)
            {
                return true;
            }
        }
        return false;
    }

    //===============================================================================
    // EVENTS
    //===============================================================================
    public void setOnclick(String v)
    {
        _hmEvents.put(Event.ON_CLICK, v);
    }

    public void setOndblclick(String v)
    {
        _hmEvents.put(Event.ON_DBL_CLICK, v);
    }

    public void setOnmouseover(String v)
    {
        _hmEvents.put(Event.ON_MOUSE_OVER, v);
    }

    public void setOnmouseout(String v)
    {
        _hmEvents.put(Event.ON_MOUSE_OUT, v);
    }

    //----------------------------------------------------------------
    public void setOnchange(String v)
    {
        _hmEvents.put(Event.ON_CHANGE, v);
    }

    public void setOnselect(String v)
    {
        _hmEvents.put(Event.ON_SELECT, v);
    }

    public void setOnfocus(String v)
    {
        _hmEvents.put(Event.ON_FOCUS, v);
    }

    public void setOnblur(String v)
    {
        _hmEvents.put(Event.ON_BLUR, v);
    }

    //----------------------------------------------------------------
    public void setOnkeypress(String v)
    {
        _hmEvents.put(Event.ON_KEY_PRESS, v);
    }

    //----------------------------------------------------------------

    //===============================================================================
    //
    //===============================================================================
//    public int getOuputType(PageContext pageContext)
//    {
//        ScreenMap screenMap = TagCommons.getScreenMap(pageContext);
//        if (screenMap != null)
//        {
//            if (screenMap.isHtmlType())
//            {
//                return HTML_OUTPUT;
//            }
//            else if (screenMap.isXulType())
//            {
//                return XUL_OUTPUT;
//            }
//        }
//        return UNKNOWN_OUTPUT;
//    }

    public void defineStyle(String sName, String sValue)
    {
        _hmStyle.put(sName, sValue);
    }

    /**
     * Return the given attribute value if not null Try to find a default value if the given value is null
     * 
     * @param tagName
     * @param nomAtt
     * @param value
     * @return attribute value ( or null )
     */
    private String getDefaultAttributeValue(String tagName, String sAttribute)
    {
        String sRetValue = null;
        //--- Try to find a default value in the current screen env.
        // A default value set with the "default" tag
        TagScreenEnv screenEnv = TagCommons.getScreenEnv(pageContext);
        if (screenEnv != null)
        {
            sRetValue = screenEnv.getDefaultAttributeValue(tagName, sAttribute);
            if (sRetValue == null)
            {
                // Try to get a default value for all tags
                sRetValue = screenEnv.getDefaultAttributeValue("*", sAttribute);
            }
        }

        if (sRetValue == null)
        {
            // Try to get a global default value (same for all screens)
            // Default values set with a properties file
            sRetValue = TagDefaultValues.getDefaultAttributeValue(tagName, sAttribute);
        }
        //        if (sRetValue == null)
        //        {
        //            //--- Try to find a variable with name "x" or "y"
        //            if (sAttribute.equals("x") || sAttribute.equals("y"))
        //            {
        //                Variables variables = Variables.getVariables(pageContext);
        //                if (variables != null)
        //                {
        //                    sRetValue = variables.get(sAttribute);
        //                }
        //                //sRetValue = getVariable(sAttribute);
        //            }
        //        }
        return sRetValue;
    }

    //    public String getAttributeValue(String sAttribute, String value)
    //    {
    //        return getAttributeValue(_sTagName, sAttribute, value);
    //    }

    protected String getHtmlTitle()
    {
        if (_sTitle != null)
        {
            return "title=\"" + _sTitle + "\"";
        }
        else
        {
            return "";
        }
    }

    /**
     * Returns a "label" tag with "for" attribute referencing the current tag Id. <br>
     * The label has no style, no class. Its Id is tag_id + "_label"
     * 
     * @param sTxt
     * @return
     */
    protected String getHtmlLabelForTag(String sTxt)
    {
        String sTagId = getId();
        return "<label for=\"" + sTagId + "\"" + " id=\"" + sTagId + "_label\"" + " >" + sTxt + "</label>";
    }

    /**
     * Return true if there at least one coordinate not null or not void
     * 
     * @return
     */
    public boolean isPositionAbsolute()
    {
        //--- If X is set => absolute
        if (_sX != null)
        {
            if (_sX.trim().length() > 0)
                return true;
        }
        //--- If Y is set => absolute
        if (_sY != null)
        {
            if (_sY.trim().length() > 0)
                return true;
        }
        // No X and no Y => not absolute
        return false;
    }

    public boolean isEventDefined(String sEventName)
    {
        return (_hmEvents.get(sEventName) != null ? true : false);
    }

    /**
     * Returns true if the string equals "true" ( after trim(), not case sensitive )
     * 
     * @param sTxt
     * @return
     */
    protected boolean isTrue(String sTxt)
    {
        if (sTxt != null)
        {
            if (sTxt.trim().equalsIgnoreCase("true"))
            {
                return true;
            }
        }
        return false;
    }

    /**
     * Returns true if the string is not null and not void
     * 
     * @param sTxt
     * @return
     */
    protected boolean notVoid(String sTxt)
    {
        if (sTxt != null)
        {
            if (sTxt.trim().length() > 0)
            {
                return true; // not void
            }
        }
        return false;
    }

    //    public void printEvents(PageContext pageContext) throws IOException
    //    {
    //        printEvents(pageContext.getOut());
    //    }

    /**
     * Print all the defined events.
     * 
     * @param out
     * @throws IOException
     */
    public void printEvents(JspWriter out) throws IOException
    {
        printEvent(out, Event.ON_CLICK);
        printEvent(out, Event.ON_DBL_CLICK);
        printEvent(out, Event.ON_MOUSE_OVER);
        printEvent(out, Event.ON_MOUSE_OUT);

        printEvent(out, Event.ON_CHANGE);
        printEvent(out, Event.ON_SELECT);
        printEvent(out, Event.ON_FOCUS);
        printEvent(out, Event.ON_BLUR);

        printEvent(out, Event.ON_KEY_PRESS);

        printEvent(out, Event.ON_SUBMIT);
        printEvent(out, Event.ON_RESET);
    }

    /**
     * Prints the given event if it is defined.
     * 
     * @param out
     * @param sEventName
     * @throws IOException
     */
    public void printEvent(JspWriter out, String sEventName) throws IOException
    {
        String sEventValue = (String) _hmEvents.get(sEventName);
        if (sEventValue != null)
        {
            if (sEventValue.trim().length() > 0)
            {
                out.print(" " + sEventName + "=\"" + sEventValue + "\" ");
            }
        }
    }

    /**
     * Prints a given event script for a given event name
     * 
     * @param out
     * @param sEventName :
     *            ie onblur, onclick, ...
     * @param sEventScript :
     *            the script associated with the event
     * @throws IOException
     */
    public void printEvent(JspWriter out, String sEventName, String sEventScript) throws IOException
    {
        if (sEventName != null && sEventScript != null)
        {
            if (sEventName.trim().length() > 0 && sEventScript.trim().length() > 0)
            {
                out.print(" " + sEventName + "=\"" + sEventScript + "\" ");
            }
        }
    }

    /**
     * Returns the full image src, according with the image directory configuration <br>
     * ie : root context URL + image directory + the given src
     * 
     * @param sSrc
     * @return
     */
    protected String getFullImageSrc(String sSrc)
    {
        if (sSrc != null)
        {
            return TagCommons.getImgDirURL(pageContext) + sSrc;
        }
        else
        {
            return "";
        }
    }

    /**
     * Returns the HttpServletRequest instance (the current request)
     * 
     * @return
     */
    protected HttpServletRequest getHttpServletRequest()
    {
        ServletRequest servletRequest = pageContext.getRequest();
        if (servletRequest == null)
        {
            // Error
            //error();
            return null;
        }
        if (servletRequest instanceof HttpServletRequest)
        {
            return (HttpServletRequest) servletRequest;
        }
        else
        {
            // Error
            return null;
        }
    }

    /**
     * Returns the HttpSession instance (or null if it doesn't exist)
     * 
     * @return
     */
    protected HttpSession getHttpSession()
    {
        HttpServletRequest httpServletRequest = this.getHttpServletRequest();
        if (httpServletRequest != null)
        {
            httpServletRequest.getSession(false);
        }
        return null;
    }

    /**
     * Returns the Telosys Screen Context ID ( 0 to N )
     * 
     * @return the context id (or -1 if not found )
     */
    public int getScreenContextId()
    {
        HttpServletRequest request = this.getHttpServletRequest();
        if (request != null)
        {
            ScreenMap screenMap = (ScreenMap) request.getAttribute(ScreenMapConst.SCREEN_MAP);
            if (screenMap != null)
            {
                return screenMap.getScreenContextId();
            }
        }
        return -1; // Not found
    }

    /**
     * Returns the Telosys Screen Name (or null if not found)
     * 
     * @return the Screen Name (or null)
     */
    public String getScreenName()
    {
        HttpServletRequest request = this.getHttpServletRequest();
        if (request != null)
        {
            ScreenMap screenMap = (ScreenMap) request.getAttribute(ScreenMapConst.SCREEN_MAP);
            if (screenMap != null)
            {
                return screenMap.getName();
            }
        }
        return null; // Not found
    }

    /**
     * Returns the screen marker name ( the screen name used to generate the 'screen marker') <br>
     * The screen marker name is the symbolic screen name if it is defined, <br>
     * else the servlet class name is used as screen identifier
     * 
     * @return
     */
    public String getScreenMarkerName()
    {
        String sScreenMarkerName = getScreenName();
        if (sScreenMarkerName != null)
        {
            //--- Use the current symbolic screen name
            return sScreenMarkerName;
        }
        else
        {
            //--- No symbolic screen name => use the servlet class name
            return getServletClassName();
        }
        //return null; // Not found
    }

    /**
     * Returns the Telosys ScreenSession instance (or null if it doesn't exist)
     * 
     * @return
     */
    protected ScreenSession getScreenSession()
    {
        HttpServletRequest request = this.getHttpServletRequest();
        if (request != null)
        {
            return ScreenSessionManager.findScreenSession(request);
        }
        else
        {
            // Error
            return null;
        }
    }

    /**
     * Returns the Telosys ScreenApplication instance (or null if it doesn't exist)
     * 
     * @return
     */
    protected ScreenApplication getScreenApplication()
    {
        HttpSession httpSession = getHttpSession();
        return ScreenApplicationManager.getScreenApplication(httpSession);
    }

    //---------------------------------------------------------------------------------
    // Current form name management (for usage in child tags)
    //---------------------------------------------------------------------------------

    private final static String CURRENT_FORM_NAME = "TELOSYS_CURRENT_FORM_NAME";

    protected void setCurrentFormName(String sFormName)
    {
        pageContext.setAttribute(CURRENT_FORM_NAME, sFormName);
    }

    protected String getCurrentFormName()
    {
        Object obj = pageContext.getAttribute(CURRENT_FORM_NAME);
        if (obj != null)
        {
            if (obj instanceof String)
            {
                return (String) obj;
            }
        }
        return null;
    }

    protected void removeCurrentFormName()
    {
        pageContext.removeAttribute(CURRENT_FORM_NAME);
    }

    //---------------------------------------------------------------------------------
    // Current form postback management (for usage in child tags)
    //---------------------------------------------------------------------------------
    private final static String CURRENT_FORM_POSTBACK = "TELOSYS_CURRENT_FORM_POSTBACK";

    protected void setCurrentFormPostback(String sPostback)
    {
        pageContext.setAttribute(CURRENT_FORM_POSTBACK, sPostback);
    }

    protected String getCurrentFormPostback()
    {
        Object obj = pageContext.getAttribute(CURRENT_FORM_POSTBACK);
        if (obj != null)
        {
            if (obj instanceof String)
            {
                return (String) obj;
            }
        }
        return null;
    }

    protected void removeCurrentFormPostback()
    {
        pageContext.removeAttribute(CURRENT_FORM_POSTBACK);
    }

    //-----------------------------------------------------------------------------
    /**
     * Print a standard trace message, using the active loggers (if the trace flag is "true")
     * 
     * @param s
     */
    protected void trace(String sMsg)
    {
        String sClassName = this.getClass().getName();
        //--- If the configuration has changed since the last acces ...
        if (TraceConfig.getCurrentState() != _iLastTraceConfigState)
        {
            //--- ScreenTitle the flag from the configuration
            _bTrace = TraceConfig.flagIsOn(sClassName);
            //--- ScreenTitle the configuration state
            _iLastTraceConfigState = TraceConfig.getCurrentState();
        }
        if (_bTrace == true)
        {
            Telosys.trace(sClassName, "Tag '" + _sTagName + "' : " + sMsg);
        }
    }

    //-----------------------------------------------------------------------------
    /**
     * Print a standard tag error message, using the active loggers
     * 
     * @param sMsg
     */
    public void error(String sMsg)
    {
        Telosys.error(this.getClass().getName(), "Tag '" + _sTagName + "' : " + sMsg);
    }

    //-----------------------------------------------------------------------------

    public void addJavascriptLine(String sLine)
    {
        //--- Get the current Javascript lines (create it if not found)
        LinkedList lines = Page.getJavascriptLines(pageContext, true);
        if (lines != null)
        {
            trace("addJavascriptLine (" + sLine + ")");
            lines.add(sLine);
        }
        else
        {
            error("addJavascriptLine(..) : Cannot get lines from page context ");
        }
    }

}