package org.objectweb.telosys.uil.taglib.widget;

import java.io.IOException;
import java.util.HashMap;

import javax.servlet.jsp.JspWriter;
import javax.servlet.jsp.PageContext;

/**
 * @author T. Poingt, L. Guerin
 *  
 */
public class Variables
{
    private static final String VARIABLES    = "SCREEN_MAP_VARIABLES";

    protected static final String X = "x";

    protected static final String Y = "y";

    private final HashMap       _hmVariables = new HashMap();

    /**
     * Return a variable storage space for the given page context (create it if necessary)
     * 
     * @param pageContext
     * @return
     */
    public final static Variables getVariables(PageContext pageContext)
    {
        //return (HashMap) pageContext.getAttribute(VARIABLES);
        //--- Try to find an existing variables storage space in the page context
        Variables variables = (Variables) pageContext.getAttribute(VARIABLES);
        if (variables == null) // If not found => not yet created
        {
            //--- Variables storage space creation
            variables = new Variables();
            //--- Keep in the page context for the next time
            pageContext.setAttribute(VARIABLES, variables);
        }
        return variables;
    }

    /**
     * Set a variable value
     * 
     * @param sVarName
     * @param sVarValue
     */
    public final void set(String sVarName, String sVarValue)
    {
        if (_hmVariables != null)
        {
            _hmVariables.put(sVarName, sVarValue);
        }
    }

//    public final void setX(String sVarValue)
//    {
//        set("x", sVarValue);
//    }
//
//    public final void setY(String sVarValue)
//    {
//        set("y", sVarValue);
//    }

    /**
     * Get a variable value
     * 
     * @param sVarName
     * @return
     */
    public final String get(String sVarName)
    {
        if (_hmVariables != null)
        {
            Object oValue = _hmVariables.get(sVarName);
            if (oValue != null)
            {
                if (oValue instanceof String)
                {
                    return (String) oValue;
                }
            }
        }
        return null;
    }
    
    public final String getX()
    {
        return get("x");
    }
    
    public final String getY()
    {
        return get("y");
    }
    
    /**
     * Unset a variable
     * 
     * @param sVarName
     */
    public final void unset(String sVarName)
    {
        if (_hmVariables != null)
        {
            _hmVariables.remove(sVarName);
        }
    }

    /**
     * Print all the declared variables on the given output
     * 
     * @param out
     * @throws IOException
     */
    public final void print(JspWriter out) throws IOException
    {
        if (_hmVariables != null)
        {
            java.util.Set keys = _hmVariables.keySet();
            if (keys != null)
            {
                Object[] names = keys.toArray();
                String sName = null;
                String sValue = null;
                for ( int i = 0 ; i < names.length ; i++ )
                {
                    sName = (String) names[i];
                    sValue = (String) _hmVariables.get(sName);
                    out.println(" . " + sName + " = " + sValue);
                }
            }
        }
        else
        {
            out.println(" (no variables) ");
        }
    }

}