package org.objectweb.telosys.uil.taglib.widget.html;

import java.io.IOException;

import javax.servlet.jsp.JspWriter;

import org.objectweb.telosys.uil.taglib.widget.Event;
import org.objectweb.telosys.uil.taglib.widget.Field;
import org.objectweb.telosys.uil.taglib.widget.GenericTag;
import org.objectweb.telosys.uil.taglib.widget.IWidget;

/**
 * HTML generation for Field widget
 * 
 * @author Laurent GUERIN
 *  
 */
public class FieldHTML extends GenericHTML implements IWidget
{
    //----------------------------------------------------------------
    public void startTag(JspWriter out, GenericTag genericTag) throws IOException
    {
        Field tag = (Field) genericTag;

        //String sPicto = tag.getPicto();
        //--- If there's a pictogram for this field
        if (notVoid(tag.getPicto()))
        {
            out.print("<span style=\"" + getStyleFragPosition(tag) + "white-space:nowrap; display:block;\" >");
            generateInputTag(out, tag, POS_NONE); // Tag without absolute position
            generateImgTag(out, tag);
            out.print("</span>");
        }
        else
        {
            generateInputTag(out, tag, POS_XY); // Tag with absolute position if x or y set
        }
    }

    private void generateInputTag(JspWriter out, Field tag, int iPositionType) throws IOException
    {
        //--- If numeric => align / right 
//        String sType = tag.getType();
//        if (sType != null)
//        {
//            if (sType.startsWith("Int") || sType.startsWith("Num"))
//            {
//                tag.defineStyle("text-align", "right");
//            }
//        }
        int iType = tag.getInternaType();
        if (iType == Field.TYPE_INT  || iType == Field.TYPE_NUM  )
        {
            tag.defineStyle("text-align", "right");
        }
        

        //--- Open the tag <input type="text" ... />
        out.print("<input ");
        if (tag.isPassword())
        {
            //--- Password field
            out.print(" type=\"password\" ");
        }
        else
        {
            //--- Standard "text field"
            out.print(" type=\"text\" ");
        }
        //--- Generic widget attributes
        out.print(getIdAttr(tag) + " ");
        out.print(getClassAttr(tag) + " ");
        //out.print(getStyleAttr(tag) + " ");
        out.print(getStyleAttr(tag, iPositionType) + " ");
        out.print(getTitleAttr(tag) + " ");
        out.print(getTabindexAttr(tag) + " ");

        //--- Specific widget attributes
        if (tag.getMaxlength() != null)
        {
            out.print(" maxlength=\"" + tag.getMaxlength() + "\"");
        }

        //--- "size" attribute ( if not set use "maxlength" )
        String sSize = tag.getSize() != null ? tag.getSize() : tag.getMaxlength();
        if (sSize != null)
        {
            out.print(" size=\"" + sSize + "\"");
        }

        String sValue = tag.getValue(); // use postback value if necessary
        if (sValue != null)
        {
            out.print(" value=\"" + sValue + "\"");
        }

        if (tag.getName() != null)
        {
            out.print(" name=\"" + tag.getName() + "\"");
        }

        if (tag.isDisabled())
        {
            out.print(" disabled=\"disabled\"");
        }

        if (tag.isReadonly())
        {
            out.print(" readonly=\"readonly\"");
        }

        //--- Events set by user
        tag.printEvents(out);
        //--- Standard field events
        if (tag.isEventDefined(Event.ON_BLUR) != true)
        {
            //--- No specific event "ON BLUR => generate standard "ON BLUR"
            String sScript = tag.getOnBlurEvent();
            tag.printEvent(out, Event.ON_BLUR, sScript);
        }
        if (tag.isEventDefined(Event.ON_KEY_PRESS) != true)
        {
            //--- No specific event => generate standard event
            String sScript = tag.getOnKeyPressEvent();
            tag.printEvent(out, Event.ON_KEY_PRESS, sScript);
        }

        //--- Close the tag
        out.print("/>");

    }

    private void generateImgTag(JspWriter out, Field tag) throws IOException
    {
        //--- Get tag "img" attributes
        String sFieldId = tag.getId();
        String sImgId = null;
        if (notVoid(sFieldId))
        {
            sImgId = sFieldId.trim() + "_picto";
        }
        String sImgSrc = tag.getFullPictoImageSrc();
        String sImgClass = tag.getPictocl();
        String sImgOnCLick = tag.getOnpicto();

        //--- Generate tag <img ... />
        out.print("<img ");
        if (notVoid(sImgId))
        {
            out.print(" id=\"" + sImgId + "\"");
        }
        if (notVoid(sImgClass))
        {
            out.print(" class=\"" + sImgClass + "\"");
        }
        //--- Style
        String sStyle = "vertical-align:top;";
        if (tag.isDisabled())
        {
            sStyle = sStyle + "visibility:hidden;";
        }
        out.print(" style=\"" + sStyle + "\" ");
        //--- Image source
        out.print(" src=\"" + sImgSrc + "\"");
        //--- "onclick" event
        if (notVoid(sImgOnCLick))
        {
            out.print(" onclick=\"" + sImgOnCLick + "\"");
        }
        out.println(" />");
    }

    //----------------------------------------------------------------
    public void endTag(JspWriter out, GenericTag genericTag) throws IOException
    {
        Field tag = (Field) genericTag;
        //--- Register the current field via Javascript ...
        registerField(tag);
    }
    //----------------------------------------------------------------
    private void registerField(Field tag) throws IOException
    {
        if (needRegister(tag))
        {
            String sFirstArgs = firstArgsToRegister(tag);
            
            String sJavascript = null ;
            if ( tag.getInternaType() == Field.TYPE_DATE )
            {
                String sFormat = tag.getFormat() ;                    
                if ( sFormat == null )
                {
                    sFormat = "DD/MM/YY" ;  // TODO : external tag parameter 
                }
                sFormat = "\"" + sFormat + "\""; // no trim
                sJavascript = "RegisterFieldDate(" + sFirstArgs + ", " + sFormat + ")";
            }
            else
            {
                sJavascript = "RegisterField(" + sFirstArgs + ")";                    
            }
            tag.addJavascriptLine(sJavascript);
        }
    }
    //----------------------------------------------------------------
}