package org.objectweb.telosys.uil.taglib.widget.html;

import java.io.IOException;

import javax.servlet.jsp.JspWriter;

import org.objectweb.telosys.common.TelosysConst;
import org.objectweb.telosys.uil.taglib.widget.Form;
import org.objectweb.telosys.uil.taglib.widget.GenericTag;
import org.objectweb.telosys.uil.taglib.widget.IWidget;

/**
 * HTML generation for 'FORM' widget
 * 
 * @author Laurent GUERIN
 *  
 */

public class FormHTML extends GenericHTML implements IWidget
{
    //----------------------------------------------------------------
    public void startTag(JspWriter out, GenericTag genericTag) throws IOException
    {
        Form tag = (Form) genericTag;

        //--- Open the tag
        out.print("<form " 
                + getIdAttr(tag) + " " 
                + getClassAttr(tag) + " " 
                + getStyleAttr(tag) + " " 
                + getTitleAttr(tag) + " ");
        if (tag.getName() != null)
        {
            out.print(" name=\"" + tag.getName() + "\" ");
        }
        if (tag.getAction() != null)
        {
            out.print(" action=\"" + tag.getAction() + "\" ");
        }
        if (tag.getMethod() != null)
        {
            out.print(" method=\"" + tag.getMethod() + "\" ");
        }
        if (tag.getTarget() != null)
        {
            out.print(" target=\"" + tag.getTarget() + "\" ");
        }
        if (tag.getEnctype() != null)
        {
            out.print(" enctype=\"" + tag.getEnctype() + "\" ");
        }
        if (tag.getAccept() != null)
        {
            out.print(" accept=\"" + tag.getAccept() + "\" ");
        }
        if (tag.getAcceptCharset() != null)
        {
            out.print(" accept-charset=\"" + tag.getAcceptCharset() + "\" ");
        }

        //--- Events set for this tag
        tag.printEvents(out);
        out.println(" >");

        //--- Generate the hidden field "SCREEN_FORM_MARKER"
        generateHiddenMarkerTag(out, tag);
    }

    //----------------------------------------------------------------
    private void generateHiddenMarkerTag(JspWriter out, Form tag) throws IOException
    {
//        String sScreenName = tag.getScreenName();
//        if ( sScreenName == null )
//        {
//            // No Screen Name (direct acces without screenmap manager) => use the servlet class name
//            sScreenName = tag.getServletClassName();
//        }
//        //sScreenName = ( sScreenName != null ? sScreenName : "" ) ;
        String sScreenName = tag.getScreenMarkerName();
        int iScreenContextId = tag.getScreenContextId();
        String sScreenContextId = ( iScreenContextId >= 0 ? ""+iScreenContextId : "" ) ;
        
        String sFormName = tag.getName();
        sFormName = ( sFormName != null ? sFormName : "" ) ;
        
        String sPostback = "" ;
        if ( tag.isSessionPostback() || tag.isRequestPostback() )
        {
            sPostback = tag.getPostback() ;
        }
        String sMarker = sScreenName + ";" + sScreenContextId + ";" + sFormName + ";" + sPostback ;
        out.print("<input type=\"hidden\" ");
        out.print(" name=\"" + TelosysConst.SCREEN_FORM_MARKER + "\" ");
        out.print(" value=\"" + sMarker + "\" ");
        out.println(" />");
    }
    //----------------------------------------------------------------
    public void endTag(JspWriter out, GenericTag genericTag) throws IOException
    {
        //--- Close the tag
        out.print("</form>");
    }
    //----------------------------------------------------------------

}