package org.objectweb.telosys.uil.taglib.widget.html;

import org.objectweb.telosys.uil.taglib.widget.GeneratorAncestor;
import org.objectweb.telosys.uil.taglib.widget.GenericTag;

/**
 * Generic class for all HTML tags generators
 * 
 * @author Laurent GUERIN
 *  
 */
public abstract class GenericHTML extends GeneratorAncestor
{
//    public final static int     POS_NONE = 1;
//
//    public final static int     POS_XY   = 2;
//
//    public final static boolean USE_VAR  = true;


//    /**
//     * Returns true if the string is not null and not void
//     * 
//     * @param sTxt
//     * @return
//     */
//    protected boolean notVoid(String sTxt)
//    {
//        if (sTxt != null)
//        {
//            if (sTxt.trim().length() > 0)
//            {
//                return true; // not void
//            }
//        }
//        return false;
//    }

    /**
     * Returns a "label" tag with "for" attribute referencing the current tag Id. <br>
     * The label has no style, no class. Its Id is tag_id + "_label"
     * 
     * @param sId
     * @param sTxt
     * @return
     */
    protected String getHtmlLabelFor(String sId, String sTxt)
    {
        return "<label for=\"" + sId + "\"" + " id=\"" + sId + "_label\"" + " >" + sTxt + "</label>";
    }

//    /**
//     * Returns the html "id" attribute ( ie : 'id="xx"' )
//     * 
//     * @param tag
//     * @return 'id="xx"' or ''
//     */
//    protected String getIdAttr(GenericTag tag)
//    {
//        String sId = tag.getId();
//        if (sId != null)
//        {
//            return "id=\"" + sId + "\"";
//        }
//        else
//        {
//            return "";
//        }
//    }
    
//    /**
//     * Returns the html "id" attribute concat with index( ie : 'id="xx_nn"' )
//     * @param tag
//     * @param iIndex to concat 
//     * @return
//     */
//    protected String getIdAttr(GenericTag tag, int iIndex)
//    {
//        String sId = tag.getId();
//        if (sId != null)
//        {
//            return "id=\"" + sId + "_" + iIndex + "\"";
//        }
//        else
//        {
//            return "";
//        }
//    }

    /**
     * Returns the html "style" attribute ( ie : 'style="xx"' )
     * including absolute position if X or Y is set. 
     * @param tag
     * @return
     */
    protected String getStyleAttr(GenericTag tag)
    {
        return getStyleAttrWithPosition(tag);
    }

    /**
     * Returns the html "style" attribute ( ie : 'style="xx"' or '' )
     * 
     * @param tag
     * @return the style attribute string (never null)
     */
    protected String getStyleAttr(GenericTag tag, int iPosType)
    {
        if (iPosType == POS_XY)
        {
            return getStyleAttrWithPosition(tag);
        }
        else
        {
            return getStyleAttrWithoutPosition(tag);
        }
    }

    protected String getStyleAttrWithoutPosition(GenericTag tag)
    {
        //return "style=\"" + getStyleFragUserStyle(tag) + getStyleFragWidthHeight(tag) + "\"";
        String s = getStyleFragUserStyle(tag) + getStyleFragWidthHeight(tag) ;
        if ( s.trim().length() > 0 )
        {
            return "style=\"" + s + "\"";                    
        }
        else
        {
            return "";                                
        }
    }

    protected String getStyleAttrWithPosition(GenericTag tag)
    {
        if (tag.isPositionAbsolute())
        {
            //return "style=\"" + getStyleFragUserStyle(tag) + getStyleFragWidthHeight(tag) + getStyleFragPosition(tag) + "\"";
            String s = getStyleFragUserStyle(tag) + getStyleFragWidthHeight(tag) + getStyleFragPosition(tag) ;
            if ( s.trim().length() > 0 )
            {
                return "style=\"" + s + "\"";                    
            }
            else
            {
                return "";                                
            }
        }
        else
        {
            return getStyleAttrWithoutPosition(tag);
        }
    }

//    /**
//     * Returns the html "class" attribute ( ie : 'class="xx"' )
//     * 
//     * @param tag
//     * @return
//     */
//    protected String getClassAttr(GenericTag tag)
//    {
//        String sClass = tag.getCl();
//        if (sClass != null)
//        {
//            return "class=\"" + sClass + "\"";
//        }
//        else
//        {
//            return "";
//        }
//    }

    /**
     * Returns the html "title" attribute ( ie : 'title="xx"' )
     * 
     * @param tag
     * @return
     */
    protected String getTitleAttr(GenericTag tag)
    {
        String sTitle = tag.getTitle();
        if (sTitle != null)
        {
            return "title=\"" + sTitle + "\"";
        }
        else
        {
            return "";
        }
    }

//    /**
//     * Returns the html "tabindex" attribute ( ie : 'tabindex="xx"' )
//     * 
//     * @param tag
//     * @return
//     */
//    protected String getTabindexAttr(GenericTag tag)
//    {
//        String sTabIndex = tag.getTabindex();
//        if (sTabIndex != null)
//        {
//            return "tabindex=\"" + sTabIndex + "\"";
//        }
//        else
//        {
//            return "";
//        }
//    }

    /**
     * Returns the style fragment for user style ( ie : 'color:...; ' )
     * 
     * @param tag
     * @return the style frament (never null, ending by ';' if not void )
     */
    protected String getStyleFragUserStyle(GenericTag tag)
    {
        String sUserStyle = tag.getStyle();
        if (sUserStyle != null)
        {
            String s = sUserStyle.trim();
            //return (s.endsWith(";") ? s : s + ";");
            if ( s.length() > 0 )
            {
                if ( ! s.endsWith(";") )
                {
                    s = s + ";" ;
                }
            }
            return s ;
        }
        else
        {
            return "";            
        }
    }

    /**
     * Returns the style fragment for "width" and "height" <br>
     * ie : 
     * if at least x or y is set : 'position:absolute;left:..px;top:..px;' 
     * if x and y are not set for the tag : ''  ( void string )
     * 
     * @param tag
     * @return
     */
    protected String getStyleFragPosition(GenericTag tag)
    {
        if ( tag.isPositionAbsolute() )
        {
            String sX = tag.getXValue(USE_VAR); // never null
            String sY = tag.getYValue(USE_VAR); // never null
            return "position:absolute;left:" + sX.trim() + "px;top:" + sY.trim() + "px; ";            
        }
        return "";
    }

    /**
     * Returns the style fragment for "absolute position" ( ie : 'width:..px; height:..px;' )
     * 
     * @param tag
     * @return
     */
    protected String getStyleFragWidthHeight(GenericTag tag)
    {
        String sWidth = tag.getWidth();
        String sHeight = tag.getHeight();
        String s = "";
        if (sWidth != null)
        {
            s = s + "width:" + sWidth + "px;";
        }
        if (sHeight != null)
        {
            s = s + "height:" + sHeight + "px;";
        }
        return s;
    }

    protected boolean needRegister(GenericTag tag)
    {
        if (tag.hasId()) // Register only fields with an id
        {
            if (tag.hasZone() || tag.hasDataBinding())
            {
                return true ;
            }
        }
        return false ;
    }
    protected String firstArgsToRegister(GenericTag tag)
    {
        //--- 1 : Tag ID
        String sId = "\"" + tag.getId() + "\"";

        //--- 2 : Zone name
        String sZone = "null";
        if (tag.hasZone())
        {
            sZone = tag.getZone();
            if (sZone != null)
            {
                sZone = "\"" + sZone.trim() + "\"";
            }
        }

        //--- 3 & 4 : Data binding : XML element & XML attribute 
        String sElement = "null";
        String sAttribute = "null";
        if (tag.hasDataBinding())
        {
            sElement = tag.getDataElement();
            if (sElement != null)
            {
                sElement = "\"" + sElement.trim() + "\"";
            }
            sAttribute = tag.getDataAttribute();
            if (sAttribute != null)
            {
                sAttribute = "\"" + sAttribute.trim() + "\"";
            }
        }

        return sId + ", " + sZone + ", " + sElement + ", " + sAttribute ;
    }
}