package org.objectweb.telosys.uil.taglib.widget.html;

import java.io.IOException;

import javax.servlet.jsp.JspWriter;

import org.objectweb.telosys.uil.taglib.widget.GenericTag;
import org.objectweb.telosys.uil.taglib.widget.IWidget;
import org.objectweb.telosys.uil.taglib.widget.RadioItem;

/**
 * HTML generation for RadioItem widget 
 * 
 * @author Laurent GUERIN
 *  
 */
public class RadioItemHTML extends GenericHTML implements IWidget 
{

    //----------------------------------------------------------------

    public void startTag(JspWriter out, GenericTag genericTag) throws IOException
    {
        RadioItem tag = (RadioItem) genericTag ;
        
        //--- Get the "txt" attribute value
        //String sTxt = getAttributeValue(TAG_NAME, "txt", _sTxt);
        String sId = tag.getId() ;
        String sTxt = tag.getTxt() ;
        //String sStyle = tag.getStyle() ;
        String sTxtPos = tag.getTxtpos() ;

        //--- If there's a text for this radio item
        if (notVoid( sTxt ))
        {
            String sLabelForTag = getHtmlLabelFor( sId, sTxt );
            String sRadioItemTag = getRadioItemTag( tag, POS_NONE );
            
            out.print("<span style=\"" + getStyleFragPosition(tag) + "white-space:nowrap;display:block;\" >");
            if (sTxtPos.toLowerCase().startsWith("l"))
            {
                //--- Text position is "left"
                out.print(sLabelForTag);
                out.print(sRadioItemTag);
            }
            else
            {
                //--- Text position is "right" ( by default )
                out.print(sRadioItemTag);
                out.print(sLabelForTag);
            }
            out.print("</span>");
        }
        else
        {
            //--- No text => no label
//            out.print("<input type=\"radio\" ");
//            out.print(getIdAttr(tag)+" ");
//            out.print(getClassAttr(tag)+" ");
//            out.print(getStyleAttr(tag, POS_XY)+" ");
//            out.print(getTitleAttr(tag)+" "); 
//            out.print(getTabindexAttr(tag)+" "); 
//            
//            out.print(getRadioItemName(tag)+" ");
//            out.print(getValueAttr(tag));
            out.print( getRadioItemTag( tag, POS_XY ) );
        }
        out.println("");
    }

    private final String getRadioItemTag(RadioItem tag, int iPositionType)
    {
        return "<input type=\"radio\" " 
        + getIdAttr(tag)+ " "
        + getClassAttr(tag)+" "
        + getStyleAttr(tag, iPositionType)+" "
        + getTitleAttr(tag)+" " 
        + getTabindexAttr(tag)+" "
        
        + getRadioItemName(tag)+" "
        + getValueAttr(tag)+" "
        + getCheckedAttr(tag)+" "
        + getDisabledAttr(tag)+ " "        
        + " />" ;
    }

    private final String getValueAttr( RadioItem tag )
    {
        String sValue = tag.getValue();
        if (sValue != null)
        {
            return " value=\"" + sValue + "\"";
        }
        else
        {
            return "";
        }
    }

    private final String getRadioItemName( RadioItem tag )
    {
        String sName = tag.getName();
        if ( sName != null) // A specific name has been set => use it
        {
            return " name=\"" + sName + "\"";
        }
        else
        {
            sName = tag.getItemNameFromParent(); // The radio group name, or id if no name 
            if (sName != null)
            {
                return " name=\"" + sName + "\"";
            }
            else
            {
                return "";
            }
        }
    }

    private final String getCheckedAttr( RadioItem tag )
    {
        if ( tag.isSelected() )
        {
            return " checked=\"checked\"";
        }
        else
        {
            return "";
        }
    }
    private final String getDisabledAttr(RadioItem tag)
    {
        boolean bDisabled = false ;
        bDisabled = tag.isDisabled() ; // The disabled attribute of the radio item
        if ( ! bDisabled )
        {
            bDisabled = tag.isParentDisabled() ; // The disabled attribute of the radio group
        }
        if (bDisabled)
        {
            return " disabled=\"disabled\" " ;
        }
        else
        {
            return "";
        }
    }

    //----------------------------------------------------------------
    //----------------------------------------------------------------
    public void endTag(JspWriter out, GenericTag genericTag) throws IOException
    {
    }

}