package org.objectweb.telosys.uil.taglib.widget.html;

import java.io.IOException;

import javax.servlet.jsp.JspWriter;

import org.objectweb.telosys.uil.taglib.JSLines;
import org.objectweb.telosys.uil.taglib.widget.GenericTag;
import org.objectweb.telosys.uil.taglib.widget.IWidget;
import org.objectweb.telosys.uil.taglib.widget.Tab;

/**
 * HTML generation for Tab widget
 * 
 * @author Laurent GUERIN
 *  
 */
public class TabHTML extends GenericHTML implements IWidget
{
    //----------------------------------------------------------------
    public void startTag(JspWriter out, GenericTag genericTag) throws IOException
    {
        Tab tag = (Tab) genericTag;
        int iTabIndex = tag.nextTabIndex(); // From 0 to N 

        out.println("");
        out.print("<!-- Tab [" + iTabIndex + "] " );
        if ( tag.isSelected() )
        {
            out.print(" (selected) " );            
        }
        out.println("--> ");
        
        if ( iTabIndex > 0 ) // Not the first tab
        {
            if ( tag.hasInternalSpacers() )
            {
                generateSpacer(out, tag);                
            }
        }
        
        String sUnselectedClass = tag.getUnselectedClass() ;
        //--- TD tag
        out.print("<td ");
        out.print(getIdAttr(tag) + " ");
        if ( sUnselectedClass != null )
        {
            out.print("class=\"" + sUnselectedClass + "\" ");
        }
        out.print(getStyleAttrWithoutPosition(tag) + " ");
        //out.print(getTitleAttr(tag) + " ");
        //--- Specific "onselect" event
        String sOnSelect = tag.getOnselect();
        if (sOnSelect == null)
        {
            out.print("onclick=\"fwkSelectTab(this)\" ");            
        }
        else
        {
            out.print("onclick=\"if(fwkSelectTab(this)){ " + sOnSelect + " }\" ");            
        }
        out.print(">");

        String sTxt = tag.getTxt();
        if (sTxt != null)
        {
            out.print(sTxt);
        }

        out.println("</td>");

        //--- SCRIPT tag ( tab initialization )
        String sTabId = tag.getId();
        if (sTabId != null)
        {
            sTabId = "\"" + sTabId + "\"";
        }
        else
        {
            sTabId = "null";
        }
        String sPanelId = tag.getPanel();
        if (sPanelId != null)
        {
            sPanelId = "\"" + sPanelId + "\"";
        }
        else
        {
            sPanelId = "null";
        }
        
        String s = " fwkInitTab( " + sTabId + ", " + iTabIndex + ", " + sPanelId + ", telosysTabSet ); " ;
        JSLines.addLine(tag.getPageContext(), JSLines.TABBOX, s);
    }

    //---------------------------------------------------------------- 
    private void generateSpacer(JspWriter out, Tab tag) throws IOException 
    {
        String sSpacerClass = tag.getSpacerClass() ;
        if ( sSpacerClass != null )
        {
            out.println("<td class=\"" + sSpacerClass + "\"></td>");
        }
        else
        {
            out.println("<!-- No spacer class : cannot generate spacer -->");            
        }
    }
    //----------------------------------------------------------------
    public void endTag(JspWriter out, GenericTag genericTag) throws IOException
    {
    }
    //----------------------------------------------------------------
}