package org.objectweb.telosys.util;

import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;

import org.objectweb.telosys.common.TelosysClassLogger;
import org.objectweb.telosys.common.TelosysException;
import org.objectweb.telosys.common.TelosysRuntimeException;

/**
 * @author Laurent GUERIN
 *
 */
public class BeanUtil
{
    private static final TelosysClassLogger $log = new TelosysClassLogger(BeanUtil.class);

    private static Method findMethod(Class beanClass, String sMethodName)
    {
        Method meth = null;
        try
        {
            meth = beanClass.getMethod(sMethodName, null);
        } catch (SecurityException e)
        {
            throw new TelosysRuntimeException("findMethod : security violation ", e);
            //return null;
        } catch (NoSuchMethodException e)
        {
            return null; // Not found
        }
        return meth;
    }

    private static Object callGetterMethod(Method method, Object oBean)
    {
        try
        {
            return method.invoke(oBean, null);
        } catch (IllegalArgumentException e)
        {
            throw new TelosysRuntimeException("callGetterMethod : IllegalArgumentException ", e);
            //return null;
        } catch (IllegalAccessException e)
        {
            throw new TelosysRuntimeException("callGetterMethod : IllegalAccessException ", e);
            //return null;
        } catch (InvocationTargetException e)
        {
            throw new TelosysRuntimeException("callGetterMethod : InvocationTargetException ", e);
            //return null;
        }
    }

    /**
     * Returns the value of the given bean field using reflection
     * @param oBean : the bean instance 
     * @param sFieldName : the field name 
     * @return : the value found 
     * @throws TelosysException if the field name is unknown or if arguments are invalid
     */
    public static Object getFieldValue(Object oBean, String sFieldName) throws TelosysException
    {
        if (oBean == null)
        {
            throw new TelosysException("getFieldValue : bean instance is null ");
        }
        if (sFieldName == null)
        {
            throw new TelosysException("getFieldValue : field name is null ");
        }

        //--- Build the "method field name" (1rst char Upper Case)
        String sFieldName2 = sFieldName.trim();
        char cFirstChar = sFieldName2.charAt(0);
        if (Character.isLowerCase(cFirstChar))
        {
            String s1 = sFieldName2.substring(0, 1);
            String s2 = sFieldName2.substring(1);
            sFieldName2 = s1.toUpperCase() + s2;
        }

        Class beanClass = oBean.getClass();
        Method getter = null;
        //--- Try to find a "getXxxx" getter method
        getter = findMethod(beanClass, "get" + sFieldName2);
        if (getter != null)
        {
            return callGetterMethod(getter, oBean);
        }
        else
        {
            //--- Try to find a "isXxxx" getter method
            getter = findMethod(beanClass, "is" + sFieldName2);
            if (getter != null)
            {
                System.out.println("get" + sFieldName2 + " : found ");
                return callGetterMethod(getter, oBean);
            }
            else
            {
                //--- Unknown field
                throw new TelosysException("getFieldValue : unknown field ");
            }
        }
    }
}
