package org.objectweb.telosys.util;

import java.lang.reflect.Field;

import org.objectweb.telosys.common.TelosysException;

/**
 * @author Laurent GUERIN
 * 
 */
public final class ClassUtil
{
    /**
     * No constructor for this class !
     */
    private ClassUtil()
    {
    }

    //-----------------------------------------------------------------------------------
    /**
     * Return the value of a static field for a given class.
     * @param cl : the class
     * @param sFieldName : the field name to find in the class
     * @return : the field value
     * @throws TelosysException
     */
    public static Object getFieldValue ( Class cl, String sFieldName ) throws TelosysException
    {
        String sErrMsg = "getStaticFieldValue(" + cl.getName() + ", " + sFieldName + ") : ";
        try
        {
            Field field = cl.getDeclaredField(sFieldName);
//            Class clType = field.getType();
//            System.out.println("Field type = " + clType.getName() );
//            Object objValue = field.get(null) ;
//            System.out.println("Field value = " + objValue );
//            return objValue ;            
            return field.get(null) ;
        } catch (SecurityException e)
        {
            throw new TelosysException( sErrMsg+e.getMessage(), e );
        } catch (NoSuchFieldException e)
        {
            throw new TelosysException( sErrMsg+e.getMessage(), e );
        } catch (IllegalArgumentException e)
        {
            throw new TelosysException( sErrMsg+e.getMessage(), e );
        } catch (IllegalAccessException e)
        {
            throw new TelosysException( sErrMsg+e.getMessage(), e );
        }
        //return null ;        
    }
}