package org.objectweb.telosys.util;

import java.text.SimpleDateFormat;
import java.util.Calendar;

/**
 * @author Laurent GUERIN
 *  
 */
public final class DateUtil
{
    /**
     * Comment for <code>sDefaultFormat</code>
     */
    private static final String DEFAULTFORMAT = "dd/MM/yyyy";

    /** */
    private DateUtil()
    {
    }

    /**
     * 
     * @return java.sql.Date :
     */
    public static java.sql.Date todaySqlDate()
    {
        //--- Date du jour
        return new java.sql.Date(Calendar.getInstance().getTimeInMillis());
    }

    /**
     * 
     * @return java.util.Date :
     */
    public static java.util.Date todayUtilDate()
    {
        //--- Date du jour
        return new java.util.Date(Calendar.getInstance().getTimeInMillis());
    }

    /**
     * 
     * @param sFormat :
     * @return String :
     */
    public static String todayString(final String sFormat)
    {
        //--- Date du jour
        java.util.Date date = new java.util.Date(Calendar.getInstance().getTimeInMillis());
        SimpleDateFormat dateFormat = new SimpleDateFormat(sFormat);
        return dateFormat.format(date);
    }

    /**
     * @param iJ :
     * @param iM :
     * @param iA :
     * @param sFormat :
     * @return :
     */
    public static String format(final int iJ, final int iM, final int iA, final String sFormat)
    {
        java.util.Date date = getUtilDate(iJ, iM, iA);
        SimpleDateFormat dateFormat = new SimpleDateFormat(sFormat);
        return dateFormat.format(date);
    }

    /**
     * @param iJ :
     * @param iM :
     * @param iA :
     * @return : date formate
     */
    public static String format(final int iJ, final int iM, final int iA)
    {
        return format(iJ, iM, iA, DEFAULTFORMAT);
    }

    /**
     * 
     * @param iJour :
     * @param iMois :
     * @param iAnnee :
     * @return java.sql.Date :
     */
    public static java.sql.Date getSqlDate(final int iJour, final int iMois, final int iAnnee)
    {
        Calendar cal = Calendar.getInstance();
        cal.set(iAnnee, iMois - 1, iJour);
        return new java.sql.Date(cal.getTimeInMillis());
    }

    /**
     * 
     * @param ts :
     * @return java.sql.Date :
     */
    public static java.sql.Date getSqlDate(final java.sql.Timestamp ts)
    {
        if (ts != null)
        {
            return new java.sql.Date(ts.getTime());
        }
        else
        {
            return null;
        }
    }

    /**
     * 
     * @param iJour :
     * @param iMois :
     * @param iAnnee :
     * @return java.util.Date :
     */
    public static java.util.Date getUtilDate(final int iJour, final int iMois, final int iAnnee)
    {
        Calendar cal = Calendar.getInstance();
        cal.set(iAnnee, iMois - 1, iJour);
        return new java.util.Date(cal.getTimeInMillis());
    }

    /**
     * 
     * @param date :
     * @return int :
     */
    public static int getDay(final java.sql.Date date)
    {
        Calendar cal = Calendar.getInstance();
        cal.setTime(date);
        return cal.get(Calendar.DAY_OF_MONTH);
    }

    /**
     * 
     * @param date :
     * @return int :
     */
    public static int getMonth(final java.sql.Date date)
    {
        Calendar cal = Calendar.getInstance();
        cal.setTime(date);
        return cal.get(Calendar.MONTH) + 1;
    }

    /**
     * 
     * @param date :
     * @return int :
     */
    public static int getYear(final java.sql.Date date)
    {
        Calendar cal = Calendar.getInstance();
        cal.setTime(date);
        return cal.get(Calendar.YEAR);
    }

    public static String toString(final java.util.Date date)
    {
        if (date != null)
        {
            SimpleDateFormat dateFormat = new SimpleDateFormat(DEFAULTFORMAT);
            return dateFormat.format(date);
        }
        else
        {
            return "null";
        }
    }

}