package org.objectweb.telosys.util;

/**
 * @author Laurent GUERIN
 * 
 */
public final class StrUtil
{
    /** */
    private StrUtil()
    {
    }

    //-----------------------------------------------------------------------------------
    /**
     * @param sVal :
     * @return boolean :
     */
    public static boolean getBoolean(final String sVal)
    {
        return sVal.equals("1");
    }

    //-----------------------------------------------------------------------------------
    /**
     * @param sVal :
     * @param iMaxLength :
     * @return String :
     */
    public static String getLimitedString(final String sVal, final int iMaxLength)
    {
        if (sVal != null)
        {
            if (sVal.length() <= iMaxLength)
            {
                return sVal;
            }
            else
            {
                return sVal.substring(0, iMaxLength);
            }
        }
        else
        {
            return "";
        }
    }

    //-----------------------------------------------------------------------------------
    /**
     * @param sVal :
     * @return String : chaine avec caractres protgs
     */
    public static String getProtectedString(final String sVal)
    {
        if (sVal != null)
        {
        	StringBuffer sbNew = new StringBuffer(100);
        	char c = '\0' ;
        	for ( int i=0 ; i < sVal.length() ; i++ )
        	{
        		c = sVal.charAt(i) ;
        		if ( c == '"')
        		{
        			sbNew.append("&quot;"); //--- Protection par '\' 
        		}
        		else
        		{
        			sbNew.append(c);
        		}
        	}
            return sbNew.toString();
        }
        else
        {
            return "";
        }
    }

    //-----------------------------------------------------------------------------------
    /**
     * @param s :
     * @return String :
     */
    public static String notNull(final String s)
    {
        if (s != null)
        {
            return s;
        }
        else
        {
            return "";
        }
    }

    /**
     * @param sVal : String value to convert 
     * @param iDefaultValue : Default value if the string is null or contains an invalid integer value
     * @return
     */
    public static int getInt(final String sVal, final int iDefaultValue )
    {
        if (sVal == null)
        {
            return iDefaultValue ;
        }

        try
        {
            return Integer.parseInt(sVal);
        }
        catch (NumberFormatException ex)
        {
            return iDefaultValue;
        }
    }
    /**
     * 
     * @param sVal :
     * @return int :
     */
    public static int getInt(final String sVal)
    {
        return getInt(sVal,0);
    }

    /**
     * @param sVal : String value to convert 
     * @param dDefaultValue : Default value if the string is null or contains an invalid value
     * @return
     */
    public static double getDouble(final String sVal, final double dDefaultValue )
    {
        if (sVal == null)
        {
            return dDefaultValue ;
        }

        try
        {
            return Double.parseDouble(sVal);
        }
        catch (NumberFormatException ex)
        {
            return dDefaultValue;
        }
    }
    /**
     * 
     * @param sVal :
     * @return int :
     */
    public static double getDouble(final String sVal)
    {
        return getDouble(sVal, 0.0);
    }

    /**
     * Return a date instance from a date in French format 
     * @param sDate : date in French format ( "DD/MM/YYYY" ) 
     * @return java.sql.Date : the date instance ( or null is the input date is not valid )
     */
    public static java.sql.Date getDateFR(final String sDate)
    {
        if (sDate == null)
        {
            return null;
        }

        if (sDate.trim().equals(""))
        {
            return null;
        }

        if (sDate.length() != 10)
        {
            return null;
        }

        try
        {
            //--- Convert Date Format : "DD/MM/YYYY" -> "YYYY-MM-DD"
            String sYYYYMMDD = sDate.substring(6, 9 + 1) + "-" + sDate.substring(3, 4 + 1) + "-"
                    + sDate.substring(0, 1 + 1);
            //--- Convert ISO String date to SqlDate 
            return java.sql.Date.valueOf(sYYYYMMDD);
        }
        catch (Exception ex)
        {
            return null;
        }
    }

    /**
     * Return a date instance from a date in ISO format 
     * @param sDate : date in ISO format ( "YYYY-MM-DD" ) 
     * @return java.sql.Date : the date instance ( or null is the input date is not valid )
     */
    public static java.sql.Date getDateISO(final String sDate) // "YYYY-MM-DD"
    {
        if (sDate == null)
        {
            return null;
        }
    
        if (sDate.trim().equals(""))
        {
            return null;
        }
    
        if (sDate.length() != 10)
        {
            return null;
        }
    
        try
        {
            //--- Convert String date to SqlDate 
            return java.sql.Date.valueOf(sDate); 
        }
        catch (Exception ex)
        {
            return null;
        }
    }


}