package org.objectweb.telosys.util;

import java.io.File;
import java.io.IOException;
import java.text.SimpleDateFormat;
import java.util.Date;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;

import org.w3c.dom.Document;
import org.xml.sax.SAXException;

/**
 * @author Laurent GUERIN
 *
 */
public final class XmlUtil
{
//    private final static String DOM_PARSER  = "javax.xml.parsers.DocumentBuilderFactory" ;
//    private final static String DOM_CRIMSON = "org.apache.crimson.jaxp.DocumentBuilderFactoryImpl" ;
//    private final static String DOM_XERCES  = "org.apache.xerces.jaxp.DocumentBuilderFactoryImpl" ;

	/**
	 * Comment for <code>$dateFormatter</code>
	 * The formatter to convert a Java Date to Xml String 
	 * International standard : ISO 8601 ( "yyyy-MM-dd" )
	 */
	private static SimpleDateFormat     $dateFormatter  = new SimpleDateFormat ("yyyy-MM-dd" ) ;
    
    /** */
    private XmlUtil()
    {
    }
    
    /**
     *  Convert "Standard String" to "Xml String"
     * 
     * @param sStdString :
     * @return String :
     */ 
    public static String xmlString(final String sStdString)
    {
        if (sStdString == null)
        {
            return "";
        }
        
        char c;
        StringBuffer sb = new StringBuffer(sStdString.length() + 40);
        for (int i = 0; i < sStdString.length(); i++)
        {
            c = sStdString.charAt(i);
            switch (c)
            {
                case '<':
                    sb.append("&lt;");
                    break;
                case '>':
                    sb.append("&gt;");
                    break;
                case '&':
                    sb.append("&amp;");
                    break;
                case '\"':
                    sb.append("&quot;");
                    break;
                case '\'':
                    sb.append("&apos;");
                    break; // Utilit  vrifier
                default:
                    sb.append(c);
                    break;
            }
        }
        return sb.toString();
    }

    public static String xmlDate(final Date date)
    {
        if (date == null)
        {
            return "";
        }
        return $dateFormatter.format(date);
    }

    public static String xmlBool(final boolean b)
    {
        if ( b )
        {
            return "1";
        }
        else
        {
            return "0";            
        }
    }

    // Convert "Xml String" to "Standard String"
    /**
     public static String stdString(String sXmlString)
     {
     if ( sXmlString == null ) return "" ;
     return "" ;
     }
     **/
    
    /**
     * Load an XML file as a "DOM document" with the default JAXP DOM Parser
     * @param sFileName : the XML file to load
     * @return : reference on the document loaded (or null if an error occurs)
     */
    public static Document loadFile(String sFileName )
    {
        //--- Create a new factory instance via JAXP
        DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
        
        //--- Configuring the Factory ...
        //factory.setValidating(true);
        //factory.setNamespaceAware(true);        

        //System.out.println("JAXP factory class = " + factory.getClass().getName() );
        
        return loadFile(sFileName, factory );
    }
    
//    /**
//     * Load an XML file as a "DOM document" (with a DOM Parser)
//     * @param sFileName : the XML file to load
//     * @param sParserImpl : the class name of the JAXP implementation
//     * @return : reference on the document loaded (or null if an error occurs)
//     */
//    public static Document loadFile(String sFileName, String sParserImpl)
//    {
//        //--- Set the JAXP system propety ( NB: change the property for all the JVM ! )
//        System.setProperty( DOM_PARSER , sParserImpl  );
//        //--- Create a new factory instance
//        DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
//        //--- Configuring the Factory ...
//        //factory.setValidating(true);
//        //factory.setNamespaceAware(true);        
//
//        return loadFile(sFileName, factory );
//    }
    
    /**
     * Load an XML file as a "DOM document" using a JAXP factory
     * @param sFileName : the XML file to load
     * @param factory : the JAXP Document Builder Factory
     * @return : reference on the document loaded (or null if an error occurs)
     */
    public static Document loadFile(String sFileName, DocumentBuilderFactory factory )
    {
        Document document = null;
        try
        {
            //--- Create a DOM parser ( instance of a builder )
            DocumentBuilder builder = factory.newDocumentBuilder();
            //--- Parse the file
            document = builder.parse(new File(sFileName));
        } catch (SAXException ex)
        {
            //--- Error generated during parsing
            ex.printStackTrace();
            document = null ;
        } catch (ParserConfigurationException ex)
        {
            //--- Parser with specified options can't be built
            ex.printStackTrace();
            document = null ;
        } catch (IOException ex)
        {
            //--- I/O error  ( FileNotFoundException, etc ... )
            ex.printStackTrace();
            document = null ;
        }
        return document ;
    }
    
}