package org.objectweb.telosys.util.web;

import java.io.IOException;
import java.io.PrintWriter;
import java.util.ListIterator;

import javax.servlet.RequestDispatcher;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.objectweb.telosys.common.Telosys;

/**
 * @author Laurent GUERIN
 *  
 */
public final class WebUtil
{

	/** */
	private WebUtil ()
	{
	}

	/**
	 * 
	 * @param sUrl :
	 * @param request :
	 * @param response :
	 * @return boolean :
	 * @throws ServletException :
	 * @throws IOException :
	 */
	public static boolean forward(final String sUrl, final HttpServletRequest request, final HttpServletResponse response) throws ServletException, IOException
	{
		RequestDispatcher rd = request.getRequestDispatcher(sUrl);
		if (rd != null)
		{
			rd.forward(request, response);
			return true;
		}
		return false;
	}

	/**
	 * @param sUrl : "HTTP redirection" to an other URL ( HTTP code "302" )
	 * @param response :
	 * @return boolean : true = OK, false = ERR
	 */
	public static boolean redirect(final String sUrl, final HttpServletResponse response)
	{
		if (sUrl != null)
		{
			try
			{
				response.sendRedirect(sUrl);
				return true ;
			}
			catch (Exception ex)
			{
			    Telosys.error( "redirect : Exception : " + ex.getMessage());
			}
		}
		else
		{
		    Telosys.error( "redirect : URL is null !");
		}
		return false ;
	}

	public static String getRootUrl(HttpServletRequest request)
	{
		/*
		return  "http://" + request.getLocalName() 
		+ ":" + request.getLocalPort() 
		+ request.getContextPath() + "/";
		*/
		
		// "context/dir/file.xx"
		String sUri = request.getRequestURI() ; 
		// "http://xxx:80/context/dir/file.xx"
		String sUrl = request.getRequestURL().toString() ; 
		//int i = sUrl.indexOf(sUri) ;
		int d = sUrl.length() - sUri.length() ;
		return sUrl.substring(0,d) + request.getContextPath() + "/";
	}

	/**
	 * 
	 * @param response :
	 * @param sMsg :
	 * @throws ServletException :
	 * @throws IOException :
	 */
	public static void info(final HttpServletResponse response, final String sMsg) throws ServletException, IOException
	{
		response.setContentType("text/html");
		PrintWriter out = response.getWriter();
		out.println("<html><body><span style='font: 8pt Tahoma;'>");
		out.println("<b>" + sMsg + "</b>");
		out.println("</span></body></html>");
	}

	/**
	 * 
	 * @param response :
	 * @param sMsg :
	 * @throws ServletException :
	 * @throws IOException :
	 */
	public static void error(final HttpServletResponse response, final String sMsg) throws ServletException, IOException
	{
		response.setContentType("text/html");
		PrintWriter out = response.getWriter();
		out.println("<html><body><span style='font: 8pt Tahoma;'>");
		out.println("<b>ERROR</b><br>");
		out.println("<b>" + sMsg + "</b>");
		out.println("</span></body></html>");
	}

	/**
	 * @param response :
	 * @param sMsg :
	 * @param iter :
	 * @throws ServletException :
	 * @throws IOException :
	 */
	public static void error(final HttpServletResponse response, final String sMsg, final ListIterator iter) throws ServletException, IOException
	{
		response.setContentType("text/html");
		PrintWriter out = response.getWriter();
		out.println("<html><body><span style='font: 8pt Tahoma;'>");
		//out.println("<b>ERREUR</b><br>");
		out.println("<b>" + sMsg + "</b><br>");
		String s = null;
		if (iter != null)
		{
			while (iter.hasNext())
			{
				s = (String) iter.next();
				out.println(" - " + s + "<br>");
			}
		}
		out.println("</span></body></html>");
	}
	public static void msgPage(HttpServletResponse response, String sTitle, String sMsg)
	{
		try
		{
			response.setContentType("text/html");
			PrintWriter out = response.getWriter();
			out.println("<html><body>");
			out.println("<h2>"+sTitle+"</h2>");
			out.println("<h4>"+sMsg+"</h4>");
			out.println("</body></html>");
			out.close();
		}
		catch (IOException e)
		{
		    Telosys.error( "Cannot get Writer from HttpResponse !");
		    Telosys.error( "Exception : " + e.getMessage());
		}
	}

}